Package: lazy.tools
Type: Package
Title: Collection of tools for lazy boys and girls
Version: 1.0.0.20260516
Author: Shin-ichi Mayekawa <mayekawa@nifty.com>
Maintainer: Shin-ichi Mayekawa <mayekawa@nifty.com>
Description: Collection of tools for lazy boys and girls.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 4.2.0)
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-16 10:39:19 UTC; mayekawa

# Generated by roxygen2: do not edit by hand

export("%&%")
export("%&&%")
export("%//%")
export("%|%")
export("%||%")
export("put_val<-")
export("put_valL<-")
export(And)
export(Exists)
export(Grep)
export(Not)
export(Or)
export(Print)
export(Printb)
export(Printm)
export(Quantile)
export(ReadLines)
export(Seq)
export(Unique)
export(WriteLines)
export(add_paren)
export(analyze_three_dots)
export(brk2midp)
export(cards)
export(cchar)
export(ceiling_n)
export(char2num)
export(check_partial_match)
export(comb)
export(comments)
export(complex_index)
export(compress_freqdist)
export(convert)
export(cprod)
export(cumsuminv)
export(decp)
export(demomat)
export(design_expand)
export(distancew)
export(distancew0)
export(distrectw)
export(dummy_compress)
export(dummy_expand)
export(eq)
export(etime)
export(expand.grid1)
export(expand_freqdist)
export(find_calling_functions)
export(find_functions)
export(freqc)
export(freqdist)
export(freqdist1)
export(from_named_list)
export(fuzz)
export(fxtabs)
export(gen01pat)
export(gen_midp)
export(gen_test_score)
export(gen_test_score_CTT)
export(generate_AndOr)
export(generate_zenkaku_sets)
export(get_range)
export(get_val)
export(get_valL)
export(get_valLL)
export(graph2pdf)
export(idplot)
export(inspect_RDS)
export(inspect_RData)
export(interpol)
export(iotable)
export(is.DBCS)
export(is.MBCS)
export(is.number)
export(is.varname)
export(jfreq)
export(keepdrop)
export(locbalpar)
export(locbalpar2)
export(locbalq)
export(loceq)
export(logistic)
export(logistic1)
export(logit)
export(logit1)
export(lv)
export(makeStack)
export(mandd)
export(mands)
export(merge2)
export(merge_df)
export(merge_tables)
export(midp2brk)
export(named_list)
export(nbyte)
export(nchar2)
export(new_name)
export(normalize_config)
export(num2char)
export(order2)
export(package_description)
export(package_info)
export(perm)
export(plot_freqdist)
export(plot_interval)
export(pplot)
export(pref2dist)
export(princ)
export(printb)
export(printm)
export(procote)
export(radarchart)
export(recode)
export(reflect_fa)
export(remove_paren)
export(reorder_fa)
export(replacec)
export(revchar)
export(revstr)
export(rmv)
export(smoothra)
export(sort_missing_data)
export(sortit)
export(stack_df)
export(standard)
export(str_olay)
export(substr2)
export(substrDBCS)
export(table2freqdist)
export(testregex)
export(trim)
export(varlist)
export(vartype)
export(was)
export(wquantile)
export(yearJAD)

#' Horizontally Stack Data Frames with Different # of Observations
#'
#' @param ... vectors, matrices or data frames
#' @param simplify = 0 not to convert the result to a matrix if possible.
#'
#' @details
#' Using the data frames in the example section, the core part is:\cr
#' \code{ res <- merge( data.frame(A1m,id=1:nrow(df1m))
#'                , data.frame(A2m,id=(1:nrow(df2m))), by="id" ,all=1) }\cr
#'
#' \cr
#' If simplify=1 is give, if possible, the result will be converted to
#' a matrix.
#'
#' @examples
#' # no overlapping col names
#' merge_df( x=1:2, y=1:5, z=1 )
#'
#' # overlapping col names
#' A <- demomat(5,4,3,cnroot="V")
#' A1 <- A[,,1]; A2 <- A[,,2]; A3 <- A[,,3]
#' A1m <- A1[1:3,1,drop=0]; A2m <- A2[1:2,1:2]; A3m <- A3[1:3,1:3];
#' merge_df( A1m, A2m, A3m )
#'
#' @return
#' A data frame or matrix
#'
#' @export
#'

merge_df <- function( ..., simplify=1 ){
 # horizontally merge objects of unequal # of rows
 # Shin-ichi Mayekawa
 # 20171111
 # all() added to |||: 20190605
 #

 # analyze three dots
 temp=analyze_three_dots( ... )
 name=temp$name
 val=temp$val
 n=length(name)


 # make them a data frame
 if( !is.data.frame(val[[1]]) ){
  val[[1]]=as.data.frame(val[[1]], stringsAsFactors=0, optional=1)
  if( is.null(colnames(val[[1]]))  || all(colnames(val[[1]])=="val[[1]]") )
   colnames(val[[1]])=name[[1]]
 }
 res=data.frame( val[[1]], key___key______=1:nrow(val[[1]])
                 , stringsAsFactors=0, check.names=0 )
 cname=colnames(val[[1]])
 if( n >= 2 ){
  for( i in 2:n ){
   if( !is.data.frame(val[[i]]) ){
    val[[i]]=as.data.frame(val[[i]], stringsAsFactors=0, optional=1)
    if( is.null(colnames(val[[i]])) || all(colnames(val[[i]])=="val[[i]]") )
     colnames(val[[i]])=name[[i]]
   }
   val[[i]]=data.frame( val[[i]], key___key______=(1:nrow(val[[i]]))
                        , stringsAsFactors=0, check.names=0 )
   # Print(i,res,val[[i]])
   res=merge( res, val[[i]], by="key___key______", all=1 )
   cname=unique( c(cname,colnames(val[[i]])) )
  }
 }

 res=res[colnames(res)!="key___key______"]

 # make the result matrix
 if( simplify ){
  vt=vartype(res)
  if( all(vt==vt[1]) ) res=as.matrix(res)
 }

 return( res )

} # merge_df

#' Frequency distribution of multivariate numeric data with case weight
#'
#' @param x numeric vector or column vector
#' @param weights case weight
#' @param npoints # of mid-points for each variable
#' @param min minimum value of mid-points
#' @param max maximum value of mid-points \cr
#'  For the multivariate x, (npoints, min, max) can be vectors \cr
#'  specifying different numbers for each variable.
#' @param prob = 1 to return relative frequency in stead of frequency
#' @param rev = 1 to change the last element of x first.
#' @param omitmiss = 1 to omit the missing grids
#' @param midpoints a list of the vectors of predefied mid-point values \cr
#'  For the multivariate x, midpoints defines the midpoints of
#'  each variable,  not their Cartesian products.
#' @param class = 1 to  output the classification of each observation
#' to the mid-point.
#' @param title tile to be used in plot
#' @param maxf max value of frequency for plot
#' @param wid Format of mid-point value (wid.dig)
#' @param dig Format of mid-point value (wid.dig)
#' @param digits Format for mid-points
#' @param print = 1 to print result
#' @param plot = 1 to plot historgam
#' @return A list consisting of \cr
#'   midpoints=  list of midpoints \cr
#'   midpoints1= list of midpoints with lost values \cr
#'   midpindex   cartesian product of midpoints number \cr
#'   grid=       cartesian product of midpoints \cr
#'               grid[,j]=midpoints[[j]][midpindex[,j]], j=1,2,...,nvar \cr
#'   freq=       column vector of frequency at midpoints \cr
#'   breaks=     breaks associated with the midpoints \cr
#'   class=      classification of each obs into the midpoints if class=1. \cr
#'   classc=     classification of each obs into the midpoints class
#'    if class=1 . \cr
#'   nobs= sum of weights\cr
#' @details
#'   (npoints, min, max)  defines the mid-points. \cr
#'    If min or max is NULL, min(x) and max(x) will be used. \cr
#'    midpoints has priority over (npoints, min, max).\cr
#' The out of range observations,  that is, those x satisfying \cr
#'  x < midpoints[1]     or   midpoints[npoints] < x \cr
#'  will be be included in midpoints[1] and midpoints[npoints], resp. \cr
#' \cr
#' Try this to see how the obs on the border are classified. \cr
#'    xx=c(1,2,3,4,5,6,7) \cr
#'    brks=c(0,2,4,6,8) \cr
#'    cc=cut(xx,brks) \cr
#'
#' @examples
#' set.seed(1701)
#' n <- 1000
#' X <- matrix(rnorm(2*n),n,2)
#' res <- freqdist( X, npoints=7,min=-3,max=3, print=1, weights=1, plot=1)
#'
#' X <- expand.grid(seq(-3,3),seq(-3,3))
#' weight <- round(n*exp(-0.5*rowSums(X^2)))
#' res1 <- freqdist( X, weight, npoints=c(7,5),min=-3,max=3, print=1, plot=1)
#' res2 <- freqdist( X, weight, npoints=c(7,5),min=-3,max=3, print=1, class=1)
#' str(res2)
#'
#' @export
#'

freqdist <- function( x, weights=1, npoints=NULL, min=NULL, max=NULL, prob=0
                      , wid=6, dig=1, maxf=NULL, rev=0, class=0, omitmiss=0
                      , midpoints=NULL, title=NULL, digits=3, print=0, plot=0 ){
 # freq dist of x using midpoints
 # Shin-ichi Mayekawa
 # 20121026
 # out-of-range observations: 121219
 # return freq: 130218
 # weights with xtabs: 130218
 # multivariate version: 130218
 # revised: 20140221,26
 # unlist midpoints when nvar=1: 140417
 # title as an arg: 140417
 # bugfix: 20140524,25
 # recover lost midpoints after cut: 20140525
 # all(wieghts==1): 20140812 dnc
 # bugfix: 20140812 dnc
 # matrix weights: 20140813,14,15 dnc
 # rev: 20140816
 # class and omitmiss: 20140816
 # error check: 20150608
 # bugfix: 20150822
 # output classc: 20241212
 #
 # Args:
 #
 #    x            a variable to be tabulated
 #    weights      assosiated weight vector or 1
 #                 Can be a matrix of multiple columns.
 #
 #    rev         = 1 to change the last element of x first.
 #    omitmiss    = 1 to omit the missing grids
 #
 #    (npoints, min, max)    definition of midpoints
 #                 If min or max is NULL, min(x) and max(x) will be used.
 #                 For the multivariate x, (npoints, min, max) can be vectors.
 #
 #     midpoints   list of midpoints of each column of x
 #                 If midpoints is NULL, (npoints, min, max) will be used.
 #
 #                 For the multivariate x, midpoints defines the midpoints
 #                 of each variable,  not their cartesian products.
 #
 #     prob      = 1 to return the relative frequency, not raw frequency
 #
 #     maxf      = maximum value of the vertical axis for univariate x
 #
 #     (wid, dig) formatC for the midpoints in plot
 #
 #     title      plot title
 #
 #     class     = 1 to output classification
 #
 #
 # Values:
 #
 #   midpoints   list of midpoints
 #   midpoints1  list of actual midpoints (lost points excluded)
 #   midpindex   cartesian product of midpoints number
 #   grid        cartesian product of midpoints
 #               grid[,j]=midpoints[[j]][midpindex[,j]], j=1,2,...,nvar
 #   freq        matrix of frequency at grid
 #   breaks      breaks associated with the midpoints
 #   class       classification of each obs into the grid if class=1
 #
 #
 # Note:
 #  The out of range observations,  that is, those x satisfying
 #     x < midpoints[1]     or   midpoints[npoints] < x
 #  will be be included in midpoints[1] and midpoints[npoints], resp.
 #
 #
 #  When rev=1, native contour does not produce correct plot
 #  because freq is not in the order of outer product of midpoints.
 #  Use the levelplot function of lattice package as:
 #
 #   res <- freqdist( ........ )
 #   levelplot( res$freq ~ res$grid[,1]*res$grid[,2], contour=1, cut=7
 #            , region=FALSE )
 #
 #
 # Needs:
 #
 #    vartype
 #
 #
 
 
 # check if x is numeric
 if( any(vartype(x) != "numeric") ){
  cat("\n\nerror1:(freqdist) variables must be numeric.\n\n")
  return(NULL)
 }
 
 # length
 if( is.vector(x) ) x=matrix(x,,1)
 n=nrow(x)
 nvar=ncol(x)
 if( is.null(rownames(x)) ) rownames(x)=paste("r",1:n,sep="")
 xname=colnames(x)
 if( is.null(xname) ) xname=paste("var",1:nvar,sep="")
 
 
 # univariate
 if( nvar == 1 ){
  res=freqdist1( x=x, weights=weights, npoints=npoints
                 , min=min, max=max, prob=prob
                 , wid=wid, dig=dig, maxf=maxf
                 , midpoints=midpoints, title=title
                 , digits=digits, print=print, plot=plot )
  return( res )
 }
 
 # define midpoints
 if( !is.null(midpoints) ){
  # midpoints given
  if( !is.list(midpoints) ){
   midpoints=list(midpoints)
   midpoints=rep(midpoints,nvar)
  }
  else if( length(midpoints) == 1 ) midpoints=rep(midpoints,nvar)
  npoints=sapply(midpoints,function(x) sum(!is.na(x)))
  min=sapply(midpoints,min); max=sapply(midpoints,max)
 }
 else{
  # midpoints not given
  if( is.null(min) ) min=apply(x,2,min)
  else if( length(min) == 1 ) min=rep(min,nvar)
  if( is.null(max) ) max=apply(x,2,max)
  else if( length(max) == 1 ) max=rep(max,nvar)
  if( length(npoints) == 1 ) npoints=rep(npoints,nvar)
  midpoints=NULL
  for( j in 1:nvar ){
   mdpj=seq(min[j],max[j],length.out=npoints[j])
   midpoints=append(midpoints, list( mdpj ))
  }
 }
 
 # breaks
 breaks=NULL
 for( j in 1:nvar ){
  breaksj=c(midpoints[[j]][2:npoints[j]],midpoints[[j]][npoints[j]]+1)
  breaksj=(breaksj+midpoints[[j]])/2
  breaksj[npoints[j]]=max(max(breaksj),max(x[,j]))+0.5
  breaksj=c(min(min(breaksj),min(x[,j]))-0.5,breaksj)
  breaks=append(breaks,list(unique(breaksj)))
 }
 
 # weights
 if( is.null(weights) ) weights=matrix(1,n,1)
 else if( length(weights) == 1 ) weights=matrix(1,n,1)
 else if( is.matrix(weights) ){
  if( nrow(weights) != n ){
   cat("\nerror(freqdist): invalid weight matrix.","\n")
   return()
  }
 }
 else if( is.vector(weights) ){
  if( length(weights) != n ){
   cat("\nerror(freqdist): invalid weight vector.","\n")
   return()
  }
  else{
   weights=matrix(weights,,1)
  }
 }
 nobs=colSums(weights)
 nf=ncol(weights)
 wname=colnames(weights)
 if( is.null(wname) ) wname=paste("freq",1:nf,sep="")
 
 
 if( print > 0 ){
  cat("\nFrequency Distribution\n")
  if( !is.null(title) ) print(title)
  Print(nvar,n,nf,nobs, rev, omitmiss, class)
  Print(npoints,min,max)
  
  cat("\n")
  for( j in 1:nvar ){
   left=breaks[[j]][-npoints[j]]
   left[1]=NA
   left[npoints[j]]=breaks[[j]][npoints[j]]
   right=breaks[[j]][-1]
   right[npoints[j]]=NA
   left=formatC(left, width=wid, digits=dig, format="f")
   right=formatC(right, width=wid, digits=dig, format="f")
   mp=formatC(midpoints[[j]], width=wid, digits=dig, format="f")
   char=cbind(left, "  <  ", mp, "  <=  ", right)
   char=rbind(
    c("left brk","   "," middp","    ","right brk"), char)
   char=matrix(apply(char,1,paste,collapse="  "),,1)
   char=format(char,justify="left")  # centre
   cat("\nCategorization of Variable",j,"\n")
   write.table(char, row.names=FALSE, col.names=FALSE, quote=FALSE)
  }
 }
 
 
 # categorize the observations  by cutting them at the break points
 c=NULL      # list of categorical values correspoinding each obs
 cc=NULL     #  list of category numbers correspoinding each obs
 lab=NULL    # list of formatted (rounded) values of midpoints
 # to be used as the names
 lost=NULL   # list of lost midpoints
 midp1=NULL  # actual midpoints: lost ones removed
 for( j in 1:nvar ){
  labelsj=format(midpoints[[j]],digits=digits)
  cj=cut( x[,j], breaks=breaks[[j]]
          , labels=labelsj
          , include.lowest=1 )
  # Print(j,cj,unique(cj),unique(as.numeric(cj)))
  
  # lost midpoints
  ucj=unique(as.numeric(cj))
  lostj=setdiff( 1:npoints[j], ucj )
  # Print(j,ucj,lostj)
  
  # category numbers
  cc=append(cc,list(unclass(cj)))
  
  # use true value of midpoints as the categorized value, not the label.
  cj=midpoints[[j]][cj]
  c=append(c,list(cj))
  lab=append(lab,list(labelsj))
  lost=append(lost,list(lostj))
  midp1=append(midp1,list(midpoints[[j]][sort(ucj)]))
 }
 
 # convert c to matrix assuming equal lengths
 # c=sapply(c, '[', 1:max(sapply(c, length)))
 c=sapply(c, unlist)
 cc=sapply(cc, unlist)
 # Print("****\n", lost,c)
 
 # append obs corresponding to the lost midpoints with zero weight
 for( j in 1:nvar ){
  lostj=lost[[j]]
  
  if( length(lostj) > 0 ){
   cj=matrix(sapply(midpoints,"[[",1),length(lostj),nvar, byrow=1)
   cj[,j]=midpoints[[j]][lostj]
   c=rbind(c,cj)
   weights=c(weights,rep(0,length(lostj)))
   cj=matrix(1, length(lostj), nvar, byrow=1)
   cj[,j]=(1:length(midpoints[[j]]))[lostj]
   cc=rbind(cc,cj)
  }
 }
 
 nlost=0
 if( nrow(c) > n ){
  nlost=nrow(c)-n
  rownames(c)=c(rownames(x),rep("dummy",nrow(c)-n))
  rownames(cc)=c(rownames(x),rep("dummy",nrow(c)-n))
  if( print >= 1 ){
   cat("\n")
   cat(" Following ", nlost, " dummy observations with 0 weights are added\n")
   cat("   in order to make the grid complete.\n")
   Print(c[(n+1):nrow(c),,drop=0], cc[(n+1):nrow(c),,drop=0])
   Print(weights[(n+1):nrow(c)])
   cat("Lost Categories\n")
   for( j in 1:nvar ){
    if( length(lost[[j]]) > 0 ){
     cat(" variable",j," categories:", lost[[j]], "\n")
    }
   }
  }
 }
 
 
 # freq dist with weights as:
 #  f=xtabs(formula = weights ~ c[[1]] + c[[2]], drop.unused.levels = 0)
 #  above f will be the npoints[1] x npoints[2] matrix.
 model="weights ~ c[,1]"
 for( j in 2:nvar ) model=paste(model,"+ c[,",j,"]", sep="")
 code=paste("f=xtabs( ", model, ", drop.unused.levels=FALSE )", sep=" ")
 eval(parse(text=code))
 
 
 # check if all the midpoints are used
 namesf=dimnames(f) # This is the formatted value of c by xtabs.
 if( nf > 1 ) namesf=namesf[-(nf+1)]
 # Print(namesf,lab,midp1)
 if( length(namesf) != length(lab) ){
  cat("\nwarning1:(freqdist) Some of the midpoints values are missing.")
  cat("\n Don't know why????\n")
 }
 
 # generate grid of midpoints
 if( nf == 1 ) dimf=dim(f)
 else dimf=dim(f)[-(nvar+1)] # 20140815
 midpindex=as.matrix( expand.grid(lapply(dimf,function(a) seq(1:a))) )
 grid=midpindex
 for( j in 1:nvar ){
  grid[,j]=midpoints[[j]][midpindex[,j]]
 }
 colnames(midpindex)=xname
 colnames(grid)=xname
 #Print(midpindex,grid,f)
 
 # convert the result of xtabs into a matrix
 dim(f)=c(prod(dimf),nf)
 f=unclass(as.matrix(f)) # removing the table attribute
 colnames(f)=wname
 # f=matrix(as.numeric(f),,1)
 gridc=expand.grid( lab )
 gc=apply(gridc,1,paste,collapse=":")
 rownames(f)=gc
 #Print(gridc,gc,f)
 
 
 if( all(colSums(f) == nobs) ) error=0
 else{
  error=1
  cat("\n\nerror1(freqdist): Sum of freq does not equal to n. \n\n")
 }
 
 # convert to prob
 if( prob == 1 ){
  if( nf == 1 ) f=f/nobs
  else f=t(t(f)/nobs)
 }
 
 if( plot > 0  &  nvar == 2 ){
  #library(lattice)
  if( is.null(title) ) title="Data"
  for( j in 1:nf ){
   if( nf == 1 ) ff=f  else ff=f[,j]
   # Below does not produce plot here byt does it when executed outside.
   # levelplot( ff~grid[,1]*grid[,2], contour=1, cut=7, region=FALSE )
   contour( midpoints[[1]], midpoints[[2]], matrix(ff,length(midpoints[[1]]))
            , main=title )
  }
 } # end of plot
 
 # omit missing cells
 if( omitmiss == 1 ){
  if( nf == 1 ) locobs=which( f != 0 ) else locobs=which( apply(f,1,sum) != 0 )
  if( nf == 1 ) f=f[locobs] else f=f[locobs,]
  grid=grid[locobs,]
  midpindex=midpindex[locobs,]
  c=c[1:(n-nlost),]
  cc=cc[1:(n-nlost),]
 }
 
 # reverse order
 if( rev == 1 ){
  code="od=order( grid[,1]"
  for( j in 2:nvar ) code=paste(code,", grid[,", j, "]", sep="")
  code=paste(code," )")
  eval(parse(text=code))
  grid=grid[od,]
  midpindex=midpindex[od,]
  if( nf == 1 ) f=f[od] else f=f[od,]
 }
 
 if( print > 0 ){
  cat("\nFrequency Distribution\n")
  Print(f, midpindex, grid)
 }
 
 if( class == 1 )
  return( list( midpoints=midpoints, midpoints1=midp1
                , midpindex=midpindex, grid=grid, freq=f, breaks=breaks
                , nobs=nobs, prob=prob, error=error, rev=rev
                , omitmiss=omitmiss, class=c, classc=cc ) )
 else
  return( list( midpoints=midpoints, midpoints1=midp1
                , midpindex=midpindex, grid=grid, freq=f, breaks=breaks
                , nobs=nobs, prob=prob, error=error, rev=rev
                , omitmiss=omitmiss ) )
 
} # end of freqdist

#' Convert levels, starting locations, or ending locations to range matrix
#'
#' @param levels vector of levels.
#' @param start vector of starting location. \cr
#' The last element must be the ending location.
#' @param end vector of ending locations. \cr
#' The last element must be the ending location.
#' @param delim vector of delimiter locations.
#'
#' @details
#' If \code{levels} is given: \cr
#' \code{ range <- cbind(cumsum(levels)-levels+1,cumsum(levels)) } \cr
#'
#' If \code{start} is given: \cr
#'  \code{range <-  cbind(start[-length(start)]} \cr
#'     \code{, c(start[2:(length(start)-1)]-1,start[length(start)]) ) } \cr
#'
#' If \code{end} is given, \cr
#'  \code{range <- cbind(c(1,end[1:(length(end)-1)]+1),end) } \cr
#'
#' If \code{delim} is given, \cr
#'  \code{range <-  cbind(c(1,delim[-length(delim)]+1)} \cr
#'     \code{, c(delim[-length(delim)]-1,delim[length(delim)])) } \cr
#'
#'  Note that \code{start[-length(start)], end} and \code{delim}
#'  must be strictly increasing sequence of numbers.
#'
#'
#' @return range: a \code{length(levels) x 2} matrix, \cr
#' where \code{length(levels) = length(start)-1 = length(end)}.
#'
#' @examples
#' get_range( c(3,4,2) )
#' get_range( start=c(1,4,8,9) )
#' get_range( end=c(3,7,9) )
#' get_range( delim=c(3,7,9) )
#'
#' levels <- c(1,2,3,1)
#' range <- get_range( levels )
#' levels2 <- range[,2]-range[,1]+1
#' identical(levels,levels2)
#'
#' @export
#'

get_range <- function( levels=NULL, start=NULL, end=NULL, delim=NULL ){
 # convert levels to range
 # Shin-ichi Mayekawa
 # 20150815
 # start and end: 20250217,18
 # delim: 20250218,20
 #

 if( is.null(start) && is.null(end) && is.null(delim) ){
  # levels is given
  range=cbind(cumsum(levels)-levels+1,cumsum(levels))
 }
 else if( !is.null(start) ){
  # start is given
  if( length(start) == 2  && start[1]<start[2] ) range=matrix(start,1,2)
  else if( length(start) <= 2 || any(diff(start) < 0) ){
   cat("\nerror1:(get_range) length(start) <= 1 or decreasing. \n")
   stop()
  }
  else
  range=cbind(start[-length(start)]
                , c(start[2:(length(start)-1)]-1,start[length(start)]) )
 }
 else if( !is.null(end) ){
  # end is given
  if( length(end) == 1 || any(diff(end) < 0) ){
   cat("\nerror1:(get_range) length(end) <= 1 or decreasing. \n")
   stop()
  }
  range=cbind(c(1,end[1:(length(end)-1)]+1),end)
 }
 else if( !is.null(delim) ){
  # delim is given
  if( length(delim) == 1 || any(diff(delim) < 0) ){
   cat("\nerror1:(get_range) length(delim) <= 1 or decreasing. \n")
   stop()
  }
  else if( delim[1] == 1 ){
   range=cbind(c(delim[1]+1,delim[-c(1,length(delim))]+1)
             , c(delim[-c(1,length(delim))]-1,delim[length(delim)]))
   }
  else{
   range=cbind(c(1,delim[-length(delim)]+1)
              , c(delim[-length(delim)]-1,delim[length(delim)]))
  }
 }

 colnames(range)=c("from","to")
 return( range)

} # end of get_range

#' Sort Matrix/Data Frame According to the Pattern of Missing Elements
#'
#' @param X a data frame or matrix to be sorted.
#' @param misslast = 1 to treat NA as the largest,
#' = 1 to treat NA as the smallest.
#' @param nobs = 1 to sort by the # of observed elements first \cr
#' = -1 to sort by the descending # of observed elements first \cr
#' = 0 not to pay any attention to the # of missing elements.
#'
#' @examples
#' set.seed(17029)
#' nn=50
#' dataframe=0
#'
#' if( dataframe ){
#'  x=sample(0:9,nn,replace=1)
#'  x[runif(nn) < 0.5]=NA
#'  x2=sample(letters[1:10],nn,replace=1)
#'  x2[runif(nn) < 0.5]=NA
#'  x3=sample(LETTERS[1:10],nn,replace=1)
#'  x3[runif(nn) < 0.5]=NA
#'  X=data.frame(x,x2,x3, stringsAsFactors=0)
#' } else{
#'  x=sample(0:9,nn,replace=1)
#'  x[runif(nn) < 0.3]=NA
#'  x2=sample(1:10,nn,replace=1)
#'  x2[runif(nn) < 0.3]=NA
#'  x3=sample(1:10,nn,replace=1)
#'  x3[runif(nn) < 0.3]=NA
#'  X=cbind(x,x2,x3)
#' }
#' res=sort_missing_data( X, misslast=1, nobs=-1 )
#' Xs=res$sorted_data
#' freqs=res$freq
#' Nobss=res$Nobs
#' od=res$od
#'
#' # Nobsss should be equal to Nobs, Xss, to Xs
#' Nobsss=ncol(X)-apply(Xs,1,function(x) sum(is.na(x)))
#' Xss=X[od,]
#' pat=rep(1:length(freqs),freqs)
#' Print(Xs,Xss,Nobss, pat,od)
#'
#' misspat=Xs[cumsum(res$freq),]
#' nobs=ncol(misspat)-apply(misspat,1,function(x) sum(is.na(x)))
#' Print(misspat,nobs)
#'
#' mands( Xs, by=pat )
#'
#'
#' @return
#' A list of (sorted_data, freq, Nobs, class, od) \cr
#' where sorted_data is of the same size as X, \cr
#' freq is the vector consisting of the # of observatons for each pattern, \cr
#' Nobs is the vector of length nrow(X)
#' containing the # of observed elements for each row of X.\cr
#' class is the interger in [1,length(freq)] indicating the classification
#' of each obs according to the missing pattern. \cr
#' od is the order to be used to sort X as X_sorted=X[od,].
#'
#'
#' @export
#'
#'

sort_missing_data <- function ( X, misslast=1, nobs=0 ){
 # sort data frame with missing data
 # Shin-ichi Mayekawa
 # 20190120,21
 # return od: 20190125dnc,26
 # sort_missing_data0 simplifed: 20200925
 # return class: 20200927
 # roxygen2: 20230117dnc
 #

 sort_missing_data0 <- function ( X, misslast=1 ){
  # sort data frame with missing data
  # The original version is from mysort function of mvnmle package
  # Shin-ichi Mayekawa
  # 20190120
  # return od: 20190125dnc
  # simplified: 20200925
  #

  #
  # This function returns:
  #   sorted_data, freq, and od
  #

  if( misslast != 1 ) misslast=0

  if( all(!is.na(X)) ){
   return(list(sorted_data=X, freq=nrow(X), od=1:nrow(X)))
  }
  if( all(is.na(X)) ){
   return(list(sorted_data=X, freq=nrow(X), od=1:nrow(X)))
  }

  # The idea of representing the missing patterns by numbers is found in
  #  "mysort" function of "mvnmle" package.
  #
  bin <- ifelse( is.na(X), as.integer(misslast), as.integer(1-misslast) )
  dec <- bin %*% as.integer(2^((ncol(X)-1):0))
  od=order(dec)
  sorted <- X[od,]
  dec <- dec[od]

  return( list(sorted_data=sorted, freq=as.vector(table(dec)), od=od) )

 } # end of sort_missing_data0



 if( is.vector(X) ){
  nv=1
  X=matrix(X,,nv)
 }
 else nv=ncol(X)

 if( nobs != 0 ){

  Nobs=nv-apply(X,1,function(X) sum(is.na(X)))
  # for each of by group
  # by function does not give us grouping info(which obs belongs to which group)
  uNobs=unique(Nobs)
  ngroup=length(uNobs)
  if( nobs > 0 ){
   uNobs=sort( uNobs )
  }
  else{
   uNobs=sort( uNobs, decreasing=TRUE )
  }
  res=NULL
  for( k in 1:ngroup ){
   loc=which( Nobs == uNobs[k] )
   resk=sort_missing_data0( X[loc,,drop=0], misslast=misslast )
   resk$od=loc[resk$od]
   res=append( res, list(resk) )
  } # k loop for group

  # res above is a list of lists: (sorted_data, freq, od)
  # sorted data and freq: horizontally stack the sorted data for each Nobs.
  rr=lapply( res, function(X) as.data.frame(X[[1]]) )
  sorted_data=Reduce( rbind, rr )
  Nobs=nv-apply(sorted_data,1,function(X) sum(is.na(X)))
  if( is.matrix(X) ) sorted_data=as.matrix(sorted_data)
  rr=lapply( res, function(X) as.vector(X[[2]]) )
  freq=Reduce( c, rr )
  rr=lapply( res, function(X) as.vector(X[[3]]) )
  od=Reduce( c, rr )
  class=rep(1:length(freq),freq)
  return( list( sorted_data=sorted_data, freq=freq, Nobs=Nobs
                , class=class, od=od ) )

 }
 else{

  res=sort_missing_data0( X, misslast=misslast )
  Nobs=nv-apply(res$sorted_data,1,function(X) sum(is.na(X)))
  class=rep(1:length(res$freq),res$freq)
  return( append( res[-3]
                  , list(Nobs=Nobs, class=class, od=res[[3]])) )

 }

} # end of sort_missing_data

#' Elapsed Time
#'
#' This program calculates the elapsed time between concecutive calls.
#'
#' @param opt = 0 to reset the clock \cr
#' = 1  to print the elapsed time. \cr
#' = 2  to print the elapsed time and reset the clock.
#' @param title a title string
#'
#' @details
#' A global variable ..start.. is used.
#'
#' @return
#' etime in seconds.
#'
#' @examples
#' etime(0)
#' x=rnorm(10000000)
#' etime(1)
#' x=rnorm(10000000)
#' etime(2)
#' x=rnorm(10000000)
#' etime(1, "This is the title string.")
#' rm(x)
#'
#' @export
#'

etime <- function( opt=0, title=NULL ){
   ## elapsed time
   ## Shin-ichi Mayekwa: 070702,03
   ## opt=2: 20150615
   ## title: 20240331
   ##
   ## etime(0) resets the clock.
   ## etime()  prints and returns the elapsed time.
   ##
   ## ..start..<-numeric(0)
   if( opt == 0 ) ..start.. <<- Sys.time()
   else if( opt >= 1 ){
      if( !is.null(title) ) cat("\n", title, "\n", sep="")
      cat( "started at ", ..start.., " " )
      print( ..start.. )
      endtime=Sys.time()
      cat( "  ended at ", endtime, " " )
      print( endtime )
      etime=unclass(endtime)-unclass(..start..)
      # cat( "elapsed time = ", etime, "seconds\n" )
      h=trunc(etime/3600); m=trunc((etime%%3600)/60); s=(etime-3600*h-60*m);
      cat( "elapsed time as H:M:S = ", h, ":", m, ":", s, "\n" )
      if( opt == 2 ) ..start.. <<- Sys.time()
      return(etime)
   }
} # end of etime

#' plot X[,1:2] with id
#'
#'
#' @param X    matrix whose rows will be plotted
#' @param id   id of the rows of X
#' @param asp  aspect ratio
#' @param type type of the plot
#' @param pos  position of the text
#' @param cex  size of the text
#' @param col  color of the text
#' @param ...  additional graphical paramaters to text function
#'
#' @details
#' Equivalent to:
#' \preformatted{
#' plot( X[,1:2], type="n", asp=1 )
#' text( X[,1],X[,2], labels=id, ... )
#' }
#'
#' @examples
#'  X=matrix(c(
#' -1,  1,  0,  1,  1,    1, -1,   0, 0,  0, 1, 0, -1, -1, 0, -1, 1, -1
#' , -.5,  .5, .5,  .5, -.5,  -.5, .5,  -.5, -2,  0, 2,  0
#' ),,2,byrow=1)
#' idplot(X, id=paste("s",1:nrow(X),sep=""))
#' idplot(X, id=paste("s",1:nrow(X),sep=""), pos=1)
#' idplot(X, id=paste("s",1:nrow(X),sep=""), pos=2, cex=2)
#'
#' @return
#' none
#'
#' @export
#'

idplot <- function( X, id=NULL, type="n", asp=1, pos=NULL, cex=1, col=NULL
                    , ... ){
 # plot X[,1:2] with id
 # Shin-ichi Mayekawa
 # 20260223,0301
 # ... in plot: 20260409
 # 20260509,11
 #

 if( is.null(id) ){
  if( is.null(rownames(X)) ) id=1:nrow(X)
  else id=rownames(X)
 }
 plot( X[,1:2], type=type, asp=asp, ... )
 text( X[,1],X[,2], labels=id, pos=pos, cex=cex, col=col )

} # end of idplot

#' Retrieve the Elements Contained in the Lists in a List
#'
#' This version is very slow!!
#'
#' @param L A list of lists or data frames
#' @param name The name of the list element, or the element number. \cr
#' Can be a vector.
#' @param removenull = 1 to remove NULL elements.
#' @param simplify = 0 not to simplify the result. \cr
#'  = 1 to simplify the result if possible \cr
#'  = 2 to simplify the result if possible when name is a vector. \cr
#'  Only works with \code{method=1}.
#' @param removeNull = 1 to remove all NULL columns.
#' @param method = 2 to use new method with sapply
#' @param debug = 1 to print the details of \code{method=2}
#'
#' @details
#' It is assumed that \code{L} is a list consisting of several lists
#' which contain the same objects.
#'
#' Specifying \code{method=2} is faster but works only when: \cr
#' 1) \code{name} is a scalar and the elements to be retrieved are
#' scalars or vectors with no missing elements, not matrices or lists. \cr
#' The result will be a vector or matrix.
#' 2) \code{name} is a vector and the elements to be retrieved are
#' scalars. \cr
#' The result will be a matrix.
#'
#' If the above conditions are not met, \code{method=1} will be used.
#'
#' Note that, Specifying \code{method=2} is equivalent to \cr
#'  \code{ res <- sapply( L, function(x) x[[name]] ) } \cr
#'
#'
#' When \code{method=1}: \cr
#'
#' If \code{simplify=1} is given, the result will be converted to a data frame
#' or matrix if possible.
#'
#' If the \code{L} contains lists or arrays, or when \code{name} is a vector,
#' the result will not be simplified.
#'
#' Try \code{l2a( lapply( "get_valLL result", a2m, method=2 ) )} .
#'
#' If \code{removenull=1} is given, and \code{name} is not valid,
#'  NULL or named list() may be returned.
#'
#' If the \code{L} consists of data frames only, better use \code{get_valL}.
#'
#' \code{removeNull=1} may be needed when not all the lists in \code{L}
#' have the same set of elements.
#' It can always be set equal to 1 but not thoroughly tested yet.
#'
#' @examples
#' # case where all the lists consists of the objects of the same size.
#' set.seed(1701)
#' n <- 100; nvar <- 3
#' ngroup <- 3
#' X <- matrix(rnorm(n*nvar)/100,n,nvar)
#' group <- sample(1:ngroup,n,replace=1)
#' X <- group+X
#' group <- letters[group]
#' md <- mandd(X,by=group)
#'
#' get_valLL(md,"mean")
#' get_valLL(md,"mean", simplify=0)
#' get_valLL(md,c("mean","cov"))
#'
#'
#' # univariate case with simplify=2
#' md2 <- lapply(md, function(x){lapply(x,"[",1)})
#' get_valLL(md2,"mean")
#' get_valLL(md2,"mean", simplify=0)
#' get_valLL(md2,c("mean","std"))
#' get_valLL(md2,c("mean","std"), simplify=2)
#'
#'
#' # general case
#' L0 <- list( A=(demomat(3,2)*10+1)
#'             , B=(1:3)*10+2
#'             , C=data.frame(demomat(5,4)*10+3)
#' )
#' L00 <- list( A=(demomat(3,2)*100+1)
#'              , B=(1:3)*100+2
#'              , C=(demomat(5,4, root="c"))
#'              , D=(demomat(5,4, root="d"))
#' )
#' L000 <- list( A=(demomat(3,2)*1000+1)
#'               , B=(1:3)*1000+2
#'               , C2=(demomat(5,4)*1000+3)
#'               , D=(demomat(4,5)*1000+4)
#' )
#' L <- list(L0,L00,L000)
#' names(L) <- c("a","b","c")
#'
#' get_valLL( L, "A" ) #   This is a list of matrices.
#' get_valLL( L, "B" ) #   This is a matrix.
#' get_valLL( L, "B", simplify=0 ) #   This is a list of vectors.
#' get_valLL( L, "C", simplify=0 ) # NULL indicates missing elements.
#' get_valLL( L, "D" )  # same as get_valLL( L, 4 )
#' get_valLL( L, 4, removenull=1  ) # missing elements were removed.
#'
#'
#' # example of removeNull
#' L21 <- list( a=1, b=1, c=1 )
#' L22 <- list( a=2, c=2 )
#' L23 <- list( a=3, b=3, c=3 )
#' L2 <- list( x=L21, y=L22, z=L23 )
#' ( get_valLL( L2, "b" ) )
#' ( get_valLL( L2, "b", removeNull=1 ) )
#'
#'
#'
#'
#'#' # Example of \code{method=2}
#'
#' # mean and std are vectors
#' get_valLL(md,"mean", method=2, debug=1)
#' get_valLL(md,"cov", method=2, debug=1) # --> method=1
#' get_valLL(md,c("mean","std"), method=2, debug=1, simplify=2) # -> method=1
#'
#'
#' # mean and std are scalar
#' md2 <- lapply(md, function(x){lapply(x,"[",1)})
#' get_valLL(md2,"mean", method=2, debug=1)
#' get_valLL(md2,c("mean","std"), method=2, debug=1)
#' get_valLL(md2,c("mean","std"), method=1, debug=1, simplify=2)
#'
#'
#' # general case: B is a scalar
#' L0 <- list( A=(demomat(3,2)*10+1)
#'             , B=(1:3)*10+2
#'             , C=data.frame(demomat(5,4)*10+3)
#' )
#' L00 <- list( A=(demomat(3,2)*100+1)
#'              , B=(1:3)*100+2
#'              , C=(demomat(5,4, root="c"))
#'              , D=(demomat(5,4, root="d"))
#' )
#' L000 <- list( A=(demomat(3,2)*1000+1)
#'               , B=(1:3)*1000+2
#'               , C2=(demomat(5,4)*1000+3)   # C is missing.
#'               , D=(demomat(4,5)*1000+4)
#' )
#' L <- list(L0,L00,L000)
#' names(L) <- c("a1","b2","c3")
#'
#' get_valLL( L, "A", method=2, debug=1 ) # --> method=1
#' get_valLL( L, "B" , method=2, debug=1)
#' get_valLL( L, "C", simplify=0, method=2, debug=1 ) # C is NULL at the 3rd list
#' get_valLL( L, 2, removenull=1, method=2, debug=1  ) # same as "B"
#'
#'
#' # example of missing list elements
#' L21 <- list( A=1, B=1, C=1 )
#' L22 <- list( A=2, C=2 )        # B is missing.
#' L23 <- list( A=3, B=3, C=3 )
#' L2 <- list( x=L21, y=L22, z=L23 )
#' get_valLL( L2, "B", method=2, debug=1 )
#' get_valLL( L2, c("A","B"), method=2, simplify=2, debug=1 )
#'
#'
#' # difference between method=1 and 2
#' get_valLL(md2,"mean", method=1)
#' get_valLL(md2,"mean", method=2)
#'
#' get_valLL(L2,"B", method=1, removeNull=1)
#' get_valLL(L2,"B", method=2)
#'
#'
#' @return
#' A list, data frame, matrix or vector consisting of the specified elements.
#'
#' @export
#'

get_valLL <- function( L, name, removenull=0, simplify=1, removeNull=0
                       , method=1, debug=0 ){
 # get values from a list of lists
 # Shin-ichi Mayekawa
 # 20190204rc,05rc
 # do not simplify arrays: 20190206rc
 # && -> &: 20190605
 # bug fix: 20191106dnc
 # removeNull: 20230919
 # name must be a scalar: 20230927
 # new treatment of vector name: 20231015,17
 # use of sapply in roxygen2: 20240401,15
 # method=2 for vector element: 20240520
 # get_valLL11.R and get_valLL12.R not used as package: 20240521
 # error trap and some bug fix: 20240522
 # names(res1)=names(L)[locobs]: 20240523
 # first version 0.1.6: 20240523
 #
 
 if( !is.list(L) ){
  cat("\nerror1:(get_valLL) The first argument must be a list.\n")
  stop()
 }
 
 # check if name contains valid names. char or num.
 nam=lapply(L,names)
 if( is.character(name) ){
  if( !all(name %in% unique(unlist(nam))) ){
   cat("\nerror1(get_valLL) Invalid character name argument.\n")
   stop()
  }
 }
 else{
  if( !( all( 0 < name )  &&  all( name <= max(sapply(nam,length))) ) ){
   cat("\nerror1(get_valLL) Invalid numeric name argument.\n")
   stop()
  }
  else{
   name=unique(unlist(nam))[name]
  }
 }
 
 #
 # method=2 works if and only if:    20240521
 # 1) all the elements are scalar (with possible missing values)
 # 2) all the elements are vector w/o any missing elements.
 #
 
 if( method == 2 ){
  
  len=sapply(L, function(x) sapply( x[ names(x) %in% name ], length ) )
  locobs=which(sapply(len, length) > 0)
  ll=length(len)
  lenlen=sapply(len,length)
  res0=sapply( L, function(x) x[ names(x) %in% name ] )
  
  if( debug >= 2 ){
   Print(len)
   Print(ll,lenlen,locobs,res0)
  }
  
  if( length(name) == 1 ){
   if( debug ) cat("\nlength(name) == 1.\n")
   
   # vector consisting of the lengths of each element of L$name
   # Print(unlist(len), ll, locobs)
   
   dims=sapply(L, function(x) sapply( x[ names(x) %in% name ], dim ) )
   
   if( all(unlist(len) == 1) ){
    # all the elements of L[[i]][name] are scalars
    if( debug ) cat("\nAll of L[[i]][name] are scalar.\n")
    res=unlist( res0 )
    if( all(is.numeric(res)) ) res1=numeric(ll)
    else res1=character(ll)
    res1=NA
    res1[locobs]=res
    names(res1)=names(L)[locobs]    # 20240523
   }
   else if( all(is.numeric(len))  &&  length(unlist(dims)) == 0 ){
    # all the elements of L[[i]][name] are vectors
    if( debug ) cat("\nAll of L[[i]][name] are vectors.\n")
    # Print(dims, unlist(dims))
    # Print(dims, !all(is.null(dims)))
    #Print("**", res0, sapply(res0,length))
    if( all( (sa <- sapply(res0,length)) == length(res0[[1]])) ){
     # vectors of the same length
     if( debug ) cat("\nAll the vectors are the same length.\n")
     res1=matrix(unlist(res0),length(res0),, byrow=1)
     rownames(res1)=names(L)
     colnames(res1)=names(res0[[1]])
    }
    else{
     # vectors of the different length
     if( debug ) cat("\nAll the vectors are NOT the same length.\n")
     # sa defined in all and if functions are available in the parent env.
     res1=matrix(NA,length(res0),max(sa))
     for( i in 1:length(res0) ){
      if( sa[i] > 0 ) res1[i,1:sa[i]]=res0[[i]]       # 20240523
     }
     rownames(res1)=names(L)
     colnames(res1)=paste("_v_",1:ncol(res1),sep="")
    }
   }
   else{
    # the elements of L[[i]][name] are not scalars or vectors or missing
    if( debug ) cat("\nThe elements of L[[i]][name] are not scalars nor vector"
                    , "or some elements are missing. Use method=1.\n")
    # use method=1
    res1=Recall( L, name, method=1, removenull=removenull
                 , simplify=simplify, removeNull=1, debug=debug )
   }
   
   return( res1 )
   
  } # end of length(name) == 1
  else{
   
   # Print(ll,len,locobs )
   #
   # length(name) > 1
   if( debug ) cat("\nlength(name) > 1.\n")
   #
   #if( all(unlist(len) == 1)  ){
   if( all(unlist(len) == 1) && all(lenlen == lenlen[1]) ){
    # all the elements of L[[i]][name] are scalars
    if( debug ) cat("\nAll the elements of L[[i]][name] are scalars.\n")
    res=unlist( res0 )
    if( all(is.numeric(res)) ) res1=numeric(ll)
    else res1=character(ll)
    res1=NA
    res1[locobs]=res
    res1=matrix(res1,length(L),, byrow=1)
    # printm(res0,len,lenlen,res1,names(L),name)
    rownames(res1)=names(L)
    colnames(res1)=name
   }
   else{
    # vector elements
    if( debug ) cat("\nAll the elements of L[[i]][name] are NOT scalars."
                    , " Use method=1.\n")
    # use method=1
    res1=Recall( L, name, method=1, removenull=removenull
                 , simplify=simplify, removeNull=1, debug=debug )
   }
   
   return( res1 )
   
  } # end of length(name) > 1
  
 } # end of method=2
 
 
 
 #
 # Here, !( method == 2  && length(name) == 1)
 # that is,   method == 1  |  length(name) > 1
 #
 
 
 # new treatment of vector name
 if( length(name) > 1  &&  simplify == 2 ){
  
  # recursive call
  # Cannot use Recall here. Reason is not clear!!!
  res1=lapply( name
             , function(x){ get_valLL( L, x, removeNull=1, removenull=0
             , simplify=1, method=1, debug=debug) } )
  # remove non existent elements
  c0=unlist(lapply(res1,function(x){ncol(x)==0}))
  loc=which(c0)
  if( length(loc) > 0 ){
   res1=res1[-loc]
   name=name[-loc]
  }
  
  c1=unlist(lapply(res1,is.matrix))
  c2=unlist(lapply(res1,ncol))==1
  lenres=all(c1 & c2)
  if( all(lenres) ){
   
   
   res=Reduce( data.frame, res1)
   colnames(res)=name
   return( res )
  }
  else{
   names(res1)=name
   return( res1 )
  }
  
 } # end of recursive call
 
 
 
 
 
 
 #####################################################################
 # main body starts here.
 #####################################################################
 
 lengthname=length(name)
 if( lengthname > 1 ) simplify=0
 
 # list of lists
 df=sapply(L, is.data.frame)
 alllist=as.numeric( all(sapply(L, mode) == "list"  &  all(!df)  ) )
 if( !alllist ){
  cat("\nerror1:(get_valLL) List L is not a list of lists.\n")
  cat(" A list of data frames can be handled by get_valL function.\n")
  stop("***")
 }
 
 # get the element names of each list
 nam=lapply(L,names)
 namlen=sapply(nam,length)
 
 # list with no names
 if( is.null(names(L)) ) names(L)=paste("_",1:length(L),sep="")
 else if( any(is.na(names(L))) ){
  names(L)[is.na(names(L))]=paste("_",1:length(L),sep="")[is.na(names(L))]
 }
 
 # Print(name, nam, namlen)
 # Print(is.character(nam))
 
 # pick up the specified list elements
 # If the element specified by name is not present in the k-th list,
 # the k-th element of the subset below is NULL.
 # subset below is a list of lists even if each element is not a list.
 
 # 20240522 comment out
 # if( is.character(nam) ){
 # subset=lapply( L, function(x){ x[names(x) %in% name] } )
 # } else{
 #  subset=lapply( L, function(x){ x[name] } )
 # }
 subset=lapply( L, function(x){ x[name] } )
 
 # name of the non-null subset
 namsubset=names(subset)[unlist(lapply(subset,length))>0]
 if( length(namsubset) == 0 ){
  subset=NULL
 } else if( lengthname == 1 ){
  # convert subset from a list of lists to a simple list
  subset=unlist( subset, recursive=FALSE )
  names(subset)=namsubset
  # str(subset)
 }
 
 # location of null elements of subset
 locnull=sapply(subset,is.null)
 namessubset=names(subset)
 # Print(name,subset,names(subset), locnull, namessubset)
 
 # remove the null elements from subset
 if( removenull ){
  namessubset=names(subset)[!locnull]
  subset=subset[!locnull]
  if( length(subset) == 0 ) subset=NULL
 }
 
 if( !simplify | is.null(subset) ){
  return( subset )
 }
 
 # simplify the result if possible
 
 # If a list is contained in subset, it cannot be simplified
 mode=sapply(subset,mode)
 df=sapply(subset, is.data.frame)
 array=sapply(subset, function(x) length(dim(x) >= 3))
 if( any(mode=="list" & !df) | any(array) ){
  return( subset )
 }
 
 lensubset=length(subset)
 for( k in 1:lensubset ){
  sk=subset[[k]]
  if( is.null(sk) ){
   # if sk is NULL, subset[[k]]=sk removes the k-th element of subset.
   sk=matrix("n_u_l_l",1)
  } else if( is.vector(sk) ){
   namsk=names(sk)
   sk=matrix(sk,1)
   if( is.null(namsk) ) colnames(sk)=paste("_v_",1:ncol(sk),sep="")
   else colnames(sk)=namsk
  }
  subset[[k]]=sk
 }
 
 # convert a list to a data frame
 res=Reduce( stack_df, subset )
 
 # convert to a matrix or data frame
 len=sapply(subset,nrow)
 names_org=rep(namessubset,len)
 vt=vartype(res)
 if( all( vt == vt[1] ) ){
  # matrix
  # res=matrix(res, nrow(res),ncol(res))
  res=as.matrix(res)
  rownames(res)=names_org
 }
 else{
  # data frame
  # names: No duplicate names allowed in df!!!!!!!!
  res=data.frame( key_key_=names_org, res
                  , stringsAsFactors=FALSE )
 }
 
 if( removeNull ){
  # all missing or n_u_l_l columns
  locnull=which( apply(res,2,function(x){ all( is.na(x)| x=="n_u_l_l" )} ) )
  if(length(locnull) > 0 ) res=res[,-locnull, drop=0]
 }
 
 return( res )
 
 
} # end of get_valLL

#' Define Break Points from Midpoints
#'
#' @param midpoints a vector of midpoints.
#' @param min value of the lowest break.
#' @param max value of the highest break.
#' @param margin value to be subtracted/added to
#'         the lowest and highest midpoints. \cr
#'         default margin = ( midpoints[2]-midpoints[1] )/2
#'
#' @details
#' Break values from midpoint values: \cr
#'  \code{brk[1]=midp[1] - margin}\cr
#'  \code{brk[2]=midp[1] + (midp[2]-midp[1])/2} \cr
#'  \code{brk[3]=midp[2] + (midp[3]-midp[2])/2} \cr
#'  \code{brk[length(midpoints)+1]=midp[length(midpoints)] + margin} \cr
#'  where \code{margin=(midp[2]-midp[1])/2}. \cr
#'  In general, the break point is located at the center of two midpoints:\cr
#'  \code{brk[k]=0.5*(midp[k-1]+midp[k])}
#'  \cr
#'
#' Midpoint values from break values: \cr
#'  \code{midp[1]=0.5*(brk[1]+brk[2])} or {midp[1]=min}\cr
#'  \code{midp[2]=0.5*(brk[2]+brk[3])} \cr
#'  \code{midp[3]=2*brk[3]-midp[2]} \cr
#'  \code{brk[length(brk)-1]=2*brk[length(brk)-1]-midp[length(brk)-2]} \cr
#' \cr
#' Note that midp[1] is treated separately and it is assumed that
#' the break point is located at the center of two midpoints.
#' \cr\cr
#' Try below to see how the obs on the border are classified. \cr
#'    \code{ xx <- c(1,2,3,4,5,6,7) } \cr
#'    \code{ brks <- c(0,2,4,6,8) } \cr
#'    \code{ cc1 <- cut(xx,brks) } \cr
#'    \code{ cc2 <- cut(xx,brks, right=FALSE) } \cr
#'    \code{ Print(xx,as.character(cc1),as.character(cc2)) }\cr
#'
#' @return
#' A vector of break points of length length(midpoints)+1.
#'
#' @examples
#' midp <- seq(0,5,length.out=11)
#' brk <- midp2brk( midp, margin=1 )
#' Print(midp,brk)
#'
#' midp <- seq(0,5,length.out=11)
#' brk <- midp2brk( midp, margin=1 )
#' Print(midp,brk)
#'
#' set.seed(1701)
#' x <- rnorm(100)+2.5
#'
#' # histogram by native hist
#' resh <- hist( x, breaks=brk )
#'
#' # histogram by native barplot
#' # freq <- resh$counts
#' # names(freq) <- midp
#' # barplot( freq, space=0 )
#' barplot( resh$count,names.arg=midp, space=0, xlab="X", col="white"
#'         , sub="top and bottom bars contain outliers." )
#'
#' # Below causes error because 'brk' does not span the range of 'x'.
#' # brk <- midp2brk( midp )
#' # resh <- hist( x, breaks=brk )
#'
#' @export
#'

midp2brk <- function( midpoints, min=NULL, max=NULL, margin=NULL ){
 # define breaks via midpoints
 # Shin-ichi Mayekawa
 # 201760719,20
 # bugfix: 20170917
 # roxgen details: 20170917,1109
 #

 npoint=length(midpoints)
 if( is.null(margin) ) margin=(midpoints[2]-midpoints[1])/2
 breaks=( midpoints + c( midpoints[2:npoint], midpoints[npoint]+2*margin ) )/2
 breaks=c( midpoints[1]-margin, breaks )
 if( !is.null(max) ) breaks[npoint+1]=max  ##
 if( !is.null(min) ) breaks[1]=min
 return( breaks )

} # midp2brk





#' Define Midpoints from Break Points
#'
#' @param brk a vector of break points.
#' @param min The value of the lowest midpoint.
#'    This has priority over margin.
#'
#' @details
#' Break values from midpoint values: \cr
#'  \code{brk[1]=midp[1] - margin}\cr
#'  \code{brk[2]=midp[1] + (midp[2]-midp[1])/2} \cr
#'  \code{brk[3]=midp[2] + (midp[3]-midp[2])/2} \cr
#'  \code{brk[length(midpoints)+1]=midp[length(midpoints)] + margin} \cr
#'  where \code{margin=(midp[2]-midp[1])/2}. \cr
#'  In general, the break point is located at the center of two midpoints:\cr
#'  \code{brk[k]=0.5*(midp[k-1]+midp[k])}
#'  \cr
#'
#' Midpoint values from break values: \cr
#'  \code{midp[1]=0.5*(brk[1]+brk[2])} or {midp[1]=min}\cr
#'  \code{midp[2]=0.5*(brk[2]+brk[3])} \cr
#'  \code{midp[3]=2*brk[3]-midp[2]} \cr
#'  \code{brk[length(brk)-1]=2*brk[length(brk)-1]-midp[length(brk)-2]} \cr
#' \cr
#' Note that midp[1] is treated separately and it is assumed that
#' the break point is located at the center of two midpoints.
#' \cr\cr
#' Try below to see how the obs on the border are classified. \cr
#'    \code{ xx <- c(1,2,3,4,5,6,7) } \cr
#'    \code{ brks <- c(0,2,4,6,8) } \cr
#'    \code{ cc1 <- cut(xx,brks) } \cr
#'    \code{ cc2 <- cut(xx,brks, right=FALSE) } \cr
#'    \code{ Print(xx,as.character(cc1),as.character(cc2)) }\cr
#'
#' @return
#' A vector of midpoints of length length(brk)-1.
#'
#' @examples
#' midp <- c(0,1,2,4,6,9,10,11)
#' brk <- midp2brk( midp )
#' Print( midp, brk, brk2midp(brk) )
#' brk[1] <- -4
#' Print( midp, brk, brk2midp(brk) )
#'
#' @export
#'

brk2midp <- function( brk, min=NULL ){
 # recover midpoints from break points
 # Shin-ichi Mayekawa
 # 20170309
 # roxgen details: 20170917,1109
 # treat midp[1] separately: 20171109
 #

 len=length(brk)-1
 midp=numeric(len)
 if( !is.null(min) ) midp[1]=min
 else midp[1]=0.5*(brk[1]+brk[2])

 midp[2]=0.5*(brk[2]+brk[3])
 for( i in 3:len ){
  midp[i]=2*brk[i]-midp[i-1]
 }

 return( midp )

} # end of brk2midp




#' Generate n Equally Spaced Midpoints in [min,max]
#'
#' @param min The minimum value.
#' @param max Them maximum value.
#' @param n # of midpoints including min and max
#'
#' @details
#' \code{gen_midp( min, max, n )} is equivalent to 
#' \code{seq( min, max, len=n )}.
#'
#' @return A vector of length n.
#'
#' @examples
#' gen_midp( 0, 10, 6 )
#' gen_midp( 1, 11, 6 )
#' gen_midp( 0, 100, 11 )
#' gen_midp( 0, 100, 11 )
#'
#' @export
#'

gen_midp <- function( min, max, n ){
 # generate n equally spaced midpoints between [min,max]
 # Shin-ichi Mayekawa
 # 20170414
 #
 # Old sas/iml domain module.
 #
 return( ((max-min)/(n-1))*(0:(n-1)) + min )

} # end of gen_midp

#' Output the Graphics to PDF Device
#'
#' @param outpdf The output pdf file name.
#'   When onefie=FALSE, give a C integer format such as "Rplot\%03d.pdf".
#' @param close = 1 to close the pdf device.
#' @param paper Size of the pdf paper: "a4", "letter", "legal"
#'  and "executive", or "a4r" and "USr" for rotated (landscape.)
#' @param height Height of the pdf paper in mm (not inches)
#' @param width Width of the pdf paper  in mm (not inches)
#' @param onefile = FLASE to create many files.
#' @param ... Other arguments to native pdf function.
#'
#' @details
#' This function calls native pdf function as:\cr
#' \code{pdf(outpdf, paper=paper, height=height, width=width}
#' \code{, onefile=onefile, ... )} \cr
#' When paper=NULL or "special", the paper size will be determined by
#' width and hight.
#' \cr
#' The defalt paper size is "a4r" which is equivalent to: \cr
#' \code{paper="special", height=210, width=297}
#'
#' @examples
#' \dontrun{
#' plot(-5:5,(-5:5)^2, type="l", main="square")
#' plot(-5:5,(-5:5)^3, type="l", main="cubic")
#'
#' # put everything in one pdf file
#' graph2pdf("testestestest.pdf")
#' plot(-5:5,(-5:5)^2, type="l", main="square")
#' plot(-5:5,(-5:5)^3, type="l", main="cubic")
#' graph2pdf(close=1)
#'
#' # create one pdf file for each graph with automatic naming
#' graph2pdf("testestestest%03d.pdf", onefile=FALSE)
#' plot(-5:5,(-5:5)^2, type="l", main="square")
#' plot(-5:5,(-5:5)^3, type="l", main="cubic")
#' graph2pdf(close=1)
#'
#' # create one pdf file for each graph with specific name
#' for( i in 1:3 ){
#'  fname=paste("f", i, ".pdf", sep="")
#'  graph2pdf(fname)
#'  plot(-5:5,(-5:5)^i, type="l", main=i)
#' }
#' graph2pdf(close=1)
#'
#' # default paper size: paper="a4r"
#' graph2pdf("test0.pdf")
#' plot(-5:5,(-5:5)^2, type="l", main="test")
#' graph2pdf(close=1)
#'
#' # ??
#' graph2pdf("test00.pdf", paper="a4")
#' plot(-5:5,(-5:5)^2, type="l", main="test")
#' graph2pdf(close=1)
#'
#'
#' # paper size with width and height
#' graph2pdf("test1.pdf", paper="a4r", width=197, height=120)
#' plot(-5:5,(-5:5)^2, type="l", main="test")
#' graph2pdf(close=1)
#'
#' # Below two produces the same pdfs.
#' graph2pdf("test2.pdf", paper=NULL, width=197, height=120)
#' plot(-5:5,(-5:5)^2, type="l", main="test")
#' graph2pdf(close=1)
#'
#' graph2pdf("test3.pdf", paper="special", width=197, height=120)
#' plot(-5:5,(-5:5)^2, type="l", main="test")
#' graph2pdf(close=1)
#'
#' }
#'
#' @export
#'

graph2pdf <- function( outpdf, close=0, paper="a4r"
                     , height=0, width=0, onefile=TRUE,  ... ){
 # draw the graphs on pdf device
 # Shin-ichi Mayekawa
 # 20171106
 # example added: 20191106
 # height and width in cm: 20200205dnc,0206
 #

 if( close == 0 ){

  # delete the pdf graphic device
  devl=dev.list()
  locpdf=which(names(devl) == "pdf")
  if( length(locpdf) > 0 ){
   lapply( locpdf, function(x) dev.off(devl[x]) )
  }
  # open pdf device

    pdf(outpdf, paper=paper
       , height=height/25.4, width=width/25.4, onefile=onefile, ... )

 } else{
  # delete the pdf graphic device
  devl=dev.list()
  locpdf=which(names(devl) == "pdf")
  if( length(locpdf) > 0 ){
   cat("\nClosing devl[locpdf] = ",devl[locpdf],"\n")
   lapply( locpdf, function(x) dev.off(devl[x]) )
  }
 }

} # end of graph2pdf

#' Return the locations of the matching parentheses in a string.
#'
#' @param exp Input string: a single (vector of length 1).
#' @param loc The starting or ending locations of the parentheses
#' whose matching pair will be found.
#'
#' @details
#' \code{loc} must be positive and less than \code{nchar(exp)}.\cr
#' \code{substring(exp,loc,loc)} must be opening or closing parens.\cr
#' Otherwise, this function returns 0.
#'
#'
#' @return The location vecotr of length(loc) of the matching parens  or  0.
#'
#' @examples
#' # Input string
#' #                1111111111222222222233333333
#' #       1234567890123456789012345678901234567
#' exp <- "a+([a+b]*(c*{d+e}))-(f-g) +(((<()>)) "
#'
#' locbalpar( exp, 3 )
#' locbalpar( exp, 19 )
#' locbalpar( exp, c(4,8) )
#' locbalpar( exp, c(13,17) )
#' locbalpar( exp, c(31,34) )
#' locbalpar( exp, c(34,31, 3,19) )
#'
#' locbalpar( exp, regexpr( "\\(", exp ) )
#'
#' # No matching paren
#' locbalpar( exp, 28 )
#'
#'
#' # As long as substr(exp,n,1) is a paren, below should always be true.
#' n <- 10; n == locbalpar( exp, locbalpar( exp, n ) )
#' n <- 36; n == locbalpar( exp, locbalpar( exp, n ) )
#'
#' @export
#'

locbalpar <- function( exp, loc ){
 # find the location of the balanced paren
 # Shin-ichi Mayekawa
 # 20111208,20150620
 # remove attributes from loc: 20150823
 # returns 0 instead of NULL when error: 20170922
 # roxygen2 error fixed: 20191129dnc
 # roxygen2 error (\\\\( fixed: 20200609dnc
 # error check: 20221011dnc
 # vector loc: 20221011dnc
 #

 # error check
 if( length(exp) > 1 ){
  cat("\n\nerror1(locbalpar)"
      , "exp must be a string vector of length 1.\n\n")
  return(0)
 }
 nstr=nchar(exp)
 nloc=length(loc)
 if( any(loc > nstr)  |  any(loc <= 0) ){
  cat("\n\nerror1(locbalpar)"
      , "loc must be positive and less than nchar(exp).\n\n")
  return(0)
 }

 # remove attributes from loc by regexpr
 attributes(loc)$match.length=NULL
 attributes(loc)$useBytes=NULL


 # recursive call when loc is a vector
 if( nloc >= 2 ){
  res=numeric(nloc)
  for( i in 1:nloc ){
   res[i]=Recall( exp, loc[i] )
  }
  return( res )
 }


 ###########################################################################
 # simple case starts here.
 ###########################################################################

 # paren
 par=substr(exp,loc,loc)

 # matching paren
 lpar=c("(","{","[","<")
 rpar=c(")","}","]",">")
 if( any( par %in% lpar ) ){
  par2=rpar[par==lpar]
  rev=0
  exp1=substr(exp,loc,nstr)
 }
 else if( any( par %in% rpar ) ){
  par2=lpar[par==rpar]
  rev=1
  exp1=revchar(substr(exp,1,loc))
  loc0=loc
  loc=1
 }
 else{
  cat("\n\nwarning1(locbalpar) substr(exp,loc,loc) is not a paren.\n\n")
  return(0)
 }

 #Print(exp,exp1,printrowname=1)

 # individual character as a vector element
 vexp=unlist(strsplit(exp1,""))
 nstrs=length(vexp)
 # Print(vexp, nstrs,printrowname=1)

 # mark the location of par and par2
 locl=grep(par,vexp,fixed=1)
 locr=grep(par2,vexp,fixed=1)
 # Print(locl,locr)

 # count from the left
 npar=numeric(nstrs)
 npar[locl]=1; npar[locr]=-1
 parcount=cumsum(npar)
 # Print(npar,parcount,printrowname=1)

 # parcount = 0 indicates the location of matching par
 locbp=which(parcount==0)[1]
 if( !is.na(locbp) ) locbp=locbp+loc-1
 else locbp=0
 if( locbp > 0  &  rev == 1 ){
  locbp=loc0-locbp+1
 }
 # Print( locbp )

 attributes(locbp)=NULL
 return( locbp )

} # locbalpar

#' Analyze Three Dots and Return the Names and the Values
#'
#' @param ... The three dots to be analyzed.
#' @param omit_NULL = 0 to include the NULL arguments.
#' @param print_the_result = 1 to print the result
#' @param debug_Debug = 1 to print intermediate info
#'
#' @details
#' ... will be analyzed and the pair of ( name, val ) and name2
#'  will be returned.\cr
#'  See the first example below.
#'
#'  Note that, when ... is used, partial match of formal and actual arguments
#'  may cause a confusing error. \cr
#'  See the last example below.
#'
#' @return A list of: \cr
#' name  A vector consisting of the names of each of ... \cr
#' val   A list consisting of the values of each of ... or NULL \cr
#' name2  For the named arguments, name2 contains the righthand side of
#' the equal signs which appear in ... . \cr
#' For the positional arguments, name2 contains the actual argument itself.
#'
#'
#' @examples
#' cc="cccc"
#' analyze_three_dots( a=1, b=c(2,3), cc, list(aa=1,2), diag(2) )
#' analyze_three_dots( a=1, b=c(2,3), cc=NULL, d=0, NULL )
#' analyze_three_dots( a=1, b=c(2,3), cc=NULL, d=0, NULL, omit_NULL=0 )
#'
#' # a function to print all of ... with names
#' mprint <- function( ... ){
#'  atd=analyze_three_dots( ... )
#'  for( i in 1:length(atd$name) ){
#'   cat(atd$name[i], ":", atd$name2[i],"\n")
#'   print(atd$val[[i]])
#'  }
#' }
#' mprint( 1:5, b=7:11 )
#'#'
#' # About partial match of the arguments
#' # "e" will be missing
#' test <- function( exp, ..., debug_Debug=0 ){
#' 	res=analyze_three_dots( ..., debug_Debug=debug_Debug  )
#' 	Print(res)
#' }
#' expr="aaa"
#' test( expr, a=1, b=2, c=3, d=4, e=5, f=6, g=7 )
#'
#'
#'
#' @export
#'

analyze_three_dots <- function( ..., omit_NULL=1
                                , print_the_result=0, debug_Debug=0 ){
	# analyze three dots and return name and val
	# Shin-ichi Mayekawa
	# 20161127
	# bugfix: 20170513
	# roxgen2 added: 20170926
	# name2: 20200609dnc
	# example bugfix: 20200701
	# roxygen2: 20220719dnc
	# omit_NULL: 20220910
	# new method to get name: 20230508
	#

 # get name and value from ... : new method 20230508
 name2=as.character(substitute(list(...)))[-1L]
	name=names( as.list(substitute(list(...)))[-1L] )
	val=list(...)

	if( is.null(name) ) name=character(length(val))
	if( is.null(names(val)) ) names(val)=character(length(val))



	if( debug_Debug ){
		Print( name,val,name2)
	}

	if( length(val) == 0 ){
		return(NULL)
	}

	if( omit_NULL ){
  # Omit NULL elements
 	locnull=which(unlist(lapply(val,is.null)))
 	if( length(locnull) > 0 ){
 		name=name[-locnull]
 		name2=name2[-locnull]    # 20220910
 		val=val[-locnull]
 	}
	}
	n=length(name)

	if( n == 0 ) return( NULL )

	# replace the name by the value name if name does not contain name
	for( i in 1:n ){
		ni=names(val[i])
		if( debug_Debug ) Print(i,name[i],names(val[i]))
		if( length(ni) > 0 && ni != "" ){
			if( name[i] == "" ) name[i]=ni               # 20230508
		}
		else{
		 if( name[i] == "" ) name[i]=name2[i]         # 20230508
		}
	}

	if( print_the_result ){
		for( i in 1:n ){
			cat(i, ": name = ", name[i], ",   name2 = ", name2[i], sep="" )
			if( !is.list(val[[i]]) && !is.array(val[[i]]) )
				cat("   value = ", val[[i]], sep="", "\n")
			else{
				cat("\n")
		 	print(val[[i]])
			}
		}
	}

	return( list(name=name, val=val, name2=name2) )

} # end of analyze_three_dots

#' logistic function with D=1.7
#'
#' @param x vector
#' @param D = 1.7
#' @details This function calculates \cr
#'   p=1/(1+exp(-D*x)) \cr
#' @return vector of the size of x
#' @export
#'

logistic <- function( x, D=1.7 ){
 # logistic function
 # Shin-ichi Mayekawa
 # 20150418 cot,19
 # no minp: 20150622
 #
 return( 1/(1+exp(-D*x)) )
} # end of logistic


#' logit function with D=1.7
#'
#' @param p vector
#' @param D = 1.7
#' @details This function calculates \cr
#'   x=log(p/(1-p))
#' @return vector of the size of p
#' @export
#'

logit <- function( p, D=1.7 ){
 # inverse of logistic function
 # Shin-ichi Mayekawa
 # no minp: 20150622
 return( log(p/(1-p))/D )
} # end of logit



#' logistic function with D and minimum value
#'
#' @param x vector
#' @param D = 1.7
#' @param minp minimum value of the function
#' @details This function calculates \cr
#'   p=1/(1+exp(-D*x)) \cr
#' and applies minp as \cr
#'   p[p<minp]=minp; p[p>1-minp]=1-minp
#' @return vector of the size of x
#' @export
#'

logistic1 <- function( x, D=1.7, minp=10e-9 ){
 # logistic function
 # Shin-ichi Mayekawa
 # 20150418 cot,19
 # renamed as 1: 20150622
 #
 p=1/(1+exp(-D*x))
 p[p<minp]=minp; p[p>1-minp]=1-minp
 return( p )
} # end of logistic


#' logit function with D and minimum value
#'
#' @param p vector
#' @param D = 1.7
#' @param minp minimum value of the function
#' @details This function calculates \cr
#'   x=log(p/(1-p))
#' at the following p values \cr
#'   p[p<minp]=minp; p[p>1-minp]=1-minp
#' @return vector of the size of p
#' @export
#'

logit1 <- function( p, D=1.7, minp=1e-9 ){
 # inverse of logistic function
 # Shin-ichi Mayekawa
 # renamed as 1: 20150622
 p[p<minp]=minp; p[p>1-minp]=1-minp
 x=log(p/(1-p))
 return( x/D )
} # end of logit

#' Comment out multiple lines
#'
#' @param ... lines to be commented out in matched quotes
#' @return None
#' @examples comments(' whatever you want except for single quotes ')
#' @export
#'

comments <- function(...){
 # block comments
 # Shin-ichi Mayekawa
 # 111126,30,1203
 #
 # use this as
 #   comments('
 #             whatever, as many lines as, ...
 #            ')
 #
 return(invisible())
} # end of comments



#' Retruns the Last Value
#'
#' @param ... Not Used
#'
#' @examples
#' \dontrun{
#' set.seed(1701)
#' rnorm(10)
#' lv()
#' }
#' 
#' @export
#'

lv <- function( ... ){
 # functoin to return the last value: (Value of Last Evaluated Expression)
 # Shin-ichi Mayekawa
 # 120125
 # renamed as lv: 20150620
 # ... added as argument: 20230808dnc
 #
 
 return(.Last.value)
} # end of lv









#' Find the mode/typeof of the columns of data
#'
#' @param data a data frame to be checked.
#' @param typeof = 1 to return typeof(var) instead of mode(var)
#'
#' @details
#'    mode   -> "numeric", "character", etc \cr
#'    typeof -> "integer", "double", "character", etc \cr\cr
#' When data contains factors, "factor" will be returned for those varaibles.
#'
#' @return a vector consisting of typeof or mode.
#'
#' @examples
#'  df=data.frame(a=1,b="a",c=as.factor(1), stringsAsFactors=0)
#'  vartype(df)
#'  vartype(df, typeof=1)
#'  ls=list(a=1:2,b=2:4,c=c("a","b"),A=diag(2))
#'  vartype(ls)
#'  vartype(ls, typeof=1)
#'
#' @export
#'
vartype <- function( data, typeof=0 ){
 # find the mode/typeof of the columns of data
 # Shin-ichi Mayekawa
 # 20140226
 # treatment of factor: 20140604,10
 # list is a vector: 20140610,20160731
 #
 #
 # Args:
 #
 #  data    data frame, matrix or vector
 #
 #  typeof  = 1 to return typeof(data) instead of mode(data)
 #
 #
 # Values
 #  returns the value of mode or typeof functon:
 #    mode   -> "numeric", "character", etc
 #    typeof -> "integer", "double", "character", etc
 #
 #  When data contains factors, "factor" will be returned for those varaibles.
 #
 #  sapply( data.frame, class ) will give you the same result.
 #  sapply( list,       class ) will be different.
 #
 #

 # location of factors
 if( is.factor(data) ) locf=TRUE
 else if( is.matrix(data) | is.vector(data) )
  locf=FALSE
 else if( is.list(data) )
  locf=sapply(data,is.factor)
 locf=which(locf)

 type=NULL
 if( typeof != 1 ){
  if( is.array(data) | (is.vector(data) & !is.list(data)) )
   type=mode(data)
  else if( is.list(data) )
   type=sapply(data,mode)
 }
 else{
  if( is.list(data) )
   type=sapply(data,typeof)
  else if( is.array(data) | is.vector(data) )
   type=typeof(data)
 }

 if( is.null(type) ) type=rep("factor",length(locf))
 else type[locf]="factor"

 return( type )

} # end of vartype




#' Displays the attribuets and the structure of the input.
#'
#' @param X an object whose info we would like to have.
#' @param opt = 1 to print structure only.\cr
#' =2 to print attributes only.
#'
#' @return A list
#'
#' @examples
#'   bool <- TRUE
#'   num <- 1
#'   cpl <- 1+1i
#'   strng <- "Hello."
#'   vec <- 1:10
#'   mat <- matrix(c(1, 2, 3), 3, 2)
#'   array <- array(1,c(2,2,2))
#'   df <- data.frame(a=1,b=letters[1:2])
#'   lis <- list(a=1:3, b="abc",NA)
#'
#'   was(bool)
#'   was(num)
#'   was(cpl)
#'   was(strng)
#'   was(vec)
#'   was(mat)
#'   was(array)
#'   was(df)
#'   was(c)
#'
#' @export
#'

was <- function( X, opt=0 ){
 # Returns info on X
 # Shin-ichi Mayekawa
 # 111031,1101
 # avoid str and opt option: 20150528
 # size -> was: 20150605
 # output format, size of df: 20181101dnc,06
 #
 # type of basic R object
 #  logical, integer, real, complex, string (or character) and raw
 #
 # mode of R object
 #
 # typeof    mode      storage.mode
 # ---------------------------------
 # logical   logical   logical
 # integer   numeric   integer
 # double    numeric   double
 # complex   complex   complex
 # character character character
 # raw       raw       raw
 # ---------------------------------
 #
 #
 # Homogeneous Data Structures: vector, factor, matrix and array.
 # Only one mode in the structure.


 # Object	Max dimension	One mode of
 # --------------------------------------------------------------------
 # vector 	1            	numeric, character, complex or logical
 # factor 	1            	numeric or character
 # matrix 	2            	numeric, character, complex or logical
 # array  	n            	numeric, character, complex or logical
 # --------------------------------------------------------------------


 # Heterogeneous Data Structures:
 #
 # list	        n             	can be of mixed mode
 # data frame	  n             	can be of mixed mode
 #


 name2=c("vector","matrix","array","list","data.frame","factor","ordered")
 res1=attributes(X)
 res1=append( res1, list(
  length=length(X),
  mode=mode(X),
  storage.mode=storage.mode(X),
  typeof=typeof(X)
 ))

 res2=c(
  is.vector(X),
  is.matrix(X),
  is.array(X),
  is.list(X),
  is.data.frame(X),
  is.factor(X),
  is.ordered(X)
 )

 if( is.data.frame(X) ){
  dim=dim(X)
  res1=append( list(dim=unlist(dim)), res1 )
 }

 # Print(res1)
 # Print(name2[res2])

 if( opt == 1 ) return(name2[res2])
 else if( opt == 2 ) return(res1)
 else  return (list(Shape=name2[res2], Size=res1))

} # end of was






#' Column standardization of a data frame, matrix, or vector
#'
#' @param X data frame, matrix, or vector \cr
#' If X contains non-mumeric variable, they will be left as they are.
#' @param center = 1 to perform column centering only.
#' @param vardef = "n-1" to use unbiased variance
#' @return The standardized input.
#' @details When X is a data frame or matrix, the colums of which
#'  will be standardized. \cr
#'  NaN will be returned for those variables with 0 variance.
#' @examples
#' standard( data.frame(id=paste("id",1:5,sep=""), diag(5), a="a") )
#'
#' @export
#'

standard <- function( X, center=0, vardef="n" ){
 # column standardization of X
 # Shin-ichi Mayekwa
 # 130110,11,12
 # proper way to remove attributes: 20140601 cot, 0604
 # vartype: 20140604,05
 # return a vector when X is a vector: 20140604
 #
 # Args:
 #
 #   X        a matrix to be centered/standardized
 #   center   = 1 to perform centering only
 #   vardef   = "n-1" to use 'R default'unbiassed' formula
 #
 # Value:
 #
 #   Z        column centered/standardized matrix
 #            non-numeric variables, including factors, are left as they are.
 #
 #
 # Needs:
 #
 #   vartype
 #


 # location of numeric varaibles excluding factors
 locn=which(vartype(X) == "numeric" )
 if( length(locn) == 0 ) return(X)

 # is X a vector?
 isvec=0
 if( is.vector(X) ) isvec=1

 # make copy of X if there are non numeric variables in X
 subset=0
 if( is.data.frame(X) )
  if( ncol(X) > length(locn) ){
   XX=X
   X=X[,locn]
   subset=1
  }

 # standardize to 0-1
 if( center != 0 )
  X=scale(X,center=TRUE,scale=FALSE)
 else{
  X=scale(X,center=TRUE,scale=TRUE)
  # vardef
  if( vardef == "n" ){
   if( is.matrix(X) )  n=nrow(X)
   else n=length(X)
   X=X/sqrt((n-1)/n)
  }
 }

 # remove attributes
 attributes(X)$`scaled:center`=NULL
 if( !is.null( attributes(X)$`scaled:scale`) )
  attributes(X)$`scaled:scale`=NULL

 # post processing
 if( subset == 1 ){
  # recover nun numeric variables
  XX[,locn]=X
  return( XX )
 }
 else{
  # vectorize
  if( isvec ) X=c(X)
  return( X )
 }

} # end of standard





#' Inverse of Cummurative Sum
#'
#' @param x A numeric vector
#'
#' @details
#' This function returns lagged difference of lagg=1 and x[1] as: \cr
#'  c( x[1], diff(x,1) )
#'
#' @return
#' A vector of the inverse of cumsum.
#'
#' @examples
#' set.seed(1701)
#' x=sample(1:9,9,replace=1)
#' y=cumsum(x)
#' Print(x,y,cumsuminv(y))
#'
#' @export
#'

cumsuminv <- function( x ){
 # inverse of cummulative sum
 # Shin-ichi Mayekawa
 # 20150620
 #
 x=c(x)
 return( c(x[1],diff(x,1)) )
} # end of cumsuminv



#' Return the Ceiling and Power of the Input using Radix (Base) of n
#'
#' @param x input vector, x > 0
#' @param n Radix or Base
#'
#' @details
#'  When n=10 \cr
#'     0   <= x <    10,    ceiling_n(x)$val=10,  ceiling_n(x)$power=1 \cr
#'    10   <= x <   100,    ceiling_n(x)$val=100,  ceiling_n(x)$power=2 \cr
#'   100   <= x <  1000,    ceiling_n(x)$val=1000,  ceiling_n(x)$power=3 \cr
#'  When n=2 \cr
#'     0   <= x <     2,  ceiling_n(x,2)$val=2,  ceiling_n(x,2)$power=1 \cr
#'     2   <= x <     4,  ceiling_n(x,2)$val=4,  ceiling_n(x,2)$power=2 \cr
#'     4   <= x <     8,  ceiling_n(x,2)$val=8,  ceiling_n(x,2)$power=3 \cr
#'
#' @return
#' A list of: \cr
#' power The power \cr
#' val   The value of ceiling = n^power
#'
#' @examples
#' ceiling_n( c(1,9,10,99,100) )
#' ceiling_n( c(1,2,3,4,7,8,15,16), n=2 )
#'
#' @export
#'

ceiling_n <- function( x, n=10 ){
 # returns the ceiling of decimal x using radix (base) of n
 # Shin-ichi Mayekawa
 # 20150601
 # error check: 20150620
 #
 # When n=10
 #    0   <= x <    10     10
 #   10   <= x <   100    100
 #  100   <= x <  1000   1000
 # When n=2
 #    0   <= x <     2     2
 #    2   <= x <     4     4
 #    4   <= x <     8     8
 #

 x=c(x)
 if( any(x <= 0) ){
  cat("\n\nerror1:(ceiling_n) Input must be positive,\n")
  return( NULL )
 }

 power=( floor( log(x)/log(n) + 1e-99998 )+1 )
 val=n^power
 return( list(power=power, val=val) )

} # end of ceiling_n



#' Returns the (approximate) # of Digits on the Both Sides of the Decimal Point
#'
#'
#' @param num A numeric vector
#' @param maxpower 10 or higher
#'
#' @return A vector or matrix containing the # of digits
#'
#' @examples
#' decp(c(-5.2,-3.99, 0, 1, 1.2, 5.23, 9999.8888, 99999999.8888888888))
#'
#' @export
#'

decp <- function( num, maxpower=10 ){
 # returns the # of digits at the both sides of the decimal point
 # Shin-ichi Mayekawa
 # IML version: 031116(AIEJ)
 # 20111122
 # vector input: 20120204
 # bugfix: 120204
 #
 # Value: a matrix of
 #   num  left  right
 #
 # Note:
 #  If you see the following warning message:
 #     probable complete loss of accuracy in modulus
 #   try to increase/decrease maxpower constant. It might help.
 #
 # This program needs to be improved.
 #

 if( is.matrix(num) ) num=as.vector(num)
 n=length(num)
 res=matrix(0,n,3); res[,1]=num
 colnames(res)=c("num","left","right")
 num=abs(num)

 for( i in 1:n ){
  numi=num[i]

  # left
  w1=floor(log(abs(numi))/log(10))+1
  if( w1 < 0 ) w1=0

  # right
  for ( w2 in 0:maxpower ){
   mod1=( (10^w2)*numi ) %% 1
   if ( abs(mod1) <= 1e-8 ){
    break
   }
   # Print(numi,w2, (10^w2)*numi,mod1,w2)
  }

  # adjust for integers
  if( numi == floor(numi) ) w2=0
  res[i,2:3]=c(w1,w2)

 } # end of i loop

 res[which(res[,1] == 0),]=0
 if( n == 1 ) res=as.vector(res)
 return( res )

} # end of decp






#' Generate all combinations of n elements taken m at a time.
#'
#'   This is very slow when n is large.
#'
#' @param n total # of elements
#' @param m # of elements to choose
#'
#' @details
#' This function uses cprod function to calculate combinations. \cr
#' For example,
#'  n choose 3 will be calculated as: \cr
#'  x <- 1:n \cr
#'  y <- cprod(x,x[-1]) \cr
#'  y <- y[y[,1]<y[,2],] \cr
#'  y <- cprod(y,x[-(1:2)]) \cr
#'  y <- y[y[,2]<y[,3],] \cr
#'
#' @return
#' A matrix of the size choose(n,m) x m.
#'
#' @examples
#' n <- 5
#' for( m in 1:n )
#'  Print( m,choose(n,m), comb(n,m) )
#'
#' @export
#'

comb <- function( n, m ){
 # Generate all combinations of the elements of x taken m at a time
 # Shin-ichi Mayekawa
 # 20150627
 #
 x=matrix(1:n,n,1)
 dimnames(x)=NULL

 if( m == 1 ) return( x )
 if( m == n ) return( t(x) )

 y=cprod(x,x[-1])
 y=y[y[,1]<y[,2],]
 dimnames(y)=NULL
 if( m == 2 ) return( y )

 for( i in 3:m ){
  y=cprod(y,x[-(1:(i-1))])
  y=y[y[,i-1]<y[,i],]
 }
 dimnames(y)=NULL
 return(y)

} # end of comb






#' Print an Object with Print function with fuzz= option
#'
#' @param obj An object to be printed
#' @param fuzz fuzz parameter of Print
#' @param ntogoback n parameter of sys.parent function
#'
#' @details
#' When used with postfix function application operator of
#' \strong{lazy.symbolic} package as
#' \code{ lv() \%//\% fuzz }, \cr
#' .fuzz. global variable will be referenced instead of fuzz parameter.
#'
#' @examples
#' Ap <- c(1e-2,1e-5,1e-7)
#' Ap
#' fuzz(Ap)
#' # Above is equivalent to: Ap %//% fuzz
#' .fuzz. <- 1e-6
#' fuzz(Ap)
#' #Ap %//% fuzz
#'
#' @export
#'
fuzz <- function( obj, fuzz=1e-4, ntogoback=1 ){
 # print an object by Print with the fuzzy option
 # Shin-ichi Mayekawa
 # 20150928DNC
 #

 if( exists( ".fuzz.", where=sys.frame(sys.parent(n=ntogoback)) ) ){
  .fuzz=.fuzz.
  if( !is.null(.fuzz)  &  is.numeric(.fuzz) ) fuzz=.fuzz
 }

 Print(obj, fuzz=fuzz)
 return( invisible() )

} # end of fuzz




#' Overlay strings
#'
#' @param ... Strings to be overlayed. Eeach string must be a scalar.
#' @param bottom = 0 to overlay on top of the last string
#' @param blanks Characters which will be treated as blank.
#' @param BLANK Blank character to be used in the final result.
#'
#' @details
#' When bottom=1, the i th string will be overlayed on top of i-1 th. \cr
#'
#' @examples
#' c1="12345 78 01"
#' c2="a c:efg x"
#' str_olay( c1,c2 )
#' str_olay( c1,c2, bottom=0 )
#' str_olay( c1,c2, blanks=": " )
#'
#' @export
#'

str_olay <- function( ..., bottom=1, blanks=" ", BLANK=" " ){
 # overlay string
 # Shin-ichi Mayekawa
 # 20151008
 #

 # blanks
 blanks=strsplit(blanks,"")[[1]]

 # analyse ...
 name=name <- as.character(substitute(list(...)))[-1L]
 val=list(...)
 locnull=which(unlist(lapply(val,is.null)))
 if( length(locnull) > 0 ){
  name=name[-locnull]
  val=val[-locnull]
 }
 val=unlist(val)
 narg=length(val)

 # Print(name,val)

 if( narg == 0 ) return( NULL )
 else if( narg == 1 ) return( val[1] )

 if( bottom != 1 ){
  val=val[narg:1]
 }

 res=strsplit(val[1],"")[[1]]
 for( i in 2:narg ){
  v=strsplit(val[i],"")[[1]]
  for( j in 1:length(v) ){
   if( !(v[j] %in% blanks) ) res[j]=v[j]
  }
 }

 # final
 res[is.na(res) | (res %in% blanks)]=BLANK

 res=paste(res,collapse="")

 # Print(res)

 return( res )

} # end of str_olay




#' Find the Complexity Index of a 0-1 String
#' Definded as the Number of Switches from "01" to "10" to Make It Simple.
#'
#' @param pat01 A matrix consisting of 0s or 1s \cr
#' or a vector whose elements are strings consisting of 0s and 1s.
#' @param rev = 1 to count the # of switches from "10" to "01".
#' @param print = 1 to print the intermediate result.
#'
#' @details
#' When rev == 0, if a string contains no 1's to the left of 0's
#' it is assumed to simple. \cr
#' This function returns the # of switches from "01" to "10"
#' in order to make the string simple. \cr
#' \cr
#' When rev == 0, the roles of 0 and 1 will be interchanged.
#' \cr\cr
#' When the input is a matrix or data frame, complexity index of each row will
#' be calculated.
#'
#' @return
#' A vector consisting of the # of switches for each string.
#'
#' @examples
#'
#' complex_index("0101", print=1 )
#' pat01=matrix(c(0,1,1,0,0,1,0,1,0,0,1,1,0,0),2)
#' res=complex_index( pat01 )
#' Print(pat01,res)
#'
#' @export
#'

complex_index <- function( pat01, rev=0, print=0 ){
 # find the # of simple switches between 01 and 10 to achieve total simplicity.
 # Shin-ichi Mayekawa
 # old iml version 19930402
 # 20161208
 #
 #

 # target
 if( rev == 0 ){
  from="01"; to="10"
 }
 else{
  from="10"; to="01"
 }
 # make the input a matrix
 if( is.data.frame(pat01) ) pat01=as.matrix(pat01)
 else if( is.vector(pat01) ){
  nc=nchar(pat01)
  if( all( nc == 1 ) ){
   # each element of the vector has 1 character
   # make it a row vector
   pat01=matrix(pat01,1)
  }
  else{
   # the # of chars in each element of the vector is different
   # make it a col vector
   pat01=matrix(pat01,,1)
  }
 }

 # concatenate if there are multiple columns
 if( ncol(pat01) > 1 ) pat01c=( apply(pat01,1,paste, collapse="") )
 else pat01c=pat01

 n=nrow(pat01)
 res=numeric(n)
 for( i in 1:n ){
  pi=pat01c[i]
  if( print ) cat(i, "   :", pi, "\n")
  c=0
  while( length(grep(from, pi)) > 0 ){
   c=c+1
   pi=sub(from, to, pi)
   if( print ) cat(c, "   :", pi, "\n")
  }
  res[i]=c
 }

 return(res)

} # end of complex_index

#' Creates html files from the help files of an R package
#'
#' @param pkgname  Package name
#' @param outdir The name of the directory to which the html files 
#' are written.
#' @param debug = 1 to print details.
#'
#' @examples
#' #  res1=package_info("lazy.tools",      "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.mat",        "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.symbolic",   "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.stat",       "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.sasef",      "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.mdpref",     "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.procrustes", "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.irt",        "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.fa",         "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.cluster",    "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.mirt",       "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.mds",        "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.accel",      "d:/Rpgm/packages/HTML_help")
#' #  res1=package_info("lazy.Bayes",      "d:/Rpgm/packages/HTML_help")
#' cat("none\n")
#'
#' @references
#'   Part of the code is from http://m884.hateblo.jp/entry/20100214/1266135349 by
#'   masaha03 (id:phosphor_m).
#'
#' @export
#'

package_info <- function( pkgname
                        , outdir="d:/Rpgm/packages/HTML_help", debug=0 ){
 # display info on package and create local html help files
 # Shin-ichi Mayekawa
 # 20150618
 # read 00Index: 20150618DNC
 # top: 20150822
 # / in the filename: 20150924,25
 # < added: 20151008
 # default outdir: 20161121
 # roxgen2: 20171007
 #

 # Args:
 #
 #  pcgname   a package name
 #  outdir    name of the directory in which the html files will be written,
 #            or NULL to skip html creation.
 #
 # Note
 #  All the information is contained in head and funclist.
 #  It is not neccesary to read 00Inde.html but it makes it much easier
 #  to construct index.html.
 #

 # execulte help(package="packagename")
 code=paste( 'pkg=help( package="', pkgname,'", help_type="text"'
             , ', try.all.packages=TRUE )', sep="" )
 eval(parse(text=code))

 # pick up the part which defines a list of functions in the package.
 # pkg below contains a string "list( c( xxxx ), c(xxxx), ...)"
 pkg=as.character(pkg)[3]

 #cat("\n***************\n")
 #pkg2=gsub('\"',"'",pkg,fixed=TRUE)
 #print(pkg2)
 #cat("\n***************\n")

 # assign the list to pkg_info
 code=paste("pkg_info=",pkg,sep="")
 eval(parse(text=code))

 # last element is NULL
 pkg_info=pkg_info[-3]

 # Here, funclist contains all package info.
 # The first element is the description, the 2nd, function list
 head=pkg_info[[1]]
 funclist=pkg_info[[2]]

 # function names
 from=which( substr(funclist,1,1) != " " )
 nfunc=length(from)
 to=from[-1]-1; to[nfunc]=length(funclist)
 funcnames=sapply( strsplit( funclist[from], " " ), "[[", 1 )

 # get path info by giving one function name.
 file=as.character( help( funcnames[1], (pkgname), help_type="text"
                          , try.all.packages=FALSE ) )
 path <- dirname(file)
 dirpath <- dirname(path)


 cat("\nInformation on package:", pkgname,"\n")
 cat("\n Stored in ", dirpath,". \n\n")
 cat("Description\n")
 print(head, quote=0)
 cat("\n\nList of Functions\n")
 Print(from,to,funcnames)

 # copy 00Index.html from R library directory
 Index=paste(dirpath,"/html/00Index.html", sep="")
 pkg_index=readLines(Index)

 # modify link to DISCRIPTION (in the same directory)
 pkg_index=gsub("../DESCRIPTION","DESCRIPTION",pkg_index)

 # modify UP link
 pkg_index=gsub("../../../doc/html/packages.html","../packages.html",pkg_index)

 # modify TOP link
 pkg_index=gsub("../../../doc/html/index.html"
                , paste(pkgname,".html",sep=""),pkg_index)


 # R.css
 Rcss=readLines(paste(dirpath,"/html/R.css", sep=""))


 if( !is.null(outdir) ){

  # output directory
  outdir=paste( outdir,"/", pkgname, sep="" )
  if( !dir.exists(outdir) ){
   dir.create(outdir, recursive=TRUE)
  }

  # Rcss
  outdirfile=paste(outdir,"/R.css",sep="")
  writeLines(Rcss, outdirfile)

  # index file
  outdirfile=paste(outdir,"/index.html",sep="")
  writeLines(pkg_index, outdirfile)

  # description
  outdirfile=paste(outdir,"/DESCRIPTION",sep="")
  writeLines(head, outdirfile)

  # each function files
  cat("\n processing .")
  for( i in seq_along(funcnames) ){
   fname=funcnames[i]
   if( regexpr("[*/<]",fname)[[1]] > 0 ) next
   outdirfile=paste(outdir,"/",fname,".html",sep="")
   if( debug ) Print(outdirfile)
   res=extract_help( fname, pkgname=pkgname, print=0, debug=debug )
   # modify the last-1st line, where, usually, we find a link to package index.
   ll=res[length(res)-1]
   ll=gsub("]",' <a href="index.html">Index</a>]',ll)
   res[length(res)-1]=ll
   writeLines( res, outdirfile )
   cat(".")
  }
  cat(" done!\n")

 } # end of html creation

 return(funclist)

} # end of package_info


extract_help <- function( funcname, pkgname=NULL, print=1, debug=0 ){
 # extract help file of func in package
 # part of the code is from http://m884.hateblo.jp/entry/20100214/1266135349
 # Shin-ichi Mayekawa
 # 20150618

 # Args:
 #
 #  funcname   function name
 #  pkgname    package name or NULL
 #  print      = 1 to print ugly result
 #  debug      = 1 to print intermediate result
 #

 if( is.null(pkgname) )
  file=as.character( help( funcname, help_type="text", try.all.packages=TRUE ) )
 else
  file=as.character( help( funcname, (pkgname), help_type="text"
                           , try.all.packages=FALSE ) )
 path <- dirname(file)
 dirpath <- dirname(path)
 pkgname2 <- basename(dirpath)
 RdDB <- file.path(path, pkgname2)
 if( debug ){
  Print(funcname,pkgname)
  Print(path,dirpath,pkgname2)
  Print(RdDB)
 }
 if (file.exists(paste(RdDB, "rdx", sep = "."))) {
  temp <- tools::Rd2HTML(tools:::fetchRdDB(RdDB, basename(file)),
                         out = tempfile("Rtxt"), package = pkgname2,
                         outputEncoding=localeToCharset()[1], no_links=TRUE)

  temp2 <- readLines(temp)
  temp2 <- gsub("(&lsquo;)|(&rsquo;)|(&ndash;)","", temp2)
  if( print == 1 ){
   cat("\n\n\n*** top of ", funcname," in ", pkgname, "\n")
   print(temp2)
   cat("********************** end temp2 ***********************\n")
  }

  return( temp2 )
 }
} # end of extract_help

#' Print arguments with their names and formats
#'
#' @param ... objects to be printed with name.
#' @param fmt A vector of formatC to be used
#' @param NAchar a character to represent NA.
#' @param removeatt = 1 to remove the attributes from each of ... .
#' @param printrowname = 0 not to print rownames when printing vector.
#' @param printrownames Same as printrowname.
#' @param fuzz Those elements which satisfy abs(X) < fuzz will be printed.
#' @param fuzzchar Replacement character(s) for those abs(X) < fuzz.
#' @param sep character to be used as the delimiter.
#' @param debug = 1 to dump intermediate result.
#'
#' @return The first element of ... invisibly.
#'
#' @details
#'
#' If ... contains lists or data.frames, they will be printed separetely.#'
#' \cr
#'   Except for the case when ... consists of a single vector,
#'   vectors will be printed as column vectors without row/col names. \cr
#'   To add rownames to vectors, specify printrowname=1. \cr
#'   "/" or "\\n" can be used as the line break. \cr
#' \cr
#'   If any of the input matrices does not have row/col names
#'   they will be generated using "r" and "c" as the roots.
#'\cr
#'   If the first argument is a number or a quoted string, it will be printed
#'   as it is (without name.) \cr
#'\cr
#'   When there are only two objects to be printed,
#'   and the first one is a constant and the second one is not a matrix,
#'   there will be no line breaks  between the first and the second objects
#'   unless the first one has "\\n". \cr
#'    compare:  \cr
#' \preformatted{
#'     Print("first", pi)
#'     Print("first\n", pi )
#'     Print("first", pi, pi )
#' }
#' \cr
#'
#' fmt  contains formats to be used separated by commas or blanks: \cr
#'  \code{fmt="FNameWid.Digits"} will use the following formatC function \cr
#'  \code{formatC( object, format="FName", width=Wid, digits=Digits )} \cr
#'  to format the object. \cr
#'  If there are more objects in ... than length(fmt),
#'  the last fmt value will be used to fill.
#'  \cr
#'
#' When fmt is given, the attributes of the object may be printed together.\cr
#' Use removeatt=1 to remove all the attributes except dimnames and dim.
#'
#'
#' The first element of ... will be returned invisibly so that
#' this function can be used with pipe operator. \cr
#' Use \code{capture.output} function, not the return value,
#' to utilize the printed output of this function.
#'
#'
#' @examples
#' bool <- TRUE; num <- 1; cpl <- 1+1i; str <- "Hello."
#' vec <- 1:10; mat <- matrix(c(1, 2, 3), 3, 2)
#' lis <- list(a=1:3, b="abc",NA); df <- data.frame(a=1:2,b=2:3,c="cc")
#' Print(bool, num, cpl, str, lis, vec, df, mat, plot, sin(0))
#'
#' A=demomat(3,2)
#' Print("This is what I need.", A)
#' Print(A/22)
#' Print(A/22, A/22, fmt="6.2, 8.4")
#' Print(A/22, fuzz=0.6)
#'
#' Print(diag(5),1:3)
#' Print(diag(5), diag(2))
#' Print(diag(5), "/", diag(5))
#' char_vec <- "This is a character vector."
#' Print("string", char_vec, t(1:2))
#'
#' B <- demomat(3,3,2,supermat=2)
#' Print(B,fmt="6.3")
#' Print(B,fmt="6.3", removeatt=1)
#'
#' # with pipe
#' C <- demomat(2,2, root="c")  |> Print(A)
#' print(C)
#' C <- demomat(2,2, root="c")  |> Print(A, .=_)
#' print(C)
#'
#' @export
#'

Print <- function( ...
                   , fmt=NULL, NAchar=".", fuzz=-1, fuzzchar="..", sep=","
                   , removeatt=0, printrowname=0, printrownames=0, debug=0 ){
 #
 # Print objects with names
 # Shin-ichi Mayekawa
 # The orginal version: 080208,11,0512, 0621
 # New versio with Mr. Takekatsu Hiramura
 #  (2010.08.09, 2011.10.28,29,30,1117,20120102,07,26)
 # table, numeric(0) etc: 20120205,08
 # fuzz: 120210
 # fmt: 120211
 # args with length 0: 120302
 # first quoted arg: 120302
 # str( ) : 120302,03
 # bugfix for the first missing: 121020
 # pass debug to printm: 121220
 # Printf: 121223c,24c
 # renamed as Print: 121225(Mitsubishi Motor Company)
 # renamed back to Print: 121227
 # name and val as arguments from Printf: 121228
 # digits as fmt=".digits": 121229
 # key -> name: 121230
 # data frame by Printdf: 121230
 # comments: 130102
 # Printdf removed, mapply modified: 130103
 # fmt in one string: 130103,04
 # new method for NAchar: 130107
 # name w/o as.character is passed to printm: 130107
 # "\n" as a line break: 130108
 # type argument from Printf: no name for all the constants: 130109
 # do not add "\n" to the first constant: 130110
 # omit line break from type: 130111
 # bug fix for the first constant: 130111
 # line break after first const and array: 130130
 # printrownames as alias: 20140809
 # bugfix: 20150526, 20150604dnc
 # remove fraction class: 20150809cot
 # aname: 20151127
 # fuzzchar: 20151215
 # name, val, type, digits removed: 20151215
 # removeat: 20151215
 # print fraction class: 20151215
 # NAchar error in print: 20160705dnc
 # printm -> mprint: 20220822
 # return the first element of ... invisibly: 20240626,27
 #
 #
 #
 # Args:
 #   ...     Objects to be printed
 #           The printing order may be different from the order in ...
 #           Vectors, matrices and arrays will be printed first.
 #
 #   fmt     A vector of formats to be used separated by commas or blanks:
 #           fmt="NameWid.Digits" will use the following formatC function
 #             formatC( object, format="Name", width=Wid, digits=Digits )
 #           to format the object
 #           If there are more objects in ... than length(fmt),
 #           the last fmt value will be used to fill.
 #           If fmt is "", format (not formatC) will be used
 #           for those objects witout format specs.
 #
 #   digits  an integer to specifity the # of digits to the right
 #           of dec point
 #           This will be used as fmt=".digits" when all the fmt are empty.
 #
 #   NAchar  a character to be used to print NA
 #   sep     a character to be used to separate printed objects
 #
 #
 #   Except for the case when ... consists of a single vector,
 #   vectors will be printed as column vectors without row/col names.
 #   To add rownames to vectors, specify printrowname=1.
 #   "/" can be used as the line break.
 #
 #   If any of the input matrices does not have row/col names
 #   they will be generated using "r" and "c" as the roots.
 #
 #   Those elements which satisfy abs(X) < fuzz will be printed
 #   using fuzzchar.
 #
 #   If the first argument is a number or a quoted string, it will be printed
 #   as it is (without name)
 #
 #   When there are only two objects to be printed,
 #   and the first one is a constant and the second one is not a matrix,
 #   there will be no line breaks  between the first and the second objects
 #   unless the first one has \n.
 #    compare:  Print("first", pi)
 #              Print("first\n", pi )
 #              Print("first", pi, pi )
 #
 #
 # Usage:
 #
 #     Print( i, A, B, C,  fmt=c("d1 f5 f5.0 f9.3") )
 #
 #
 # Needs:
 #  mprint.R,  ccbind2c, arraytomatc.R
 #  decomp_fmt.R,  gen_formatC.R
 #

 # old arguments: 20151215
 name=NULL; val=NULL; type=NULL; digits=NULL

 if( is.null(name) ){
  # store ...
  name <- as.list(substitute(list(...)))[-1L]
  val <- list(...)
 }


 # fraction class of MASS
 for( i in 1:length(val) ){
  if( "fractions" %in% class(val[[i]])  ){
   vali=val[[i]]
   clv=class(vali)
   loc=which(clv == "fractions")
   if( length(clv) > 1 ) clv=clv[-1]
   else clv=NULL
   class(val[[i]])=clv
   # Only this may be sufficient
   val[[i]]=format( matrix(attributes(vali)$fracs,nrow(vali)),justify="right")
  }
 }

 # remove zero-length objects
 lenval=lapply(val,length)
 if( debug ){ ##################################
  cat("\n\n**** Top of Print ****")
  cat("\n name:\n"); print(name)
  cat("\n val:\n"); print(val)
  cat("\n lenval:\n"); print(lenval)
  cat("\n fmt:"); print(fmt) ;cat("\n")
 } #############################################

 # initialize format
 if( is.null(fmt) ) fmt=rep("",length(name))
 else{
  if( length(fmt) == 1  && fmt != "" ){
   temp=strsplit(fmt,split="[\\, ]", fixed=0)[[1]]
   temp=temp[temp != ""]
   fmt=temp
  }
 }
 if( length(fmt) < length(name) )
  fmt[(length(fmt)+1):length(name)]=fmt[length(fmt)]

 # convert digits to fmt
 if( !is.null(digits)  && all(fmt == "") ){
  fmt=paste(".", digits,sep="")
  fmt=rep(fmt,length(name))
 }
 if( debug > 0 ){ ###########################################
  cat(" initial fmt =", fmt, " length =", length(fmt), "\n")
 } ##########################################################



 # remove zero-length objects
 loc=which( lenval == 0 )
 if( length(loc) > 0 ){
  cat("\nThe lengths of the following areguments are zero.\n")
  print(name[loc])
  if( length(grep("str\\(",name)) > 0 ){
   cat(" str function cannot be printed by Print.\n")
  }
  name=name[-loc]
  val=val[-loc]
  fmt=fmt[-loc]
 }
 if( length(name) == 0 ){
  return()
 }
 nname=length(name)

 # type of arguments: 130109
 # logical, double, integer, complex, character, symbol, language, etc
 # Printf and Print are the only functions to define type
 if( is.null(type) )
  type=sapply( name, typeof )


 # remove the type of line break (which is 'character')
 loc=which( name == "/"  |  name == "\n" )
 if( length(loc) > 0 ) type=type[-loc]

 # check if all are vector or matrix or array class
 morvora=( sapply( val, is.array ) + sapply( val, is.matrix )
           + sapply( val, is.vector )*!sapply( val, is.list ) )
 morvora=morvora*!sapply( val, is.table )
 locmorvora=which(morvora >= 1)
 locother=which(morvora == 0)

 # extract data frames
 # locdf=which( sapply( val, is.data.frame ) )
 # locother=setdiff( locother, locdf )

 # In this version, locdf is NOT used.
 locdf=NULL

 # alias
 if( printrownames != 0 ) printrowname=printrownames

 if( debug ){ ############################
  cat("\n(Print) after removing NULL objects\n")
  cat("\n nname = ",nname,"\n")
  cat("\n name = \n")
  print(name)
  cat("\n loc = ")
  print(loc)
  cat("\n type = ")
  print(type)
  cat("\n morvora = ")
  print(morvora)
  cat("\n locmorvora = ")
  print(locmorvora)
  cat("\n locother = ")
  print(locother)
  if( !is.null(locdf) ){
   cat("\n locdf = ")
   print(locdf)
  }
 } #######################################


 if( any(morvora >= 1 ) ){
  # use mprint.R

  # print the first constant w/o name and remove it from the list
  # name1=as.character(name)[locmorvora]
  name1=name[locmorvora]
  # name1=as.character(name[locmorvora])
  val1=val[locmorvora] # This is how to subset a list:  not [[ loc ]]
  fmt1=fmt[locmorvora]
  if( debug ){ #######################################################
   cat("\n name1 is as.character(name)[locmorvora]:\n")
   cat("name1[[1]]=", name1[[1]],":\n")
   cat("val1[[1]][[1]]=", val1[[1]][[1]],":\n")
   cat("fmt1=", fmt1[1],":\n")
  } ##################################################################

  # remove attributes if requested: 20151215
  if( removeatt ){
   for( i in 1:length(name1) ){
    att=attributes(val1[[i]])
    attributes(val1[[i]])=att[names(att) %in% c("dimnames","dim")]
   }
  }

  if( type[1] %in% c("character", "double", "integer"
                     , "complex", "logical") ){
   if( debug > 0 ) cat("\n(Print): First arg. is a constant.\n")

   # print it
   cat(val1[[1]]   ,"\n") # \n added 20150604

   if( nname == 1 ){
    cat("\n")
    return(invisible(val1[[1]]))
   }

   # remove the printed
   name1=name1[-1]
   val1=val1[-1]
   fmt1=fmt1[-1]
   type=type[-1]

   if( debug > 0 ){ ################################################
    cat("\nval1[1][[1]]=", val1[1][[1]],"\n")
    cat("type[1]=", type[1],"\n")
    cat("typeof(val1[1][[1])=\n", typeof(val1[1][[1]]),"\n")
    cat("is.matrix(val1[1][[1])=\n", is.matrix(val1[1][[1]]),"\n")
   } ###############################################################
   # If the object next to the first constant is matrix or array, NEWLINE.
   if( is.matrix(val1[1][[1]]) || is.array(val1[1][[1]]) ) cat("\n")
  }

  nname1=length(name1)



  if( nname1 > 0 ){
   if( debug > 0 ){ ###################################
    cat("\n nname1 = ", nname1, "\n")
   } ##################################################

   # subsets defined by line breaks
   loc=NULL
   loc0=which( val1 == "/" | val1 == "\n" ) # the locations of line breaks
   if( debug ){ #################################################
    cat("\n location of line breaks \n")
    print(loc0)
   } ############################################################
   if( length(loc0) > 0 ){
    # remove consecutive line breaks
    if( length(loc0) >= 2 ){
     loc=loc0[1]
     for( i in 2:length(loc0)){
      if( loc0[i] != loc0[i-1]+1 ) loc=c(loc,loc0[i])
     }
    }
    else loc=loc0

    # index of subsets
    loc=unique( c(1,loc,nname) )
    from=loc[1]; to=loc[2];
    if( length(loc) > 2 ){
     for( i in 2:(length(loc)-1) ){
      from=c(from,loc[i]); to=c(to,loc[i+1])
     }
    }
   }
   else{
    # no line breaks
    from=1; to=nname1;
   }
   nsub=length(from)

   if( debug ){ #################################################
    cat("\n # of subsets to be passed to mprint =", nsub
        , " with the following 'from' and 'to':\n")
    print(from); print(to)
   } ############################################################

   # when input consists of "\n" only. 20150604dnc
   if( is.na(to[1]) ) return(invisible())


   # for each subset, use mprint
   for( i in 1:nsub ){
    loc1=setdiff(from[i]:to[i],loc0)
    if( length(loc1) > 0 ){
     # remove NA or NULL elements (don't know where they come from!)
     name11=name1[loc1]
     loc11=!is.na(name11) & (1-unlist(lapply(name11,is.null)))
     name11=name11[loc11]
     val11=val1[loc1][loc11]
     fmt11=fmt1[loc1][loc11]
     if( debug ){ #######################################################
      cat("\n the following arguments are passed to mprint (",i,")\n")
      cat(" loc1 = ", loc1,  ", loc11 = ", loc11, "\n")
      cat(" name11 = ")
      print( name11 )
      cat(" val11 = ")
      print( val11 )
      cat(" fmt11 = ", fmt11, "length(fmt11)="
          , length(fmt11), "\n")
     } ##################################################################
     # use mprint
     if( length(name11) > 0 )  # 20150526
      mprint( name=name11, value=val11, fuzz=fuzz, fuzzchar=fuzzchar, fmt=fmt11
              , digits=digits, sep=sep, NAchar=NAchar
              , novecrname=1-printrowname, debug=debug, type=type )
    }
   } # end of i loop
  } # end of nname1 > 0

  # return(val1[[1])

 } # end of matrix etc


 # data frame: Not used
 #  if( length(locdf) > 0 ){
 #   name1=name[locdf]
 #   val1=val[locdf]
 #   fmt1=fmt[locdf]
 #   if( debug > 0 ){ ###################################################
 #    cat("\n**The following objects will be handled by Printdf\n")
 #    cat("name1\n"); print(name1)
 #    cat("val1\n"); print(val1)
 #   } ##################################################################
 #   for( i in 1:length(locdf) ){
 #    Printdf( name=name1[[1]], val1[[1]], fmt=fmt1[[1]], debug=debug
 #           , NAchar=".", digits=NULL, printrowname=0, fuzz=-1, sep="," )
 #   }
 #   # return()
 #  }


 if( length(locother) > 0 ){
  if( any(morvora >= 1 ) ) cat("\n")
  # original method by Mr. Hiramura
  if( debug > 0 ){ ###########################################
   cat("\n****HHHH**** locother=\n")
   print(locother)
  } ##########################################################
  name1=as.character(name[locother])
  val1=val[locother]
  mapply(
   function(k, v){
    cat(k, "= ")
    cat("\n"); print(v); cat("\n")
   }
   ,  # end of funcion
   name1, val1) # end of mapply
  cat("\n")
 }  # end of list etc

 return( invisible(val1[[1]]) )
 # temp=gc()

} # end of Print



mprint <-  function( ..., sep=",", nsep=2, matname=1, digits=NULL, type=NULL
                     , NAchar=".", novecrname=1, fuzz=0, fuzzchar=".."
                     , fmt=NULL, debug=0
                     , name=NULL, value=NULL){
 # print several matrices in a row
 # Shin-ichi Mayekawa
 # 20111202,
 # rn1, cn1, ets as args to ccbin2c: 111203
 # matprint renamed as printm: 111203
 # trim all removed to cope with negative elements: 111205
 # array print: 120102
 # (name,value) as input: 120102
 # NAchar: 120102
 # 120116
 # no col name for a vector: 120125,26,28,30
 # use format when nmat==1: 120210
 # fuzz: 120210,0303
 # bugfix (nmat): 121220
 # new fmt definition for formatC from Printf: 121223c
 # digits option disabled: 121225(Mitsubishi Mortor Company)
 # comments: 130102
 # name w/o as.character is passed from Print: 130109
 # type argument: 130109
 # quote=F -> quote=0: 130110
 # vec names: 20140809
 # fuzzchar: 20151215
 # NAchar error: 20160705dnc
 # printm -> mprint: 20220822
 #
 #
 # Args:
 #   ...      matrices to be printed
 #   sep      a character to be used to separate matrices (first row only)
 #   nsep     # of blanks to separate matrices
 #   matname  not used
 #   digits   # of digits to be used in format function
 #   NAchar   characters to represent NA
 #   novecrname = 1 to omit the rownames for vector arguments
 #
 #   Below two are not usually used.
 #     name     matrix names as a vector
 #     value    matrices as a list
 #
 #
 # Needs:
 #   ccbind2c, arraytomatc,    decomp_fmt, get_formatC
 #
 #
 #  Except for the case of single vector input,
 #  vectors will be printed as column vectors without row/col names.
 #  To add rownames to vectors, specify novecrname=0.
 #
 #  If an input matrix does not have row/col names
 #  they will be generated using "r" and "c" as the roots.
 #
 #  NA  will be printed using NAchar
 #
 #


 arraytomatc <- function( A, fwd="", debug=0 ){
  # convert an array to a character matrix for smart printing
  # to be used in Print
  # Shin-ichi Mayekawa
  # 120102
  # matrix as an array: 130115
  # aname: 20151127,1221
  #

  if( debug  > 0 ){
   cat("\n Top of arraytomatc\n")
   cat(" input = ")
   print(A)
  }
  n3=dim(A)[3]
  nr=dim(A)[1]
  nc=dim(A)[2]

  if( debug  > 0 ){
   cat("nr =",nr,"\n")
   cat("nc =",nc,"\n")
   cat("n3 =",n3,"\n")
  }


  # cannot handle NULL dimnames:
  if( is.null(dimnames(A))){
   dimnames(A)=
    list(paste("r",1:dim(A)[1],sep=""), paste("c",1:dim(A)[2],sep="")
         , paste("a",1:dim(A)[3],sep=""))
  }

  cname=dimnames(A)[[2]]; aname=dimnames(A)[[3]]
  blanks=matrix(rep("",nc),1)    # code=gen_formatC(fX2, X2, fwd2)
  # fA=formatC(A, format=fwd[1]
  #         , width=as.integer(fwd[2]), digits=as.integer(fwd[3]) )
  code=gen_formatC(fA, A, fwd)
  eval(parse(text=code))
  if( debug > 0 ){ ########################################
   cat("\n arraytomatc: code = >>", code, "<<\n")
  } #######################################################

  if( is.matrix(fA) ) fA=array(fA,dim(nr,nc,1))
  if( debug > 0 ){
   cat(" fA =")
   print(fA)
  }

  # Print(cname,blanks,fA)

  # row bind all the sub matrices
  if( is.null(aname) )
   aname=1:n3

  an=aname[1]
  mat=rbind(
   matrix(c(paste(", ,", an),rep("",nc-1)),1), blanks, cname, fA[,,1]
   )
  if( n3 >= 2 )
   for( i in 2:n3 ){
    an=aname[i]
    mat=rbind( mat, blanks,
      matrix(c(paste(", ,", an), rep("",nc-1)),1), blanks, cname, fA[,,i]
    )
   }

  #print(mat, quote=F)

  # remove cname, which was created when rbinded
  rnm=rownames(mat)
  rnm=sub("cname","",rnm)
  rownames(mat)=rnm

  # delete col names
  colnames(mat)=rep("",nc)

  # print(mat, quote=F)

  return( mat )

 } # end of arraytomatc



 ccbind2c <- function( fX1, fX2, name1, name2, rn1, cn1, rn2, cn2
                       , nsep=1, sep=",", matname=1, debug=0
                       , noX1name=0, noX2name=0 ){
  # cbind two character matrices fX1 and fX2 to form matrix big
  # Shin-ichi Mayekawa
  # 20111202
  # tuned to mprint.R: 111203
  # sep=sep -> sep=",": 120102
  # comments: 120102
  # noX2name: 130108
  #
  # Args:
  #  fX1, fX2      Two character matrices to be printed
  #  name1, name2  names for the input matrices
  #  rn1, cn1      row/col names for the first mat
  #  rn2, cn2      row/col names for the 2nd mat
  #  nsep          # of blanks to separate matrices
  #  sep           a character to be used to separate matrices at first row
  #  matname   = 1 to print the matrix name to the left of the body
  #            = 2 to print the matrix name to the left of the column names
  #            = 3 to print the matrix name above the body
  #                method 2 and 3 are not yet available
  #  noX1name    = 1 to skip printing the first matrix name and rownames
  #  noX2name    = 1 to skip printing the second matrix name and rownames
  #

  r1=nrow(fX1); r2=nrow(fX2); c1=ncol(fX1); c2=ncol(fX2);

  if( debug ){
   cat("\nIn ccbind2c:  fX1 and fX2\n")
   cat("fx1="); print(fX1)
   cat("rn1="); print(rn1);
   cat("cn1="); print(cn1);
   cat("noX1name="); print(noX1name)
   cat("fx2="); print(fX2)
   cat("rn2="); print(rn2);
   cat("cn2="); print(cn2);
   cat("noX2name="); print(noX2name)
  }

  # offset = 1 if there are no matrix name for the first one
  offset=0; if( noX1name == 1 ) offset=2;    # 2 for name and rownames
  offset2=0; if( noX2name == 1 ) offset2=2;  # 2 for name and rownames

  # big is the space where X1 and X2 will be stored
  # X1 consists of name1, rowname1, fX1, nsep
  rb=max(r1,r2); cb=c1+c2+nsep+2+2-offset-offset2;
  big=matrix(" ",rb,cb) # fill all the cells with blanks

  # first matrix name and rownames
  if( noX1name == 0 ){
   # first matrix name
   big[1,1]=paste(name1," =")
   # first matrix row names
   big[1:r1,2]=rn1;
  }
  # first matrix body
  big[1:r1,(3-offset):(c1+2-offset)]=fX1;
  # separator: use "?" which will be replaced by blank when printed.
  big[,(c1+3-offset):(c1+3+nsep-1-offset)]="?";
  # comma at row one to separate matrices
  big[1,(c1+3-offset)]=sep

  # second matrix name
  if( noX2name == 0 ){
   big[1,c1+nsep+3-offset]=paste(name2, " =")
   # second matrix row names
   big[1:r2,(c1+nsep+4-offset)]=rn2
  }
  # second matrix body
  big[1:r2,(c1+nsep+5-offset-offset2):cb]=fX2

  if( noX1name == 1 ){
   if( noX2name == 1 )
    colnames(big)=c(cn1,rep(" ",nsep), cn2)
   else
    colnames(big)=c(cn1,rep(" ",nsep), " ", " ", cn2)
  }
  else{
   if( noX2name == 1 )
    colnames(big)=c(" ", " ", cn1,rep(" ",nsep), cn2)
   else
    colnames(big)=c(" ", " ", cn1,rep(" ",nsep), " ", " ", cn2)
  }
  rownames(big)=rep("",rb)

  #  if( matname == 2 ){
  #   if( length(attributes(X1)$title) == 0 ){
  #    title=paste(name1
  #        , paste(unlist(rep(" ",lenfX1-nchar(name1)+1)), sep="",collapse="")
  #        , sep="",collapse="")
  #   }
  #   title=paste(title
  #        , paste(unlist(rep(" ",2*nsep)), sep="",collapse="")
  #        , name2
  #        , paste(unlist(rep(" ",lenfX2-nchar(name2)+1)), sep="",collapse="")
  #        , sep="",collapse="")
  #
  #   attributes(big)$title=title
  #   cat(title,"\n")
  #   print(big[1:rb,1:cb],na.print="",quote=F)
  #  }

  if( debug ) print(big, na.print="",quote=0)

  return(big)

 } # end of ccbind2c



 ############## main body of mprint starts here. ####################

 # blanks
 BLK="                                                                  "

 if( debug > 0 ){ ###############################
  cat("\n\n*** Top of mprint ***\n\n")
  cat("name ="); print(name)
  cat("value ="); print(value)
  cat("typeof(name) ="); print(typeof(name))
 } ###############################################

 # type of arguments: 130109
 # logical, double, integer, complex, character, symbol, language, etc
 if( is.null(type) )
  type=sapply( name, typeof )

 # make name character
 name=as.character(name)


 if( debug > 0 ){ ###############################
  cat("type ="); print(type)
  cat("name ="); print(name)
 } ###############################################


 # decompose fmt:   resulting fwd is 3 x length(fmt)
 fwd=sapply(fmt, decomp_fmt, debug=debug)
 if( debug > 0 ){
  cat("\nfwd\n")
  print(fwd)
 }

 # process ... args  if name and value are not given
 if( is.null(name) ){
  name=as.character( as.list(substitute(list(...)))[-1L] )
  value=list(...)
 }
 nmat=0
 for( i in 1:length(name) ){
  if( !is.na(name[[i]]) ) nmat=nmat+1
 }
 name=name[1:nmat]; value=value[1:nmat]; type=type[1:nmat];
 fwd=fwd[,1:nmat,drop=0]

 if( debug ){ ########################################################
  cat("(mprint): # of arguments =", nmat, "\n")
  for( i in 1:nmat ){
   cat(i, " name =", name[[i]], ", value =", value[[i]]
       , ", fmt =", fmt[i], ", type =", type[i], "\n")
  }
 } ###################################################################

 if( nmat == 1 ){

  # constant or not
  if( type[1] %in%  c("character", "double", "integer"
                      , "complex", "logical") ){
   cat(value[[1]],"\n")
   return()
  }

  temp0=value[[1]]
  # cat("value[[1]]=",value[[1]],"::")
  if( is.vector(temp0) ){
   # row vectrize
   temp=matrix(temp0,1)
   colnames(temp)=names(temp0)  # 20140809
   rownames(temp)=""
   if( length(temp0) > 0   &&  is.null(colnames(temp)) )
    dimnames(temp)=list(rep("",1),paste("cx",1:ncol(temp),sep=""))
  }
  else temp=temp0

  if( debug > 0 ){ ########################################################
   cat("\n  matrix to be printed stored in temp =", temp,"\n")
  } #######################################################################

  # Here, temp is matrix or array
  if( nrow(temp) == 1  &&  ncol(temp) == 1 ){
   # temp is a scalar
   if( debug > 0 ){ ####################################
    cat("\n temp is a scalar.\n")
   } ###################################################
   code=gen_formatC(temp, temp, fwd)
   eval(parse(text=code))
   colnames(temp)=""
   #### 20130105
   # if( name == temp ) cat(temp)
   # else cat(name, "= ",temp )
   cat(name, "= ",temp )
  }
  else{
   # temp is a matrix or array
   if( debug > 0 ){ ####################################
    cat("\n temp is not a scalar.\n")
   } ###################################################
   rn1=rownames(temp);
   if( length(rn1) == 0 ) rownames(temp)=paste("r",1:nrow(temp),sep="")
   cn1=colnames(temp);
   if( length(cn1) == 0 ) colnames(temp)=paste("c",1:ncol(temp),sep="")

   if( is.numeric(temp) ){
    #temp[abs(temp) <= fuzz]=fuzzchar
    locfz=which(abs(temp) <= fuzz)
    code=gen_formatC(temp, temp, fwd)
    eval(parse(text=code))
    fzc=paste(substr(BLK,1,max(nchar(temp))-nchar(fuzzchar)-1),fuzzchar
              ,collapse=""); temp[locfz]=fzc
    if( debug  > 0 ){ ##################################################
     cat("\n NAchar =:", NAchar,":  temp before NAchar=\n")
     print(temp)
    } #################################################################
    #    temp[grep("NA*",temp)]=NAchar
    #temp=sub("NA",NAchar,temp)
    temp=sub("^NA$",NAchar,temp)
    if( debug  > 0 ){ ##################################################
     cat("\n fwd=", fwd, " with length(fwd)=", length(fwd),"****\n")
     cat(" result of gen_formatC is >>", code, "<< ***********\n")
    } #################################################################
   }
   cat(name, "= ");  cat("\n");
   print(temp, quote=0);
  }
  cat("\n")
  return(invisible())

 } # end of nmat == 1.   returned


 # check if all are vector or matrix class
 morvora=(sapply( value, is.array )) +
  (sapply( value, is.matrix ) + sapply( value, is.vector ) ) *
  !sapply( value, is.list )
 if( any(morvora == 0 ) ){
  locnmorvora=which(morvora == 0)
  nn=name[locnmorvora]
  cat("\nerror:(mprint) Input must be vectors,  matrices or array.\n")
  cat("The following argument(s)  are not vector, matrix nor array.\n")
  cat("    ", nn, "\n")
  return(invisible())
 }

 if( debug ){ #####################################################
  cat("\nTop of nmat >= 2\n")
  for( i in 1:nmat ){
   cat(i, " name =", name[[i]], ", value =", value[[i]]
       , ", fmt =", fmt[i], "\n")
  }
 } ################################################################

 # first two matrices
 X1=value[[1]]; X2=value[[2]];
 nam1=names(X1); nam2=names(X2)    # 20140809
 fwd1=fwd[,1]; fwd2=fwd[,2]
 if( is.array(X1) == 1 && length(dim(X1)) >= 3 )
  X1=arraytomatc( X1, fwd=fwd1, debug=debug )
 if( is.array(X2) == 1 && length(dim(X2)) >= 3 )
  X2=arraytomatc( X2, fwd=fwd2, debug=debug )
 if( is.vector(X1) ){
  # column vectorization
  X1=matrix(X1,,1); rownames(X1)=nam1   # 20140809
  if( nrow(X1) == 1 ) rownames(X1)=""
 }
 if( is.vector(X2) ){
  X2=matrix(X2,,1); rownames(X2)=nam2
  if( nrow(X2) == 1 ) rownames(X2)=""
 }
 name1=sub("^ +", "", name[1]); name1=sub(" +$", "", name1)
 name2=sub("^ +", "", name[2]); name2=sub(" +$", "", name2)

 # small values
 locfz1=integer(0); locfz2=integer(0)
 if( is.numeric(X1) ){
  # X1[which(abs(X1)<=fuzz)]=fuzzchar
  locfz1=which(abs(X1) <= fuzz)
 }
 if( is.numeric(X2) ){
  # X2[which(abs(X2)<=fuzz)]=fuzzchar
  locfz2=which(abs(X2) <= fuzz)
 }
 # matrices 1 and 2 to be column combined
 if( debug ) cat("2.1**************\n")
 # fX1=formatC(X1, format=fwd1[1]
 #           , width=as.integer(fwd1[2]), digits=as.integer(fwd1[3]) )

 code=gen_formatC(fX1, X1, fwd1)
 eval(parse(text=code))
 fzc=paste(substr(BLK,1,max(nchar(fX1))-nchar(fuzzchar)-1),fuzzchar
           ,collapse=""); fX1[locfz1]=fzc
 if( debug ){ ###############################################
  cat("2.2***********", code, "**********\n")
  print(fwd1)
  cat("2.3**************\n")
  print(X1)
  print(fX1)
 } ##########################################################

 # fX2=formatC(X2, format=fwd2[1]
 #            , width=as.integer(fwd2[2]), digits=as.integer(fwd2[3]) )
 code=gen_formatC(fX2, X2, fwd2)
 eval(parse(text=code))
 fzc=paste(substr(BLK,1,max(nchar(fX2))-nchar(fuzzchar)-1),fuzzchar
           ,collapse=""); fX2[locfz2]=fzc
 if( debug ){ #######################################################
  cat("3.2***********", code, "**********\n")
  print(fwd2)
  cat("3.3**************\n")
  print(X2)
  print(fX2)
 } #################################################################
 r1=nrow(fX1); r2=nrow(fX2); c1=ncol(fX1); c2=ncol(fX2);

 # row/col names;
 rn1=rownames(fX1); if( length(rn1) == 0 ) rn1=paste("r",1:r1,sep="")
 cn1=colnames(fX1); if( length(cn1) == 0 ) cn1=paste("c",1:c1,sep="")
 rn2=rownames(fX2); if( length(rn2) == 0 ) rn2=paste("r",1:r2,sep="")
 cn2=colnames(fX2); if( length(cn2) == 0 ) cn2=paste("c",1:c2,sep="")
 if( r1*c1 == 1 || is.vector(value[[1]]) ){ if(novecrname) rn1=""; cn1="" }
 if( r2*c2 == 1 || is.vector(value[[2]]) ){ if(novecrname) rn2=""; cn2="" }


 # replace character NA by DOT
 # fX1[grep("NA *",fX1)]=NAchar
 fX1=gsub("^NA$",NAchar,fX1)
 # fX1=trim(fX1,all=1)
 #fX2[grep("NA *",fX2)]=NAchar
 fX2=gsub("^NA$",NAchar,fX2)
 # fX2=trim(fX2,all=1)

 lenfX1=max(nchar(apply(fX1,1,function(x){paste(x,sep="",collapse="")})))
 lenfX2=max(nchar(apply(fX2,1,function(x){paste(x,sep="",collapse="")})))
 lenfX1=lenfX1+c1+max(nchar(rn1))+1+1
 lenfX2=lenfX2+c2+max(nchar(rn2))+1

 if( type[1] %in%  c("character", "double", "integer"
                     , "complex", "logical") ) nX1n=1
 else nX1n=0
 if( type[2] %in%  c("character", "double", "integer"
                     , "complex", "logical") ) nX2n=1
 else nX2n=0

 # combine first two
 res=ccbind2c( fX1, fX2, name1, name2, rn1, cn1, rn2, cn2, nsep=nsep, sep=sep
               , noX1name=nX1n, noX2name=nX2n)

 if( nmat == 2 ){
  # replace separator mark by blanks
  res=chartr("?", " ", res)
  print( res, quote=0 )
  return( invisible() )
 }

 # add third matrix and later
 for( i in 3:nmat ){

  if( debug > 0 ){ #########################################
   cat("\n\nTop of nmat >= 3\n")
  } ########################################################
  # matrices to be column combined
  fX1=res;
  X2=value[[i]];   if( is.vector(X2) ) X2=matrix(X2,,1)
  fwd1=fwd[,i]
  if( debug ){ ################################################
   cat("\n in the i loop \n")
   cat(" i=", i, " X2 =", X2, ", fwd1 =", fwd1,"\n")
  } ###########################################################

  if( is.array(X2) == 1 && length(dim(X2)) >= 3 )
   X2=arraytomatc( X2, fwd=fwd1, debug=debug )
  if( is.numeric(X2) ){
   # X2[which(abs(X2)<=fuzz)]=fuzzchar
   locfz2=which(abs(X2) <= fuzz)
  }

  # fX2=formatC(X2, format=fwd1[1]
  #          , width=as.integer(fwd1[2]), digits=as.integer(fwd1[3]) )
  code=gen_formatC(fX2, X2, fwd1)
  eval(parse(text=code))
  fzc=paste(substr(BLK,1,max(nchar(fX2))-nchar(fuzzchar)-1),fuzzchar
            ,collapse=""); fX2[locfz2]=fzc
  if( debug ){ #############################################################
   cat("5***********", code, "**********\n")
   print(fwd1)
  } ########################################################################

  name2=name[i]
  r1=nrow(fX1); r2=nrow(fX2); c1=ncol(fX1); c2=ncol(fX2);

  # replace character NA
  #fX1[grep("NA *",fX1)]=NAchar
  fX1=gsub("^NA$",NAchar,fX1)
  # fX1=trim(fX1,all=1)
  #fX2[grep("NA *",fX2)]=NAchar
  fX2=gsub("^NA$",NAchar,fX2)
  # fX2=trim(fX2,all=1)

  # row/col names;
  rn1=rownames(fX1); if( length(rn1) == 0 ) rn1=paste("r",1:r1,sep="")
  cn1=colnames(fX1); if( length(cn1) == 0 ) cn1=paste("c",1:c1,sep="")
  rn2=rownames(fX2); if( length(rn2) == 0 ) rn2=paste("r",1:r2,sep="")
  cn2=colnames(fX2); if( length(cn2) == 0 ) cn2=paste("c",1:c2,sep="")
  if( r1*c1 == 1 ){ rn1=""; cn1="" }
  if( r2*c2 == 1 || is.vector(value[[i]]) ){ if(novecrname) rn2=""; cn2="" }
  if( r2 == 1 ) rn2=""   # 20140809

  # character length: to be used to check if the final result is short enough
  # in the later version.
  lenfX1=max(nchar(apply(fX1,1,function(x){paste(x,sep="",collapse="")})))
  lenfX2=max(nchar(apply(fX2,1,function(x){paste(x,sep="",collapse="")})))
  lenfX1=lenfX1+c1+max(nchar(rn1))+1+1
  lenfX2=lenfX2+c2+max(nchar(rn2))+1

  if( type[i] %in%  c("character", "double", "integer"
                      , "complex", "logical") ) nX2n=1
  else nX2n=0


  res=ccbind2c( fX1, fX2, "", name2, rn1, cn1, rn2, cn2
                , nsep=nsep, debug=debug, noX1name=1, noX2name=nX2n, sep=sep )
  if( debug ){ ##############################################################
   cat("\nafter ccbind2c\n")
   print(res)
  } #########################################################################

 } # end of i loop

 # replace separator mark by blanks
 res=chartr("?", " ", res)

 print(res, quote=0)
 return( invisible( ) )


} # end of mprint




decomp_fmt <- function( fmt, debug=1 ){
 # decompose format string of type:  NameWidth.Digits
 # Shin-ichi Mayekawa
 # 121223c
 # 130103
 #
 # Assuming fmt has a structure: name + width + dot + digits,
 # return the vector consisting of name, width, and digits
 #
 # The default name is "f".
 #   If fmt = ".5",  name="f", width="", digits="5".
 #
 #  fmt is a scalar, not a vector.
 #
 # We could execute formatC from here if we check all the 2^3 possible
 # combinations of fmt[1] -- fmt[3]
 #


 digits=""
 format=""    # defailt format name when nothing is given.
 width=""
 fwd=c("","","")
 if( fmt != "" ){

  # remove digits, if any.
  res=strsplit( fmt, split="\\." )[[1]]
  if( debug > 0 )
   cat("\n decomp_fmt:1", "length(res)=", length(res), " res=", res, "\n")
  if( length(res) >= 1 ){
   digits=res[2]
   res=res[-2]
  }
  if( debug > 0 )
   cat(" decomp_fmt:1", "res=", res, "\n")

  # if there are no name, use "f" as default and return
  if( length(grep("[[:alpha:]]", res)) == 0 ){
   format="f"
   width=res
   if( debug > 0 )
    cat(1.5, res, format,width,digits,"\n")
   fwd[1]=format; fwd[2]=width; fwd[3]=digits
   return( fwd )
  }

  # If res contains "fg", return
  res=strsplit( res, split="fg", fixed=1 )[[1]]
  if( debug > 0 )
   cat(" decomp_fmt:2", "length(res)=", length(res), " res=", res, "\n")
  if( length(res) >= 1  & res[1] == "" ){
   # the first element of strsplit("fg5","fg")[[1]] is NA.
   format="fg"
   width=res[2]
   fwd[1]=format; fwd[2]=width; fwd[3]=digits
   if( debug > 0 )
    cat(" decomp_fmt:2", "res=", res, "fwd=", fwd, "\n")
   return( fwd )
  }

  # Other format type than "fg".
  format=substr(res[1],1,1)
  res=strsplit( res, split="[defgisEG]", fixed=0 )[[1]]
  if( debug > 0 )
   cat(" decomp_fmt:3", "length(res)=", length(res), " res=", res, "\n")
  if( length(res) >= 1 ){
   if( res[1] != "" ) format="f"
   width=res[2]
  }
 }

 # old fortran I format.
 if( toupper(format) == "I" ) format="d"
 if( toupper(format) == "A" ) format="s"
 fwd[1]=format; fwd[2]=width; fwd[3]=digits

 if( debug > 0 )
  cat(" decomp_fmt:final", "fwd=", fwd, "\n")
 return( fwd )

} # end of decomp_fmt



gen_formatC <- function( res, arg, fwd ){
 # generate
 #  code="res=formatC(, arg, format=fwd[1], width=fwd[2], digits=fws[3]) )"
 # and execute it.
 # Shin-ichi Mayekawa
 # 121223c,24c,25
 # suppressWarnings: 20151215
 #
 #  If typeof(arg) is either complex or logical, or
 #  if fwd is empty, format will be used instead of formatC.
 #
 #
 # This module causes error if called as: gen_formatC( A )
 #

 fwdname=as.character(substitute(fwd))
 type=typeof(arg)
 # cat("\n in gen_formatC: typeof(arg) = ", typeof(arg), "\n")
 # cat("\n fwdname=",fwdname,"\n")
 # cat('all(fwd == "") =', all(fwd == ""), "\n")
 if( !(type %in% c("logical", "complex"))  &  !all(fwd == "") ){
  # type is not logical or complex: use formatC
  fwd[is.na(fwd)]=""    #  somewhere fwd[2] may become NA!!
  code=paste( substitute(res)
              , "=suppressWarnings(formatC( ", substitute(arg), sep="" )
  if( fwd[1] != "" )
   code=paste(code, ", format='",fwd[1], "'", sep="")
  if( fwd[2] != "" )
   code=paste( code, ", width=as.integer(", fwdname, "[2])", sep="" )
  if( fwd[3] != "" )
   code=paste( code, ", digits=as.integer(", fwdname, "[3])", sep="" )
  code=paste( code, " ))", sep="" )
 }
 else{
  # use format if type is logical or complex.
  code=paste( substitute(res)
              , "=suppressWarnings(format( ", substitute(arg), " ))", sep="" )
 }
 return( code )

} # gen_formatC

#' Sort an Object according to the Keys
#'
#' @param x an object to be sorted.\cr
#' If x is a matrix, all the column will be sorted.
#' @param ... vectors to be uses as keys.\cr
#' Must be the same length as x.
#' @param descending = 1 to sort descendingly when ... is present.\cr
#' @param cols vector consisting of the column numbers of x
#' which will be used as keys.\cr
#' This has priority over ... .
#'
#' @details
#' In order to sort descendingly using cols argument,
#'  specify negative column number in cols= \cr
#' or use - sign in numeric ... ariguments. \cr
#' Or, use descending=1 option.
#'
#' @return Object of the same size as x.
#'
#' @examples
#' # sort the rows of x according to the order of ... .
#' sortit( demomat(4,2), c(4,3,1,2) )
#' sortit( demomat(4,2), c(4,3,1,2), c("a","a","b","b") )
#'
#' # sort the rows of x using the colums specified in cols= as keys.
#' sortit( diag(4), cols=1:4 )
#'
#' # sorting the rows of a matrix
#' X <- as.matrix( expand.grid( 1:2, 3:5 ) )
#' colnames(X)=c("xx1","xx2")
#' sortit( X, cols=c(1,2) )
#' Print( sortit( X, cols=c(1,-2) ), sortit( X, X[,1], -X[,2] ) )
#'
#' # the last column first or the first column first.
#' Print( sortit( X, cols=1:2 ), sortit( X, cols=2:1 ) )
#'
#' # sort the result of unique function
#' set.seed(1701)
#' Y <- matrix( sample(letters[1:3],99, replace=1),,3 )
#' Yu <- unique(Y)
#' Print( Yu, sortit(Yu, cols=1:3) )
#' Print( sortit(Yu, Yu[, 1], Yu[, 2], Yu[, 3]) )
#' Print( Yu, sortit(Yu, cols=-(1:3)) )
#' Print( sortit(Yu, Yu[, 1], Yu[, 2], Yu[, 3], descending=1) )
#'
#' @export
#'

sortit <- function( x, ..., descending=0, cols=NULL ){
 # sort x by multiple keys and return the sorted x
 # 20111123,24,25
 # drop=F -> drop=0: 130110
 # as.logical(desceinding): 20140510
 # cols: 20140510
 # descending bug fix: 20140520
 # vector x: 20141217,18
 # bugfix for cols: 20150825
 # roxgen comments: 20161104c,1121,20170917
 # bugfix for R4.2.1: 20220806
 #
 # Args:
 #  x          an object whose rows will be sorted
 #             When x is a matrix, all the columns will be sorted.
 #  ...        list of the keys
 #             These are NOT key names/numbers, but values.
 #             Each key must have the same length as nrow(x)
 #            ... can be NULL when sorting vector x by its value.
 #  descending = 1 to sort descendingly for all the keys when ... is given.
 #
 #  cols     = list of columns of x to be used as keys
 #             negative column number means descending sort for that key
 #             cols has priority over ... .
 #
 #

 # is x a vector? 20141217,18
 isvec=is.vector(x)
 # if( isvec ){
 #  if( is.null(cols) ) cols=1
 #  if( cols < 0 ) descending=1
 #  return( sort(x,decreasing=as.logical(descending)) )
 # }

 # is cols given?
 if( !is.null(cols) ){
  # name=deparse(substitute(x))
  name="x"   # 20150825
  #Print(name, cols)
  loc=which( cols < 0 )
  cols=abs(cols)
  b=paste(name,"[,",cols,"]", sep="")
  bs=paste("length(",name,"[,",cols,"])-"
           ,"rank(",name,"[,",cols,"], ties.method='min')"
           , sep="")
  #Print(b,bs, size(b),loc)
  if( length(loc) > 0 ){
   b[loc]=bs[loc]
  }
  b=paste(b,collapse=",")
  #Print(b)
  code=paste("od=order(",b,")")
  # Print(code)
  eval(parse(text=code))
  # sort x
  if ( isvec ){
   x=x[od]
  }
  else{
   x=x[od,,drop=0]
  }
  return(x)
 } # end of cols != NULL

 # Here key is given as ...

 # process ... argument
 value=list(...)
 name=as.character( as.list(substitute(list(...)))[-1L] )
 #Print(name,value,names(value))
 nkeys=length(name)
 lenv=as.numeric(lapply(value,length))
 #Print(name,value,nkeys,lenv,descending)


 # vec or matrix/data.frame
 if ( isvec ) len=length(x)
 else len=nrow(x)

 # error check
 if( any( len != lenv ) ){
  cat("\n\nerror:(sortit) The length of the body and the key(s) differ. \n\n")
  Print(len,lenv)
  return( NULL )
 }

 # no key case
 if ( nkeys == 0 ){
  if ( is.matrix(x) && ncol(x) > 1 ){
   cat("\n\nerror:(sortit) To sort a matrix, a key must be given. \n\n")
   return( NULL )
  }
  else{
   if( is.matrix(x) ) return(
    matrix(sort(x,decreasing=as.logical(descending)),,1) )
   else return( sort(x,decreasing=as.logical(descending)) )
  }
 }

 # find order and take care of descending option
 dec=rep(0,nkeys)
 if( descending ) dec=rep(1,nkeys)
 od=order(..., decreasing=descending)

 # sort x
 if ( isvec ) x=x[od] else x=x[od,,drop=0]

 return(x)

} # end of sortit

#' Reorder the rows and columns of factor pattern matrix
#'
#' @param A Factor Pattern matrix or an object with $loadings.
#' @param unclass = 1 to unclass the result
#' @param sortcols = 0 not to sort factors (columns of A) according to
#' factor contributions.
#' @param print = 1 to print the result
#'
#' @details
#' First, if sortcols == 1, the columns of \code{A} are reordered
#' according to the magnitude of \code{diag(t(A)\%*\%A)}. \cr
#' Then, for each row, the column number of the maximum elements of
#' \code{abs(A)}
#' is recorded (as \code{max.col(abs(A))}) and the rows of \code{A}
#'are sorted according
#' to these column numbers. \cr
#' Finally, within each max.col group, the rows are again sorted
#' according to the magnitude of \code{A}.
#'
#' @examples
#' set.seed(1701)
#' n <- 20; ndim <- 4
#' A0 <- matrix(runif(n*ndim),n,ndim)
#' dimnames(A0)=list( paste("v",1:n,sep=""),paste("f",1:ndim,sep=""))
#' A0v <- varimax(A0)$loadings
#' A0v <- A0v*matrix(sample(c(-1,1,1),n*ndim,replace=TRUE),n,ndim)
#' A0v[,3:4] <- -A0v[,3:4]
#'
#' resrc <- reorder_fa(A0v)
#' A0vrc <- resrc$loadings
#' Print(A0v,A0vrc, fmt="6.3", fuzz=0.7)
#'
#' resr <- reorder_fa(A0v, sortcols=0)
#' A0vr <- resr$loadings
#' Print(A0v,A0vr, fmt="6.3", fuzz=0.7)
#'
#' # reordered matrix from original
#' dif1 <- A0vrc-A0v[resrc$row_order,resrc$col_order]
#'
#' # original from reordered matrix
#' A0v2 <- A0vrc; A0v2[resrc$row_order,resrc$col_order] <- A0vrc
#' rownames(A0v2)[resrc$row_order] <- rownames(A0vrc)
#' Print(A0v,A0v2, fmt="6.3")
#' dif2 <- A0v2-A0v
#' Print(dif1, dif2)
#'
#' # reducing the number of negative signs
#' A0vrcs <- reflect_fa( A0vrc )$loadings
#' Print(A0vrc, A0vrcs, fmt="6.3")
#'
#' # attributes
#' print(reorder_fa(A0v)$loadings)
#' print(reorder_fa(A0v, unclass=1)$loadings)
#'
#' @return A list of \cr
#' loadings = reordered loadings matrix \cr
#' row_order = sort order of the rows \cr
#' col_order = sort order of the cols
#'
#' @export
#'

reorder_fa <- function( A, unclass=0, sortcols=1, print=0 ){
 # reordering the rows of A as  proc factor reorder
 # Shin-ichi Mayekawa
 # 20140528
 # non matrix input: 20140612
 # renamed as reorder_fa: 20150705
 # sort by factor contributions: 20151215 (no implementation)
 # attributes, unclass: 20151215
 # names and sortcols: 20151224
 # roxygen: 20210421
 #

 # store attributes
 attrib=attributes(A)
 A=unclass(A)
 n=nrow(A)

  # names
 if( is.null(rownames(A)) ) rownames(A)=paste("r",1:nrow(A),sep="")
 if( is.null(colnames(A)) ) colnames(A)=paste("c",1:ncol(A),sep="")
 rname=rownames(A); cname=colnames(A)

 odc=1:ncol(A)
 names(odc)=cname
 if( sortcols == 1 ){
  # sort by diag(t(A)%*%A)
  fc=colSums(A*A)
  names(fc)=cname
  odc=order(-fc)
  A=A[,odc]
  cname=colnames(A)
 }

 # sort according to the location(col. number) of the row maximum
 maxcol=max.col(abs(A))

 index=1:n
 od=order(maxcol)
 B=A[od,]
 index=index[od]
 maxcols=maxcol[od]
 # rownames(B)=name[od]
 rname=rownames(B)

 um=unique(maxcols)
 num=length(um)

 # sort according to the value of A within the subgroups
 for( i in 1:num ){
  umi=um[i] # subgroup id = column number
  loc=which(maxcols == umi)
  od=order(-(B[loc,umi]))
  B[loc,]=B[loc,,drop=0][od,,drop=0]
  rname[loc]=rname[loc][od]
  index[loc]=index[loc][od]
 }
 rownames(B)=rname

 if( print > 0 ){
  cat("\nReordered Matrix\n")
  Print(index,B,fmt="3.0 6.2")
 }

 if( !unclass ){
  dn=dimnames(B)
  attributes(B)=attrib
  dimnames(B)=dn
 }

 return( list( loadings=B, row_order=index, col_order=odc ) )

} # end of reorder_fa

#' Recode the Values of obj from "from" to "to".
#'
#' @param obj a vector, matrix or list of those.
#' @param from a vector of old values
#' @param to   a vector or new values
#' @param ft   the conversion table data frame
#' @param method = 1 to use \code{merge}, \cr
#' = 2 to use R style teble look-up using names attribute of a vector.
#' @param useNA = 1 to mark out-of-range elements
#' @param debug = 1 to print intermediate result.
#'
#' @details
#'  When \code{ft} is not given,
#'  \code{cbind(from,to)}  makes the conversion table \code{ft}. \cr
#'  \code{ft} has priority over from and to.
#'
#'  The core of this function is:\cr\cr
#'  If \code{method==1} \cr
#'  \code{New <- merge( cbind(from,to), data.frame(from=c(obj))
#'  , by="from", sort=FALSE, all.y=TRUE )} \cr
#'  If \code{method==2} \cr
#'  \code{names(to) <- from; New <- to[as.character(c(obj))]} \cr
#'
#'  Method 2 is faster than method 1. \cr
#'
#'  If the conversion table is not large enough,
#'  if \code{method=2} and \code{useNA=1},
#'  out-of-range elements will be set to NAs,
#'  otherwise, the original values will not  be changed.
#'
#'  When the types of \code{obj} and \code{from} are different,
#'  they will be converted to character.
#'
#'  \code{lazy.tools::new_name} will be deprecated.
#'
#' @return
#'  A matrix or vector depending on the \code{obj}.
#'  When \code{obj} is a list, a list will be returned.
#'
#' @examples
#' n <- 10
#' n1 <- n-2
#' obj <- cbind( letters[1:n], rev( letters[1:n] ) )
#' obj[1,1] <- NA; obj[3,2] <- NA
#' rownames(obj) <- paste("RR",1:n,sep="")
#' colnames(obj) <- paste("CC",1:ncol(obj),sep="")
#' from <- letters[1:n1]
#' to <- LETTERS[1:n1]
#' # to <- 1:n1
#' res <- recode( obj, from, to )
#' Print(obj,res,from,to)
#' res2 <- recode( obj, from, to, useNA=1 )
#' Print(obj,res2,from,to)
#'
#'
#' n=10
#' n1=n-2
#' obj=cbind( 1:n, n:1 )
#' rownames(obj)=paste("RR",1:n,sep="")
#' colnames(obj)=paste("CC",1:ncol(obj),sep="")
#' from=1:n1
#' to=from^2
#' res=recode( obj, from, to, method=2 )
#' Print(obj,res, from, to)
#'
#'
#' n <- 10
#' n1 <- n-2
#' obj <- cbind( 1:n, n:1 )        # This is numeric.
#' rownames(obj) <- paste("RR",1:n,sep="")
#' colnames(obj) <- paste("CC",1:ncol(obj),sep="")
#' from <- as.character(1:n1)      # This is character.
#' to <- LETTERS[1:n1]
#' res <- recode( obj, from, to )
#' Print(obj,res, from, to)
#'
#' @export
#'

recode <- function( obj, from=NULL, to=NULL, ft=NULL, method=2
                    , useNA=0, debug=0 ){
 # recode the values of obj from from to to
 # Shin-ichi Mayekawa
 # 20221022
 # warning: 20221026
 # merge2 -> merge: 20221026
 # debug: 20221103
 # useNA: 20221222
 #

 # check
 if(  !is.null(ft) && (is.null(from) || is.null(to)) ){
  cat("\nerror1(recode): from and to  or  ft must be given.\n")
  stop()
 }
 lf=length(from)
 lt=length(to)
 if( lf != lt ){
   cat("\nerror1(recode): The lengths of from and to must be the same.\n")
   stop()
 }


 ###########################################################
 # if list, call recode recursively
 #
 if( is.list(obj) ){
  lname=names(obj)
  res=lapply( obj, recode, from, to, ft )
  names(res)=lname
  return( res )
 }
 ###########################################################

  # shape of obj
 isvec=is.vector(obj)
 ismat=is.matrix(obj)
 if( isvec ){
  nr=length(obj); nc=1
  dimname=list( names(obj), NULL )
 }
 if( ismat ){
  nr=nrow(obj); nc=ncol(obj)
  dimname=dimnames(obj)
 }


 if( method == 1 ){

 # conversion table
 if( is.null(ft) ){
   ft=data.frame( old=from, new=to )
 }
 if( debug ){ cat("ft\n"); str(ft) }

 # reshape obj
 vecobj=c(obj)
 lobj=length(vecobj)
 vecobj=data.frame( old=vecobj, ..id..=1:lobj )
 if( debug ){ cat("vec(obj)\n"); str(vecobj) }

 # uo=unique(c(vecobj))
 # if( !all(uo %in% from) ){
 #  cat("\nwarning1(recode): Somve values of obj are missing from 'from'\n")
 # }

 # merge
 New=merge( ft, vecobj, by="old", sort=FALSE, all.y=TRUE )
 if( debug ){ cat("merged\n"); str(New) }
 New=New[order(New$..id..), "new"]

 }
 else{

  if( !is.null(ft) ){
   from=ft[,1]; to=ft[,2]
  }
  names(to)=from
  New=to[as.character(c(obj))]

  if(0){
    # Above New already has NAs for the out-of-range elements.
    # So we do not need below
    if( useNA == 1 ){
      uf=unique(from)
      uo=unique(c(obj))
      locd=which( !(uo %in% uf) )
      loco=which( c(obj) %in% uo[locd] )
      Print(uo,uf,locd, loco)
      New[loco]=NA
    }
  }
 }

 # back to the original shape
 New=matrix( New, nr, nc )
 dimnames(New)=dimname
 if( isvec ){
   New=c(New)
   names(New)=dimname[[1]]
 }

 if( useNA == 0 ){
   # copy the original values to NA.
   New[is.na(New)]=obj[is.na(New)]
 }


 return(New)

} # recode

#' Dummmy Expantion of Categorical Data
#'
#' @details
#'  In compressed format, Uc[i,j]=k means that subject i answers to item j
#'  by selecting option k, k=0,1,...,ncat[j]. \cr
#'  This function expands this compressed format to uncompressed
#'  dummy expanded format: \cr
#'  That is, each item takes ncat[j] colums of U[, fromP[j]:toP[j]], \cr
#'  and the k+fromP[j]-th colum will be 1, and 0 elsewhere.
#' @param Uc Item Response matrix in Compressed Format
#' @param ncat a vector consisting of # of categories of each item
#' @param type NOT used.
#' @param zero = 0 to exclude the zero-th category from output
#'
#' @return  A list of \cr
#' U, ncat, type, zero, fromP, toP
#'
#' @examples
#' # generate item response
#' # in uncompressed format
#' # set.seed(1701)
#' # res <- gendataIRT( 1, paramS1, npoints=7, thmin=-3, thmax=3 )
#' # U=res$U
#' # in compressed format
#' # set.seed(1701)
#' # res1 <- gendataIRT( 1, paramS1, npoints=7, thmin=-3, thmax=3, compress=1 )
#' # Uc=res1$U
#' #
#' # uncompressed data: ncat=c(2,2,4)
#' U <- "
#' Q1_0 Q1_1 Q2_0 Q2_1 Q3_0 Q3_1 Q3_2 Q3_3
#' 1    0    1    0    1    0    0    0   
#' 1    0    1    0    0    1    0    0   
#' 1    0    1    0    1    0    0    0   
#' 1    0    1    0    0    1    0    0   
#' 1    0    0    1    0    0    1    0   
#' 0    1    0    1    0    0    0    1   
#' 0    1    0    1    0    0    0    1   
#' "; U=cards( U, header=1 )
#' #
#' # compressed data: ncat=c(2,2,4)
#' Uc <- "
#'  Q1 Q2 Q3
#'  0  0  0 
#'  0  0  1 
#'  0  0  0 
#'  0  0  1 
#'  0  1  2 
#'  1  1  3 
#'  1  1  3 
#' "; Uc=cards( Uc, header=1 )
#'
#'
#' # from compressed to uncompressed
#' U1=dummy_expand(Uc)$U
#' # back to compressed
#' Uc1=dummy_compress(U)$Uc
#' Print(U-U1)
#' Print(Uc-Uc1)
#'
#' # zero=0 option
#' U2=dummy_expand(Uc, zero=0)$U
#' Uc2=dummy_compress(U2, zero=0, ncat=c(2,2,4))$Uc
#' Print(Uc-Uc2)
#'
#'
#' @export
#'

dummy_expand <- function( Uc, ncat=NULL, type=NULL, zero=1 ){
 # dummy expand the categorical valued variable u
 # Shin-ichi Mayekawa
 # 20121022
 # matrix input: 121024
 # missing value: 121025
 # make the data integer when possible: 121126
 # vectorize fromP: 20151113
 # 20161210
 
 nitems=ncol(Uc)
 npoints=nrow(Uc)
 iname=colnames(Uc)
 
 if( is.null(ncat) ){
  ncat=matrix(apply(Uc,2,max,na.rm=1),,1)+1
 }
 toP=cumsum(ncat)
 fromP=c(toP-ncat+1)
 # Print(nitems,npoints,ncat,toP,fromP, iname)
 U=matrix(as.integer(0),npoints,toP[nitems])
 for( j in 1:nitems ){
  uj=Uc[,j]
  locobs=!is.na(uj)
  min=0; max=ncat[j]-1
  factuj=factor(uj, levels=min:max)
  Uj=model.matrix(~factuj-1)
  #Print(uj,factuj,Uj)
  #  attributes(Uj)$assign=NULL
  #  attributes(Uj)$contrasts=NULL
  #  colnames(Uj)=min:max
  U[locobs,fromP[j]:toP[j]]=as.integer(Uj)
 }
 if( is.null(iname) )
  iname=paste("Q",1:nitems,sep="")
 Pname=character(sum(ncat))
 catname=outer(paste(iname,"_",sep=""),0:max(ncat),paste,sep="")
 for( j in 1:nitems ){
  Pname[fromP[j]:toP[j]]=catname[j,1:ncat[j]]
 }
 colnames(U)=Pname
 rownames(U)=rownames(Uc)
 
 if( zero != 1 ){
  # remove the 0-th category
  U=U[,-fromP]
  fromP=fromP-(0:(nitems-1))
  toP=toP-(1:nitems)
 }
 
 return( list(U=U, ncat=ncat, type=type, zero=zero, fromP=fromP, toP=toP) )
 
} # end of dummy_expand




#' Compress the Dummmy Expanded Categorical Data
#'
#' @details
#'  In compressed format, Uc[i,j]=k means that subject i answers to item j
#'  by selecting option k, k=0,1,...,ncat[j]. \cr
#'  This function recovers this compressed format from the dummy expanded
#'  uncompressed format: \cr
#'  That is, each item takes ncat[j] colums of U[, fromP[j]:toP[j]], \cr
#'  and the k+fromP[j]-th colum will be 1, and 0 elsewhere.
#' @param U The Item Response matrix in Uncompressed Format
#' @param ncat a vector consisting of # of categories of each item
#' @param type NOT used.
#' @param zero = 0 if the input does not have the zero-th column.
#'
#' @return  A list of \cr
#' Uc, ncat, type, zero, fromP, toP
#'

#' @examples
#' # generate item response
#' # in uncompressed format
#' # set.seed(1701)
#' # res <- gendataIRT( 1, paramS1, npoints=7, thmin=-3, thmax=3 )
#' # U=res$U
#' # in compressed format
#' # set.seed(1701)
#' # res1 <- gendataIRT( 1, paramS1, npoints=7, thmin=-3, thmax=3, compress=1 )
#' # Uc=res1$U
#' #
#' # uncompressed data: ncat=c(2,2,4)
#' U <- "
#' Q1_0 Q1_1 Q2_0 Q2_1 Q3_0 Q3_1 Q3_2 Q3_3
#' 1    0    1    0    1    0    0    0   
#' 1    0    1    0    0    1    0    0   
#' 1    0    1    0    1    0    0    0   
#' 1    0    1    0    0    1    0    0   
#' 1    0    0    1    0    0    1    0   
#' 0    1    0    1    0    0    0    1   
#' 0    1    0    1    0    0    0    1   
#' "; U=cards( U, header=1 )
#' #
#' # compressed data: ncat=c(2,2,4)
#' Uc <- "
#'  Q1 Q2 Q3
#'  0  0  0 
#'  0  0  1 
#'  0  0  0 
#'  0  0  1 
#'  0  1  2 
#'  1  1  3 
#'  1  1  3 
#' "; Uc=cards( Uc, header=1 )
#'
#'
#' # from compressed to uncompressed
#' U1=dummy_expand(Uc)$U
#' # back to compressed
#' Uc1=dummy_compress(U)$Uc
#' Print(U-U1)
#' Print(Uc-Uc1)
#'
#' # zero=0 option
#' U2=dummy_expand(Uc, zero=0)$U
#' Uc2=dummy_compress(U2, zero=0, ncat=c(2,2,4))$Uc
#' Print(Uc-Uc2)
#'
#' @export
#'

dummy_compress <- function( U, ncat=NULL, type=NULL, zero=1 ){
 # compress the dummy_expanded data
 # Shin-ichi Mayekawa
 # 20161210
 # bugfix: 20170308
 #
 
 # recover ncat
 if( is.null(ncat) ){
  if( zero == 0 ){
   cat("\nerror1(dummy_compress): zero must be 1 when ncat is not given.\n")
   stop()
  }
  toP=which( apply(apply(U,1,cumsum),1,var) == 0 )
  if( length(toP) == 0 ){
   cat("\nerror1(dummy_compress): Input matrix is not a dummy expanded data."
       , "\n")
   cat(" zero-th category may be missing.\n")
   stop()
  }
  fromP=c(1,toP+1)[-length(toP)-1]
  ncat=toP-fromP+1
 }
 else{
  nitems=length(ncat)
  toP=cumsum(ncat)
  fromP=toP-ncat+1
  if( zero != 1 ){
   fromP=fromP-(0:(nitems-1))
   toP=toP-(1:nitems)
  }
 }
 # Print(zero, ncat, fromP, toP)
 
 # const
 npoints=nrow(U)
 nitems=length(ncat)
 iname=colnames(U)[fromP]
 iname=sub("_[[:digit:]]","",iname)
 
 # create Uc from U
 Uc=matrix(0,npoints,nitems)
 
 if( zero == 1 ){
  for( j in 1:nitems ){
   loc=apply(U[,fromP[j]:toP[j],drop=0], 1, which.max)
   Uc[,j]=loc-1
  }
 }
 else if( zero != 1 ){
  for( j in 1:nitems ){
   loc=apply(U[,fromP[j]:toP[j],drop=0], 1, which.max)
   Uc[,j]=loc
   loc=which( apply(U[,fromP[j]:toP[j],drop=0], 1, sum) == 0 )
   Uc[loc,j]=0
  }
 }
 
 colnames(Uc)=iname
 
 return( named_list(Uc, ncat, type, zero, fromP, toP) )
 
} # end of dummy_compress

#' Generate all possible 0-1 patterns of given length,
#' or binary partition index matrix
#'
#' @param nitems  The length of the pattern
#' @param sort    =  1 to sort the result from the left \cr
#'                = -1 to sort the result from the right \cr
#'                =  2 to sort the result by rowSums and from the left \cr
#'                = -2 to sort the result by rowSums and from the right \cr
#' @param partition = 1 to generate binary partition index matrix
#' @param print = 1 to print the result
#'
#' @details
#' Schatzoff et al method is used. \cr
#' nitems must be less than log(.Machine$integer.max)/log(2). \cr
#' \cr
#' Essentially the same  as \cr
#'   cprod( rep(list(0:1),nitems) ) \cr
#' but gen01pat is much faster.
#'
#' @references
#' Schatzoff, M., Tsao, R.  and Fienberg, S. (1968)
#' Efficient Calculation of All Possible Regressions.  Technometrics.
#' Vol. 10, No. 4, pp. 769-779
#'
#' @examples
#' gen01pat( 4 )
#' gen01pat( 4, partition=1, sort=-2 )
#'
#' pindex=gen01pat( 4, partition=1 )
#' left=apply(pindex,1,function(x) letters[1:4][as.logical(x)])
#' right=lapply(left,function(x){setdiff(letters[1:4],x)})
#' res=mapply( function(x,y){list(left=x,right=y)}, left,right, SIMPLIFY=FALSE)
#' Print(res)
#'
#'
#' @return
#' \code{2^nitems x nitems} matrix \\
#' or when \code{partition=1} is given \\
#' \code{2^{nitems-1}-1 x nitems} matrix
#'
#'
#'
#' @export
#'
#'

gen01pat <- function( nitems, sort=0, partition=0, print=0 ){
 #  generation of all the possible 0-1 patterns of the length nitems
 #  by the Schatzoff et al method.
 #  Shin-ichi Mayekawa
 #  sas/matrix and iml version: 890127-011118
 #  20150729
 #  bugfix: 20150822
 #  partition: 20230314
 #  reference: 20230926
 #  sort=-2,2; 20231004
 #  nitems/2: 20231004
 #
 #  See sas proc matrix manual for details.
 #  Large # of items with 2##nitems > maximum integer
 #   must be handled by operating the modulus.  not ready yet!!
 #

 if( nitems > log(.Machine$integer.max)/log(2)-1 ){
  stop("\nerror1:(gen01pat) nitems must be less than or equal to ",
       round(log(.Machine$integer.max)/log(2)-1),". \n")
 }

 npat=2^nitems
 xall=matrix(integer(0),npat,nitems)

 n21=2^nitems-1+.00001

 if( partition ){
  if( nitems <= 1 ){
   stop("\nerror1:(gen01pat) nitems must be greater than 1.\n")
  }
  n21=2^(nitems-1)-1+.00001
  xall=matrix(integer(0),npat/2,nitems)
 }


 x=rep(0,nitems) # location of 1s
 mode(x)="integer"
 mode(xall)="integer"
 #Print(1,typeof(x), typeof(xall))
 xall[1,]=x

 # this is the loop for all the possible patterns
 for( k in 1:n21){
  # find the location to add/remove
  i=1; kk=k;
  while( kk %% 2 == 0 ){ i=i+1; kk=kk/2 }
  # here i is the number of trailng zeros of binary(k) + 1.
  # add/remove
  x[i]=!x[i]
  xall[k+1,]=x
 } # of k loop

 if( partition ){
   xall=xall[-1,,drop=0]
   loc=rowSums(xall) > nitems/2
   xall[loc,]=1-xall[loc,]
 }

 if( sort == -1 )
  xall=sortit( xall,  cols=ncol(xall)-(1:ncol(xall))+1 )
 else if( sort == 1 )
  xall=sortit( xall,  cols=1:ncol(xall) )
 else if( sort == -2 )
  xall=xall[do.call( order, data.frame(rowSums(xall),-xall) ),]
 else if( sort == 2 )
   xall=xall[do.call( order, data.frame(rowSums(xall),xall) ),]


 if( print ){
  Print(partition, sort)
  Print(xall)
 }

 return( xall )

} # end of gen01pat

#' Vertically Stack Data Frames /  Matrices with Different Sets of Variables
#'
#' @param ... data frames or matrices
#' @param sort = 1 to alphabetically sort the columns of the result \cr
#' Otherwise, the order of the original data frames will be kept.
#'
#' @details
#' Conceptually, \code{stack_df( df1, df2 )} is equivalent to
#' \preformatted{
#'  t( merge( cbind(t(df1),cn=colnames(df1)), cbind(t(df2),cn=colnames(df2))
#'          , by="cn", all=1 ) )
#' }.
#'
#' Using the data frames in the example section, the core part is:\cr
#' \preformatted{
#' res <- merge( data.frame(df1m,id=1:nrow(df1m))
#'             , data.frame(df2m,id=(1:nrow(df2m))+nrow(df1m)), all=1 )
#'  }
#' which vertically stacks data frames because the values of
#' the common variable id are unique.
#' \cr
#' Note that, since check.names=0 and optional=1 are used
#' when creating data frames,
#' the resulting data frame may not have valid column names.
#' \cr
#' \cr
#' When all the row names are unique, it will be used as the row names
#' of the resulting data frame. Otherwise, native \code{make.unique} will be
#' used to mark the duplicate row names which appends ".n" to the original names.
#'
#'
#' @examples
#' A <- demomat(5,4,3,cnroot="")
#' A1 <- A[,,1]; A2 <- A[,,2]; A3 <- A[1:2,,3]
#' colnames(A1) <- sub("4","v4",colnames(A1))
#' colnames(A2) <- sub("4","v4",colnames(A2))
#' colnames(A3) <- sub("4","v4",colnames(A3))
#' rownames(A1) <- paste("a1_",1:nrow(A1),sep="")
#' rownames(A2) <- paste("a2_",1:nrow(A2),sep="")
#' rownames(A3) <- paste("a3_",1:nrow(A3),sep="")
#' rownames(A3)[2] <- "a1_2"
#' A1m <- A1[,c(1,4)]; A2m <- A2[,c(1,3), drop=0]; A3m <- A3[,c(4,2,3)]
#' rownames(A2m) <- NULL
#'
#' print(A1m)
#' print(A2m)
#' print(A3m)
#'
#' res <- stack_df( A1m, A2m, A3m )
#' Print(res)
#'
#' # no factors allowed.
#' df1m <- as.data.frame(A1m, stringsAsFactors=0)
#' df2m <- as.data.frame(A2m, stringsAsFactors=0)
#' df2m[,2] <- paste("c",df2m[,2],sep="")
#' df3m <- as.data.frame(A3m, stringsAsFactors=0)
#'
#' res <- stack_df( df1m, df2m, df3m )
#' Print(df1m, df2m, df3m)
#' Print(res)
#' res2 <- stack_df( df2m, df3m, df1m )
#' Print(res2)
#' res3 <- stack_df( df1m, df2m, df3m, sort=1 )
#' Print(res3)
#'
#' # use of native Reduce function
#' res0 <- Reduce( stack_df, list(df1m, df2m, df3m) )
#'
#' res4 <- stack_df( A=demomat(3,2), b=letters[1:3] )
#' Print(res4)
#'
#'
#' @return
#' A data frame or a matrix
#'
#' @export
#'

stack_df <- function( ..., sort=0 ){
 # vertically stack several data frames
 # Shin-ichi Mayekawa
 # 20170921,23,26redcross
 # new_name -> order2: 20170927
 # check.names=optional=1: 20170928
 # all() added to ||: 20190605
 # roxygen2 example: 20200920
 # allmat: 20200921
 # make.names: 20200921
 # delete res=res[order(res$key___key___),]: 20200921
 # example of Reduce: 20201009
 #

 # analyze ...
 temp=analyze_three_dots( ... )
 name=temp$name
 val=temp$val
 n=length(name)

 allmat=unlist( lapply(val,is.matrix) )

 # error
 if( n == 0 ){
  cat("\nerror1:(stack_df) The argument is empty. \n")
  stop()
 }

 # stack vertically
 if( !is.data.frame(val[[1]]) ){
  val[[1]]=as.data.frame(val[[1]], stringsAsFactors=0, optional=1)
  if( is.null(colnames(val[[1]]))  || all(colnames(val[[1]])=="val[[1]]") )
   colnames(val[[1]])=name[[1]]
 }
 res=data.frame( val[[1]], key___key___=1:nrow(val[[1]])
                 , stringsAsFactors=0, check.names=0 )
 cname=colnames(val[[1]])
 if( is.null(rownames(val[[1]])) ) rn=1:nrow(val[[1]])
 else rn=rownames(val[[1]])
 if( n >= 2 ){
  for( i in 2:n ){
   if( !is.data.frame(val[[i]]) ){
    val[[i]]=as.data.frame(val[[i]], stringsAsFactors=0, optional=1)
    if( is.null(colnames(val[[i]])) || all(colnames(val[[i]])=="val[[i]]") )
     colnames(val[[i]])=name[[i]]
   }
   val[[i]]=data.frame( val[[i]], key___key___=(1:nrow(val[[i]]))+nrow(res)
                        , stringsAsFactors=0, check.names=0 )
   res=merge( res, val[[i]], all=1 )
   cname=unique( c(cname,colnames(val[[i]])) )
   if( is.null(rownames(val[[i]])) ) rn=c(rn,paste(i,1:nrow(val[[i]]),sep="_"))
   else rn=c(rn,rownames(val[[i]]))
  }
 }

 # sort by key___key___ and remove key___key___.
 res=res[order(res$key___key___),]
 res=res[,colnames(res)!="key___key___",drop=0]

 if( length(unique(rn)) == nrow(res) ) rownames(res)=rn
 else rownames(res)=make.unique(rn)

 # variable names
 if( sort >= 1 ){
  # sort colnames(res)
  od=order(colnames(res))
 }
 else{
  # sort the colnames(res) according to the original order
  od=order2( colnames(res), cname[cname!="key___key___"] )
 }
 res=res[,od, drop=0]

 if( all(allmat) ) res=as.matrix(res)

 return( res )

} # end of stack_df

#' Smart Sequence Generation
#'
#' @param from the starting value of the sequence
#' @param to the ending value of the sequence
#' @param decreasing = 1 to generate the decreasing sequence
#'
#' @details
#' This function is defined as
#' \preformatted{
#'   from-1 + seq_len(max(0,to-from+1))
#' }
#' so that
#' when \code{from > to} \code{Seq( from, to )} returns \code{numeric(0)}.
#'
#' Basically, same as
#' \preformatted{
#' Seq <- function(from, to) {
#'   if (from <= to) {
#'      return(from:to)
#'   } else {
#'      return(numeric(0))
#'   }
#' }
#' }
#'
#' @examples
#' cat("Seq(3,5)\n")
#' for( i in Seq(3,5) ) print(i)
#' cat("Seq(5,3)\n")
#' for( i in Seq(5,3) ) print(i)
#' cat("seq(5,3)\n")
#' for( i in seq(5,3) ) print(i)
#' cat("\nSeq(3,5, decreasing=1)\n")
#' for( i in Seq(3,5, decreasing=1) ) print(i)
#' cat("Seq(5,3, decreasing=1)\n")
#' for( i in Seq(5,3, decreasing=1) ) print(i)
#'
#' @export
#'

Seq <- function( from, to, decreasing=0 ){
 # smart sequence generation
 # Shin-ichi Mayekawa
 # consulted with chatGPT: 20240217
 # as a function: 20240311
 # decreasing: 20240626
 #

 if( decreasing )  rev( to-1 + seq_len(max(0,from-to+1)) )
 else from-1 + seq_len(max(0,to-from+1))

} # end of Seq

#' Conversion between Japanese Era and AD
#'
#' @param yearJ a vector of Japanese years
#'  with Japanese era name,  e.g.  M01, T15, S27, H1, R04
#' @param yearAD a vector of years in AD \cr
#' This has priority over yearJ.
#' @param EraName a Japanese Era Name to be used for the conversion from AD
#'
#' @details
#'  \code{ yearJAD( "R1" ) } returns 2019. \cr
#'  \code{ yearJAD( yearAD=2019 ) } returns "R01". \cr
#'  \code{ yearJAD( yearAD=2019, EraName="S" ) } returns "S94".
#'
#' @examples
#' # conversion table
#' yJ <- c("M1","M2","M44","M45","T1","T2","T14","T15","S1","S2","S63","S64"
#'      , "H01","H02","H30","H31","R1","R2","R4")
#' yAD <- yearJAD( yJ )
#' yJfromAD <- yearJAD( yearAD=yAD )
#' Print(yJ,yAD,yJfromAD)
#'
#' # using out-of-range Era Name
#' yearJAD(yearAD=2017, EraName="R")  # negative number
#' yearJAD(yearAD=2018, EraName="R")  # year 0 means a year before year 1.
#' yearJAD(yearAD=2019, EraName="R")
#'
#' yearJAD(yearAD=1952, EraName="T")
#' yearJAD(yearAD=1952, EraName="S")
#' yearJAD(yearAD=1952, EraName="H")  # negative number
#'
#' @export
#'

yearJAD <- function( yearJ=1952, yearAD=NULL, EraName="" ){
 # convert yearJ to yearAD
 # Shin-ichi Mayekawa
 # 20221220dnc
 #

 if( is.null( yearAD) ){

  nyear=length(yearJ)
  yearAD=numeric(nyear)

  for( i in 1:nyear){
   YearJ=yearJ[i]
   EraName=toupper(substr(YearJ,1,1))
   if( !( EraName %in% c("M","T","S","H","R") ) ){
    cat("\nerror1:(yearJAD) yearJ must start with a Roman character.\n")
    return(NULL)
   }
   YearJ=as.numeric(substring(YearJ,2))
   if( EraName == "M" ) base=1867
   else if( EraName == "T" ) base=1911
   else if( EraName == "S" ) base=1925
   else if( EraName == "H" ) base=1988
   else if( EraName == "R" ) base=2018
   yearAD[i]=base+YearJ
  }
  return(yearAD)

 }
 else if( EraName == "" ){

  nyear=length(yearAD)
  yearJ=numeric(nyear)
  for( i in 1:nyear ){
   YearAD=yearAD[i]
   if( YearAD > 2018 ){
    YearJ=paste("R", formatC(YearAD-2018,format="d",flag="0",width=2),sep="")
   }
   else if( YearAD > 1988 ){
    YearJ=paste("H", formatC(YearAD-1988,format="d",flag="0",width=2),sep="")
   }
   else if( YearAD > 1925 ){
    YearJ=paste("S", formatC(YearAD-1925,format="d",flag="0",width=2),sep="")
   }
   else if( YearAD > 1911 ){
    YearJ=paste("T", formatC(YearAD-1911,format="d",flag="0",width=2),sep="")
   }
   else if( YearAD > 1867 ){
    YearJ=paste("M", formatC(YearAD-1867,format="d",flag="0",width=2),sep="")
   }
   else{
    YearJ=""
   }
   yearJ[i]=YearJ
  }
  return( yearJ )

 }
 else{

  nyear=length(yearAD)
  yearJ=numeric(nyear)
  for( i in 1:nyear ){
   YearAD=yearAD[i]
   if( EraName =="R" ){
    YearJ=paste("R", formatC(YearAD-2018,format="d",flag="0",width=2),sep="")
   }
   else if( EraName =="H" ){
    YearJ=paste("H", formatC(YearAD-1988,format="d",flag="0",width=2),sep="")
   }
   else if( EraName =="S" ){
    YearJ=paste("S", formatC(YearAD-1925,format="d",flag="0",width=2),sep="")
   }
   else if( EraName =="T" ){
    YearJ=paste("T", formatC(YearAD-1911,format="d",flag="0",width=2),sep="")
   }
   else if( EraName =="M" ){
    YearJ=paste("M", formatC(YearAD-1867,format="d",flag="0",width=2),sep="")
   }
   else{
    YearJ=""
   }
   yearJ[i]=YearJ
  }
  return( yearJ )

 }

} # end of yearJAD

#' Find Calling Functions
#'
#' @param func A function  object (no quote) or
#' a list/vector of the function object names to be searched.
#' @param resff The result of find_functions from which func will be searched.
#' @param from A name of a function or a list of functions from which
#' func will be searched. (resff has priority over from.)
#' @param nonative : argument of find_functions
#' @param noglobal : argument of find_functions
#' @param packages : argument of find_functions
#' @param print = 0 not to print the result.
#'
#' @details
#' \cr
#' When resff is not given, this function  requires "codetools" package.
#' \cr
#'
#' This function matches each element of func to the elements of resff.
#' \cr
#' resff can be created as \cr
#' \code{ resff <- find_functions( from, nonative=nonative, unlist=0
#' , noglobal=noglobal, packages=packages ) }
#' \cr
#'
#'
#' @examples
#' \dontrun{
#' library(codetools)
#' library(lazy.tools)
#'
#' res=find_calling_functions( vartype, from=ls("package:lazy.tools") )
#' res2=find_calling_functions( c("vartype","varlist"), resff=res$resff )$res
#' # res2 can be obtained by
#' #  find_calling_functions( c("vartype","varlist")
#' #                        , from=ls("package:lazy.tools") )$res
#' # but find_function takes long time.
#' }
#'
#' @return
#' A list of: \cr
#' res The list of function names which contains each of func. \cr
#' resff The reslt of find_functions.
#'
#'
#'
#' @export
#'



find_calling_functions <- function( func=NULL, resff=NULL, from=NULL
                        , nonative=1, noglobal=0, packages=NULL, print=1 ){
 # find functions which call func.
 # Shin-ichi Mayekawa
 # 20180203
 # roxygen2: 20220927dnc

 if( is.null(func) ){
  return(NULL)
 }

 # result of find functions: find functions in from
 if( is.null(resff) ){
  resff=find_functions( from, nonative=nonative, unlist=0
                        , noglobal=noglobal, packages=packages )
 }

 if( mode(func) == "function" ) func=as.character(substitute(func))
 # Here, func contains function names

 # Print( resff, func)

 # first func
 res2=unlist(
  lapply( resff, function(x){ if( func[1]  %in% x ) TRUE else FALSE  } )
 )
 res=list(names(resff)[res2])

 if( length(func) > 1 ){
  for( i in seq(along=(1:length(func)))[-1] ){
   res2=unlist(
    lapply( resff, function(x){ if( func[i]  %in% x ) TRUE else FALSE  } )
   )
   res=append( res, list(names(resff)[res2]) )
  }
 }

 names(res)=func

 if( print ){
  Print(res)
 }

 return( named_list( res, resff ) )

} # end of find_calling_functions

#' Find unique values of a vector or unique rows of a matrix and their location
#'
#' @param x a vector or a matrix
#' @param sort = 1 to sort the result
#'
#' @return
#'  A list of \cr
#'  \code{value} The unique values \cr
#'  \code{loc} The list of length \code{length(value)} containing
#'  the locations of the original elements.
#'
#' @examples
#' ( temp <- Unique( c(1,2,1,5,6,3,5,4,6) ) )
#' sum( unlist(lapply(temp$loc,length)) )
#' ( temp <- Unique( c(1,2,1,5,6,3,5,4,6), sort=1 ) )
#'
#' ( temp2 <- Unique( rbind(demomat(5,4),demomat(3,4)) ) )
#' ( temp2 <- Unique( rbind(demomat(5,4),demomat(3,4)), sort=1, decreasing=1 ) )
#'
#' @export
#'

Unique <- function( x, sort=0, decreasing=0 ){
 # returns a set of unique values of x and their locations
 # Shin-ichi Mayekawa
 # 20230902cot
 # sort: 20230903
 # factor added: 20230909
 #
 
 if( !is.vector(x) && !is.matrix(x) && !is.factor(x) ){
  cat("\nerror1(Unique) cannot handle arrays.\n")
  stop()
 }
 
 xu=unique(x)
 
 if( is.vector(x) || is.factor(x) ){
  
  lenu=length(xu)
  loc=vector( mode="list", length=lenu )
  for( i in 1:lenu ){
   loc[[i]]=which( x == xu[i] )
  }
  names(loc)=xu
  
  if( sort ){
   od=order(xu, decreasing=decreasing)
   xu=xu[od]
   loc=loc[od]
  }
  
 }
 else{
  
  lenu=nrow(xu)
  loc=vector( mode="list", length=lenu )
  for( i in 1:lenu ){
   loc[[i]]=which( apply( x, 1, function(y){all(y==xu[i,])} ) )
   names(loc[[i]])=NULL
  }
  
  if( sort ){
   nc=ncol(xu)
   code=paste("od=order(", paste("xu[,",1:nc,"]", sep="", collapse=",")
              , ", decreasing=decreasing)", sep="")
   eval(parse(text=code))
   Print(code)
   xu=xu[od,]
   loc=loc[od]
  }
  
 }
 
 return( list( value=xu, loc=loc ) )
 
 
} # end of Unique

#' Perfectly Flat Cross Table by native xtabs
#'
#' This function makes the result of xtabs perfectly flat.
#'
#' @param formula a formula object of the form lhs~rhs
#' @param data an optional data frame containing the variables in
#' the formula formula.
#' @param remove_zeros = 1 to remove zero frequency cells
#' @param print = 1 to print the result
#' @param debug = 1 to print the intermediate results
#' @param subset xtabs parameter
#' @param sparse xtabs parameter
#' @param na.action xtabs parameter
#' @param na.rm xtabs parameter
#' @param addNA xtabs parameter
#' @param exclude xtabs parameter
#' @param drop.unused.levels xtabs parameter
#'
#' @details
#' \code{formula} and \code{data} parameters function the same way
#' as native xtab function. \cr
#' In addtion, those parameters marked as "xtabs parameter",
#' see \link[stats:xtabs]{stats::xtabs}.
#'
#' The core part of thus function is as follows:
#' \preformatted{
#'  res <- ftable( xtabs( lhs~rhs,  data=data ) )
#' }
#' The above returns a matrix when \code{lhs} is a matrix.
#' However, as shown in the examples,
#' when \code{lhs} is a vector or missing, the result is a cross table. \cr
#' Note that, by adjusting \code{row.vars} and \code{col.vars} options of
#'  \code{ftable}, it is possible to produce a completely flat table.
#'
#'
#' @examples
#'
#' seed=1701
#' set.seed(seed)
#' n <- 50
#' x1 <- sample(1:3,n,replace=TRUE)
#' x2 <- sample(LETTERS[1:2],n,replace=TRUE)
#' x3 <- sample(letters[1:2],n,replace=TRUE)
#' c1 <- sample(c(0:3),n,replace=TRUE)
#' c2 <- sample(c(0:5),n,replace=TRUE)
#' data=data.frame(x1,x2,c1,c2)
#'
#' # 3 by variables
#' fxtabs( cbind(c1,c2)~x1+x2+x3, data=data)
#' ftable(xtabs( cbind(c1,c2)~x1+x2+x3, data=data))
#' fxtabs( c1~x1+x2+x3, data=data)
#' ftable(xtabs( c1~x1+x2+x3, data=data))
#' fxtabs( ~x1+x2+x3, data=data)
#' ftable(xtabs( ~x1+x2+x3, data=data))
#'
#' # 2 by variables
#' fxtabs( cbind(c1,c2)~x1+x2, data=data)
#' ftable(xtabs( cbind(c1,c2)~x1+x2, data=data))
#' fxtabs( c1~x1+x2, data=data)
#' ftable(xtabs( c1~x1+x2, data=data))
#' fxtabs( ~x1+x2, data=data)
#' ftable(xtabs( ~x1+x2, data=data))
#'
#' # single by variable
#' fxtabs( cbind(c1,c2)~x1, data=data)
#' ftable(xtabs( cbind(c1,c2)~x1, data=data))
#' fxtabs( c1~x1, data=data)
#' ftable(xtabs( c1~x1, data=data))
#' fxtabs( ~x1, data=data)
#' ftable(xtabs( ~x1, data=data))
#'
#'
#' @return
#' a matrix of flat contingency table.
#'
#' @export
#'

fxtabs <- function( formula = ~., data = parent.frame(), subset
                    , sparse = FALSE, na.action, na.rm = FALSE, addNA = FALSE
                    , exclude = if (!addNA) c(NA,NaN)
                    , drop.unused.levels = FALSE
                    , remove_zeros=0, print=0, debug=0 ){
 # Completely Flat Cross Table by native xtabs
 # Shin-ichi Mayekawaq
 # 20260514
 #
 # note
 # The original version was aggregate_freqdist which was renamed as fxtabs0.
 #
 #

 # analyze the input formula
 fmc=gsub(" ","", deparse(substitute(formula)))
 countname=strsplit(fmc,"~")[[1]][1]
 xname=strsplit(fmc,"~")[[1]][2]
 xname=strsplit(xname,"\\+")[[1]]
 nxvar=length(xname)

 if( debug ){
  cat("\n\n")
  printm(fmc,countname,xname)
 }

 #
 # Note that, flatten does not flatten when there are single count variable
 # and two by variables (classification variables.)
 # Therefore, when count is a single varible,
 #  we must fake xtabs by cbinding c_c_c_=1 to the given count.
 # The easiest way to do so is to add c_c_c_=1 always.
 #
 # Also, when the count variable is missing, we must add c_c_c_=rep(1,n)
 # before faking xtabs.
 # The resulting count will be in .count. variable.
 #


 # fake xtabs by adding .count.=rep(1,n) when there are only one count variable.
 if( countname == "" ){
  # get n: we need n when the count is missing.
  m <- match.call(expand.dots = FALSE)
  drop=c("print","debug","remove_zeros")
  m=m[ !(names(m) %in% drop) ]
  m[[1L]] <- quote(stats::model.frame)
  mf <- eval(m, parent.frame())
  n=nrow(mf$data)
  rm(mf,m)
  if( debug ) printm(n)
  formula <- update(formula, cbind(.count.=rep(1,n)) ~ ., evaluate=FALSE)
 }
 # always fake xtabs by adding c_c_c_=1
 formula <- update(formula, cbind(c_c_c_=1,.) ~ ., evaluate=FALSE)

 # use xtabs
 temp=xtabs( formula=formula, data=data, sparse = sparse   # , subset=subset
             , na.rm = na.rm, addNA = addNA
             , exclude = exclude, drop.unused.levels = drop.unused.levels
 )

 if( debug ){
  printm("xtabs", temp)
  str(temp)
 }

 # flatten the xtabs output
 ft=ftable(temp)
 rname=attributes(ft)$row.vars
 cname=attributes(ft)$col.vars[[1]]

 if( debug ){
  cat("\nFlattened table by ftabs\n")
  print(ft)
  printm(rname,cname)
  str(ft)
 }

 # make the last variable change first: rev reverses the columns of data frame
 id2=rev(expand.grid(rev(rname)))
 # merge id2
 ft=unclass(ft)
 colnames(ft)=cname

 # convert to num or char
 for( j in 1:ncol(id2) ){
  id2[,j]=type.convert(as.character(id2[,j]), as.is = TRUE)
 }

 locnz=apply( ft,1, function(x) !all(x==0) )
 ft=cbind(id2,ft )

 if( remove_zeros ){
  ft=ft[locnz,]
  if( debug ) ft=ft[locnz]
 }

 # remove fake var
 locfake=which(colnames(ft)=="c_c_c_")
 if( length(locfake) > 0 ) ft=ft[,-locfake, drop=FALSE]


 if( print ) printm(ft)

 return( ft )


} # end of fxtabs

#' Dummy Expantion of a Vector
#'
#' @param x  a vector to be expanded
#' @param type = -1 for non-full-rank design matrix used in SAS proc GLM \cr
#'              = 0  for zero-sum design matrix \cr
#'              = 1  for drop-one-column design matrix \cr
#' @param drop  = "last"   or    "first"  when type != -1\cr
#' @param levels = a string vector consisting of the labels for the levels
#' @param froot = A character to be used to name the columns of X matrix
#' @param sort = 1 to sort the levels as the native factor function does. \cr
#' This is the default behavior of native model.matrix function.
#'
#' @details
#' By default, the zero-sum design matrix with the last level dropped
#' will be generated.
#'
#' @return Design matrix of the size length(x) times length(factor(x))
#' or length(x) times length(factor(x))-1
#'
#' @examples
#' design_expand( c("a","b","c") )
#' design_expand( c("a","xb","c") )
#' design_expand( c("a","b","c"), sort=1 )
#'
#' design_expand( c("a","b","c"), type=-1 )
#' design_expand( c("a","b","c"), levels=c("c","a","b"), type=-1 )
#'
#' design_expand( 1:3, type=0 )
#' design_expand( c("a","b","c"), froot="A" )
#'
#' @export
#'

design_expand <- function( x, type=0, drop="last", levels=NULL
                           , froot="level", sort=0 ){
 # generate desing matrix form a vector consisting of level names
 # Shin-ichi Mayekawa
 # 20150729
 # names: 20150730dnc
 # sort: 20190120
 # levels: 20221021
 #
 # Args:
 #  x           vector or column vector consisting of levels
 #  type        type of desing matrix to be generated
 #              = -1 for non-full-rank design matrix used in SAS proc GLM
 #              = 0  for zero-sum design matrix with the last level omitted
 #              = 1  to drop one column
 #  drop        = "last"   or    "first"
 #

 if( is.factor(x) ) x=as.vector(x)
 else if( is.matrix(x) ) x=as.vector(x)

 if( is.null(levels) ){
  if( sort == 0 ){
   xf=factor( x, levels=unique(x) )
  }
  else{
   xf=factor( x )
  }
 }
 else{
  xf=factor( x, levels=levels )
 }

 X=model.matrix( ~xf-1 )
 attributes(X)=attributes(X)[1:2]
 colnames(X)=paste(froot,1:ncol(X), sep="")

 if( type == 0 ){
  if( drop == "last" ) loc=ncol(X)
  else loc=1
  X[X[,loc]==1,]=-1
  X=X[,-loc,drop=0]
 }
 if( type == 1 ){
  if( drop == "last" ) loc=ncol(X)
  else loc=1
  X=X[,-loc,drop=0]
 }

 return( X )

} # end of design_expand

#' Frequency Distribution of Many Discrete Variables
#'
#' @param ... Discrete/categorical variables to be tabulated
#' @param sortrow = 0  not to sort the rows of the result (category values)
#' @param sortcol = 1  to sort the column of the result (variable names)
#' @param useNA = "ifany" if the frequency of NA is needed.
#' @param debug = 1 to print intermediate result
#'
#' @details
#' If \code{X} is a matrix consisting of discrete variables, \cr
#' \code{freqc(X)} is equivaleet to
#' \preformatted{
#'   freq=lapply(as.list(X),table)
#'   res2=mapply( function(x,name){
#'     y=matrix(x,1); colnames(y)=names(x); rownames(y)=name;
#'     return(as.data.frame(y))  }
#'     ,  freq, names(freq) )
#' }
#' 
#' Space or Blank values will be denoted as V1.
#'
#' @return
#' A values x variables matrix containing the frequencies. \cr
#' \code{F <- freqc( a, b )} returns a matrix \code{F}
#' whose i-j element contains the frequency of value \code{rownames(F)[i]}
#' in the variable \code{colnames(F)[j]}.
#'
#' @examples
#' set.seed(1701)
#' a <- sample(1:9,20,replace=1)
#' b <- letters[sample(1:9,20,replace=1)]
#' c <- sample(3:5,20,replace=1)
#' d <- letters[sample(5:9,20,replace=1)]
#'
#' a[9] <- NA
#' b[3:4] <- NA
#'
#' data <- data.frame( a, b )
#'
#' res1 <- freqc(a,b,c,d)
#' res2 <- freqc(data,c,d)
#' res3 <- freqc(c,d,a,b)
#' Print(res1,res2,res3)
#' res4 <- freqc(c,d,a,b, sortrow=0)
#' res5 <- freqc(c,d,a,b, sortcol=1)
#' Print(res1,res4,res5)
#'
#' res11 <- freqc(a,b,c,d, useNA="ifany")
#' res21 <- freqc(data,c,d, useNA="ifany")
#' Print(res11,res21)
#'
#' @export
#'

freqc <- function( ..., useNA="no", sortrow=1, sortcol=0, debug=0 ){
 # univariate freq dist of many discrete varaibles
 # Shin-ichi Mayekawa
 # 20200830c, 0921
 # input as ...: 20200922
 # bugfix for factor: 20221023
 # simplify in mapply: 20221024
 # When ... consists of a single matrix: 20230118
 # sort as.numeric: 20230118
 # 
 
 # analyze ...
 temp=analyze_three_dots( ... )
 name=temp$name
 val=temp$val
 n=length(name)
 
 allmat=unlist( lapply(val,is.matrix) )
 
 # error
 if( n == 0 ){
  cat("\nerror1:(freqc) The argument is empty. \n")
  stop()
 }
 
 val2=NULL
 name2=NULL
 for( i in 1:n ){
  if(is.vector(val[[i]]) | is.factor(val[[i]]) ){
   val2=append(val2,list(val[[i]]))
   name2=c(name2,name[i])
  }
  else{
   vv=data.frame(val[[i]])
   val2=append(val2, as.list(vv))
   name2=c(name2,colnames(vv))
   n=length(val2)
  }
 }
 names(val2)=name2
 
 
 
 
 if( debug ){
  Print(name2)
  if( debug >= 9 ) Print(name2,val2)
 }
 
 # tabulate all:  result is a list
 freq=lapply(val2, table, useNA=useNA)
 names(freq)=name2
 
 if( debug ) Print(freq)
 
 # change NA in the names to "<NA>"
 freq=lapply( freq, function(x){ names(x)[is.na(names(x))]="<NA>"; x} )
 
 if( debug ) Print("NA",freq)
 
 if(0){
  # blanks
  freq=lapply( freq, function(x){
   names(x)[grep("[ ]+",names(x))]="sp";
   return(x)
  })
 }
 
 # convert the list elements to a data frame with rownames
 res2=mapply( function(x,name){
  y=matrix(x,1);
  colnames(y)=names(x);
  rownames(y)=name;
  return(as.data.frame(y))  }
  ,  freq, names(freq), SIMPLIFY=FALSE )
 
 if( debug ) Print("df", res2, names(res2))
 
 if( n == 1 ){
  res3=matrix(unlist(res2),,1)
  rownames(res3)=names(res2[[1]])
  colnames(res3)=names(res2)[[1]]
 }
 else{
  # stack them all
  res3=Reduce( stack_df, res2 )
  
  if( debug ) Print("n>1", res3)
  
  # transpose the result
  if( debug ) Print(colnames(res3))
  res3=t(res3)
 }
 
 if( debug ) Print("after t", res3)
 
 # sort by the rownames=category values
 if( sortrow == 1 ){
  if( is.number(rownames(res3)) ) od=order(as.numeric(rownames(res3))) 
  else od=order(rownames(res3))
  res3=res3[od,,drop=0]
 }
 if( sortcol == 1 ){
  od=order(colnames(res3))
  res3=res3[,od,drop=0]
 }
 
 # move "<NA>" to the very bottom
 locna=which(rownames(res3) == "<NA>")
 if( length(locna) > 0 ){
  res3=rbind(res3[-locna,,drop=0],res3[locna,,drop=0])
 }
 
 return( res3 )
 
} # end of freqc

#' Get Values from an Array
#'
#' @param A An array object
#' @param i Vector consisting of the subscripts for the 1st dimension
#' @param j Vector consisting of the subscripts for the 2nd dimension
#' @param k Vector consisting of the subscripts for the 3rd dimension
#' @param l Vector consisting of the subscripts for the 4th dimension
#'
#' @param subs string specifying the subscripts such as "1:2, , 4, 2:2" \cr
#' or a numeric vector consisting of the scalar subscript for each dim.
#'
#' @param value The value to be assigned.
#'
#' @details
#' It is assumed that \code{A} is an array. \cr
#' When \code{A} is a list of data frames, use \code{get_valL} instead.\cr
#' When numeric \code{subs} is given it must be a vector of length
#' \code{length(dim(A))} specifying a scalar element of \code{A}. \cr
#' \cr
#' \code{get_val(A,i,j,k,l)} returns \code{A[i,j,k,l]}. \cr
#' \code{put_val(A,i,j,k,l) <- value} puts the value at \code{A[i,j,k,l]}.
#' \cr
#' Therefore, when \code{A} is an array, better use \code{A[i,j,k,l]}.
#' \cr
#'
#' \code{get_val(A,i,j,k,l)} is equivalent to \code{get_val(A,subs=subs)}
#' \cr where \code{subs} contains the values of \code{i,j,k,l} as a string.
#' \cr For example, \cr
#' \code{get_val(A, 1:2, , 3)} is equivalent to
#' \code{get_val(A, subs="1:2, , 3")}. \cr
#' \cr
#' Use \code{subs=} when the # of dimensions is more than 5.
#' \cr
#'
#' \code{subs=} has priority over \code{i,j,k,l}.
#'
#' \cr
#' The result will be simplified to a matrix or vector if possible.
#'
#' @examples
#' # set up an array B
#' dims=c(5,4,3,2)
#' B <- array(0,dims)
#' B[,,,1] <- demomat(5,4,3)*10+1
#' B[,,,2] <- demomat(5,4,3)*10+2
#' dimnames(B) <-
#'  mapply( function(x,y){paste(x,1:y,sep="")}, letters[1:length(dims)],dims)
#'
#' # get B[5,4,3,2]
#' res1=get_val(B,5,4,3,2)
#' res2=get_val(B,subs=c(5,4,3,2))
#' res3=get_val(B,subs="5,4,3,2")
#' Print(res1,res2,res3)
#'
#' # get B[2,,3,]:  cannot use numeric subs here.
#' res1=get_val(B,2,,3)
#' res3=get_val(B,subs="2,,3,")
#' Print(res1, res3 )
#'
#' # modify B[,1,2:3,1]
#' ( temp <- B[,1,2:3,1] )
#' put_val(B,,1,2:3,1) <- -9999
#' B[,1,2:3,1]
#' put_val(B,subs=",1,2:3,1") <- temp
#' B[,1,2:3,1]
#' put_val(B,subs=c(1,1,1,2)) <- -9999
#' B[1,1,1,2]
#'
#'
#' @return
#'  \code{get_val(A, i,j.,k,l)} returns
#'  \code{A[i,j,k,l]}. \cr
#'  \code{put_val(A, i,j,k,l) <- value} replaces
#'  \code{A[i,j,k,l]} with \code{value}.
#'
#' @export
#'

get_val <- function( A, i=NULL, j=NULL, k=NULL, l=NULL, subs=NULL ){
 # get value of A at subs
 # Shin-ichi Mayekawa
 # 20160915c
 # i,j,k,l: 20171030
 #

 if( is.list(A) ){
  cat("\nerror1:(get_val) Use get_valL instead for list A.\n")
  stop()
 }

 # is subs given?
 if( !is.null(subs) ){

  cond=0
  if( all(is.numeric(subs)) && length(subs) == length(dim(A)) ){
   # numeric subs specifying a scalar element
   subs=paste(subs,collapse=",")
   cond=1
  }
  if( length(subs) == 1  && is.character(subs) ){
   # string subs
   cond=2
  }

 if( cond != 0 ){
  code=paste("Asub=A[",paste(subs,collapse=","),"]",sep="")
  eval(parse(text=code))
  return(Asub)
 }
 else{
   cat("\nerror1:(get_val) The length of numeric sub must be the same",
       "as the # of dimensions of A.\n")
   stop()
  }

 }

 # subs not given
 dimA=dim(A)
 ndim=length(dimA)

 if( ndim >= 5 ){
  cat("\nerror1:(get_val) Use subs= when length(dim(A)) >= 5. \n")
  stop()
 }

 code="Asub=A["
 for( s in 1:ndim ){  # 3 for i,j,k

  if( s == 1 ) ss=i
  else if( s == 2 ) ss=j
  else if( s == 3 ) ss=k
  else if( s == 4 ) ss=l

  if( !is.null(ss) ){
   isub=paste("c(", paste(ss,collapse=","),")", sep="")
  }
  else{
   isub=paste("c(", paste(1:dim(A)[s],collapse=","),")", sep="")
  }
  # Print(s,isub)
  code=paste(code,isub,collapse="")
  if( s < ndim ) code=paste(code, ",", collapse="")
  # Print(code)
 }
 code=paste(code, "]", collapse="")
 # Print(code)

 eval(parse(text=code))
 return(Asub)

} # end of get_val


#' @rdname get_val
#' @export
#'

`put_val<-` <- function( A, i=NULL, j=NULL, k=NULL, l=NULL, subs=NULL
                         , value ){
 # modify the value of A at subs
 # Shin-ichi Mayekawa
 # 20160915c
 # i,j,k,l: 20171030

 if( is.list(A) ){
  cat("\nerror1:(get_val) Use get_valL instead for list A.\n")
  stop()
 }

 if( !is.null(subs) ){

  cond=0
  if( all(is.numeric(subs)) && length(subs) == length(dim(A)) ){
   subs=paste(subs,collapse=",")
   cond=1
  }
  if( length(subs) == 1  && is.character(subs) ){
   cond=2
  }
  if( cond != 0 ){
   code=paste("A[",paste(subs,collapse=","),"]=value",sep="")
   eval(parse(text=code))
   return(A)
  }
  else{
   cat("\nerror1:(put_val) The length of numeric sub must be the same",
       "as the # of dimensions of A.\n")
   stop()
  }
 }



 # subs not given
 dimA=dim(A)
 ndim=length(dimA)

 if( ndim >= 5 ){
  cat("\nerror1:(put_val) Use subs= when length(dim(A)) >= 5. \n")
  stop()
 }

 code="A["
 for( s in 1:ndim ){  # 3 for i,j,k

  if( s == 1 ) ss=i
  else if( s == 2 ) ss=j
  else if( s == 3 ) ss=k
  else if( s == 4 ) ss=l

  if( !is.null(ss) ){
   isub=paste("c(", paste(ss,collapse=","),")", sep="")
  }
  else{
   isub=paste("c(", paste(1:dim(A)[s],collapse=","),")", sep="")
  }
  # Print(s,isub)
  code=paste(code,isub,collapse="")
  if( s < ndim ) code=paste(code, ",", collapse="")
  # Print(code)
 }
 code=paste(code, "]=value", collapse="")
 # Print(code)

 eval(parse(text=code))
 return(A)


} # end of `put_val<-`

#' Find the Location of the Quote to the Right or Left of the One at loc.
#'
#' @param exp A string.
#' @param loc The location of a quote.
#' @param right = 0 to find the one to the left.
#'
#' @details
#' If exp contains double quotes, exp must be defined by using single quotes
#' or must be escaped by the use of \.
#'
#' @return
#' The location of the quote found.
#'
#' @examples
#' exp='regexp("a", "12""a56")'
#' loc=regexpr('"',exp)
#' locn=locbalq(exp, loc)
#' Print(substr(exp,loc,locn))
#' loc=13
#' locn=locbalq(exp, loc)
#' Print(substr(exp,loc,locn))
#'
#' # location of the non-matching quote
#' loc=10
#' locn=locbalq(exp, loc)
#' Print(substr(exp,loc,locn))
#'
#' exp="regexp('a', '12''a56')"
#' locbalq(exp, regexpr("'",exp))
#' loc=16
#' locn=locbalq(exp, loc, right=0)
#' Print(substr(exp,locn,loc))
#'
#' @export
#'

locbalq <- function( exp, loc, right=1 ){
 # location of matching quote
 # Shin-ichi Mayekawa
 # 20171115,16
 #

 # error check
 nch=nchar(exp)
 if( loc > nch  |  loc <= 0 ){
  cat("\n\nerror1(locbalq)"
      , "loc must be positive and less than nchar(exp).\n\n")
  return(0)
 }

 # remove attributes from loc by regexpr
 attributes(loc)$match.length=NULL
 attributes(loc)$useBytes=NULL

 # quote
 q=substr(exp,loc,loc)

 # matching paren
 qqDS=c("'",'"')
 if( q %in% qqDS ){
  q2=q
  rev=0
  exp1=substr(exp,loc,nch)
 }
 else{
  cat("\n\nwarning1(locbalq) substr(exp,loc,loc) is not a quote.\n\n")
  return(0)
 }

 # DQ=rawToChar(as.raw(2))
 # SQ=rawToChar(as.raw(1))
 # e=exp
 # e=gsub("\"\"",DQ,e,fixed=1)
 # e=gsub("\'\'",SQ,e,fixed=1)
 # Print(e)

  e=strsplit( exp, "" )[[1]]

  locq=which(e==q)
  llq=length(locq)
  locqq=which(locq == loc)
  if( right ){
   if( locqq < llq ) locn=locq[locqq+1]
   else{
    cat("\nerror1:(locbalq) No matching quote in the right direction.\n")
    stop()
   }
  }
  else{
   if( 1 < locqq ) locn=locq[locqq-1]
   else{
    cat("\nerror1:(locbalq) No matching quote in the backward direction.\n")
    stop()
   }
  }

  return( locn )

} # end of locbalq

#' Add Enclosing Parentheses
#'
#' @param exp a string vector consisting of symbolic expressions..
#' @param check = 1 to avoid adding redundant parens.
#' @param trim = 1 to remove leading and trailing blanks from exp.
#'
#' @return an object of the same size as exp
#'
#' @examples
#' exp=c("a","(a+b)","(a+b)-(c-d)")
#' add_paren( exp )
#' add_paren( exp, check=1 )
#'
#' @export
#'

add_paren <- function( exp, check=0, trim=0 ){
 # add enclosing parentheses
 # Shin-ichi Mayekawa
 # 20150817,18
 # 20170921
 #

 exp[]=unlist( lapply( exp,function(x){
  if( trim == 1 ) x=trim(x,leading=1)
  if( check == 1 ){
   if( substr(x,1,1) == "("  &&  locbalpar(trim( x,leading=1),1) == nchar(x) )
    return(x)
   else return( paste("(",x,")",sep="") )
  }
  else return( paste("(",x,")",sep="") )
 }  ) )

 return( exp )

} # add_paren

#' Read Lines or Wite Lines with or without Encoding Option
#'
#' @param filename an input text file name
#' @param encoding = "Shift-JIS" | "UTF-8" | NULL
#' where NULL is for system default and should be equal to "native.enc".
#' @param text a text vector to be written
#' @param asDBCS = 1 to create double byte characters when
#'  encoding="Shift-JIS" is specified.
#' @param ... aditional parameters to readLines and writeLines
#'
#' @details
#' Note that, as described in the Arguments section of native readLines,
#' specifying encoding option when calling readLines cannot read lines
#' from a file using the specified encoding. \cr
#' This function specifies encoding when defining the connection
#' and closes it upon return.
#'
#' The following code
#' \preformatted{
#'  con <- file( filename, encoding="Shift-JIS" )
#'  return( readLines( con, ... )  )
#' }
#' makes the Zenkaku characters 3 byte chracters.
#' Therefore, when asDBCS=1, native \code{readLines} will be used.
#'
#' When writing UTF-8 characters which are not included in native encoding,
#' specify \code{useBytes=TRUE} option.\cr
#' Check to see that \code{Encoding(text)} returns "UTF-8", not "unknown".
#'
#' Above can avoid the following default action of \code{writeLines}:
#' \preformatted{
#'  useBytes is for expert use. Normally (when false) character strings with
#'  marked encodings are converted to the current encoding before being passed
#'  to the connection (which might do further re-encoding).
#'  useBytes = TRUE suppresses the re-encoding of marked strings so they are
#'  passed byte-by-byte to the connection: this can be useful when strings
#'  have already been re-encoded by e.g. iconv.
#'  (It is invoked automatically for strings with marked encoding "bytes".)
#'  }
#'
#' @examples
#' # In the examples below, TestData1.txt is encoded in Shift-JIS
#' # and  TestData2.txt, in UTF-8. The last line of TestData2.txt contains
#' # non Shif-JIS characters.
#' #
#' # These files are stored in the folder where lazy.tools package is stored.
#'
#' # reading Shift-JIS files on from R.4.1.3
#' nobs <- 10
#' infile1 <- system.file("TestData1.txt",package="lazy.tools")
#' if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 ){
#'  temp1 <- ReadLines(infile1, n=nobs+2)[-(1:2)]
#' } else{
#'  # The result will be Double Byte.
#'  temp1 <- ReadLines(infile1, n=nobs+2, asDB=1)[-(1:2)]
#'  # The result will be Triple Byte.
#'  temp11 <- ReadLines(infile1, n=nobs+2)[-(1:2)]
#'
#'  nbyte(temp1)
#'  nbyte(temp11)
#'  identical(temp1[-nobs],temp11[-nobs])  # true
#'  identical(nbyte(temp1[-nobs]),nbyte(temp11[-nobs])) # false
#' }
#'
#'
#'
#' # reading UTF-8
#' nobs <- 10
#' infile2 <- system.file("TestData2.txt",package="lazy.tools")
#' temp2 <- ReadLines(infile2, n=nobs+2, encoding="UTF-8")[-(1:2)]
#' nbyte(temp2)
#'
#' identical(temp1[-nobs],temp2[-nobs])  # true
#' if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) <= 0 ){
#'  identical(temp11[-nobs],temp2[-nobs])  # true
#'  identical(nbyte(temp11[-nobs]),nbyte(temp2[-nobs]))  # false
#' }
#'
#'
#'
#' # reading UTF-8 with BOM
#' nobs <- 10
#' infile3 <- system.file("TestData3.txt",package="lazy.utf8")
#' temp3 <- ReadLines(infile3, n=nobs+2, encoding="UTF-8")[-(1:2)]
#' nbyte(temp3)
#' identical(temp2[-nobs],temp3[-nobs])  # true
#'
#'
#'
#' \dontrun{
#' # These two work
#' WriteLines( temp1, "temp11.txt" )
#' WriteLines( temp1, "temp12.txt", encoding="UTF-8" )
#'
#' # These two work exept for the last obs which has non Shift-JIS characters.
#' WriteLines( temp2, "temp21.txt" )
#' WriteLines( temp2, "temp22.txt", encoding="UTF-8"  )
#'
#' # This works.
#' con <- file( "temp222.txt", encoding="native.enc" )
#' writeLines( temp2, con , useBytes=TRUE )
#' close(con)
#' }
#'
#'
#' # creating data frame using column specifications
#' # column spec
#' from <- c(1, 7,25,26,27,28,29,30,31,32,34)
#' to <-   c(5,23,25,26,27,28,29,30,31,32,39)
#' name <- c("id", "name",paste("Q",1:8,sep=""),"comment")
#'
#' # convert to a character matrix by stacking each row (record, observation)
#' datamat2 <- Reduce( rbind, substr2(temp2, from,to), init=NULL )
#' colnames(datamat2) <- name
#' # matrix to data frame
#' df2 <- as.data.frame(datamat2)
#'
#' # Alternative way by stacking each column (variable)
#' datamat22 <- Reduce( cbind, mapply( function(x,y) substr2(temp2,x,y)
#'                    , from, to, SIMPLIFY=FALSE ), init=NULL )
#' colnames(datamat22)=name
#' # matrix to data frame
#' df22 <- as.data.frame(datamat22)
#'
#' identical(datamat2,datamat22)  # true
#' identical(df2,df22)  # true
#'
#' print(datamat2)
#'
#' # character to numeric conversion
#' df2[,paste("Q",1:8,sep="")] <- apply(df2[,paste("Q",1:8,sep="")],2
#'                                     ,as.numeric)
#' str(df2)
#'
#' # cannto print UTF-8 data frame nicely by native print.data.frame.
#' print(df2)
#' print.listof(df2)
#'
#'
#' @return ReadLines returns a vector of strings.
#'
#' @export
#'

ReadLines <- function( filename, encoding="Shift-JIS", asDBCS=0, ... ){
 # Read Lines with encoding
 # Shin-ichi Mayekawa
 # 20220811cot,12cot
 # asDBCS: 20220818
 # rozygen2 example: 20220823
 #

 # This enables the use of DBCS.
 if( encoding == "Shift-JIS"  &&  asDBCS == 1 ){
   return( readLines( filename, ... ) )
 }

 # in order to close the connection, it seems necessary to define an object
 if( !is.null(encoding) ){
  con=file( filename, encoding=encoding )
 }
 else{
  con=file( filename )
 }

 # close connection upon return
 on.exit(close(con))

 return( readLines( con, ... )  )

} # end of ReadLines



#' @rdname ReadLines
#' @export
#'

WriteLines <- function( text, filename, encoding="Shift-JIS", ... ){
 # Wite Lines with encoding
 # Shin-ichi Mayekawa
 # 20220811cot,12cot
 #

 # in order to close the connection, it seems necessary to define an object
 if( !is.null(encoding) ){
  con=file( filename, encoding=encoding )
 }
 else{
  con=file( filename )
 }

 # close connection upon return
 on.exit(close(con))

 writeLines( text, con, ... )

 invisible( NULL )

} # end of WriteLines





#' Returns the number of bytes used to represent each character
#' in each of the string.
#'
#' @param x String vector
#'
#' @return a list consisting of the number of bytes per character
#'
#' @details
#' Let x = "abc". The core of this function is \cr
#' \code{lapply(
#'   strsplit(x,""), function(y) unlist(lapply(lapply(y,charToRaw),length))
#'   ) } \cr
#' It counts the length of the Raw Expression of each character.
#'
#' About rawToChar and charToRaw functions: \cr
#' To get the raw expression of character, use\cr
#' \code{ raw <- charToRaw(paste(c(0:9," ",letters,LETTERS),collapse="")) }\cr
#' To get the numeric equivalent of raw, use\cr
#' \code{ num <- as.numeric(raw) }.\cr
#' To recover the original characters, use\cr
#' \code{ char <- rawToChar( as.raw(num) ) }.\cr
#'
#' @examples
#' # Example of one, two, three amd four byte characters on UTF-8 machines
#' if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 ){
#'  num=c(97, 206, 177, 227, 129, 130, 240, 160, 174, 183)
#'  char=num2char(num)
#'  nbyte(char)
#' }
#'
#' # A string consisting of single and multi byte characters
#' # The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
#' {if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
#'  nza <- c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
#' else
#'  nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
#' za <- rawToChar(as.raw(nza))
#'
#' # location of the multi byte characters
#' nchar(za)
#' is.DBCS(za)
#' printm( nchar(za), length(charToRaw(za)), nchar2(za) )
#' sza=unlist(strsplit(za,""))
#' printm(sza,nbyte(za),is.DBCS(za))
#'
#' cat("When using Shif-JIS encoding, ",
#'     "it seems that 'strsplit' makes the result 3 byte characters.\n")
#' printm(sza,unlist(nbyte(sza)),unlist(is.DBCS(sza)))
#' psza <- paste(sza,collapse="")
#' printm(sza,nbyte(psza),is.DBCS(za))
#'
#' identical(za,psza)
#' identical(nbyte(za),nbyte(psza))
#'
#' # Instead of strsplit, use below
#' sza2 <- substring(za,1:nchar(za),1:nchar(za))
#' printm( sza, unlist( nbyte( sza2 ) ) )
#' psza2 <- paste(sza2,collapse="")
#' nbyte(psza2)
#'
#' identical(za,psza2)
#' identical(nbyte(za),nbyte(psza2))
#'
#'
#' @export
#'

nbyte <- function( x ){
 # returns a list of the # of bytes in a string vector
 # Shin-ichi Mayekawa
 # 20170703
 # nbyte: 20220811cot
 # avoid strsplit: 20220818
 # use nchar with width=: 20220819
 # rozygen2 example: 20220823
 # new method: 20220826
 #

 #
 # old version: strsplit makes the result 3 byte characters!!!
 # nbyte=lapply( strsplit(x,"")
 #             , function(y) unlist(lapply(lapply(y,charToRaw),length)) )
 #
 # old version: better use nchar with byte option
 # nbyte=lapply( x, function(y){
 #  res=numeric(nchar(y))
 #  for( i in 1:nchar(y) ){
 #   res[i]=length(charToRaw(substr(y,i,i)))
 #  }
 #  return( res )
 #  }
 # )

  nbyte=lapply( x, function(y){
    unlist(lapply(1:nchar(y), function(i){nchar(substr(y,i,i), type="byte")} ))
  } )

 if( length(nbyte) == 1 ) nbyte=unlist(nbyte)
 return( nbyte  )

} # end of nbyte




#' Count the number of characters using bytes, not characters, as the unit.
#'
#' @param x A string
#'
#'
#' @details
#'
#' This function is equivalent to native nchar function with type="byte"
#' option.
#'
#'
#' About rawToChar and charToRaw functions: \cr
#' To get the raw expression of character, use\cr
#' \code{ raw <- charToRaw(paste(c(0:9," ",letters,LETTERS),collapse="")) }\cr
#' To get the numeric equivalent of raw, use\cr
#' \code{ num <- as.numeric(raw) }.\cr
#' To recover the original characters, use\cr
#' \code{ char <- rawToChar( as.raw(num) ) }.\cr
#'
#' @examples
#' # A string consisting of single and multi byte characters
#' # The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
#' {if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
#'  nza <- c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
#' else
#'  nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
#' za <- rawToChar(as.raw(nza))
#' # za <- num2char(nza)
#'
#' # location of the multi byte characters
#' nchar(za)
#' is.DBCS(za)
#' printm( nchar(za), length(charToRaw(za)), nchar2(za) )
#' printm(unlist(strsplit(za,"")),nbyte(za),is.DBCS(za))
#'
#'
#' @return A number or a numeric vector.
#'
#' @export
#'

nchar2 <- function( x ){
 # # of characters using bytes, not characters, as the unit
 # Shin-ichi Mayekawa
 # 20171011
 # vector x: 20220812cot
 #
 return( sapply(lapply(x,charToRaw),length) )

} # end of nchar2


#' Starting and Ending Columns of Each Character in a Text File
#'
#' @param x A string vector
#'
#' @details
#' This function can be used to get the substring of a string
#' by specifying the starting and the ending columns. \cr
#' It is assumed that the double or triple byte characters takes two columns
#' of a text file.  (Zenkaku Character)
#'
#' If \code{x} contains a string consisting of single and multibyte characters,
#' \preformatted{
#'  ft <- cchar(x)
#'  substr( x, which(ft[,1] == start), To=which(ft[,2] == stop) )
#' }
#' will return the characters between columns \code{start} and \code{stop}.
#'
#' @examples
#' # A string consisting of single and multi byte characters
#' # The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
#' {if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
#'  nza <- c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
#' else
#'  nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
#' za <- rawToChar(as.raw(nza))
#' # za <- num2char(nza)
#'
#' # location of the multi byte characters
#' nchar(za)
#' is.DBCS(za)
#' printm( nchar(za), nchar2(za), nchar(za)+sum(is.DBCS(za)) )
#' printm(1:nchar(za), unlist(strsplit(za,"")),nbyte(za),cchar(za))
#'
#' # When input is a string vector
#' cchar(c(za,za))
#'
#' # convert column position to character position
#' # from and to as column position
#' from <- c(1,2,3,5,6,7,9,11)
#' to <- c(1,2,4,5,6,8,10,11)
#' # character position corresponding to (from,to) to be used in substr(ing)
#' FTc <- cchar(za,from,to)[[1]]
#' # substrings
#' ssg <- substring(za,FTc[,1],FTc[,2])
#' # substrings using substr2 which can hadle column (from,to)
#' ss2 <- substr2(za,from,to)
#' za1 <- strsplit(za,"")[[1]]   # to print each character
#' printm(za1, from, to, FTc, ssg,ss2)
#'
#' # use of last option
#' cchar( za, 1:11, 1:11 )
#' cchar( za, 1:11, 1:11, last=0 )
#'
#' cchar( za, 3,4 )
#' cchar( za, 3,5 )
#' cchar( za, 3,7 )
#' cchar( za, 3,7, last=0 )
#'
#' # When input is a string vector
#' cchar( c(za,za), 1:11, 1:11, last=0 )
#'
#'
#' @return
#' When \code{from} is not given, a list of matrices
#' \code{FT[[i]]} where \code{FTT[[i]][j,1]} and \code{FTT[[i]][j,2]},
#' resp, are the starting and ending columns of the \code{j}-th character
#' of the \code{i}-th string.
#' When the input is a string, \code{FT[[1]]} will be returned.
#' \cr
#' When \code{from} is given, a list of matrices
#' \code{FTc[[i]]} corresponding to each of the strings where
#' \code{FTc[[i]][j,1], FTc[[i]][j,2]} are, resp,
#' the starting and ending characters corresponding to
#' \code{from[j]} and \code{to[j]} of the i-th string.
#'
#' @export
#'

cchar <- function( x, from=NULL, to=NULL, last=TRUE  ){
 # starting and ending columns of each character in a string vector
 # Shin-ichi Mayekawa
 # 20220811cot,14,19,
 # nchar with width: 20220821
 # start,stop, res -> from,to, FT: 20220821
 # convert_code included: 20220821
 # new method: 20220910cot
 #

 FT=lapply(x, function(lin){
  nb2=sapply(strsplit(lin,""),nchar,type="width")
  to=cumsum(nb2)  # ending column of each character
  from=to - nb2 + 1 # starting column of each character
  ft=cbind(from,to)
  colnames(ft)=c("from","to")
  return(ft)
 })

 if( is.null(from) ){
  if( length(FT) == 1 ) FT=FT[[1]]
  return( FT )
 }

 #
 # old convert_column starts here.
 #

 lenx=length(x)
 FT2=vector(mode="list", length=lenx)
 for( i in 1:lenx){
  ft=FT[[i]]

  # old method
  # lfrom=length(from)
  # FTnew=matrix(NA,lfrom,2)
  # for( j in 1:lfrom ){
  #  loc=which(ft[,1] == from[j])
  #  if(length(loc) > 0 ) FTnew[j,1]=loc
  #  loc=which(ft[,2] == to[j])
  #  if(length(loc) > 0 ) FTnew[j,2]=loc
  #  else if( last ) FTnew[j,2]=nchar(x[i])
  # }
  # end of old method

  # new
  FTbigF=rep(NA,nchar(x[i],type="width"))
  FTbigT=FTbigF
  FTbigF[ft[,1]]=1:nrow(ft)
  FTbigT[ft[,2]]=1:nrow(ft)

  FTnew=cbind(FTbigF[from],FTbigT[to])
  if( last ) FTnew[is.na(FTnew[,2]),2]=nchar(x[i])

  FT2[[i]]=FTnew
 }

 # if( lintext == 1 ) FT2=FT2[[1]]
 return( FT2 )

} # end of cchar







#' Substring Using Column Location as the Unit.
#'
#' @param x A string vector
#' @param from The first column (location in a line of a text file)
#' @param to The last column or 0 meaning the end of string
#' @param strict = 1 to check if start and stop are valid.
#' @param trim  = 1 to trim the result by lazy.tools::trim
#' @param ... options for lazy.tools::trim
#'
#' @details
#' It is assumed that the double or triple byte character takes two colums
#' of a text file.  (Zenkaku Character) \cr
#' See the description of \code{cchar}.
#'
#' When scrict=1, if the start or stop value is invalid, NA will be returned.\cr
#' When strict=0, if the start value is valid and the stop value is invalid,
#' stop will be set to the location of the last character.
#'
#' When trim is used, the result will be re-encoded to UTF-8.
#'
#' @examples
#'
#' # A string consisting of single and multi byte characters
#' # The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
#' {if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
#'  nza <- c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
#' else
#'  nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
#' za <- rawToChar(as.raw(nza))
#' # za <- num2char(nza)
#'
#' printm(za)
#' # location of the multi byte characters
#' is.DBCS(za)
#' printm( nchar(za), nchar2(za), nchar(za)+sum(is.DBCS(za)) )
#' printm(unlist(strsplit(za,"")),nbyte(za),cchar(za))
#' # The first to the fourth characters
#' substr(za,1,4)
#' # The characters of za in the first to the fourth column.
#' substr2(za,1,4)
#'
#'
#' # Another example
#' {if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
#'  nn <- c(87, 101,  32, 104,  97, 118, 101,  32, 228, 184, 141, 232, 184
#'         , 176, 231, 159, 169,  32, 121, 101, 116,  33)
#' else
#'  nn <- c(87, 101,  32, 104,  97, 118, 101,  32, 149, 115, 230, 250, 139
#'          , 233, 32, 121, 101, 116,  33)}
#' za2=num2char(nn)
#' printm( nchar(za2), nchar2(za2), nchar(za2)+sum(is.DBCS(za2)) )
#' printm(1:nchar(za2),unlist(strsplit(za2,"")),nbyte(za2),cchar(za2))
#'
#' # Note that start and stop must be the numbers contained in, resp,
#' # the first and the second columns of the returning matrix from
#' # cchar function.
#' substr2(za2,4,7)    # ok
#' substr2(za2,4,9)    # NG
#' substr2(za2,4,10)   # ok
#' substr2(za2,4,11)   # NG
#' substr2(za2,4,12)   # ok
#' substr2(za2,4,9, strict=1)    # NG
#' substr2(za2,4,11, strict=1)   # NG
#' substr2(za2,9,10, strict=1)   # ok
#' substr2(za2,9,11, strict=1)   # NG
#'
#'
#'
#'
#' # reading Shift-JIS files on from R.4.1.3
#' nobs <- 10
#' infile1 <- system.file("TestData1.txt",package="lazy.tools")
#' if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 ){
#'  temp1 <- ReadLines(infile1, n=nobs+2)[-(1:2)]
#' } else{
#'  # The result will be Double Byte.
#'  temp1 <- ReadLines(infile1, n=nobs+2, asDB=1)[-(1:2)]
#'  # The result will be Triple Byte.
#'  temp11 <- ReadLines(infile1, n=nobs+2)[-(1:2)]
#'
#'  nbyte(temp1)
#'  nbyte(temp11)
#'  identical(temp1[-nobs],temp11[-nobs])  # true
#'  identical(nbyte(temp1[-nobs]),nbyte(temp11[-nobs])) # false
#' }
#'
#' # reading UTF-8
#' nobs <- 10
#' infile2 <- system.file("TestData2.txt",package="lazy.tools")
#' temp2 <- ReadLines(infile2, n=nobs+2, encoding="UTF-8")[-(1:2)]
#' nbyte(temp2)
#'
#' # starting and ending columns of each field
#' from=c(1, 7,25,26,27,28,29,30,31,32,34)
#' to=c(  5,23,25,26,27,28,29,30,31,32,39)
#'
#' # character locations corresponding to from and to: should be equal.
#' FTc1=cchar( temp1, from, to )
#' FTc2=cchar( temp2, from, to )
#' char=strsplit(temp1[1],"")[[1]]
#' printm( char, from,to, FTc1[[1]],FTc2[[1]] )
#'
#' # substringing using FTc
#' substring( temp2[1], FTc1[[1]][,1], FTc2[[1]][,2] )
#' substring( temp2[2], FTc1[[2]][,1], FTc2[[2]][,2] )
#'
#' # get the field values as substrings  using substr2 function
#' s12=substr2(temp1,from[2],to[2])
#' s14=substr2(temp1,from[4],to[4])
#' s111=substr2(temp1,from[11],to[11])
#' s22=substr2(temp2,from[2],to[2])
#' s24=substr2(temp2,from[4],to[4])
#' s211=substr2(temp2,from[11],to[11])
#'
#' # comparison
#' printm(s14,s24)
#'
#' # Note that s111 and s211 are trimmed because they come from
#' # the variable length last columns.
#' # (On R4.1.3  s21 and s11 cannot be printmed together by printm.)
#' print.listof(list(s12=s12,s22=s22,s111=s111,s211=s211))
#'
#' # On R4.1.3,
#' # substr or substr2 does not change the encoding.
#' # lazy.tools::trim changes the encoding.
#' # s12t=trim(s12)
#' # printm( s12[1], s12t[1] )
#' # printm( nbyte(s12[1]),nbyte(s12t[1]) )
#' # This does not.
#' # s12tt=substr(s12,1,nchar(trim(s12)))
#' # printm(s12[1],s12tt[1])
#' # printm(nbyte(s12[1]),nbyte(s12tt[1]))
#'
#'
#' @return A vector consisting of the substrings of the input vector.
#' The result will be a list if the input is a string vector.
#'
#' @export
#'

substr2 <- function( x, from, to=0, strict=0, trim=0, ... ){
 # substr using bytes, not characters, as the unit
 # Shin-ichi Mayekawa
 # 20170701,02,03
 # comments: 20181101dnc
 # UTF-8: 20220811cot
 # strict: 20220813cot
 # new method: 20220821
 # start,stop -> from,to: 20220821
 # convert_code -> cchar: 20220821
 # trim: 20220821
 # roxygen2: 20220823
 #

 #  convert (from,to) to FTc
 if( strict )
   FTc=cchar( x, from, to, last=0 )
 else
   FTc=cchar( x, from, to, last=1 )

 # use substring which can handle vector start and stop
 res=mapply( function(y,ftc){ substring(y,ftc[,1],ftc[,2]) }, x, FTc
             , SIMPLIFY=FALSE )
 names(res)=names(x)

 if( length(x) == 1 ) res=res[[1]]
 if( length(from) == 1 ) res=unlist(res)

 if( trim ){
  res=trim(res,...)
 }
 return( res )

} # end of substr2



#' Checks if the Characters in a String are Multi Byte Characters.
#'
#' @param x String vector
#'
#' @details
#' Locale of your Windows machine can be obtained by the following:
#' \preformatted{
#' shell("ver")
#' shell("PowerShell  Get-WinSystemLocale")
#' shell("chcp > chcp.txt")  # chcp.txt will be created somewhere.
#' strsplit(readLines("chcp.txt")," ")[[1]][3]
#' }
#' Code 923 means Shift-JIS.
#'
#' Version and locale of R can be obtained as follows;
#' \preformatted{
#' version
#' Sys.getlocale()
#' }
#' \cr
#' T check the character encoding, use: \code{Encoding(char)}.
#' The result will be UTF-8 or unknown meaning system encoding.
#' \cr
#'
#' About rawToChar and charToRaw functions: \cr
#' To get the raw expression of character, use\cr
#'  \code{ char <- paste(c(0:9," ",letters,LETTERS),collapse="")}
#'  \code{ raw <- charToRaw(char) }\cr
#' To get the numeric equivalent of raw, use\cr
#'  \code{ num <- as.numeric(raw) }.\cr
#' To recover the original characters, use\cr
#'  \code{ char <- rawToChar( as.raw(num) ) }.\cr
#' \cr
#' Hex numbers used to express raw expression can be bypassed by using \cr
#'  \code{num <- char2num(char)} \cr
#'  \code{char <- num2char(num)} \cr
#'
#' @examples
#'
#' # A string consisting of single and multi byte characters
#' # The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
#' {if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
#'  nza=c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
#' else
#'  nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
#' za <- rawToChar(as.raw(nza))
#' # za <- num2char(nza)
#'
#' # location of the multi byte characters in a string
#' is.DBCS(za)
#'
#' # Pick up the single byte characters.
#' strsplit(za,"")[[1]][!is.DBCS(za)]
#' # Pick up the multi byte characters.
#' strsplit(za,"")[[1]][is.DBCS(za)]
#'
#' # strsplit changes the encoding from Shift-JIS (double byte) to UTF-8.
#' Encoding(strsplit(za,"")[[1]])
#' Encoding(substring(za,1:nchar(za),1:nchar(za)))
#' unlist(nbyte(strsplit(za,"")[[1]]))
#' unlist(nbyte(substring(za,1:nchar(za),1:nchar(za))))
#'
#'
#' @return A logical vector or list of logical vectors indicating if
#' each character in the input string vector is a multi byte chractor or not.
#'
#' @export
#'

is.DBCS <- function( x ){
 # Checks if the Characters in a String are Multi Byte Characters
 # Shin-ichi Mayekawa
 # 20170703
 # ==2 -> > 1: 20220811cot
 # roxygen2 example: 20220812cot,22
 #

 loc=lapply( strsplit(x,"")
             , function(y) unlist(lapply(lapply(y,charToRaw),length)) > 1 )
 if( length(loc) == 1 ) loc=unlist(loc)
 return( loc  )
} # end of is.DBCS


#' @rdname is.DBCS
#' @export
#'

is.MBCS <- is.DBCS



#' Substr Using Column Location as the Unit.
#'
#' @param x A string vector
#' @param start The first location
#' @param stop The last location
#'
#' @details
#'
#' This is for strictly DUOBLE byte characters. NOT for UTF-8 characters.
#' For strictly double byte characters, this is faster than susbtr2.
#'
#' About rawToChar and charToRaw functions: \cr
#' To get the raw expression of a string, use\cr
#' \code{ raw <- charToRaw(paste(c(0:9," ",letters,LETTERS),collapse="")) }\cr
#' To get the numeric equivalent of a raw (vector),  use\cr
#' \code{ num <- as.numeric(raw) }.\cr
#' To recover the original string, use\cr
#' \code{ char <- rawToChar( as.raw(num) ) }.\cr
#'
#' \cr
#' If the n-th byte is the beginning or the ending location of \cr
#' a double byte character, \code{substrDBCS(x,n,n)} is invalid.\cr
#' Consult \code{is.DBCS(x)} and use with caution.
#'
#' @examples
#' # The exa,ple below causes error on R 4.2.0 and above.
#' \dontrun{
#' # A string consisting of single and double byte characters
#' nza <- c(97,98,130,131,100,101,130,134,130,135,104)
#' za <- rawToChar(as.raw(nza))
#' Print(za)
#' # location of the double byte characters
#' is.DBCS(za)
#' # The first to the fourth characters
#' substr(za,1,4)
#' # The characters stored in the first to the fourth bytes of za.
#' substrDBCS(za,1,4)
#' }
#'
#' @return A vector consisting of the substrings of the input vector.
#'
#' @export
#'

substrDBCS <- function( x, start, stop=max(nchar(x,type="byte")) ){
  # substr using bytes, not characters, as the unit
  # Shin-ichi Mayekawa
  # 20170701,02,03
  # comments: 20181101dnc
  # substrDBCS renamed as substrDBCS
  #

  # Raw=charToRaw(x)
  # Char=rawToChar(Raw[start,stop])

  if( start > stop ) return( "" )
  unlist( lapply( lapply( lapply(x,charToRaw), "[", start:stop ), rawToChar ) )

 } # substrDBCS

#' Weighted Quantile
#'
#' This program calculates the cdf of x from the frequency distribution
#'  (x, freq) and reverse interpolates it to calculate quantiles.
#'
#' @param x A vector of the random variable
#' @param freq A vector of probability or frequency associated with x
#' @param probs A vector of percents/100
#' @param interpol Name of interpolation function: approx or spline
#' @param ... additional parameters to interpol function.
#'
#' @details
#' When \code{length(unique(x)) <= 10 }, \code{quantile(x,probs=probs)}
#' will be returned.
#' 
#' Quantile and wquantile differs in the way the interpolate (cdf,x). \cr
#' wquantile modifies (cdf,x) to (cdf, xb ) where xb is the mid point of
#' neighboring unique values of x.
#' 
#'
#' @return
#' A vector containing quantile values with name.
#'
#' @examples
#' seed <- 1701
#' set.seed(seed)
#'
#' # original real random variable
#' n <- 100
#' x0 <- 100+10*rnorm(n)
#' # round to integers
#' x <- round(x0)
#' # (xc, f) summarizes the dist of x
#' f <- table(x)
#' xc <- as.numeric(names(f))
#' # expand (xc, f) to (xxf, 1) : These two contains the same info.
#' xxf <- unlist(mapply( rep,xc,f ))
#'
#' # quantiles
#' qtx0 <- quantile(x0)
#' qtx <- quantile(x)
#' qtxxf <- quantile(xxf)
#' qtxc <- quantile(xc)
#' wqtxc <- wquantile( xc, f )
#'
#' # comparison
#' qt <- cbind(qtx0,qtx,qtxxf,qtxc,wqtxc)
#' Print(qt)
#' # difference from the real quantiles
#' dif <- abs(qt-qt[,1])
#' mad <- matrix(colMeans(dif),,1); rownames(mad)=colnames(dif)
#' Print(dif,mad)
#'
#' @export
#'

wquantile <- function( x, freq=1, probs=c(0, .25, .5, .75, 1)
                     , interpol=approx, ... ){
 # weighted quantile
 # Shin-ichi Mayekawa
 # 20150312,17,18, 19 O'Hare, 20
 # wqtl renamed: 20150616
 # wqtl2.R -> wquantile3.R: 20171027
 # use of native quantile when # of obs is small: 20171027
 # midp2brk instead of int option: 20171027
 # use native quantile when freq=1: 20171027
 # when length(unique(freq))==1: 20171110
 # comments: 20230720
 #
 #
 #  Args:
 #   (x, freq)     freq dist of x:  Pr( X=x[i] ) = freq[i]
 #                 x does not have to be uniqe.
 #   prob          vector of probabilities
 #   int           = 1  to add 1/2 to x
 #   interpol      name of the interpolation function: approx or spline
 #
 #  Values:
 #    quantiles    vector of length(probs)
 #
 #  Needs:
 #    interpol, mid2brk
 #

 # unweighted case
 if( length(unique(freq)) == 1 ){
  return( quantile( x=x, probs=probs, na.rm=TRUE ) )
 }


 # unique value of x
 nux=length(unique(x))
 if( nux <= 10 ){
  return( quantile(x, probs=probs, na.rm=TRUE) )
 }


 # normalize freq to p
 p=freq/sum(freq)

 # weighted frequency distribution of (f,x): This takes care of non unique x.
 f=unclass( xtabs( p~x ) )
 attributes(f)=NULL

 # cdf of x
 cdf=cumsum(f)/sum(f)
 xu=sort(unique(x))
 xu=midp2brk(xu)[-1]

 # get quantile by linear interpolation or ( cdf, xu ) to ( orob, quantile )
 # qt=spline( cdf, xu, xout=probs, method="natural" )$y
 # plot( cdf,xu )
 qt=interpol( cdf, xu, xout=probs, ... )$y
 # Print(cdf,xu,probs, qt)
 qt[is.na(qt)]=min(x)
 qt[which(probs == 0)]=min(x)
 qt[which(probs == 1)]=max(x)
 names(qt)=probs

 return( qt )

} # end of wquantile

#' Reverse string
#'
#' @param x Input character vector
#'
#' @return
#' A vector containing the reversed elements.
#'
#' @details
#' revstr is the alias for this function.
#'
#' @examples
#' revchar(c("ncc","1701","a"))
#'
#' @export
#'

revchar <- function( x ){
 # reverse the string
 # from the help of strsplit strReverse
 # Shin-ichi Mayekawa
 #
 # 20150620
 #
 return( sapply(lapply(strsplit(x, NULL), rev), paste, collapse="") )
} # revchar



#' Reverse string
#'
#' @param x Input character vector
#'
#' @return
#' A vector containing the reversed elements.
#'
#' @details
#' revchar is the alias for this function.
#'
#' @examples
#' revstr(c("ncc","1701","a"))
#'
#' @export
#'

revstr <- function( x ){
 # reverse the string
 # from the help of strsplit strReverse
 # Shin-ichi Mayekawa
 #
 # 20150620
 #
 return( sapply(lapply(strsplit(x, NULL), rev), paste, collapse="") )
} # revstr

#' Remove Objects If They Exists in the Parent Frame
#'
#' @param ... The objects to be removed, as names (unquoted)
#'
#' @examples
#' \dontrun{
#' a=1; b=1
#' rm( a, b, c )
#' rmv( a, b, c )
#' print(a)
#' print(b)
#' }
#'
#' @export
#'

rmv <- function( ... ){
 # Remove Objects
 # Shin-ichi Mayekawa
 # 20151008
 #

 # suppressWanings is needed to avoid removing non-removable objects.

 name=as.character(substitute(list(...)))[-1L]
 suppressWarnings( rm( list=name, envir=parent.frame() ) )

} # end of rmv

#' Interprete sas-like abbreviated variable list
#'
#' @param var string containing sas-like abbreviated variable list such as\cr
#'              "q5-q7  x1  height--gender"\cr
#'         Comma can be a denominator.
#' @param varlistf full list of variables on the basis of which double - is
#' interpreted. \cr
#' Use varlistf=colnames(data)\cr
#' This can be NULL when double - is not used.
#' @param location = 1 to output the locatin of var in varlistf \cr
#'  If location == 1, varlistf must be given.
#' @param regexpr a string containing a regular expression to be used
#' to select the variables. \cr
#' This has priority over var.
#'
#' @return vector or list
#' @details
#' varlist( "q2-q5 a c--d", varlistf=c("a", "bb","c","d", "e", "gender") ) \cr
#' will return c( "q2", "q3", "q4", "q5", "a",  "c",  "d" )\cr
#'   When location == 1, in addition to the vector above, \cr
#'    the location of the variable names contained in varlistf, if any,
#'    will be output as the 2nd element of the list, which is\cr
#'    c(1,3,4)
#'    \cr
#'   Expression such as abc: can be used to designate all the variables
#'   whose names start with abc.
#'
#' @examples
#' varlist( "x3-x1, y5, x8-x9" )
#' varlistf=c("a", "b","c","d", "e", paste("Q", 1:5, sep=""), "fff" )
#' varlist( "q2-q5 a e--c", varlistf=varlistf )
#' varlist( "q2-q5 a e--c Q:", varlistf=varlistf )
#' varlist( regexpr="^Q[123]", varlistf=varlistf )
#'
#' @export
#'

varlist <- function( var=NULL, varlistf=NULL, location=0, regexpr=NULL ){
 # interprete sas-like abbreviated variable list
 # Shin-ichi Mayekawa
 # 20140527,28
 # check if var is a string: 20170920
 # root:: 20180419dnc,20
 # regexpr: 20180420
 # && -> &: 20190605,1230cot
 #
 #
 # Args:
 #
 #   var       string containing sas-like abbreviated variable list such as
 #              "q5-q7  x1  height--gender"
 #             comma can be a denominator.
 #   varlistf  full list of variables on the basis of which -- is interpreted
 #              varlistf=colnames(data)
 #
 #   location = 1 to output the locatin of var in varlistf
 #             If location == 1, varlistf must be given.
 #
 # Values:
 #
 #   When location == 0, this function outputs
 #    the expanded variable list as a vector as
 #       c("q5","q6","x1","height",....,"gender")
 #   When location == 1, in addition to the vector above,
 #    the location of each variable in var will be output as the 2nd element
 #    of the list.
 #

 if( !is.null(var) ){
  # is var a string
  if( !( is.vector(var)  &  typeof(var) == "character" ) ){
   cat("\n\nerror1:(varlist) var must be a string of length one.\n")
   stop()
  }
  # error
  if( any(regexpr("--",var) > 0)  && is.null(varlistf) ){
   cat("\n\nerror1:(varlist) \n"
       , " When you specify --, you must specify varlistf=.\n" )
   return(NULL)
  }
 }

 if( !is.null(regexpr)  && is.null(varlistf) ){
  cat("\n\nerror1:(varlist) \n"
      , " When you specify regexpr, you must specify varlistf=.\n" )
  return(NULL)
 }


 # Is regexpr given?
 if( !is.null(regexpr) ){
  # regexpr given

  varlist=varlistf[ grep( regexpr, varlistf ) ]

 }
 else{
  # regexpr is not given.

  vl=unlist( strsplit( var, "[, ]") )
  vl=vl[which(vl != "")]
  lenvl=length(vl)

  # Print(lenvl,vl)

  varlist=NULL
  for( i in 1:lenvl ){

   if( regexpr("--",vl[i]) > 0 ){

    if( regexpr(":",vl[i]) > 0 ){
     cat("\n\nerror1:(varlist)  -- and : cannot be mixed.\n")
     return(NULL)
    }

    #Print("--",i,vl[i],varlistf)
    var=unlist(strsplit(vl[i],"--"))
    if( length(var) < 2 ){
     cat("\n\nerror1:(varlist)   Must have two variable names.\n")
     return(NULL)
    }
    loc=which(varlistf %in% var)
    #Print(loc)
    if( length(loc) < 2 ){
     cat("\n\nerror1:(varlist)   Invalid variable name.\n")
     return(NULL)
    }
    if( var[1] == varlistf[loc[1]] )
     vari=varlistf[loc[1]:loc[2]]
    else
     vari=varlistf[loc[2]:loc[1]]

   }
   else if( regexpr(":",vl[i]) > 0 ){

    if( regexpr("--",vl[i]) > 0 ){
     cat("\n\nerror1:(varlist)  -- and : cannot be mixed.\n")
     return(NULL)
    }

    #Print(":",i,vl[i],varlistf)
    root=unlist(strsplit(vl[i],":"))
    if( length(root) >= 2 | root == "" ){
     cat("\n\nerror1:(varlist)   root: please.\n")
     return(NULL)
    }
    loc=grep(root, varlistf)
    loc=which(regexpr(root,varlistf)>0)
    # Print(root,loc)
    vari=varlistf[loc]

   }
   else if( regexpr("-",vl[i]) > 0 ){

    roots=unlist(strsplit(vl[i],"[[:digit:]-]"))
    roots=roots[which(roots != "")]
    #Print(roots)
    if( is.na(roots[2]) ){
     cat("\n\nerror1:(varlist)  Must have the same alpha roots",
         " on the both sides of --.\n")
     return(NULL)
    }
    if( roots[1] != roots[2] ){
     cat(
      "\n\nerror1:(varlist)"
      , "Must have the same alpha roots on the both sides"
      , " of -.\n")
     return(NULL)
    }
    #Print(i,vl[i],roots)
    num=unlist(strsplit(vl[i],paste("[",roots[1],"-]",collapse="")))
    num=num[which(num!="")]
    #Print(num)
    vari=paste(roots[1],num[1]:num[2],sep="")
    #Print(vari)

   }
   else{
    vari=vl[i]
   }
   varlist=c(varlist,vari)

  } # end of i loop

 }

 # location in varlistf
 if( location > 0 ){
  if( is.null(varlistf) ){
   cat("\n\nerror1:(varlist) When location=1 varlistf must be given.\n^n")
   return(NULL)
  }
  loc=loceq(varlistf,varlist)
  res=list(varlist=varlist, loc=loc)
 }
 else res=varlist

 return( res )

} # end of varlist

#' Mathematica like postfix function application
#' @param exp argument of the function
#' @param func function name
#' @details
#' Equivalent to func( exp ). \cr
#' \cr
#' Implemented as:
#' \code{if( mode(func) == "function" ) return( func( exp ) ) }
#'
#' @examples
#' temp <- demomat(4,3)
#' temp %//% t
#'
#' @export
#'
"%//%" <- function( exp=.Last.value, func ){
 # Postfix Function Application
 # Shin-ichi Mayekawa
 # 20150924
 # moved to lazy.tools: 20171010
 # lv() removed from examples for version 4.2: 20230625
 #
 if( mode(func) == "function" ){
  return( func( exp ) )
 }
 else{
  cat("\n 2nd argument (right arg) of %//% is not a function.\n")
  return( exp )
 }
}

#' trim leading and trailing blanks
#'
#' @param x string vector or matrix
#' @param leading = 1 to remove the leading blanks
#' @param trailing = 1 to remove the trailing blanks
#' @param middle = 1 to replace consecutive blanks by one (compress)
#' @param all = 1 to remove the leading and trailing blanks and compress\cr
#' = 2 to remove all the blanks
#' @param blank = a character to define the blank
#'
#' @return trimed string vector
#'
#' @examples
#' testc="   c a  p r i    ci  ous   "
#' trim( testc )
#' trim( testc, leading=1, trailing=0 )
#' trim( testc, middle=1, trailing=0 )
#' trim( trim( testc, middle=1 ), leading=1 )
#' trim( testc , all=1 ) # same as above.
#' trim( testc, all=2 )
#' A=demomat(3,3,root="a"); A[1,1]="  a1 1"; A[2,2]="   a2 2 "
#' trim(A,all=1)
#' trim(A,all=2)
#'
#' @export
#'

trim <- function( x, leading=FALSE, trailing=TRUE, middle=FALSE, all=FALSE
                 , blank=" " ){
 # trim leading and trailing blanks
 # Shin-ichi Mayekawa
 # 20111127,1202
 # zenkaku: 120323
 # data frame warning: 120323
 # zen -> blank: 20150604dnc
 # matrix input: 20150803
 # names: 20150809cot
 # all=2: 20171010dnc
 # bugfix for middle: 20220816cop
 #
 # Args:
 #  leading   = 1 to remove the leading blanks
 #  trailing  = 1 to remove the trailing blanks
 #  middle    = 1 to replace consecutive blanks by one (compress)
 #   excluding leanding and trailing blanks
 #  all       = 1 to remove the leading and trailing blanks and compress
 #  blank     = a blank character to be used instead of " ".
 #
 #

 if( is.data.frame(x) ){
  cat("warning:(trim)  Use sapply(data.frame, trim, options...).\n")
  return(NULL)
 }


 if( all == 1 ){
  leading=1; trailing=1; middle=1
 }

 ismat=is.matrix(x)
 if( ismat ){
  nr=nrow(x); nc=ncol(x)
 }
 islist=is.list(x)
 len=length(x)

 c1=paste("^",blank,"+",sep="")
 c2=paste(blank,"+$",sep="")
 # c32=paste(blank,"+([^",blank,"]+)",sep="") # excluding tail
 # c33=paste("(.)",blank,"+",sep="") # excluding head
 c3=paste("([^",blank,"]+)",blank,"{2,}([^",blank,"]+)",sep="")
 y=x

 if( all == 2 ) y=gsub(blank, "", y)
 else{
  if( leading == 1 )  y=sub(c1, "", y)
  if( trailing == 1 ) y=sub(c2, "", y)
  if( middle == 1 ){
   y=gsub(c3, paste("\\1",blank,"\\2",sep=""), 
          gsub( c3, paste("\\1",blank,"\\2",sep=""), y ) )
  }
 }

 if( ismat ){
  y=matrix(y,nr,nc)
  dimnames(y)=dimnames(x)
 }
 if( islist ){
  y=as.list(y)
  names(y)=names(x)
 }

 return( y )

} # end of trim

#' Find Functions used in a Function or Functions in a List of Function Names
#'
#' @param func A function object (no quote) or
#' a list/vector of the function object names.
#' @param nonative = 0 to print the R native functions.
#' @param noglobal = 1 not to print the functions defined in the global env.
#' @param sort = 0 not to sort the result
#' @param unlist = 0 to output the result in a list when func is a
#'  list or vector.
#' @param packages A character vector containing the names of the packages.
#' \cr The functions included in those packages will not be printed.
#'
#' @details
#' \cr
#' This function  requires "codetools" package.
#' \cr
#'
#' @examples
#' \dontrun{
#' library(codetools)
#' library(lazy.tools)
#'
#' find_functions( is.varname )
#'
#' res1 <- find_functions( ls("package:lazy.tools") )
#' res2 <- find_functions( ls("package:lazy.tools"), unlist=0 )
#' res3 <- find_functions(
#'          ls("package:lazy.tools"), unlist=0, packages="lazy.tools" )
#' Print(res1,res2,res3)
#'
#' # funclist <- ls("package:lazy.tools")
#' # res <- sort( unique( unlist( lapply( funclist, find_functions ) ) ) )
#' # res <- unique( unlist( lapply(
#'             funclist, find_functions, packages="lazy.tools" )
#'              ) )
#' }
#'
#' @return
#' A character vector containing the function names.
#'
#' @references
#' The answer by kohske (answered Aug 9 '12 at 7:51) at: \cr\cr
#'   http://stackoverflow.com/questions/11872879/
#'    finding-out-which-functions-are-called-within-a-given-function
#'
#' @export
#'

find_functions <- function( func, nonative=1, noglobal=0
                            , sort=1, packages=NULL, unlist=1 ) {
 # Find all the functions called from a function
 # Shin-ichi Mayekawa
 # 20161106c,07
 # list or vector input: 20161119,20,21
 # roxgen2: 20180203
 #
 # This is the modified version
 # of the the code posted by kohske (answered Aug 9 '12 at 7:51) at: \cr\cr
 #  http://stackoverflow.com/questions/11872879/
 #        finding-out-which-functions-are-called-within-a-given-function
 #

 # library(codetools)
 #

 # recursive call
 if( length(func) > 1 ){

  # func is a list or vector
  res=lapply( func, find_functions, unlist=1
      , nonative=nonative, noglobal=noglobal, packages=packages )
  names(res)=func

  # remove charactre(0)
  loc=which( unlist(lapply( res, length )) == 0 |
              unlist( lapply( res, is.null ) ) )
  if( length(loc) > 0 ) res=res[-loc]

  if( length(res) == 0 ) return( NULL )

  # sort and unlist
  if( sort ) res=lapply( res, sort )
  if( unlist ){
   res=unique( unlist( res ) )
   if( sort ) res=sort(res)
  }
  return( res )

 } # end of list/vector input


 # unlist
 if( is.list(func) ) func=unlist(func)

 # check if func is a func.
 if( mode(func) != "function" ){
  # func is not a function
  if( mode(func) %in% c("numeric","logical","list") ){
   # func is not a character
   return( NULL )
  }
  else{
   # func is a character
   if( !exists( func, envir=parent.frame() ) ){
    # no object named func
    return( NULL )
   }
   # an object named func exists
   func=get( func, envir=parent.frame() )
   if( mode(func) != "function" ){
    # object named func is not a function
    return( NULL )
   }
  }
 } # end of mode(func) != "function"

 # Here, func contains the body of a function


 ##### coded by kohske starts here. #####
 leaf <- function (e, w) {
  r <- try(eval(e), silent = TRUE)
  if(!is.null(r) && is.function(r)) ret <<- c(ret, as.character(e))
 }
 call <- function (e, w) {
  walkCode(e[[1]], w)
  for (a in as.list(e[-1])) if (!missing(a)) walkCode(a, w)
 }
 ret <- c()
 walkCode(body(func), makeCodeWalker(call = call, leaf = leaf, write = cat))
 res=unique(ret)
 ##### coded by kohske ends here. #####


 # locations (packages) to exclude
 exclude_loc_list=NULL
 if( nonative ){
  exclude_loc_list=c( "package:base",  "package:stats", "package:graphics"
              , "package:grDevices", "package:utils", "package:datasets"
              , "package:methods" )
 }
 if( noglobal == 1 ) exclude_loc_list=c(exclude_loc_list,".GlobalEnv")
 if( !is.null(packages) ){
  exclude_loc_list=c(exclude_loc_list, paste("package:",packages,sep=""))
 }

 # list of functions to be excluded defined in exclude_loc_list
 exclude_func_list=lapply( exclude_loc_list, ls )
 names(exclude_func_list)=exclude_loc_list
 exclude_func_list=unlist(exclude_func_list)

 # exclude
 res=setdiff( res, exclude_func_list )

 # null?
 if( length( res) == 0 ) return( NULL )

 # sort
 if( sort ) res=sort(res)
 names(res)=NULL

 return( res )

} # end of find_functions

#' Principal Component Analysis of Y with possible missing elements
#'
#' @param Y matrix of n x p
#' @param ndim # of dimensions: Must be less than or equal to p.
#' @param estmu = 0 not to estimate mu  (No standardization of Y)
#' @param maxiter Maximum # of iterations for missing elements
#' @param eps criterion for convergence
#' @param init = 0 to use random number as the first estimate of missing\cr
#' = 1 to use the gramd mean \cr
#' = 2 to use the column mean
#' @param print = 1 to print the result
#' @details This function minimizes the following RSS \cr
#'      RSS = tr( ( Y - 1 mu' - F A')'( Y - 1 mu' - F A') ) \cr
#' where mu is the p x 1 vector, \cr
#' F is the n x ndim orthonormal score matrix, \cr
#' and A is the p x ndim matrix of loadings.
#' @return A list of:\cr
#' Yhat=1mu'-FA', F, A, mu, iter, eps, rmse=sqrt(RSS)
#' @examples
#' n <- 20; np <- 5;  pmiss <- 0.1
#' Y <- matrix( rnorm(n*np),n )
#' Y[which(matrix(runif(n*np),n,np) < pmiss)]=NA
#' princ(Y,2, print=10, estmu=1, init=1)
#' @export
#'

princ <- function( Y, ndim=1, estmu=1, maxiter=1000, eps=1e-8, init=1
                   , print=1 ){
 # pca with missing data
 # Shin-ichi Mayekawa
 # 120227,29,30
 # renamed from pcam to princ: 120302
 # return rmse: 120926
 # Print format: 130126
 # init: 130127
 # error check: 20150605
 # ndim==np: 20161003
 #
 # This program minimizes
 #  RSS = tr( ( Y - 1 mu' - F A')'( Y - 1 mu' - F A') )
 # with respect to mu, F and A and missing elements of Y.
 # F will be column centerd and F'F=n I.
 # A will satisry A'A =  diag .
 #
 #
 # Args:
 #   Y       n x np matrix
 #   ndim    # of dimensions. i.e., # of columns of F and A
 #   estmu   = 1 to estimate mu (np x 1)
 #   init    initial value of the missing elements of Y
 #           = 0 to use random numbers
 #           = 1 to use the grand mean
 #           = 2 to use the column means
 #
 #

 # error check
 if( !is.matrix(Y) ){
  cat("\n\nerror1:(princ) Input must be a matrix.\n\n")
  return()
 }

 # constants
 n=nrow(Y); np=ncol(Y)
 sqrtn=sqrt(n)
 one=matrix(1,n,1)

 if( np < ndim ){
  cat("\n\nerror1:(princ) ndim must be less than or equal to ncol(Y).\n\n")
  return()
 }

 # location of missing values
 locmiss=is.na(Y)
 pmiss=sum(locmiss)/(n*np)

 if( init == 1 ){
  # missing values replaced by grand mean
  Y[locmiss]=mean(Y[!locmiss])
 }
 else if( init == 2 ){
  for( j in 1:np ){
   Y[is.na(Y[,j]),j]=mean(Y[,j],na.rm=1)
  }
 }
 else{
  Y[locmiss]=rnorm(sum(locmiss))
 }


 rmsep=999999
 conv=0
 mu=matrix(0,np)

 for( llll in 1:maxiter ){

  if( estmu == 1 ){
   # estimate mu
   mu=matrix(colMeans(Y),np,1)
  }

  # column centered Y
  Z=Y-one%*%t(mu)

  # svd
  UDV=svd(Z,ndim,ndim)

  # rescale to get F and A
  F=matrix(UDV$u,n)*sqrtn
  A=matrix(UDV$v,np)/sqrtn
  if( ndim == 1 ) D=UDV$d[1]
  else D=diag(UDV$d[1:ndim])
  A=A%*%D

  # model value and rss
  Yhat=F%*%t(A)+one%*%t(mu)
  rmse=sqrt( sum((Y-Yhat)^2)/(n*np) )
  rmseimp=rmsep-rmse

  if( print >= 3 )
   Print(llll, rmsep, rmse, rmseimp, fmt="i3 f10.7")
  if( rmseimp <= eps ){
   conv=1
   break
  }

  # next iteration
  rmsep=rmse

  # replace the missing Y by model value
  Y[locmiss]=Yhat[locmiss]

 } # of llll loop

 if( conv != 1 & print >= 0 ){
  cat("\n\nwarning:(princ) Iteration did not converge.\n\n")
 }


 if( print > 0 ){
  cat("\n\nPrincipal Component Analysis\n\n")
  cat(" # of rows =", n, ", # of columns =", np
      , ",  # of dimensions =", ndim,"\n")
  cat(" estimation of mu =", estmu, "\n")
  cat("  percentage of missing elements in Y =", 100*pmiss, "%\n")
  cat(" method for the initial values =", init,"\n")
  cat(" max # of iterations =", maxiter, "  with eps =",eps,"\n")
  cat(" iterations required to converge =", llll, " with rmse =", rmse, "\n")
  Print( mu, F, A, fmt="8.4" )
 }
 if( print >= 2 ){
  Print(t(F)%*%F, t(A)%*%A, fmt="8.4")
 }

 return( list(Yhat=Yhat, F=F,A=A,mu=mu, iter=llll, eps=eps, rmse=rmse) )

} # end of princ

#' Generation of character infix operators: `&`, `&&`, `|`, `||`
#'
#' @details
#' This function generates infix operators: `&`, `&&`, `|`, `||`
#' in the global environment, \cr
#' which are almost equivalent to, resp, `\%&\%`, `\%&&\%`, `\%|\%`, `\%||\%` .
#' \cr
#' The difference is that `&` has precedence over `|`  but `\%&\%` does NOT.
#' \cr For example: \cr\cr
#' \code{"a" | "b" & "c"} produces \code{"(a) | ((b) & (c))"} \cr
#' but \cr
#' \code{"a" \%|\% "b" \%&\% "c"} produces \code{"((a) | (b)) & (c)"} .
#' \cr\cr
#' See the examples below.
#' \cr\cr
#' \code{ "a" & "b" } is equivalent to  \code{And( "a", "b" )}
#' and generates \code{"(a) & (b)"}.\cr
#' \code{"a" && "b"} is equivalent to  \code{And( "a", "b", oper="&&" )}
#' and generates \code{"(a) && (b)"}.\cr
#' \code{ "a" | "b" } is equivalent to \code{Or( "a", "b" )}
#' and generates \code{"(a) | (b)"}.\cr
#' \code{ "a" || "b"} is equivalent to  \code{Or( "a", "b", oper="||" )}
#' and generates \code{"(a) || (b)"}.\cr
#' \cr
#' Use \code{eval( parse(text=condition) )} to evaluate the expression.
#' \cr
#' \cr
#' To remove these operators: \cr
#'  \code{ rm(  `&`, `&&`, `|`, `||` ) }
#'
#' @return
#' Generates four infix operators in the global environment.
#'
#' @examples
#' generate_AndOr()
#'
#' "a" & "b"
#' "a" %&% "b"
#' And( "a", "b" )
#' "a" | "b"
#' "a" %|% "b"
#' Or( "a", "b" )
#'
#' # precedences
#' "a" | "b" & "c"
#' "a" %|% "b" %&% "c"
#'
#' "a" & "b" | "c"
#' "a" %&% "b" %|% "c"
#'
#'
#' @export
#'

generate_AndOr <- function(){
 # generate infix operators: `%&%`, `%&&%`, `%|%`, `%||%`
 # Shin-ichi Mayekawa
 # 20171006,07
 # roxygen2 error fixed: 20191129dc
 #

 percent=0

 if( percent == 1 ){

  `%&%` <<- function( A, B=NULL ){
   # & operator
   # Shin-ichi Mayekawa
   # 20171006

   if( is.character(A) | is.character(B) ){
    if( is.null(B) ) return( And(TRUE,A))
    else return( And(A,B) )
   }
   else{
    if( is.null(B) )  return(  .Primitive("&")(TRUE,A) )
    else return(  .Primitive("&")(A,B) )
   }
  } # end of &

  `%|%` <<- function( A, B=NULL ){
   # | operator
   # Shin-ichi Mayekawa
   # 20171006

   if( is.character(A) | is.character(B) ){
    if( is.null(B) ) return( Or(TRUE,A))
    else return( Or(A,B) )
   }
   else{
    if( is.null(B) )  return(  .Primitive("|")(FALSE,A) )
    else return(  .Primitive("|")(A,B) )
   }
  } # end of |

  `%&&%` <<- function( A, B=NULL ){
   # & operator
   # Shin-ichi Mayekawa
   # 20171006

   if( is.character(A) | is.character(B) ){
    if( is.null(B) ) return( And(TRUE,A, oper=" && "))
    else return( And(A,B, oper=" && ") )
   }
   else{
    if( is.null(B) )  return(  .Primitive("&")(TRUE,A) )
    else return(  .Primitive("&")(A,B) )
   }
  } # end of &&

  `%||%` <<- function( A, B=NULL ){
   # | operator
   # Shin-ichi Mayekawa
   # 20171006

   if( is.character(A) | is.character(B) ){
    if( is.null(B) ) return( Or(TRUE,A, oper=" || "))
    else return( Or(A,B, oper=" || ") )
   }
   else{
    if( is.null(B) )  return(  .Primitive("|")(FALSE,A) )
    else return(  .Primitive("|")(A,B) )
   }
  } # end of ||

 }
 else{

  .GlobalEnv$`&` <- function( A, B=NULL ){
   # & operator
   # Shin-ichi Mayekawa
   # 20171006

   if( .Primitive("|")(is.character(A), is.character(B)) ){
    if( is.null(B) ) return( And(TRUE,A))
    else return( And(A,B) )
   }
   else{
    if( is.null(B) )  return(  .Primitive("&")(TRUE,A) )
    else return(  .Primitive("&")(A,B) )
   }
  } # end of &

  .GlobalEnv$`|` <- function( A, B=NULL ){
   # | operator
   # Shin-ichi Mayekawa
   # 20171006

   if( .Primitive("|")(is.character(A), is.character(B)) ){
    if( is.null(B) ) return( Or(TRUE,A))
    else return( Or(A,B) )
   }
   else{
    if( is.null(B) )  return(  .Primitive("|")(FALSE,A) )
    else return(  .Primitive("|")(A,B) )
   }
  } # end of |

  .GlobalEnv$`&&` <- function( A, B=NULL ){
   # & operator
   # Shin-ichi Mayekawa
   # 20171006

   if( .Primitive("|")(is.character(A), is.character(B)) ){
    if( is.null(B) ) return( And(TRUE,A, oper=" && "))
    else return( And(A,B, oper=" && ") )
   }
   else{
    if( is.null(B) )  return(  .Primitive("&")(TRUE,A) )
    else return(  .Primitive("&")(A,B) )
   }
  } # end of &&

  .GlobalEnv$`||` <- function( A, B=NULL ){
   # | operator
   # Shin-ichi Mayekawa
   # 20171006

   if( .Primitive("|")(is.character(A), is.character(B)) ){
    if( is.null(B) ) return( Or(TRUE,A, oper=" || "))
    else return( Or(A,B, oper=" || ") )
   }
   else{
    if( is.null(B) )  return(  .Primitive("|")(FALSE,A) )
    else return(  .Primitive("|")(A,B) )
   }
  } # end of ||

  cat( "\n\n ** String Infix Operators ( `&`, `&&`, `|`, `||`  ) are"
      , " now available in the global environment. \n" )
  cat( "    To remove these operators, submit the following code:"
      , "  rm(  `&`, `&&`, `|`, `||` ) .     **\n\n" )

 }


} # end of generate_AndOr

#' Remove Brackets and Double \\s from R type Regular Expression
#'
#' @param pattern R type regular expression
#' @param expr A string vector containing expressions, \cr
#'  or a string which contains several expressions separated by a comma, \cr
#'  or "".
#'
#' @details
#'  A pattern w/o brackets will be printed which can be pasted to \cr
#'  \code{https://jex.im/regulex/#!flags=&re=\%5E(a\%7Cb)*\%3F\%24}.
#'
#' @return
#'  NONE
#'
#' @examples
#' testregex( "^[[:alpha:].][[:alnum:]_.]*$" )
#' testregex( "^[[:alpha:].][[:alnum:]_.]*$"
#'         , c( "a1", "b", ".c1.", "@d_", "1_2", "._" ) )
#' testregex( "^((([[:alpha:]]|[.][._[:alpha:]])[._[:alnum:]]*)|[.])$"
#'         , "a1, b, .c1., @d_, 1_2, ._" )
#'
#'
#' @export
#'

testregex <- function( pattern, expr="" ){
  # testing regular expression
  # Shin-ichi Mayekawa
  # 20200616dnc,17
  # 20230430cot
  # Old testregexpr (trx) in lazy.symbolic renamed: 20230509dnc
  # \ added before % in url: 20250305
  #
  # args:
  #  pattern regular expression in R format
  #  expr expressions to be evaluated separated by commas and blanks
  #
  # values
  #  none
  #
  # details
  #  regular expression without [[ and \\ will be printed as pat1
  #  which can be pasted to
  #  \code{https://jex.im/regulex/#!flags=&re=%5E(a%7Cb)*%3F%24}.
  #

 # remove [ and ]
 pat1=gsub("\\\\", "\\", pattern, fixed=1)
 pat1=gsub("[:", ":", pat1, fixed=1)
 pat1=gsub(":]", ":", pat1, fixed=1)
 Print(pat1)

 if( all(expr == "") ) return( invisible() )

 # split expr by comman or blank
 if( length(expr) == 1 ){
  expr=unlist(strsplit(expr,"[ ,]"))
  expr=expr[expr!=""]
 }

 res=regexpr(pattern, expr)

 cat("\npattern\n")
 cat(pattern, "\n")
 cat("\n Result of regexpr: expr starts from column 6.\n")
 resf=formatC(res,width=4,format="d")
 res1=paste(resf, expr, sep=" ")
 cat(res1, sep="\n")

} # end of testregex

#' Interpolation of data
#'
#' @param x input vector
#' @param y input vector
#' @param xout vector at which y is calculated,  or NULL.
#' @param method = "constant" or "linear" or "natural" \cr
#' "spline" can be uses as an alias for "natural".
#' @param n # of points in (min(x), max(x)) at which y is calculated
#' when xout is NULL.
#' @param print = 1 to print the result
#' @param plot = 1 to plot the result
#' @return A matrix: cbind( xout, yout )
#'
#' @examples
#' x <- c(1,2,3,4,5,6)
#' y <- c(1,4,9,7,6,8)
#' xout <- seq(1, 7, length=10)
#' sp=interpol( x, y, xout, method="constant", plot=1 )
#' sp=interpol( x, y, xout, method="linear", plot=1 )
#' sp=interpol( x, y, xout, method="spline", plot=1 )

#' @export
#'

interpol <- function( x, y, xout=NULL, method="natural", n=3*length(x)
                    , print=0, plot=0 ){
 # interpolation by approx or spline
 # Shin-ichi Mayekawa
 # 20141216
 # error check: 20150605
 # rule option added to approx: 20160727
 #
 # Args:
 #
 #  x, y    vectors giving the coordinates of the points to be interpolated.
 #  xout    vector of new x values  where interpolation is to take place.
 #
 #  n       If xout is NULL, interpolation takes place at
 #          n equally spaced points spanning the interval [min(x), max(x)].
 #          That is, xout=seq(min(x),max(x),length=n) will be used.
 #
 #  plot    = 1 to plot the result
 #
 #
 # Values:
 #
 #  matrix of cbind( xout, yout )
 #

 # error
 if( !( is.vector(x) & is.vector(y) ) ){
  if( !( ncol(x) == 1  &  ncol(y) == 1 ) ){
   cat("\n\nerror1:(interpol) Input must be vectors.\n\n")
   return()
  }
  else{
   x=c(x); y=c(y)
  }
 }

 if( is.null(xout) ) xout=seq(min(x),max(x),length=n)
 if( method == "spline" ) method="natural"
 if( method %in% c("linear","constant") ){
  res=approx( x, y, xout=xout, method=method, rule=2 )
 }
 else{
  res=spline( x, y, xout=xout, method=method )
 }

 if( plot == 1 ){
  plot(x,y,type="p", main=method)
  if( method == "linear" )
   lines(x,y)
  else if( method == "constant" )
   lines( approx( x, y, n=100, method=method ) )
  else
   lines( spline( x, y, n=30, method=method ) )
  points( res, col=2 )
 }

 res=cbind(res$x, res$y)
 colnames(res)=c("x","y")
 if( print == 1 )
  Print(res)

 return( res )

} # end of interpol

#' Calculates mean, std, variance and n with case weight
#'
#' @param data data vector, matrix or data frame.
#' @param weight case weight vector of length nrow(data) or 1.
#' @param probs vector of probabilities to calculate quantiles
#' @param sandk = 1 to calculate skewness and kurtosis
#' @param by a factor or a list of factors, each of length nrow(data) \cr
#' to be used as the INDICES of by function.
#' @param byvarname vector containing the names of by varialbes to be used.
#'  (Not Yet Implemented.)
#' @param vardef = "n-1" to use n-1 as denominator for variance.
#' @param print = 1 to print the result
#' @param simplify = 0 to return class by object when by= is given.
#'
#' @details
#' The means and vars will be calculated using native weighted.mean function.
#' \cr
#' The kurtosis = weigted.mean( scale(data, scale=FALSE)^4 ) / std^4 - 3
#' The quantiles will be calculated using wquantile function. \cr
#'
#'
#' @return A matrix of statistics(mean, std, var, nobs, n)  x  variables \cr
#' where nobs is sum(weights) for nonmissing obs, and n is nrow(data). \cr
#' In addition, if \code{probs} is given, quantiles will be appended to
#' the above. \cr
#' \cr
#' When by is present, this function returns a class by object
#' consisting of above for each subgroup if simplify=0. \cr
#' If by is not a list and simplify=1,
#' the result is a statistics x variables x subgroups array. \cr
#' If by is a list of several factors, simplify=1 does not affet the result.
#' \cr
#' When X is univariate, the result is a statistics x subgroup matrix.
#' \cr
#' \cr
#' When data is NULL, NULL will be returned.
#'
#' @examples
#' # univariate example
#' y <- 1:9
#' pdf <- exp( -0.5*(y-5)^2/(1.3^2) )
#' pdf <- pdf/sum(pdf)
#' group <- c(1,1,1,1,1,2,2,2,2)
#' mands( y )
#' mands( y, pdf, sandk=1 )
#' mands( y, by=group, sandk=1 )
#'
#' # bivariate example
#' ndims <- 2; npoints <- 11; xmin <- -3; xmax <- 3; corrmat <- 0.2
#' nobs <- 100000
#'
#' # dispersion matrix
#' Dmat <- matrix(corrmat,ndims,ndims)+(1-corrmat)*diag(ndims)
#' Dmat[1,1] <- .5
#'
#' # grid X
#' X <- seq(xmin,xmax,length.out=npoints)
#' if( ndims >= 2 ){
#'  for( i in 2:ndims )
#'   X <- cprod(X,seq(xmin,xmax,length.out=npoints))
#' }
#'
#' # weight
#' pdf <- exp( -0.5*diag( X%*%solve(Dmat)%*%t(X)) )
#' pdf <- pdf/sum(pdf)
#' freq <- round(nobs*pdf+.5)
#'
#' # all data
#' mands( X )
#' mands( X, freq )
#' mands( X, freq, probs=c(0.25,0.5,0.75,0.975) )
#'
#' # grouping variable
#' set.seed(1701)
#' if( is.matrix(X) ) n <- nrow(X) else n <- length(X)
#' byvar <- sample(1:3, n, replace=1)
#' byvar <- sort(byvar)
#'
#' # for each subgroup
#' mands( X, by=byvar )
#' mands( X, freq, by=byvar )
#' mands( X, freq, by=byvar, probs=c(0.25,0.75) )
#'
#' @export
#'

mands <- function( data, weight=1, sandk=0, probs=NULL
                   , by=NULL, byvarname=NULL
                   , vardef="n", print=0, simplify=1 ){
  # weighted statistics: mean, std, variance and n
  # Shin-ichi Mayekawa
  # 20140725 at Madison
  # n added: 20150605
  # no covariance version named mands: 20170906dnc
  # by: 20170919dnc
  # simplify: 20170919,20
  # byvarname: 20170920
  # is.null(data): 20171004
  # var name when x is a vector: 20171025
  # probs: 20171027,29
  # bugfix: 20171110
  # nobs=sum(weight[nomiss]): 20180227dnc,28
  # NaN -> NA: 20190606dnc
  # nrow(data) == 1: 20200830cot
  # all NA: 20210208,10
  # sandk: 20220303
  # bugfix for one-row data: 20230117dnc
  # 
  #
  # Args:
  #   data     data frame or matrix or vector
  #   weight   weight vector or 1
  #   vardef   "n" or "n-1"
  #
  #
  # Value:
  #   A matrix of mean, std, var, nobs, n
  #  where
  #   nobs is the sum of weights
  #

  # error
  if( is.null(data) ){
    return( NULL )
  }

  # # of by variables
  nby=0
  if( is.null(by) ) nby=0
  else if( is.list(by) ) nby=length(by)
  else if( is.vector(by) ) nby=1
  else if( is.matrix(by) && ncol(by) == 1 ){
    nby=1
    by=c(by)
  }
  else if( is.matrix(by) && ncol(by) == 1 ){
    nby=1
    by=as.vector(by)
  }
  else{
    cat("\nerror1:(mands) by must be a vector or list of variables.\n")
    return(NULL)
  }

  # make data a data frame so that, when by is cbinded, the name "by"
  # is used instead of "INDICES".
  if( is.vector(data) ){
    vname=as.character(substitute(data))
    if( all(is.na(vname)) ) vname="V1"
    if( length(vname) >= 2 ) vname=vname[2]   # mands(c(1,2))  20210208
    data=matrix(data,length(data),1)
    if( all(regexpr("^[[:alpha:]][[:alnum:]]*$",vname) > 0 ) )
      colnames(data)=vname
  }

  if( is.matrix(data) ) data=as.data.frame(data, stringsAsFactors=0 )
  vname=colnames(data)
  n=nrow(data)
  if( length(weight) == 1 ) weight=rep(1,n)
  

  # numeric only
  # Note that all NA variable will be classified as logical!! 20210208
  locallna=which(apply(data,2,function(x) all(is.na(x))))
  locnonnum=which(vartype(data) != "numeric")
  # locdel=union(locallna,locnonnum)
  # if( length(locdel) > 0 ) data=data[,-locdel, drop=FALSE]
  if( length(locnonnum) > 0 ) data=data[,-locnonnum, drop=FALSE]
  nvar=ncol(data)
# Print(locnonnum,nvar,data, is.na(X))
  if( (n==1 && all(is.na(data))) || nvar <= 0 ){
    cat("\nerror1:(mands) Must have numeric variables.\n")
    res=matrix(NA,5,ncol(data))
    rownames(res)=c("maen","std","var","nobs","n")
    res[4,]=0; res[5,]=nrow(data)
    return(res)
  }



  ##################################
  # analysis of byvar
  #
  # See matReplace.R in lazy.symbolic.
  # See also test_parent.R in lay.tools directory
  #####################################


  ############## recursive call when by is present #############
  if( !is.null(by) ){

    res=by( cbind(data, weight), by, FUN=function(x){
      mands( x[,-ncol(x)],  weight=x[,ncol(x)], by=NULL, probs=probs
             , sandk=sandk, vardef=vardef, print=print-8 )
    } )

    if( nby == 1 ){
      if( simplify ){
        names=dimnames(res[[1]])
        # names[[3]]=paste(1:length(res),sep="")
        names[[3]]=names(res)
        res=array( unlist(res), dim = c(dim(res[[1]]), length(res)) )
        dimnames(res)=names
        if( ncol(data) == 1 ){
          names=dimnames(res)
          dim(res)=dim(res)[c(1,3)]
          dimnames(res)=names[c(1,3)]
        }
      }
    }

    if( print ) print(res)

    return( res )

  }



  ############# simple call when by is  not present ############


  if( tolower(vardef) == "n" ) method="ML"
  else method="unbiased"

  # use native weighted.mean function
  nvar=ncol(data)
  vname=colnames(data)
  locobs=!apply(data,2,is.na)  # result is the same size as data 20210208
# Print(data,locobs)
  if( nrow(data) == 1 ){       # 20200830
    locobs=!is.na(data)
  }
  nobs=numeric(nvar)
  for( j in 1:nvar ){
    nobs[j]=sum(weight[locobs[,j]])
  }

  mean=apply( data, 2, weighted.mean, w=weight, na.rm=1 )
  mean2=apply( data^2, 2, weighted.mean, w=weight, na.rm=1 )
  var=mean2-mean^2
  if( vardef != "n" ){
    var=(nobs/(nobs-1))*var
  }
  std=sqrt(var)

  # NaN to NA
  mean[is.nan(mean)]=NA
  std[is.nan(std)]=NA
  var[is.nan(var)]=NA


  stat=rbind(mean,std,var,nobs,n)
  rownames(stat)=c("mean","std","var","nobs","n")
  colnames(stat)=vname

  if( !is.null(probs) ){
    qt=apply( data, 2, wquantile, weight, probs )
    qt=matrix(qt,,nvar)
    rownames(qt)=
      paste("Q",formatC(100*probs, width=3,digits=1,format="f",flag="0")
            ,sep="")
    stat=rbind(stat,qt)
  }


  if( sandk == 1 ){
    skew=apply(scale(data,scale=FALSE)^3, 2
               ,  weighted.mean, w=weight, na.rm=1 ) / std^3
    kurt=apply(scale(data,scale=FALSE)^4, 2
               ,  weighted.mean, w=weight, na.rm=1 ) / std^4 - 3
    skew[is.nan(skew)]=NA
    kurt[is.nan(kurt)]=NA
    sk=rbind(skew,kurt)
    rownames(sk)=c("skew","kurt3")
    stat=rbind(stat,sk)
  }

  if( print> 0 ) Print( stat )

  return( stat )

} # end of mands

#' Remove Redundant Parentheses
#' (very very primitive version)
#'
#' @param exp expression vector or matrix
#' @param method 1, 2 or 3 \cr
#' If exp contains any logical operator, method=1 will be used.\cr
#' @param loc the location of the paren to be removed. \cr
#' Use with caution when the paren is preceded by - sign,
#' or exp is a vector or matrix.
#' @param debug = 1 to print intermediate result.
#'
#' @details
#'
#' When loc > 0 is given, those parentheses located at the loc-th position
#' of the elements of exp will be removed.\cr
#' No signs will be distributed: \cr
#' \code{remove_paren("-(a+b)",loc=2)} will result in \code{-a+b}.
#' \cr
#'
#' First,  (+....)  will be replaced by  (...) .\cr
#' When method = 1, a pair of parentheses which encloses a single term
#' w/o divisions will be removed. \cr
#' When method = 2, a pair of parentheses which is not preceded by [*/-]
#' and not trailed by [*/] will be removed. \cr
#' When method = 3, patterns like  -(-a) or -(-(a)) will be processed.
#' \cr\cr
#' Use Expand to distribute multiplicative factors.
#'
#' \cr\cr
#' \code{method=2} may return wrong result: \cr
#' \code{remove_paren("-((a+b))", method=2)} becomes \code{-a+b}.
#' \cr
#' \cr
#'
#' @return
#' A vector or matrix of the same size as the input
#'
#' @examples
#' remove_paren( c("(x)", "-(x)", "(x+y)", "-(x+y)") )
#' remove_paren( c("a*(x)*b", "a*(x+y)*b", "(-x)*a", "-(x+y)*a") )
#' remove_paren( c("a%*%(x)%*%b", "a%*%(x+y)%*%b", "(-x)%*%a", "-(x+y)%*%a") )
#' remove_paren( c("a%@%(x)%*%b", "a%@%(x+y)%@%b", "(-x)%@%a", "-(x+y)%@%a") )
#'
#' exp <- "((a)) - ((a)+f(b)) + (a-b) - (x-gg(y)) + (((z+1)))"
#' remove_paren( exp )
#'
#' exp2 <- c(exp, "(a+b)*(c)-d+1")
#' remove_paren( exp2 )
#'
#' ( temp <- And(Or("a==A","b>0"),"c") )
#' remove_paren(temp)
#'
#' remove_paren("!(!(a))")
#' remove_paren("!(!(a))",loc=4)
#' remove_paren("!( !(a) )",loc=9)
#'
#' @export
#'

remove_paren <- function( exp, method=1:9, loc=0, debug=0 ){
 # remove redundant parentheses
 # Shin-ichi Mayekawa
 # 20150823,24
 # (+....) -> (...):  20150905
 # bugfix: 20150906,07
 # regexpr()[[1]]: 20150907
 # 1/(a): 20150907
 # parens preceded by alpha: 20150913
 # %*%: 20150915
 # special operator: 20150922
 # (Use remove_paren6.R if this version does not work.)
 #
 # moved from symbolic to tools: 20171007
 # remove double matching paren: 20171008
 # new method 1 with  &|!: 20171008
 # loc: 20171010dnc
 # roxgen2 wrong result: 20171024dnc
 # bug fix for special ops;  ` added: 20200520
 # lv() removed from examples for version 4.2: 20230625
 #

 # mark
 MARK=";"

 # is exp a scalar?
 if( is.vector(exp) ) lenexp=length(exp)
 else lenexp=nrow(exp)*ncol(exp)

 # if not scalar, use recursive call
 if( lenexp > 1  |  is.matrix(exp) ){
  exp[]=unlist(lapply( exp, remove_paren, method=method, loc=loc, debug=debug ))
  return( exp )
 }


 ############# main starts here. #####################################


 #loc
 if( loc >= 1  &&  substr(exp,loc,loc) %in% c("(",")") ){
  loc2=locbalpar(exp,loc)
  if( loc2 > 0 ){
   substr(exp,loc,loc)=MARK
   substr(exp,loc2,loc2)=MARK
   exp=gsub( MARK,"", exp )
   exp=gsub( " ", "", exp )
   exp=gsub( "+-", "-", exp, fixed=1 )
   exp=gsub( "++", "+", exp, fixed=1 )
   exp=gsub("!#", "#", exp, fixed=1)
   exp=gsub("!!", "", exp, fixed=1) ########### 20171010dnc
   return( exp )
  }
 }


 # compress exp
 exp00=exp
 exp=gsub( " ", "", exp )


 # remove leading plus sign:   (+....) -> (...)  20150905
 # must escape ( and +
 exp=gsub( "\\(\\+", "\\(", exp )

 # find the location of left parens
 temp=unlist(strsplit(exp,""))
 loc1=which( unlist( gregexpr( "\\(", temp ) ) == 1 )

 np=length(loc1)

 if( debug ) Print(exp, np,loc1)

 # Is there any paren?
 if( np == 0 ) return(exp00)


 # Omit those parens preceded by alnum, i.e.,  function calls
 rm=NULL
 for( i in 1:np ){
  if( loc1[i] == 1 ) next
  if( regexpr( "[[:alnum:]_.]", temp[loc1[i]-1] )[[1]] > 0 )
   rm=c(rm,i)
 }
 if( length(rm) > 0 ) loc1=loc1[-rm]
 np=length(loc1)
 if( np == 0 ) return(exp00)


 # find the matching parens
 # loc2=loc1
 # for( i in 1:np ){
 # loc2[i]=locbalpar( exp, loc1[i] )
 # }
 loc2=unlist(lapply( loc1, function(x) locbalpar( exp, x ) ))

 # range=mapply( ":", loc1, loc2 )
 range=cbind(loc1,loc2)
 if( debug ) Print(temp,range, printrownames=1)


 # if exp contains logical operators, use method 1 only.
 if( regexpr("[&|!]", exp) > 0 ) method=1


 # fixed point iteration
 exp0=exp
 for( llll in 1:np ){


  ################################
  # remove matching double parens
  removed=numeric(0)
  for( i in 1:np ){
   from=range[i,1]; to=range[i,2]
   f=substr(exp,from,to)
   if( substr(exp,from+1,from+1) == "(" ){
    #Print(i, f, from, to)
    #Print(locbalpar(exp,from+1))
    #Print(locbalpar(exp,locbalpar(exp,from+1)))
    if( locbalpar(exp,from+1) == to-1 ){
     if( debug ) Print("double paren removed.")
     substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
     removed=c(removed,i)
    }
   }
   # Print("end of", i)
  } # end of method == 0


  ################################
  if( 1 %in% method   &  np > 0 ){

   # remove (single term)
   if( debug ) cat("llll=",llll,"M1 Removing (single term)...\n")
   removed=numeric(0)
   for( i in 1:np ){

    from=range[i,1]; to=range[i,2]
    f=substr(exp,from,to)
    if( debug ) Print(i,f, from, to)

    if( from == 1){
     # no preceding anything
     if( regexpr( "[-+|=><]", f )[[1]] == -1 ){
      # inside is a factor
      if( debug ) cat("M11\n")
      substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
      removed=c(removed,i)
     }
    } # from==1
    else{
     # check if ( is preceeded by / or !
     cond1=regexpr("[/!%]",substr(exp,from-1,from-1))[[1]]
     if( debug ) Print(cond1,substr(exp,from-1,from-1))

     if( cond1 == -1 ){
      if( regexpr( "[-+|=><]", f )[[1]] == -1 ){
       # inside is a factor
       if( debug ) cat("M12\n")
       substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
       removed=c(removed,i)
      }
     }
     else if( regexpr("/",substr(exp,from-1,from-1))[[1]] > -1 ){
      # .../()
      if( regexpr( "[-+/*]", f )[[1]] == -1 ){
       if( debug ) cat("M13\n")
       # inside is a factor and does not contain /
       substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
       removed=c(removed,i)
      }
     }
     else if( regexpr("!",substr(exp,from-1,from-1))[[1]] > -1 ){
      # ...!()
      if( regexpr( "[-+*&|]", f )[[1]] == -1 ){
       # inside is a single element
       if( debug ) cat("M14\n")
       substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
       removed=c(removed,i)
      }
     }
     else if( regexpr("%",substr(exp,from-1,from-1))[[1]] > -1 ){
      # ...%()
      if( regexpr( "[-+*/%&|]", f )[[1]] == -1 ){
       # inside is a single element
       if( debug ) cat("M15\n")
       substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
       removed=c(removed,i)
      }

     }


    } # from>1

   } # end of i loop for method 1
   if( debug ) Print(removed, exp)
   # update range
   if( length(removed) > 0 ) range=range[-removed,,drop=0]
   np=nrow(range)
   if( debug ) Print(range)

  } # end of method 1


  #################################
  if( 2 %in% method   &  np > 0  ){

   # remove (terms) w/o preceding * or - and trailing *
   if( debug ) cat("llll=",llll
       ,"M2 Removing (terms) w/o preceding * or - and trailing *...\n")
   removed=numeric(0)
   for( i in 1:np ){

    from=range[i,1]; to=range[i,2]
    f=substr(exp,from,to)
    if( debug ) Print(i,f)

    cond1=-1       #   ` added  20200520
    if( from > 1 ) cond1=regexpr("[-$&~*/%`]",substr(exp,from-1,from-1))[[1]]
    cond2=-1       #   ` added  20200520
    if( to < nchar(exp) )
      cond2=regexpr("[$&~*/%`]",substr(exp,to+1,to+1))[[1]]
    if( cond1 == -1   &  cond2 == -1 ){
     substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
     removed=c(removed,i)
    }

   } # end of i loop for method 2
   if( debug ) Print(removed,exp)
   # update range
   if( length(removed) > 0 ) range=range[-removed,,drop=0]
   np=nrow(range)
   if( debug ) Print(range)

  } # end of method 2

  ###############################
  if( 3 %in% method  &  np > 0 ){

   # remove -(-a)
   if( debug ) cat("llll=",llll,"M3 Removing -(-a) or -(-(a)) ...\n")
   removed=numeric(0)
   for( i in 1:np ){

    from=range[i,1]; to=range[i,2]
    f=substr(exp,from,to)
    if( debug ) Print(i,f)

    cond1=0
    if( from > 1  &  substr(exp,from-1,from-1) == "-" ){
     # if( regexpr( "^\\(-[[:alnum:]_.]+\\)$", f ) > 0 ) cond1=1
     if( regexpr( "^\\(-[[:alnum:]_.*/]+\\)$", f )[[1]] > 0 ) cond1=1
    }
    if( cond1 ){
     substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
     substr(exp,from-1,from-1)="+"; substr(exp,from+1,from+1)=MARK
     removed=c(removed,i)
    }

    cond1=0
    if( from > 1  &  substr(exp,from-1,from-1) == "-" ){
     if( regexpr( "^\\(-\\(.*\\)\\)$", f )[[1]] > 0 ){
      if( locbalpar(f,3) == nchar(f)-1 ) cond1=1
     }
    }
    if( cond1 ){
     substr(exp,from,from)=MARK;  substr(exp,to,to)=MARK
     substr(exp,from-1,from-1)="+";
     substr(exp,from+1,from+2)=MARK
     substr(exp,to-1,to-1)=MARK
     removed=c(removed,i)
    }

   } # end of i loop for method 3

   if( debug ) Print(removed,exp)
   # update range
   if( length(removed) > 0 ) range=range[-removed,,drop=0]
   np=nrow(range)
   if( debug ) Print(range)

  } # end of method 3


  # convergence
  if( all( exp == exp0 )  |   np == 0 ) break
  exp0=exp

 } # end of llll loop


 # remove MARK
 exp=gsub( MARK,"", exp )

 # take care of the case exp="x+(-2)" or "+(+2)"
 exp=gsub( "+-", "-", exp, fixed=1 )
 exp=gsub( "++", "+", exp, fixed=1 )

 exp=gsub("!#", "#", exp, fixed=1)
 exp=gsub("!!", "", exp, fixed=1) ########### 20171010dnc

 if( debug ) Print(exp)

 return( exp )


} # end of remove_paren

#' expand.grid with rev and colnames option
#'
#' @param ... vectors, factors or a list containing these.
#' @param rev = 1 to make the last elements of ... chage first.
#' @param colnames = 1 to use  ... as the column names.
#' @param KEEP.OUT.ATTRS
#'    a logical indicating the "out.attrs" attribute (see below) should be
#'    computed and returned.
#' @param stringsAsFactors
#'      logical specifying if character vectors are converted to factors.
#' @return A data frame.
#'
#' @details See comments of native expand.grid.\cr
#'   Compare \code{expand.grid( 1:2, 2:3 )} and
#'   \code{expand.grid1( 1:2, 2:3 )}.\cr
#'   \code{expand.grid1( 1:2, 2:3, rev=0 )} should be equal to
#'    native \code{expand.grid( 1:2, 2:3 )}. \cr
#'  \cr
#'  When the first element of ... is a list, expand.grid1 works
#'  as if it is called only with the first element.
#'  See the example below.\cr
#'
#'  \cr
#'  When ... contains a matrix, better use \code{cprod}.
#'
#' @examples
#'
#' Print( expand.grid( 1:2, 2:3 ), expand.grid1( 1:2, 2:3 ) )
#'
#' res=expand.grid( 1:2, c("a","b") )
#' res1=expand.grid1( 1:2, c("a","b") )
#' Print(res,res1)
#'
#'
#' ll=list(aaa=c("a","b"),bbb=1:3)
#' expand.grid1(ll, colnames=0)
#' expand.grid1(ll, colnames=1)
#' expand.grid1(ll, colnames=0, rev=0)
#' expand.grid1(ll, colnames=1, rev=0)
#'
#' expand.grid(ll,ll)
#' expand.grid1(ll,ll)
#'
#'
#' x=c("a","b"); y=1:3
#' expand.grid1(x,y, colnames=0)
#' expand.grid1(x,y, colnames=1)
#' expand.grid1(x,y, colnames=0, rev=0)
#' expand.grid1(x,y, colnames=1, rev=0)
#'
#'
#'
#' @export
#'

expand.grid1 <- function( ..., rev=1, colnames=0
                          , KEEP.OUT.ATTRS=TRUE, stringsAsFactors=FALSE  ){
 # expand.grid with rev option
 # Shin-ichi Mayekawa
 # 20140816
 # name: 20150609
 # list input: 20150730
 # analyze_three_dots: 20171127
 #
 # Same as the native expand.grid exept that
 #  when rev == 1, the last elements in ... changes first.
 #
 # Value:
 #
 #   data frame containing the resulting cartesian product of ... .
 #
 #   Character variables will remain as characters.
 #

 if( rev != 1 ){
  # rev != 1 : use native expand.grid
  res=expand.grid( ...
                   , KEEP.OUT.ATTRS=KEEP.OUT.ATTRS
                   , stringsAsFactors=stringsAsFactors )
  if( colnames == 0 ) return( res )
  else{
   resa=analyze_three_dots( ... )
   name=resa$name; val=resa$val
   if( is.list(val[[1]]) ){
    name=names(val[[1]])
   }
   colnames(res)=name
   return( res )
  }
 }
 else{
  # rev == 1
  resa=analyze_three_dots( ... )
  name=resa$name; val=resa$val

  # if the first of ... is a list, use the elements of it as the input.
  if( is.list(val[[1]]) ){
   name=names(val[[1]])
   val=val[[1]]
   #   names(val)=NULL
  }

  if( length(val) == 1 ){
   if( length(name) == 0 ){
    return( data.frame("Var1"=val[[1]]) )
   }
   else{
    res=data.frame(val)
    names(res[,1])=name
    return( res )
   }
  }

  vt=sapply(val,mode)
  res=t( apply(
   expand.grid( rev(val)
                , KEEP.OUT.ATTRS=KEEP.OUT.ATTRS
                , stringsAsFactors=stringsAsFactors )
   , 1, rev ) )

  if( colnames == 1 ) colnames(res)=name
  else colnames(res)=paste("Var",1:ncol(res),sep="")
  res=as.data.frame(res, stringsAsFactors=FALSE)
  for( j in 1:ncol(res) ) if( vt[j] == "numeric" ) res[,j]=as.numeric(res[,j])
  # res[,vt=="numeric"]=as.numeric(res[,vt=="numeric"])

  return( res )
 }

} # end of expand.grid1

#' Keep or Drop the variables in the data frame
#' 
#' @param df The data frame whose variables are to be kept/dropped.
#' @param keep A vector consisting of the names of the variables to keep\cr
#' or sas-like abbreviated list of variables to keep as a string.
#' @param drop A vector consisting of the names of the variables to drop\cr
#' or sas-like abbreviated list of variables to drop as a string.
#'
#' @examples
#' df <- as.data.frame(demomat(5,7))
#' names(df) <- c("x1","x2","x3","a","s","m","i")
#'
#' keepdrop( df, keep=c("x1", "a", "x3") )
#' keepdrop( df, keep="x1 a x3" )
#' keepdrop( df, keep="x1-x3 m" )
#' keepdrop( df, keep="x1--m" )
#'
#' keepdrop( df, drop=c("x1", "a", "x3") )
#' keepdrop( df, drop="x1 a x3" )
#' keepdrop( df, drop="x1-x3 m" )
#' keepdrop( df, drop="x1--m" )
#'
#' @details
#' This functin first expands the abbreviated variable list by varlst function.
#' \cr
#' Then, it creats a data frame with the variables in the keep list. \cr
#' Finally, it drops those variable in the drop list.
#'
#'
#' @return
#' A data frame
#'
#' @export
#'

keepdrop <- function( df, keep=NULL, drop=NULL ){
 # keep or drop variables from a data frame
 # Shin-ichi Mayekawa
 # 20170912dnc
 # roxgen comment: 20170922
 #
 if( !is.null(keep)){
  keep=varlist( keep, varlistf=names(df) )
  df=df[,keep, drop=0]
 }
 if( !is.null(drop)){
  drop=varlist( drop, varlistf=names(df) )
  df=df[,!(names(df) %in% drop), drop=0]
 }
 return( df )
} # end of keepdrop

#' Extened Orthogonal Procrustes Rotation
#'
#' @param A The input matrix to be rotated
#' @param C The target matrix
#' @param estm = 1 to estimate column means
#' @param estd = 1 to estimate scaling constant
#' @param maxiter max # of iterations
#' @param eps convergence criterion for rmse
#' @param print = 0 to surpress the output
#'
#' @return A list of: \cr
#' B=d * A \%*\% T + matrix(1,nv,1)\%*\%t(m), \cr
#'  T, m, d, Cm the updated target matrix, rmse=sqrt(RSS)
#'
#' @examples
#'  set.seed(1701)
#'  nv <- 10; nr <- 2
#'  pmiss <- 0.3
#'
#'  A <- matrix(rnorm(nv*nr),nv,nr)
#'  dum <- matrix(rnorm(nv*nr),nv,nr)
#'  T <- eigen(t(dum)%*%dum)[[2]]
#'  m <- matrix(rnorm(nr),nr,1)
#'  d <- 10*runif(1)
#'  C <- d*A%*%T + matrix(rep(1,nv),nv,1)%*%t(m)
#'
#'  res <- procote( A, C, estm=0, estd=0, print=1 )
#'  res2 <- procote( A, C, estm=1, estd=1, print=1 )
#'
#'  # missing elements in the target
#'  dum <- matrix(runif(nv*nr),nv,nr)
#'  C[which(dum < pmiss)] <- NA
#'
#'  res3 <- procote( A, C, estm=0, estd=0, print=1 )
#'  res4 <- procote( A, C, estm=1, estd=1, print=1 )
#'
#'
#' @details
#' This program finds those A, m, d  and orthonormal T which minimize \cr
#'   \deqn{RSS = tr( ( C  - Chat )' ( C - Chat ) ) }\cr
#'   where \eqn{Chat = d * A \%*\% T + matrix(1,nv) \%*\% t(m)}, \cr
#'   \eqn{d} is a scalar, \eqn{T} is an orthonormal rotation matrix, \cr
#'   and \eqn{m} is the column mean vector. \cr
#'\cr
#' Note that when \code{estd=0} and \code{estm=0}
#' , and the target matrix does not
#' have missing elements, \cr
#' this is equivalent to the usual orthogonal Procrustes rotation.
#'
#' @export
#'

procote <- function( A, C, estm=0, estd=0, maxiter=200, eps=1e-6, print=2 ){
 # Extended Orthogonal Procrustes Rotation
 # Shin-ichi Mayekawa
 # translated from IML 880623-071222: 111109,10
 # ssq removed: 111204
 # bugfix for m: 111220
 # printm -> Print, print B: 20140816
 # lmax -> maxiter: 20170530dnc
 # remove ;  at the end: 20260516
 # Print -> printm: 20260516
 # tr, Diag replaced: 20260516
 #
 # Args:
 #  A       nv x nr matrix to be rotated
 #  C       nv x nr target matrix
 #  estm    = 1 to estimate m
 #  estd    = 1 to estimate d
 #  maxiter    max # of iterations when there are NAs in C
 #  eps     eps for the relative improvement of RMSE
 #  print   to control the print level
 #
 # Returns:
 #  list(B=B, T=T, m=m, d=d, Cm=Cm, rmse=rmse)
 #  where B is the result of the rotation,
 #  Cm is the nv x nr matrix with the missing elements estimated by LS.
 #
 # The program minimizes
 #   RSS = TR( ( C  - (d A T + 1*m`) )` ( C - (d A T + 1*m`) ) )
 #  w.r.t T, m and d subject to T'T = TT' = I  and d > 0.
 # The missing elements of C matrix, if any,
 # will be estimated so that they also
 # minimize RSS.
 #
 # rmse = sqrt(RSS/(nv*nr)) will be reported.
 #

 # const
 nv=nrow(A); nr=ncol(A)

 Aorg=A
 if (estm == 1){
  amean=as.matrix(apply(A,2,mean),ncol(A),1)
  A=A-matrix(1,nv,1)%*%t(amean)
 }
 traceAtA=sum(diag(t(A)%*%A))

 # missing target elements
 Cm=C
 locmiss=is.na(C)   # logical matrix of the same size as C
 nmiss=sum(locmiss)
 pmiss=nmiss/(nv*nr)
 if (nmiss > 0)
  Cm[locmiss]=A[locmiss]

 converged=0;

 if (print >= 1){
  cat("\n"); cat("\n"); cat("\n");
  cat("Extended Orthogonal Procrustes Rotation:\n")
  cat(" minimizing  TR( ( C  - (d A T + 1*m`) )` ( C - (d A T + 1*m`) ) )\n")
  cat("  w.r.t T, m and d subject to T'T = TT' = I  and d > 0 \n")
  cat(" The missing elements of the targetmatrix C, if any,",
      "will be updated.\n")
  cat("\n")
  cat("Input parameters:\n")
  cat(" Size of A and C matrices: ", nv, " x ", nr, "\n")
  cat(" estimation of m = ", estm, ", estimation of d = ", estd, "\n")
  cat(" max # of iterations = ", maxiter, ", eps for relative rmseimp = ", eps
  , "\n")
  cat(" print = ", print, "\n")
   cat(" # of missing elements in C = ", nmiss, "(", pmiss, ")\n")
 }


 # initial values
 d=1; m=matrix(0,nr,1)
 T=diag(nr); Tp=T
 rmsep=sqrt(sum((Cm-A)^2)/(nv*nr))
 rmse0=rmsep


 # main iteration
 for (llll in 1:maxiter){

  # update orthogonal T;
  CAAC=t(Cm)%*%A%*%t(A)%*%Cm
  ev=eigen(CAAC); eva=matrix(ev$values,length(ev$values),1); eve=ev$vectors
  rm(ev)
  loc=which(eva > 0); eva=eva[loc]; eve=eve[,loc,drop=FALSE]
  SQR=eve%*%diag(sqrt(1/eva),nrow=nr,ncol=nr)%*%t(eve)
  T=t(A)%*%Cm%*%SQR

  B=A%*%T

  # update m
  if (estm == 1) {
   m=as.matrix(apply(Cm-d*A%*%T,2,mean),nr,1)
  }

  # update d
  if (estd == 1 ) {
   d=sum(diag(t(B)%*%Cm))/traceAtA
   # make d positive
   if ( d < 0 ){
    d=-d; T=-T
   }
  }

#  Print(d, A, T, m)
  B=d*A%*%T + matrix(1,nv,1)%*%t(m)

  # check convergence
  rmse=sqrt( sum((Cm-B)^2) / (nv*nr) )
  rmseimp=rmsep-rmse
  if( rmsep > 0 ) rmseimpr=rmseimp/rmsep
  else rmseimpr=0
  maxadt=max(abs(T-Tp))

  if ( print >= 2 )
   printm(llll, rmsep, rmse, rmseimp, rmseimpr, maxadt, fmt="3 .4")

   if ( (rmseimpr < eps) || (nmiss == 0 && estd == 0 && estm == 0) ) {
    converged=1
    break
   }

   # update the target
   if ( nmiss > 0 ){
    Cm[locmiss]=B[locmiss]
   }

   # next iteration
   Tp=T; rmsep=rmse

  } # end of llll loop


  if ( !converged )
   cat("\n Warning(procote): Iteration did not converge.\n\n")

  # update m using the original A
  if (estm == 1) {
   m=as.matrix(apply(Cm-d*Aorg%*%T,2,mean),nr,1)
  }

  B=d*Aorg%*%T+matrix(1,nv,1)%*%t(m)
  rmse=sqrt( sum((Cm-B)^2) / (nv*nr) )

  if ( print > 0 ){
   if ( estd == 1 || estm == 1 || nmiss > 0 ){
    cat("Iteration Terminated.\n")
   printm(llll, rmsep, rmse, rmseimp, rmseimpr, maxadt, fmt=" 3 .4");
   }
   cat(" Final RMSE = ", rmse, "\n")
   cat("\n")
   printm(B, fmt=".4")
   printm(T, m, d, fmt=".4")
  };

  # returning the result
  res=list(B=B, T=T, m=m, d=d, Cm=Cm, rmse=rmse)
  return( res )


 } # end of procote

#' Reflect the signs of the columns of the factor pattern matrix
#' so that the number of positive loadings is larger than the negative ones.
#'
#' @param A Factor Pattern matrix or an object with $loadings.
#' @param unclass = 1 to unclass the result
#' @param sortcols = 1 to sort factors (columns of A) according to
#' factor contributions.
#' @param print = 1 to print the result
#'
#' @details
#' First, if sortcols == 1, the columns of \code{A} are reordered
#' according to the magnitude of \code{diag(t(A)\%*\%A)}. \cr
#' Then, for each column of \code{A},
#' the sign will be reflected if the number of negative loadings is
#' greater than the positive ones.
#'
#' @examples
#' set.seed(1701)
#' n <- 20; ndim <- 4
#' A0 <- matrix(runif(n*ndim),n,ndim)
#' dimnames(A0)=list( paste("v",1:n,sep=""),paste("f",1:ndim,sep=""))
#' A0v <- varimax(A0)$loadings
#' A0v <- A0v*matrix(sample(c(-1,1,1),n*ndim,replace=TRUE),n,ndim)
#' A0v[,3:4] <- -A0v[,3:4]
#'
#' A0vs <- reflect_fa(A0v)$loadings
#' Print(A0v,A0vs, fmt="6.3")
#'
#' A0vsc <- reflect_fa(A0v, sortcol=1)$loadings
#' Print(A0v,A0vsc, fmt="6.3")
#'
#' # attributes
#' print(reflect_fa(A0v)$loadings)
#' print(reflect_fa(A0v, unclass=1)$loadings)
#'
#' @return A list of \cr
#' loadings = reordered loadings matrix \cr
#' col_order = sort order of the cols \cr
#' sign = a vector indicating if the signs were preserved or not
#'
#' @export
#'

reflect_fa <- function( A, sortcols=0, unclass=0 ){
 # reeflect the signs of the columns
 # Shin-ichi Mayekawwa
 # 20210416cot,17cot
 # lazy.tools: 20210421
 #

 # store attributes
 attrib=attributes(A)
 A=unclass(A)
 n=nrow(A)

  # names
 if( is.null(rownames(A)) ) rownames(A)=paste("r",1:nrow(A),sep="")
 if( is.null(colnames(A)) ) colnames(A)=paste("c",1:ncol(A),sep="")
 rname=rownames(A); cname=colnames(A)

 odc=1:ncol(A)
 names(odc)=cname
 if( sortcols == 1 ){
  # sort by diag(t(A)%*%A)
  fc=colSums(A*A)
  names(fc)=cname
  odc=order(-fc)
  A=A[,odc]
  cname=colnames(A)
 }

 # sig[j]=1 if the # of negative loadings is less than positive ones.
 sig=2*( (colSums(A>0)-nrow(A)/2) >= 0 )-1
 A=A%*%diag( sig )
 rownames(A)=rname
 colnames(A)=cname

 if( !unclass ){
  attributes(A)=attrib
  rownames(A)=rname
  colnames(A)=cname
 }


 return( list(loadings=A, sign=sig, col_order=odc ) )

} # end of reflect_fa

#' Inside-Out table
#'
#' @param x Input vector matrix
#' @param id id to be used instead of rownames(x).
#' @param var sas-like abbreviated list of variables to be used.
#' @param rank = 1 to retrun the rank in stead of the row name.
#' @param abs = 1 to use the absolute value of x to sort.
#' @param descending = 0 to sort ascendingly.
#' @param dec a vector indicating if the sort is descending order
#' for each varaible.
#' @param mark A character to mark the negative numbers.
#' @param print = 1 to print the result.
#' @details
#'  This function sort the id according to the magnitude of each variable.\cr
#'  When rank = 0 and abs = 1, those names corresponding to\cr
#'    the negative values  will be marked by - sign. \cr
#' @examples
#' data <- matrix(c(c(2,1,4,5,3),1:5),,2)
#' rownames(data) <- paste("r",1:5,sep="")
#' colnames(data) <- paste("v",1:2,sep="")
#' data[4,2]=-9
#' iotable( data, descending=1, print=0 )
#' iotable( data, descending=1, abs=1, print=0 )
#'
#' @return A vector or matrix of the same size as x.
#'
#' @export
#'

iotable <- function( x, id=NULL, var=NULL, rank=0, abs=0, descending=1
                     , dec=NULL, mark="-", print=1 ){
 # inside-out table
 # Shin-ichi Mayekawa
 # 20140520,21
 # id as an argument: 20140527
 # var as an argument: 20140528
 #
 # Args:
 #
 #     x      vector or matrix whose (row) names are to be printed
 #     id     id variable name to be used
 #     var    sas-like abbreviated variable list
 #     rank   = 1 to retrun the rank in stead of the row name
 #     descending = 1 to sort descendingly for all variables
 #     dec    vector indicating if the sort is descending order
 #     abs    = 1 to use the absolute value of x
 #
 # Values:
 #
 #     vector or matrix of names sorted according to the magnitude
 #     of each variable
 #     or
 #     rank for each variable
 #
 #     When rank = 0 and abs = 1, those names corresponding to
 #     the negative values  will be marked by - sign.
 #
 #
 # Needs:
 #
 #     varlist, loceq
 #

 # variable selection and ordering by varlist
 if( !is.null(var) ){
  temp=varlist( var, colnames(x), 1)
  varf=temp[[2]]
  loc=temp[[2]]
  rm(temp)
  # Print(var,varf,colnames(x),loc)
  x=x[,loc]
 }

 # abs?
 if( abs > 0 ){
  locneg=sign(x)
  if( length(locneg) > 0 ) x=abs(x)
  else abs=0
 }

 # if input is a vector
 if( is.vector(x) ){
  if( rank == 0 ){
   od=order(x, decreasing=descending)
   if( is.null(id) ) name=names(x) else name=id
   if( is.null(name) ) name=paste("r",1:length(x),sep="")
   res=name[od]
   if( abs > 0 ){
    locneg1=which(locneg[od]<0)
    res[locneg1]=paste(mark,res[locneg1],sep="")
   }
  }
  else {
   res=rank(x)
   if( descending == 1 ) res=length(x)-res+1
  }
  return( res )
 }

 # here x is a matrix
 nr=nrow(x)
 if( is.null(id) ){
  name=rownames(x)
  if( is.null(name) ) name=paste("r",1:nrow(x),sep="")
 }
 else name=id
 nc=ncol(x)

 # descending or not
 if( is.null(dec) ){
  dec=rep(0,nc)
  if( descending > 0 ) dec=rep(1,nc)
 }
 else{
  if( length(dec) != nc ){
   cat("\nerror1:(io_plot) length(dec) must be equal to # of columns.\n")
   return( NULL )
  }
 }

 # main body
 if( rank == 0 ){
  res=apply( x, 2, function(y)name[order(y, decreasing=dec)] )
  rownames(res)=1:nrow(res)
  if( abs > 0 ){
   for( j in 1:nc ){
    od=order(x[,j], decreasing=dec)
    locneg1=which( locneg[od,j] < 0 )
    res[locneg1,j]=paste(mark,res[locneg1,j],sep="")
   }
  }
 }
 else{
  res=apply( x, 2, rank )
  for( j in 1:nc ){
   if( dec[j] == 1 ) res[,j]=nr-res[,j]+1
  }
 }

 if( print > 0 ){
  cat("\n\nInside-Out Table\n")
  cat(" # of observations =", nr, ",   # of varaibles =", nc
      , "\n descending =", descending, ",  abs =", abs,"\n")
  print(res,quote=0)
 }

 return( res )

} # end of iotable




#' Profile Plot
#'
#' @param df data frame
#' @param var sas-like abbreviated list of variables to be used.
#' @param standard = 1 to standardize the variables \cr
#' = 2 to scale the variables using min and max
#' @param title plot title string.
#' @param  pty A vector to specify point symbol: \cr
#'     Default 16 (closed circle), if you don't plot data points, \cr
#'       it should be 32. This is repeatedly used for data series. \cr
#' @param  pcol  A vector of color codes for plot data: \cr
#'      Default 1:8, which are repeatedly used.
#' @param  plty  A vector of line types for plot data: \cr
#'      Default 1:6, which are repeatedly used.
#' @param  plwd   A vector of line widths for plot data: \cr
#'    Default 1, which is repeatedly used.
#' @param  ... Miscellaneous arguments to be given for plot.
#' @param  legend  = 1 to display legend.
#' @param  lpos  upper left corner of the legend.
#' @param  lcex  font size for the legend.
#' @param  llwd  line width for the legend.
#' @param  lbty  the type of box to be drawn around the legend.
#' @param  pch plot character
#' @param  lwd line width
#' @param  nobspc # of observations in one chart.
#' @param  across # of graphs in a row.
#' @param  down # of graphs in a column.
#' @param  mar margins for multiple graphs per page.
#' @param  axislabcol not used.
#' @param  vlabels not used.
#' @param  caxislabels  not used.
#' @param  paxislabels not used.
#'
#' @examples
#' data=data.frame(a=c(1,2,3,4),b=c(0,1,0,1),c=c(4,3,2,1))
#' pplot( data)
#' pplot( data, nobspc=2 )
#' pplot( data, nobspc=2, across=2 )
#'
#' @return None
#' @export
#'

pplot <- function(  df, var=NULL, pty=16, pcol = 1:8, pch=2, lwd=2
                    , plty = 1:8, plwd = 2, axislabcol = "blue"
                    , title = ""
                    , vlabels = NULL, caxislabels = NULL, paxislabels = NULL
                    , legend=1, lpos=c(1.5,-.2), lcex=1, llwd=1, lbty="n"
                    , standard=0, nobspc=0, down=1, across=1
                    , mar=c(8,2,1,2)
                    , ... ){
 # profile plot
 # Shin-ichi Mayekawa
 # 20140601,02
 # bugfix: 20140604
 # standard: 20140604,05
 # nobspc=1: 20140610 dnc
 # remove non-numeric variables: 20140610
 # standard == 0: 20151021
 #

 # Args:
 #

 #  df
 #      The data frame to be used to draw profile plot.
 #      The number of columns (variables) must be more than 2.

 #  var
 #      abbreviated list of variables to be used

 #  axistype
 #      The type of axes, specified by any of 0:5. 0 means no axis label. #
 #      1 means center axis label only. 2 means around-the-chart label only.
 #      3 means both center and around-the-chart (peripheral) labels.
 #      4 is *.** format of 1, 5 is *.** format of 3. Default is 0.

 #  seg
 #      The number of segments for each axis (default 4).

 #  pty
 #      A vector to specify point symbol:
 #      Default 16 (closed circle), if you don't plot data points,
 #      it should be 32. This is repeatedly used for data series.
 #  pcol
 #      A vector of color codes for plot data:
 #      Default 1:8, which are repeatedly used.
 #  plty
 #      A vector of line types for plot data:
 #      Default 1:6, which are repeatedly used.
 #  plwd
 #      A vector of line widths for plot data:
 #      Default 1, which is repeatedly used.

 #  title
 #      if any, title should be typed.
 #  maxmin
 #      Logical. If true, data frame includes possible maximum values as row 1
 #      and possible minimum values as row 2.
 #      If false, the maximum and minimum values for each axis will be
 #       calculated as actual maximum and minimum of the data. Default TRUE.
 #  ...
 #      Miscellaneous arguments to be given for plot.default().
 #
 #  legend
 #      = 1 to display legend
 #  lpos
 #      upper left corner of the legend
 #  lcex
 #      font size for the legend
 #  llwd
 #
 #  lbty
 #
 #
 #  standard
 #      = 1  each variable will be standardized and
 #      the graph will be drawn  using ylim=c( min(dfs), max(dfs) )
 #      = 2
 #      Othersize, each variable will be scaled so that min=0 and max=1.
 #
 #  nobspc
 #      # of observations in one chart
 #
 #  across
 #      # of graphs in a row
 #  down
 #      # of graphs in a column
 #  mar
 #      margins for multiple graphs per page

 #
 # Values:
 #
 #

 # Needs:
 #
 #    varlist, loceq, standard, vartype
 #




 # const
 nobs=nrow(df)
 nvar=ncol(df)

 # errors
 if( nvar < 2 ){
  cat("The number of variables must be 3 or more.\n")
  return()
 }


 # variable selection and ordering by varlist
 if( !is.null(var) ){
  temp=varlist( var, colnames(df), 1)
  varf=temp[[2]]
  loc=temp[[2]]
  rm(temp)
  # Print(var,varf,colnames(x),loc)
  df=df[,loc]
  nvar=ncol(df)
 }

 if (!is.data.frame(df)) {
  #  cat("The data must be given as dataframe.\n")
  # return()
  df=as.data.frame(df)
 }
 if( is.null(rownames(df)) ) rownames(df)=paste("r",1:nobs,sep="")


 # convert factors to integer
 loc=which( vartype(df) == "factor" )
 if( length(loc) > 0 ) df[,loc]=as.integer(df[,loc])
 # renove non-numeric variables
 loc=vartype(df)
 loc=which( loc == "numeric" )
 if( length(loc) <= 2 ){
  cat("The number of numeric variables must be 3 or more.\n")
 }
 else df=df[,loc,drop=0]
 nvar=ncol(df)

 # standardization or minmax
 if( standard == 1 ){
  dfs=standard(df)
  ylim=c(min(dfs),max(dfs))
 }
 else if( standard == 2 ){
  # for each variable, make min=0 and max=1
  dfmax <- apply(df, 2, max)
  dfmin <- apply(df, 2, min)
  dfs=t( (t(df)-dfmin)/(dfmax-dfmin) )
  rownames(dfs)=rownames(df)
  ylim=0:1
 }
 else{
  dfs=df
  ylim=c(min(dfs),max(dfs))
 }


 # graphical parameters
 if( across < 1  |  across > 14 ){
  cat("\nwarning1:(pplot)  across must be in 1:14.   Set to one.\n")
  across=1
 }
 if( down < 1  |  down > 14 ){
  cat("\nwarning1:(pplot)  down must be in 1:14.   Set to one.\n")
  down=1
 }
 # save the current graphical parameter
 oldpar <- par(no.readonly = TRUE)

 # set the graphical parameters
 par(mfrow = c(down,across), mar=mar + 0.1, xpd=TRUE)

 # main plot type, color etc
 if (length(pty)  < nobs)   ptys = rep(pty, nobs) else  ptys = pty
 if (length(pcol) < nobs)   pcols = rep(pcol, nobs) else   pcols = pcol
 if (length(plty) < nobs)   pltys = rep(plty, nobs) else   pltys = plty
 if (length(plwd) < nobs)   plwds = rep(plwd, nobs) else  plwds = plwd


 # nobspc observations in one chart
 if( nobspc == 0 ) nobspc=nobs
 nobspc=min(nobspc,nobs)
 nloop=ceiling(nobs/nobspc)


 # loop for each chart
 for( llll in 1:nloop ){

  fromto=( (llll-1)*nobspc+1 ):min(llll*nobspc,nobs)
  nobsllll=min(nobspc,length(fromto))

  # matplot
  matplot( t(dfs[fromto,,drop=0]), type="b", pch=pch, lwd=lwd, lty=pltys[fromto]
           , xaxt="n", ylim=ylim
           , ylab="", main=title, col=pcols[fromto] )
  axis(1, at = 1:ncol(dfs), labels=colnames(dfs), cex.axis = 1)
  legend( lpos[1],ylim[1]+lpos[2]*(ylim[2]-ylim[1])
          , rownames(dfs[fromto,,drop=0])
          , col=pcols[fromto], lty=pltys[fromto]
          , cex=lcex, lwd=llwd, merge=1, ncol=nobsllll, bty=lbty )

 } # end of llll loop

 on.exit(par(oldpar))


} # end of pplot



#' Radar Chart
#'
#'  This is a modified version of \code{radarchart} in \code{fmsb} package.
#'
#' @param df data frame
#' @param var sas-like abbreviated list of variables to be used.
#' @param standard = 1 to standardize the variables
#' @param title plot title string.

#' @param axistype The type of axes, specified by any of 0:5. \cr
#'    0 means no axis label. \cr
#'    1 means center axis label only. 2 means around-the-chart label only. \cr
#'    3 means both center and around-the-chart (peripheral) labels. \cr
#'    4 is *.** format of 1, 5 is *.** format of 3. Default is 0. \cr

#' @param seg The number of segments for each axis (default 4).


#' @param  pty A vector to specify point symbol: \cr
#'     Default 16 (closed circle), if you don't plot data points, \cr
#'       it should be 32. This is repeatedly used for data series. \cr
#' @param  pcol  A vector of color codes for plot data: \cr
#'      Default 1:8, which are repeatedly used.
#' @param  plty  A vector of line types for plot data: \cr
#'      Default 1:6, which are repeatedly used.
#' @param  plwd   A vector of line widths for plot data: \cr
#'    Default 1, which is repeatedly used.


#' @param  pdensity A vector of filling density of polygons: \cr
#'      Default NULL, which is repeatedly used.
#' @param pfcol A vector of color codes for filling polygons: \cr
#'      Default NA, which is repeatedly usd.
#' @param pangle angle
#' @param  cglty Line type for radar grids: Default 3, which means dotted line.
#' @param  cglwd Line width for radar grids: Default 1,
#' which means thinnest line.
#' @param  cglcol Line color for radar grids: Default "navy"
#' @param  axislabcol Color of axis label and numbers: Default "blue"
#'
#' @param  centerzero Logical. If true, this function draws charts with scaling
#'      originated from (0,0). \cr
#'      If false, charts originated from (1/segments). Default FALSE.
#' @param  vlabels Character vector for the names for variables. \cr
#'      If NULL, the names of the variables as colnames(df) are used.
#' @param  caxislabels  Character vector for center axis labels, \cr
#'      overwriting values specified in axistype option. \cr
#'      If NULL, the values specified by axistype option are used.
#' @param  caxisoffset offset
#'
#' @param  caxiscex cex
#'
#' @param  paxislabels Character vector for around-the-chart (peripheral)
#'  labels, \cr
#'      overwriting values specified in axistype option. \cr
#'      If NULL, the values specified by axistype option are used.
#' @param  clockwise = 1 if the graph places the variables in clockwise
#' from the top
#' @param  frame = 1 to draw frame around the graph

#' @param  legend  = 1 to display legend.
#' @param  lpos  upper left corner of the legend.
#' @param  lcex  font size for the legend.
#' @param  llwd  line width for the legend.

#' @param  nobspc # of observations in one chart.
#' @param  across # of graphs in a row.
#' @param  down # of graphs in a column.
#' @param  mar margins for multiple graphs per page.
#' @param  ... additional parameters for plot
#'
#' @examples
#' data=data.frame(a=c(1,2,3,4),b=c(0,1,0,1),c=c(4,3,2,1))
#' radarchart( data, nobspc=2, across=2, lpos=c(.9,.1) )
#'
#' @return None
#' @export
#'

radarchart <- function(
 df, var=NULL, axistype = 4, seg = 4, pty = 16, pcol = 1:8
 , plty = 1:8, plwd = 1, pdensity = NULL, pangle = 45, pfcol = NA
 , cglty = 3, cglwd = 1, cglcol = "navy", axislabcol = "blue"
 , title = "", centerzero = FALSE
 , vlabels = NULL, caxislabels = NULL, paxislabels = NULL
 , frame=1, clockwise=1, caxisoffset=-.1, caxiscex=.9
 , legend=1, lpos=c(1.1,-.1), lcex=.8, llwd=1
 , standard=0, nobspc=0, down=1, across=1
 , mar=c(1, 1, 1, 1)
 , ... ){
 #
 # Radar Chart
 # original found in fmsb package
 # Shin-ichi Mayekawa
 # 20140527,28,29
 # nobspc, down, across: 20140601
 # var: 20140601,02
 # llwd: 20140603
 # standardization: 20140605
 # nobspc=1: 20140610 dnc
 # remove non-numeric variables: 20140610
 # standard == 0: 20151021
 #

 # Args:
 #

 #  df
 #      The data frame to be used to draw radarchart.
 #      The number of columns (variables)  must be more than 2.

 #  var
 #      abbreviated list of variables to be used

 #  axistype
 #      The type of axes, specified by any of 0:5. 0 means no axis label. #
 #      1 means center axis label only. 2 means around-the-chart label only.
 #      3 means both center and around-the-chart (peripheral) labels.
 #      4 is *.** format of 1, 5 is *.** format of 3. Default is 0.

 #  seg
 #      The number of segments for each axis (default 4).

 #  pty
 #      A vector to specify point symbol:
 #      Default 16 (closed circle), if you don't plot data points,
 #      it should be 32. This is repeatedly used for data series.
 #  pcol
 #      A vector of color codes for plot data:
 #      Default 1:8, which are repeatedly used.
 #  plty
 #      A vector of line types for plot data:
 #      Default 1:6, which are repeatedly used.
 #  plwd
 #      A vector of line widths for plot data:
 #      Default 1, which is repeatedly used.
 #  pdensity
 #      A vector of filling density of polygons:
 #      Default NULL, which is repeatedly used.
 #  pfcol
 #      A vector of color codes for filling polygons:
 #      Default NA, which is repeatedly usd.
 #  cglty
 #      Line type for radar grids: Default 3, which means dotted line.
 #  cglwd
 #      Line width for radar grids: Default 1, which means thinnest line.
 #  cglcol
 #      Line color for radar grids: Default "navy"
 #  axislabcol
 #      Color of axis label and numbers: Default "blue"

 #  title
 #      if any, title should be typed.
 #  centerzero
 #      Logical. If true, this function draws charts with scaling
 #      originated from (0,0).
 #      If false, charts originated from (1/segments). Default FALSE.
 #  vlabels
 #      Character vector for the names for variables.
 #      If NULL, the names of the variables as colnames(df) are used.
 #      Default NULL.
 #  caxislabels
 #      Character vector for center axis labels,
 #      overwriting values specified in axistype option.
 #      If NULL, the values specified by axistype option are used.
 #      Default is NULL.
 #  caxisoffset
 #
 #  caxiscex
 #
 #  paxislabels
 #      Character vector for around-the-chart (peripheral) labels,
 #      overwriting values specified in axistype option.
 #      If NULL, the values specified by axistype option are used.
 #      Default is NULL.
 #  ...
 #      Miscellaneous arguments to be given for plot.default().
 #
 #  clockwise
 #      = 1 if the graph places the variables in clockwise from the top
 #  frame
 #      = 1 to draw frame around the graph
 #  legend
 #      = 1 to display legend
 #  lpos
 #      upper left corner of the legend
 #  lcex
 #      font size for the legend
 #
 #  standard
 #      When standard == 1 each variable will be standardized and
 #      the graph will be drawn  using ylim=c( min(dfs), max(dfs) )
 #      Othersize, each variable will be scaled so that min=0 and max=1.
 #
 #  nobspc
 #      # of observations in one chart
 #  across
 #      # of graphs in a row
 #  down
 #      # of graphs in a column
 #  mar
 #      margins for multiple graphs per page
 #

 #
 # Values:
 #
 #

 # Needs:
 #
 #    varlist, loceq, standard, vartype
 #


 initialize_web <- function(){

  # initialize plot
  plot(c(-1.2, 1.2), c(-1.2, 1.2), type = "n", frame.plot = frame
       , axes = FALSE, xlab = "", ylab = "", main = title, asp = 1
       , ...)

  # drawing empty web and axis
  for (i in 0:seg) { # for each level
   polygon(xx * (i + CGap)/(seg + CGap), yy * (i + CGap)/(seg + CGap)
           , lty = cglty, lwd = cglwd, border = cglcol)
   if (axistype == 1 | axistype == 3)
    CAXISLABELS <- paste(i/seg * 100, "(%)")
   if (axistype == 4 | axistype == 5)
    CAXISLABELS <- sprintf("%3.2f", i/seg)
   if (!is.null(caxislabels) & (i < length(caxislabels)))
    CAXISLABELS <- caxislabels[i + 1]
   if (axistype == 1 | axistype == 3 | axistype == 4 | axistype == 5)
    text(caxisoffset, (i + CGap)/(seg + CGap), CAXISLABELS
         , col = axislabcol, cex=caxiscex )
  } # end of i loop

  if (centerzero) {
   arrows(0, 0, xx * 1, yy * 1
          , lwd = cglwd, lty = cglty, length = 0, col = cglcol)
  }
  else {
   arrows(xx/(seg + CGap), yy/(seg + CGap), xx * 1, yy * 1
          , lwd = cglwd, lty = cglty, length = 0, col = cglcol)
  }

  # axis labels
  PAXISLABELS <- colnames(df)
  if (!is.null(paxislabels))
   PAXISLABELS <- paxislabels
  if (axistype == 2 | axistype == 3 | axistype == 5) {
   text(xx[1:nvar], yy[1:nvar], PAXISLABELS, col = axislabcol)
  }
  VLABELS <- colnames(df)
  if (!is.null(vlabels)) VLABELS <- vlabels
  text(xx * 1.2, yy * 1.2, VLABELS)

 } # end of initialize_web





 # const
 nobs=nrow(df)
 nvar=ncol(df)

 # errors
 if ((nvar <- length(df)) <= 2) {
  cat("The number of variables must be 3 or more.\n")
  return()
 }

 # variable selection and ordering by varlist
 if( !is.null(var) ){
  temp=varlist( var, colnames(df), 1)
  varf=temp[[2]]
  loc=temp[[2]]
  rm(temp)
  # Print(var,varf,colnames(x),loc)
  df=df[,loc]
  nvar=ncol(df)
 }


 # reverse col order
 if( clockwise ) df=df[,c(1,nvar:2)]

 if (!is.data.frame(df)) {
  #  cat("The data must be given as dataframe.\n")
  # return()
  df=as.data.frame(df)
 }

 # convert factors to integer
 loc=which( vartype(df) == "factor" )
 if( length(loc) > 0 ) df[,loc]=as.integer(df[,loc])
 # renove non-numeric variables
 loc=vartype(df)
 loc=which( loc == "numeric" )
 if( length(loc) <= 2 ){
  cat("The number of numeric variables must be 3 or more.\n")
 }
 else df=df[,loc,drop=0]
 nvar=ncol(df)


 # standardization or minmax
 if( standard == 1 ){
  dfs=standard(df)
  minmax=c(min(dfs),max(dfs))
  # minmax=closest( ylim, seq(-3,3,length.out=21) )
  dfs=t( (t(dfs)-minmax[1])/(minmax[2]-minmax[1]) )
  rownames(dfs)=rownames(df)
  caxislabels=seq(minmax[1],minmax[2],length.out=seg+1)
  caxislabels=formatC(caxislabels,1,4,format="f")
 }
 else if( standard == 2 ){
  # for each variable, make min=0 and max=1
  dfmax <- apply(df, 2, max)
  dfmin <- apply(df, 2, min)
  dfs=t( (t(df)-dfmin)/(dfmax-dfmin) )
  rownames(dfs)=rownames(df)
  ylim=0:1
 }
 else{
  dfs=df
  minmax=c(min(dfs),max(dfs))
  # minmax=closest( ylim, seq(-3,3,length.out=21) )
  dfs=t( (t(dfs)-minmax[1])/(minmax[2]-minmax[1]) )
  rownames(dfs)=rownames(df)
  caxislabels=seq(minmax[1],minmax[2],length.out=seg+1)
  caxislabels=formatC(caxislabels,1,4,format="f")
 }



 # initialize plot
 theta <- seq(90, 450, length = nvar + 1) * pi/180
 theta <- theta[1:nvar]
 xx <- cos(theta)
 yy <- sin(theta)
 rm(theta)
 CGap <- ifelse(centerzero, 0, 1)

 # main plot type, color etc
 if (length(pty)  < nobs)   ptys = rep(pty, nobs) else  ptys = pty
 if (length(pcol) < nobs)   pcols = rep(pcol, nobs) else   pcols = pcol
 if (length(plty) < nobs)   pltys = rep(plty, nobs) else   pltys = plty
 if (length(plwd) < nobs)   plwds = rep(plwd, nobs) else  plwds = plwd
 if (length(pdensity) < nobs)  pdensities = rep(pdensity, nobs) else
  pdensities = pdensity
 if (length(pangle) < nobs)  pangles = rep(pangle, nobs) else pangles = pangle
 if (length(pfcol)  < nobs)  pfcols = rep(pfcol, nobs) else  pfcols = pfcol

 CGr=CGap/(seg + CGap)
 segr=seg/(seg + CGap)


 # graphical parameters
 if( across < 1  |  across > 14 ){
  cat("\nwarning1:(radarchart)  across must be in 1:14.   Set to one.\n")
  across=1
 }
 if( down < 1  |  down > 14 ){
  cat("\nwarning1:(radarchart)  down must be in 1:14.   Set to one.\n")
  down=1
 }
 # save the current graphical parameter
 oldpar <- par(no.readonly = TRUE)

 # set the graphical parameters
 par(mfrow = c(down,across), mar=mar + 0.1)


 # nobspc observations in one chart
 if( nobspc == 0 ) nobspc=nobs
 nloop=ceiling(nobs/nobspc)

 # loop for each chart
 for( llll in 1:nloop ){

  # clear and drow empty web
  initialize_web()

  fromto=( (llll-1)*nobspc+1 ):min(llll*nobspc,nobs)

  for( i in fromto ){

   scale=CGr+dfs[i,]*segr

   xxs=xx*scale  # sin*scale
   yys=yy*scale  # cos*scale

   if (is.null(pdensities)) {
    polygon( xxs, yys, lty = pltys[i], lwd = plwds[i]
             , border = pcols[i], col = pfcols[i] )
   } else {
    polygon( xxs, yys, lty = pltys[i], lwd = plwds[i]
             , border = pcols[i]
             , density = pdensities[i], angle = pangles[i], col = pfcols[i] )
   } # end of pdensities

   points(xxs, yys, pch = ptys[i], col = pcols[i])

  } # end of i loop

  legend(lpos[1],lpos[2], rownames(dfs[fromto,,drop=0]), col=pcols[fromto]
         , cex=lcex,  lwd=llwd, merge = TRUE)

 } # end of llll loop

 on.exit(par(oldpar))


} # end of radarchart

#' Check if a string vector contains numbers only.
#'
#' @param arg a vector or a list
#' @param pat = 1 to match non-negative integers \cr
#'  = 2 to match signed real numbers \cr
#'  = 3 to match integers
#'
#' @details
#' The leading and trailing blank characters will be ignored.

#' @examples
#' vec=c("a","1","1.4","b.","-b","-2","2-"," 12 ","5 5","0.4",".4","-0.","-0.8")
#' is.number(vec)
#' printm(vec,is.number(as.list(vec)),is.number(as.list(vec), 2))
#' is.number( list( c("1","2","a"), c(1,2) ) )
#' is.number( list( 2, -2,"-2","-2." ), 3 )
#'
#' @return
#' If arg is a vector or matrix, a scalar TRUE or FALSE. \cr
#' If arg is a list, a vector of the same length as the arg consisting of
#' TRUEs or FALSEs.
#'
#' @export
#'

is.number <- function( arg, pat=1 ){
 # check if all the elements are numbers.
 # Shin-ichi Mayekawa
 # 20160731,0801
 # new method and pat=3: 20220927dnc
 #

 # pattern to match
 pattern=c( "^ *[[:digit:]]+ *$",
 "(^ *-?[[:digit:]]+\\.?[[:digit:]]* *$)|(^ *-?[[:digit:]]*\\.?[[:digit:]]+ *$)"
 ,"^ *-?[[:digit:]]+ *$" )


 if( is.list(arg) ){
  res=unlist( lapply( arg, function(x){ regexpr(pattern[pat], x) > 0 } ) )
 }
 else{
  if( is.numeric( arg ) ) res=TRUE
  else{
   resg=grep(pattern[pat], arg)
   if( length(arg) == length(resg) ) res=TRUE
   else res=FALSE
  }
 }

 return( res )

} # is.number

#' Rename the elements of the input by the new set of names
#' given in the table n2o or by the consecutive numbers.
#'
#' @param old An object whose elements are to be replaced by numbers.
#' @param n2o Conversion table (data frame) from old to new or NULL. \cr
#' Must have two columns named "new" and "old" and "new" must be 1:nrow(n2o).
#' @param sort = 1 to sort the new name when n2o is NULL. \cr
#' = 2 to sort  the new name when n2o is NULL by naturalsort package.
#' @param print = 1 to print the conversion summary.
#'
#' @return A list of the following: \cr
#'   new     renamed version of old:  same shape and same order as old \cr
#'   n2o     conversion table from new to old \cr
#'   o2n     sorted version of n2o by old \cr
#'   freqnew frequency of new names \cr
#'           cbind( n2o,freqnew) will give you the frequency table. \cr
#'   od      order to be used to sort an object
#'
#' @details
#' By default, the unique names in old will be renamed by the
#' consecutive numbers in the order of appearance. \cr
#' When old is a data frame, new may be a matrix.
#' \cr\cr
#' naturalsort package is required to use sort=2 option.
#' \cr\cr
#' 
#' @examples
#' # add consecutive numbers in the order of appearance.
#' ( new_name( matrix(c("IA","NY","WY","AR","NY","IA"),3) ) )
#'
#' # renaming old
#' seed <- 1701; set.seed(seed)
#' old <- sample( letters[1:6], 10, replace=TRUE )
#' # order of appearance
#' res1 <- new_name( old, print=1 )
#' # sorted order
#' res2 <- new_name( old, print=1, sort=1)
#' # using renaming table
#' old2 <- sample( old, 10, replace=TRUE )
#' res <- new_name( old2, res2$n2o, print=1 )
#'
#' # Rename "a" as "Z", "b" as "Y", "c" as "X".
#' new <- c( "Z", "Y", "X" )
#' old <- c( "a", "b", "c" )
#' n2o <- data.frame( new=new, old=old, stringsAsFactors=0 )
#' temp <- new_name( old, n2o )
#' Print( old, temp$new )
#' old2 <- sample( old,10,replace=1 )
#' temp2 <- new_name( old2, n2o )
#' Print( old2, temp2$new  )
#'
#' # sorting vectors
#' # Suppose that you hava a vector
#' #   vec = c( "A", "D", "B", "E", "C", "D", "A" )
#' # and you wish to sort it as
#' #  vec_sorted <- vec[od,]
#' # so that
#' #  vec_sorted = c("B", "D", "D", "A", "A", "C", "E") .
#' # This can be accomplished by using order2 function as:
#' # vec <- c( "A", "D", "B", "E", "C", "D", "A" )
#' # od <- order2( vec, order=c("B", "D", "A", "C", "E") )
#' # vec_sorted <- vec[od]
#' #
#' # The same result can be obtained as:#'
#' vec <- c( "A", "D", "B", "E", "C", "D", "A" )
#' n2o=data.frame(new=1:5, old=c("B", "D", "A", "C", "E"))
#' od <- new_name( vec, n2o )$od
#' vec_sorted <- vec[od]
#'
#'
#'
#' @export
#'

new_name <- function( old, n2o=NULL, sort=0, print=0 ){
 # assign consecutive numbers to the unique elements of old
 # Shin-ichi Mayekawa
 # 20150430 DNC, 0502
 # naturalsort: 20150508
 # factor input: 20150811
 # typo fix: 20160801
 # return od: 20160804
 # when new is a factor: 20160805
 # freqnew: 20161003
 # roxgen2: 20170926
 # bugfix: 20170927
 # roxygen2 error fixed: 20191129dnc
 #
 #
 # Args:
 #
 #   old     vector or matrix which contains old names
 #
 #   n2o     optional matrix or data.frame which contains new and old
 #           where old[i] will be renamed as new[i].
 #
 #   sort    = 1 to assign consecutive numbers to the sorted elements of old.
 #           = 2 to use package naturalsort
 #
 #
 # Values: list of the following
 #
 #   new     renumbered version of old:  same shape and same order as old
 #   n2o     conversion table from new to old
 #   o2n     sorted version of n2o by old
 #   freqnew frequency vector of new names
 #
 # Needs:
 #
 #   naturalsort package
 #

 # original data type
 if( is.factor(old) ){
  old=as.vector(old)
  type=1; nr=length(old); nc=1
 }
 if( is.vector(old) ){
  type=1; nr=length(old); nc=1
 }
 else if( is.matrix(old) ){
  type=2;  nr=nrow(old); nc=ncol(old)
 }
 else if( is.data.frame(old)){
  type=2;  nr=nrow(old); nc=ncol(old)
  for( j in 1:nc ){
   if( is.factor(old[,j]) ) old[,j]=as.vector(old[,j])
  }
 }

 # vectorize
 old=c(as.matrix(old))

 # conversion table
 if( is.null(n2o) ){
  n2oin=0
  oldu=unique(old)
  if( sort == 1 ) oldu=sort(oldu)
  else if( sort == 2 ) oldu=naturalsort(oldu)
  n=length(oldu)
  n2o=data.frame( new=1:n, old=oldu, stringsAsFactors=0 )
 }
 else{

  n2oin=1
  n2o=as.data.frame(n2o)
  n=nrow(n2o)

  # Print(loceq(old,n2o[,2]), old%in%n2o[,2])
  if( !all( old %in% n2o[,2] ) ){
   cat("\nerror1:(new_name)"
       , " Given conversion table is not sufficient for the new input.\n")
   Print(old,n2o)
   stop()
  }

 }

 # rename old by new
 # add tempid
 old=data.frame( old=old, tempid=1:length(old), stringsAsFactors=0 )
 # merge df n2o and df old by var old as key
 new=merge( n2o, old, by="old", sort=FALSE, stringsAsFactors=0 )
 # sort back to the original order
 new=new[order(new$tempid),]
 # back to original mode
 if( type == 1 ) new=c(as.vector(new$new))
 else if( type == 2 ) new=matrix(as.vector(new$new),nr)

 # freq
 freqnew=c( tapply( new, new, function(x) length(x) ) ) # This keeps the names!


 if( print ){
  tt=c("vector", "matrix"); n2oin1=c("not given","given")
  cat("\nRenaming the elements of old using the table given in n2o.\n")
  cat(" type of old =", tt[type], "\n")
  cat(" total # of elements in old =", nr, "\n")
  cat(" # of unique elements of old =", n, "\n")
  cat(" Conversion table from old to in is ",n2oin1[n2oin+1],".\n")
  if( n2oin == 0 ) cat(" sort option =", sort, "\n")
  cat(" Conversion table\n")
  Print(n2o)
  cat(" Input (old) and Output (new) \n")
  res=data.frame(matrix(old[,-ncol(old)],nr),new, stringsAsFactors=0)
  if( type == 1 ) colnames(res)=c("old","new")
  else colnames(res)=c(paste("old",1:nc,sep=""),paste("new",1:nc,sep=""))
  print(res)
 }

 o2n=n2o[order(n2o$old),]
 res=list( new=new, n2o=n2o, o2n=o2n, freqnew=freqnew, od=order(new) )
 return( res )

} # end of new_name

#' Cartesian product of many vectors and matrices
#'
#' @param ... vectors, factors or matrices \cr
#' Or  ... can be a list containing the levels vectors as its elements.
#' @param rev = 0 to make the fist elements of ... change first
#' @param as.data.frame = 1 to make the result a data frame.
#' @param rownames = 1 to return the compressed version as the rownames.
#' @param colnames = 1 to use the names of ... as the colnames.
#'
#' @return A matrix or data frame.
#'
#' @details When all the inputs are vectors, \code{cprod} is equivalent to
#' \code{expand.grid1}.\cr
#' 
#' When both \code{A} and \code{B} are matrices, 
#' \code{cprod( A, B )} is equivalent to
#' \preformatted{
#'   Reduce( rbind, lapply(as.data.frame(t(A))
#'        ,  function(x) cbind(repmat(matrix(x,1,ncol(A)),nrow(B)),B)) )
#' }
#'
#' @examples
#' cprod(1:2,3:5)
#' cprod(1:2,3:4, rev=0)
#' cprod(matrix(1:6,3,2, byrow=1),1:2)
#' cprod(matrix(1:6,3,2, byrow=1),matrix(1:4,2,2,byrow=1))
#'
#' # when ... is a list
#' ll <- list(a=c("a","b"),bbb=1:3)
#' cprod(ll, colnames=0)
#' cprod(ll, colnames=1)
#'
#' # ANOVA data
#' levels <- c(2,3)
#' ll <- lapply(levels,function(x)seq(1:x))
#' names(ll) <- c("A","B")
#' data <- cprod(ll, colnames=1)
#'
#' # generate 3D grid of x0
#' x0 <- 1:3
#' ndim <- 3
#' x <- cprod( rep(list(x0),ndim) )
#' Print(x)
#'
#' @export
#'

cprod <- function( ..., rev=1, as.data.frame=0, rownames=0, colnames=0 ){
 # cartesian product of many vectors
 # Shin-ichi Mayekawa
 # 130211
 # old version (111121,27,130110) is obsolete.
 # remove NULL elements from ...: 130219
 # comments: 20140221
 # matrix input: 20140222,23
 # bugfix: 20140815 dnc
 # bugfix: 20150626,0712c
 # bugfix for cprod(1,1): 20150729
 # list input: 20150729
 # name=name<- fixed: 20150803
 # arg type removed: 20150804
 # character mat: 20200723
 #
 # Args:
 #  x1, x2, ..., xq  vectors of length n1, n2, ..., nq
 #
 #  rev              = 0 if x1 varies first
 #  rownames         = 1 to return the compressed version as the rownames
 #  colnames         = 1 to use names of ... as the column names
 #  as.data.frame    = 1 to make the result a data frame
 #
 # Value:
 #  If several vectos are given:
 #   (n1 x n2 x ... x nq) x q  matrix of the cartesian products
 #    where x1 varies the fastest when rev=F.
 #

 name <- as.character(substitute(list(...)))[-1L]
 val=list(...)
 locnull=which(unlist(lapply(val,is.null)))
 # Print(name,val,locnull)
 if( length(locnull) > 0 ){
  name=name[-locnull]
  val=val[-locnull]
 }
 if( is.list(val[[1]]) ){
  name=names(val[[1]])
  # if( colnames == 0 ) name=paste("Var",1:length(name), sep="")
  val=val[[1]]
  names(val)=NULL
 }
 nval=length(val)

 # Print(name, val, nval)

 # mark matrix elements of ... and replace them by index vectors
 # mat has non null dimensions which do not contain 1
 locmat0=!unlist(lapply(lapply(val,dim),function(a) any(a==1) | is.null(a)))
 locmat=which(locmat0)
 nmat=length(locmat)

 matlist=list(); ncollist=0; colnameslist=list()
 if( nmat > 0 ){
  matlist=val[locmat]
  val[locmat]=lapply(matlist,function(a) seq(1:nrow(a)))
  ncollist=unlist(lapply(matlist,ncol))
  colnameslist=lapply(matlist,colnames)
 }

 #Print(matlist,val,name)

 # Here val contains vectors
 if( rev != 1 ){
  res=expand.grid( val )
  if( !as.data.frame )
   res=as.matrix(res,nrow(res))
 }
 else{
  res=t( apply( expand.grid( rev(val) ), 1, rev ) ) # This is a matrix.
  colnames(res)=rev(colnames(res))

  #  if( nrow(res) == 1 ){
  #   res=t(res); colnames(res)="Var1"
  #  } # 20150729
  if( as.data.frame )
   res=data.frame(res)
 }

 # if( rev ) colnames(res)=rev(colnames(res))
 if( colnames == 1 ){
  # if( rev ) name=rev(name)
  colnames(res)=name
 }

 rownames(res)=1:nrow(res)
 if ( rownames == -1 )
  rownames(res)=NULL
 else if( rownames == 1 )
  rownames(res)=apply(res,1,function(a){paste(a,collapse=":")})

 # restore matrix elements
 if( nmat > 0 ){
  res1=as.data.frame( matrix(0,nrow(res),ncol(res)+sum(ncollist)-nmat) )
  nextcol=1; nextmat=1
  for( j in 1:nval ){
   if( locmat0[j] ){
    res1[,nextcol:(nextcol+ncollist[nextmat]-1)]=
     matlist[[nextmat]][as.integer(res[,j]),]
    nextcol=nextcol+ncollist[nextmat]
    nextmat=nextmat+1
   }
   else{
    res1[,nextcol]=res[,j]
    nextcol=nextcol+1
   }
  }
  res=res1
 }

 if( all(sapply(res,mode) == "numeric") ||
     all(sapply(res,mode) == "character") ) res=as.matrix(res) # 20200723

 return( res )

} # end of cprod

#' Check If the Objects Listed Exist in the Parent Frame
#'
#' @param varname A string vector containing object names
#' @param ntogoback n for \code{parent.frame}
#' @param mode A string vector containing what to look for.
#' @param get = 1 to retrieve the value of those objects.
#' @param debug = 1 to print intermediate results
#'
#' @return
#'  If \code{get=0}: \cr
#'   \code{ex} A vector indicating the existence of each object. \cr
#'  If \code{get=1}: \cr
#'   A list of \code{ex} and a list, \code{value}, which contains
#'   the values of the objects found.
#'
#' @examples
#' a=0; b=0; c=0; d=0; e=0
#' rm(a,b,c,d,e)
#'
#' a="A0"
#' b="B0"
#' d=function(a){print(a)}
#' Exists( varname=c("a","b","c","d","e") )
#' Exists( varname=c("a","b","c","d","e"), get=1 )
#' Exists( varname=c("a","b","c","d","e"), get=1, mode="function" )
#'
#' func1 <- function(){
#'  a="A1"
#'  b="B1"
#'  res=Exists( varname=c("a","b"), get=1 )
#'  Print(res)
#'  res=Exists( varname=c("a","b"), get=1, ntogoback=2 )
#'  Print(res)
#' }
#' func1()
#'
#'
#' @export
#'

Exists <- function( varname, ntogoback=1
                  , mode=c("numeric", "character","logical")
                  , get=0, debug=0 ){
 # get varname from the calling directory
 # Shin-ichi
 # old test_parent2: 20170920,20221112cot
 # modified as Exists: 20230509dnc
 #

 # nvar
 nvar=length(varname)

 if( debug ){
  # parent (calling, not enclosing) env
  sysparent=sys.parent(n=ntogoback)
  parentframe=parent.frame(n=ntogoback)

  Print(sysparent)
  cat("parent environment given by parent.frame(n=",ntogoback,") is "
      , sep="")
  print(parentframe)
  Print(varname,nvar)
 }

 value=vector("list",length=nvar)
 names(value)=varname
 ex=numeric(nvar)
 names(ex)=varname
 for( i in 1:nvar ){

  temp=varname[i]
  exi=exists( temp, where=parent.frame(n=ntogoback) )
  if( exi ){
   temp2=get(temp,pos=parent.frame(n=ntogoback))
   if( mode(temp2) %in% mode ){
    if( get == 1 ) value[[i]]=temp2
    ex[i]=TRUE
   }
   else{
    ex[i]=FALSE
   }
   rm(temp2)
  } # end of exists

 } # end of variable name loop

 if( debug ){
  Print(ex)
  Print(value)
 }

 if( get != 1 ) return( ex )
 else return( named_list(ex,value) )

} # end of Exists

#' Convert the columns of a data frame to a specific type/class.
#'
#' @param X A Data Frame or a Matrix
#' @param what A character vector of class names, or "ANY" to match any class
#' \cr where class names can be "factor", "character", "numeric" etc.
#' @param func The function to be used for the conversion.
#'
#' @details
#' This function returns \cr
#' \code{ rapply( X, func, classes=what, how="replace" ) }
#' \cr\cr
#' The same can be obtained by:\cr
#' \code{ loc <- which( sapply(vartype(X), "\%in\%", what ) ) }\cr
#' \code{ X[loc] <- lapply( X[loc], func ) }\cr
#'
#' @examples
#' A <- demomat(5,3,root="")
#' Af <- as.data.frame(A)
#' Af[,2]=as.numeric(Af[,2])
#' str(Af)
#'
#' Ac <- convert( Af )
#' str(Ac)
#'
#' Ac <- convert( Af, what=c("factor","numeric") )
#' str(Ac)
#'
#' @return X with the converted values.
#'
#' @export
#'

convert <- function( X, what="factor", func=as.character ){
 # convert all factors to character
 # Shin-ichi Mayekawa
 # 20170917
 #

 if( length(dim(X)) != 2 ){
  cat("\n\nerror1:(convert) Input must be a matrix or data frame.\n\n")
  stop()
 }
 else if( is.matrix(X) ) X=as.data.frame(X)
 X=rapply( X, func, classes=what, how="replace" )
 return( X )
} # end of convert

#' Check if the Partial Match is correct.
#'
#' @param fargs1 as sys.call() formal arguments
#' @param fargs2 as match.call()  result of the matching
#' @param print = 1 or 2 to print the details.
#'
#' @details
#' This function checks if the arguments listed in ... are correctly
#' matched to the formal arguments of teh called function. \cr
#' The error may occur when ... contains single-lettered arguments.
#' \cr
#' This function should be called at the beginning of any function
#' which has ... as its formal argument.
#'
#' @examples
#' test1 <- function( alpha=0, beta=0, gamma=0, ... ){
#'  #
#'  cat("\nTop of test1\n")
#'
#'  check_partial_match( sys.call(), match.call() )
#'
#'  cat("\n Back to test1\n")
#'  Print(alpha,beta,gamma)
#'  val=list(...)
#'  cat("\n ... contains below:\n")
#'  print(val)
#'  cat("\nBottom of test1\n")
#'
#' } # end of test1
#'
#'
#' test2 <- function(a){
#'   res=test1( alpha=1, b=rep(9,3), d=2,xx=4, g=8, print=1 )
#' } # end of test2
#'
#'
#' test3 <- function(a){
#'   res=test1( alpha=1, beta=2, gamma=3, b=rep(9,3), d=2,xx=4, g=8, print=1 )
#' } # end of test2
#'
#'
#' test3()
#'
#' # If run, below caues error.
#' # test2()
#'
#' @export
#'

check_partial_match <- function( fargs1, fargs2, print=0 ){
  # check if the formal args match the ones in three dots
  # Shin-ichi Mayekawa
  # 20220624,25
  #
  #
  
  fargs11=deparse(fargs1)
  fargs22=deparse(fargs2)
  
  fargs111=names(as.list(fargs1))[-1]
  fargs222=names(as.list(fargs2))[-1]
  
  
  callingfunc1 = as.list(sys.call(-1))[[1]]
  callingfunc2 = as.list(sys.call(-2))
  if( length(callingfunc2) == 0 ) callingfunc2="global env."
  else callingfunc2=callingfunc2[[1]]
  calledfunc = as.list(sys.call())[[1]]
  parent_formals <- formals(sys.function(sys.parent(n = 1)))
  pfargs <- names(parent_formals)
 
  if( print >= 2 ){
    Print("\n\nin check_partial_match")
    Print(callingfunc1,callingfunc2,calledfunc)
    print(paste(callingfunc1, "  was calling  ", calledfunc, sep=""))
    print(paste(callingfunc2, "  was calling  ", callingfunc1, sep=""))
    parent_formals <- formals(sys.function(sys.parent(n = 1)))
    Print(pfargs, fargs1, fargs2)
  }
  
  if( print >= 1 ){
    cat("\nThe Formal Argument of ", callingfunc1, " is "
        , paste(pfargs, collapse=", "),"\n")
    cat("From ", callingfunc2,", ", callingfunc1, " was called as:\n"
        , fargs11,"\n" )
    cat("In ", callingfunc1, " formal arguments are matched as\n"
        , fargs22, "\n")
  }
  
  # Print(fargs111,fargs222, pfargs)
  
  # remove positional arguments at the begining
  if( any(fargs111 == "") ) fargs111=fargs111[-which(fargs111 == "")]
  locname=which(!(fargs111 %in% fargs222))
  # Print(locname)
  
  if( length(locname) > 0 ){
    locname2=which( substr(pfargs,1,1) %in% fargs111[locname] )
    cat("\nerror1:(check_partial_match)\n")
    cat(" When ", callingfunc1," was called from ", callingfunc2, "as\n")
    cat("  ", fargs11,"\n")
    cat(" the partial matching of the arguments in ... listed below failed!!")
    cat("\n ", fargs111[locname], "\n" )
    cat(" Specify the formal arguments whose first character matches"
        , "the above, namely,\n  ", paste(pfargs[locname2], collapse=", ")
        , "\n by their name explicitly when calling ", callingfunc1, ".\n")
    stop()
  }
  
  
} # end of check_partial_match

#' Frequency distribution of univariate numeric data with case weight
#'
#' @param x numeric vector or column vector
#' @param weights case weight
#' Can be a scalar, vector of size length(x) or matrix of length(x) x nf.\cr
#' If nf > 1, frequency for each column of weights will be calculated.
#' @param npoints # of mid-points
#' @param min minimum value of mid-points
#' @param max maximum value of mid-points
#' @param prob = 1 to return relative frequency in stead of frequency
#' @param midpoints vector of predefied mid-point values
#' @param title tile to be used in plot
#' @param maxf max value of frequency for plot
#' @param wid Format of mid-point value (wid.dig)
#' @param dig Format of mid-point value (wid.dig)
#' @param digits Format for mid-points
#' @param print = 1 to print result
#' @param plot = 1 to plot historgam
#' @return A list consisting of\cr
#'   midpoints =  midpoints \cr
#'   freq =       column vector of frequency at midpoints \cr
#'   breaks =     breaks associated with the midpoints \cr
#'   class =      classification of each obs into the midpoints \cr
#'   nobs =       sum of case weights \cr
#'   error =      error flag
#' @details
#'   (npoints, min, max)  defines the mid-points. \cr
#'    If min or max is NULL, min(x) and max(x) will be used. \cr
#'    midpoints has priority over (npoints, min, max).\cr
#' The out of range observations,  that is, those x satisfying \cr
#'  x < midpoints[1]     or   midpoints[npoints] < x \cr
#'  will be be included in midpoints[1] and midpoints[npoints], resp. \cr
#' \cr
#' Try this to see how the obs on the border are classified. \cr
#'    xx=c(1,2,3,4,5,6,7) \cr
#'    brks=c(0,2,4,6,8) \cr
#'    cc=cut(xx,brks) \cr
#' @examples
#' set.seed(1701)
#' n <- 10000
#' npoints <- 21
#' x <- rnorm(n)
#' res <- freqdist1( x, npoints=npoints,min=-3,max=3, print=1, plot=1 )
#' res1 <- freqdist1( x, midpoints=res$midpoints, print=1 )
#' weight=c(0,1,3,4,2,2,1)
#' freqdist1(seq(-3,3,1),weight, min=-3,max=3,npoints=7, print=1,plot=1)
#' rm(x)
#'
#' @export
#'

freqdist1 <- function( x, weights=1, npoints=7, min=NULL, max=NULL, prob=0
                       , midpoints=NULL, wid=6, dig=1, maxf=NULL
                       , title=NULL, digits=3, print=0, plot=0 ){
 # univariate freq dist of x using midpoints or (npoints, min, max)
 # Shin-ichi Mayekawa
 # 20121026
 # out-of-range observations: 121219
 # return freq: 130218
 # weights with xtabs: 130218
 # title as an arg: 20140417
 # formatC midpoints, maxf: 20140418
 # bugfix for the min/max of breaks: 20140418
 # print categorization: 20140418
 # print # of obs: 20140424
 # prob: 20140424
 # bug fix: 20140523
 # new print method for categorization: 20140524
 # return classification: 20140524
 # order of the arguments: 20140525
 # matrix weights: 20140813,14
 # bug fix for weights: 20150605
 # nobs added to return list: 20150608
 #
 # Args:
 #
 #    x           a variable to be tabulated
 #    weights     assosiated weight vector or 1
 #                can be a matrix of multiple columns
 #
 #    (npoints, min, max)    definition of midpoints
 #               If min or max is NULL, min(x) and max(x) will be used.
 #
 #    midpoints   midpoints of x
 #                If midpoints is NULL, (npoints, min, max) will be used.
 #
 #     prob      = 1 to plot probability, not frequency
 #     maxf      = maximum value of the vertical axis
 #
 #     (wid, dig) formatC for the midpoints in plot
 #
 #    title      plot title
 #
 # Values:
 #
 #   midpoints   midpoints
 #   freq        column vector of frequency at midpoints
 #   breaks      breaks associated with the midpoints
 #   class       classification of each obs into the midpoints
 #   nobs        sum of weights
 #
 #
 # Note:
 #  The out of range observations,  that is, those x satisfying
 #     x < midpoints[1]     or   midpoints[npoints] < x
 #  will be be included in midpoints[1] and midpoints[npoints], resp.
 #
 #
 ##
 # try this to see how the obs on the border are classified.
 # xx=c(1,2,3,4,5,6,7)
 # brks=c(0,2,4,6,8)
 #  cc=cut(xx,brks)
 ##
 #

 # x must be a vector
 if( !is.vector(x) ){
  if( ncol(x) > 1 ){
   cat("\nerror1:(freqdist1) Input must be a vector. Use freqdist.\n")
   return()
  }
  else x=c(x)
 }

 # length
 n=length(c(x))

 # midpoints
 if( is.null(min) ) min=min(x)
 if( is.null(max) ) max=max(x)
 if( is.null(midpoints) ){
  midpoints=seq(min,max,length.out=npoints)
  width=midpoints[2]-midpoints[1]
 }
 else{
  npoints=length(c(midpoints))
  width=NA
 }


 # breaks
 breaks=c(midpoints[2:npoints],midpoints[npoints]+1)
 breaks=(breaks+midpoints)/2
 breaks[npoints]=max(max(breaks),max(x))+0.5
 breaks=c(min(min(breaks),min(x))-0.5,breaks)


 # weights
 if( is.null(weights) ) weights=matrix(1,n,1)
 else if( length(weights) == 1 ) weights=matrix(weights,n,1)
 else if( is.matrix(weights) ){
  if( nrow(weights) != n ){
   cat("\nerror(freqdist1): invalid weight matrix.","\n")
   return()
  }
 }
 else if( is.vector(weights) ){
  if( length(weights) != n ){
   cat("\nerror(freqdist1): invalid weight vector.","\n")
   return()
  }
  else{
   weights=matrix(weights,,1)
  }
 }
 nobs=colSums(weights)
 nf=ncol(weights)

 # categorize the observations  by cutting them at the break points
 ccc=cut( x, breaks=breaks, labels=format(midpoints,digits=digits)
          , include.lowest=1)

 # freq dist with weights
 f=xtabs( weights ~ ccc )

 # convert factors to level labels
 ccc=as.numeric( levels(ccc)[ccc] )
 # Print(ccc)

 #Print("***",weights,ccc,f)

 # f=table(cut(x, breaks=breaks, labels=format(midpoints,digits=digits)
 #           , include.lowest=1))

 freq=as.matrix(f); colnames(freq)=paste("freq",1:nf,sep="")
 # colnames(f)="freq"

 # if( sum(f) == length(c(x)) ) error=0
 if( sum(f) == sum(weights) ) error=0
 else{
  error=1
  cat("\n\nerror1(freqdist1): Sum of freq does not equal to n. \n\n")
 }

 if( print > 0 ){
  cat("\nFrequency Distribution\n")
  if( !is.null(title) ) print(title)
  Print(n,nobs,"/",npoints,min,max,width)

  left=breaks[-npoints]
  left[1]=NA
  left[npoints]=breaks[npoints]
  right=breaks[-1]
  right[npoints]=NA
  left=formatC(left, width=wid, digits=dig, format="f")
  right=formatC(right, width=wid, digits=dig, format="f")
  mp=formatC(midpoints, width=wid, digits=dig, format="f")
  #  char=paste(left, "  <  ", mp, "  <=  ", right, "       ", freq, sep="")
  char=cbind(left, "  <  ", mp, "  <=  ", right, "       ", freq)
  char=rbind(
   c("left brk","   "," middp","    ","right brk", "   ", colnames(freq))
   , char)
  char=matrix(apply(char,1,paste,collapse="  "),,1)
  char=format(char,justify="left")  # centre

  cat("\nCategorization of Variable:\n")
  write.table(char, row.names=FALSE, col.names=FALSE, quote=FALSE)
  #  if( print >= 4 )  Print(x,as.matrix(c),weights)
 }

 if( plot > 0 ){
  if( is.null(title) ) title="Data"
  mp=midpoints
  if( prob == 1 ) f=f/nobs
  if( !is.null(wid) ) mp=formatC(midpoints, width=wid, digits=dig, format="f")
  for( j in 1:nf ){
   ff=freq[,j]
   if( is.null(maxf) ) maxf=max(ff)
   par(new=FALSE)
   barplot(height=ff, names.arg=mp, ylim=c(0,maxf)
     , main=paste("Histogram of ", title, ":   # of obs = ", nobs[j], sep="")
     , sub=paste( "# of midpoints = ", npoints
     , ",  min = ", midpoints[1], ",   max = ", midpoints[npoints]
                        , ",   width =", width
                        , sep="")
           , offset=0  )
  }

 }
 return( list( midpoints=midpoints, freq=freq, breaks=breaks
               , class=ccc, nobs=nobs, error=error ) )

} # end of freqdist1

#' Replace Characters by New Ones
#'
#' @param from A character string to be replaced by "to".
#' By default, there are no need to escape special characters.
#' @param to A character string to replace "from".
#' @param char A character vector where replacement occur.
#' @param n # of times to replace
#' @param start The location from which the replacement starts.
#' @param fixed A parameter to be passed to gregexpr and gsub.
#' @param ... Additional parameters to gregexpr and gsub.
#'
#' @details Do not use regular expressions in from such as [a-b] etc. \cr
#' When from does not contain regular expression, \cr
#' fixed=0 and ignoe.case=1 \cr
#' should work.
#'
#' @return A string with replacement.
#'
#' @examples
#' char <- paste( paste(letters,collapse=""),"0123456789",'"'
#'           , "'`!#$%&'()[]{}<>*+-,.;:/^~=_|", sep="", collapse="" )
#' lchar <- nchar(char)
#' char <- paste(char,char,char,sep=" ",collapse="")
#'
#'
#' # one to one
#' replacec( "b", "B", char )
#' replacec( "b", "B", char, 2 )
#' replacec( "b", "B", char, 1, 135)
#'
#' # many to one
#' replacec( "bc", "B", char )
#' replacec( "bc", "B", char, 2 )
#' replacec( "bc", "B", char, 1, 135 )
#'
#' # one to many
#' replacec( "b", "BC", char )
#' replacec( "b", "BC", char, 2 )
#' replacec( "b", "BC", char, 1, 135 )
#'
#' # to null string
#' replacec( "b", "", char, 99 )
#'
#' # special characters
#' replacec( "(", "[paren]", char )
#' replacec( "^", "[hat]", char )
#' replacec( "!#$", "[esd]", char )
#'
#' # ignore.case
#' replacec("B","[B]",char, fixed=0, ignore.case=1 )
#'
#' @export
#'

replacec <- function( from, to, char, n=1, start=1, fixed=1, ... ){
 #  Replace the occurence of "from" in "char" by "to".
 #  Search starts at column "start" and ends when "n" replacements
 #  have accomplished.
 # Shin-ichi Mayekawa
 # original SAS macro/iml version: 930623-040622
 # 20150620,21
 #

 lchar=nchar(char)

 # error check
 if( start < 0  |  lchar < start ){
  cat("\n\nerror1:(replace) start out out range.\n")
  Print(lchar, start)
  return()
 }
 if( n <= 0 ){
  cat("\n\nerror1:(replace) n out of range.\n")
  Print(lchar, n)
  return()
 }
 if( nchar(from) <= 0 ){
  cat("\n\nerror1:(replace) from must contain at least one chracter.\n")
  Print(nchar(from), from)
  return()
 }

 loc=gregexpr( from, char, fixed=fixed, ... )[[1]]
 loc=loc[loc >= start]
 if( length(loc) == 0 ) return(char)
 if( length(loc) > n ) loc=loc[1:n]
 lloc=length(loc)
 mid=substr(char,loc[1],loc[lloc]+nchar(from)-1)
 # Print(loc,lloc,mid, nchar(from))
 mid=gsub( from, to, mid, fixed=fixed, ... )
 top=NULL; tail=NULL
 if( loc[1] > 1 ) top=substr(char,1,loc[1]-1)
 if( loc[lloc] < lchar ) tail=substr(char,loc[lloc]+nchar(from),lchar)
 res=paste(top,mid,tail, sep="", collapse="" )

 return( res )

} # end of replacec

#' Merge two data frames and return the result with the contribution indices
#'
#' @param x First data frame.
#' @param y Second data frame.
#' @param ... parameters to the native merge frunction.
#' @param .inx. The name of the contribution index for x.
#' @param .iny. The name of the contribution index for y.
#'
#' @details
#' The SAS like contribution indices will be returned as the variables
#' of the merged data frame.
#'
#' In the followint example, those observations with \code{in.x=1} are
#' contributed from data frame x, and those with \code{in.y=1}, from y.
#'
#' @examples
#' d1 <- data.frame(x=c(11:15, 17:20),y=c(111:115, 117:120))
#' d1[2,2] <- NA
#' d2 <- data.frame(x=c(11:13, 16, 18:21),y=c(211:213, 216, 218:221))
#' d2[5,2] <- NA
#' d12 <- merge2(d1,d2, by="x", all=TRUE)
#' printm( d1, d2, d12 )
#'
#' D12 <- merge2(d1,d2, by="x", all=TRUE, .inx.="IN1")
#' printm( d1, d2, D12 )
#'
#' @export
#'


merge2 <- function( x, y, ..., .inx.="in.x", .iny.="in.y" ){
 # merge two data frames and return the result with the contribution indices
 # Shin-ichi Mayekawa
 # 20220908
 #

 D12=merge( data.frame(x, ..id..x=1:nrow(x) )
          , data.frame(y, ..id..y=1:nrow(y) )
          , ... )
 in1=as.integer(!is.na(D12$..id..x))
 in2=as.integer(!is.na(D12$..id..y))

 D12=D12[,!(colnames(D12) %in% c("..id..x","..id..y"))]
 cname=colnames(D12)
 D12=data.frame(D12,in1,in2)
 colnames(D12)=c(cname, .inx., .iny.)

 return( D12 )

} # end of merge2

#' Create a named list
#'
#' @param ... elements of the list to be created
#'
#' @details When ... contains objects with valid variable names,
#' those names will be used as the names of the objcts in the list.\cr
#'
#' @examples
#' a=1; b=1:2
#' named_list( a, a1=a, b, 3:4, d=5:6, "E" )
#'
#' @return A list.
#'
#' @export
#'

named_list <- function( ... ){
 # create a named list from ...
 # Shin-ichi Mayekawa
 # 20150803
 # bugfix: 20170513
 #

 # analyze ...
 namval=analyze_three_dots(...)

 if( length(namval) == 0 ) return(NULL)

 name=namval$name
 val=namval$val
 # Print(name,val)

 # values
 res=list()
 for( i in 1:length(val) ){
  res=append( res, list( val[[i]] ) )
 }

 # Print(name, !is.varname(name), val, names(val),length(names(val)) )
 # names
 nam=name
 if( length(names(val)) > 0 )
  nam[!is.varname(nam)]=names(val)[!is.varname(nam)]
 nam[!is.varname(nam)]=""

 names(res)=nam

 return( res )

} # named_list




#' Retrieve Objects from a Named List
#'
#' @param nl A named list
#'
#' @details
#' This function defines or replaces the objects in the parent frame. \cr
#'
#' mapply( assign, names(nl), nl, MoreArgs=list(envir=parent.frame()) )
#'
#'
#' @examples
#'
#' # in global env.:
#' a=1; a1=2; b=1:2
#' nl=named_list( a, a1=a+3, b, 3:4, d=5:6, "E" )
#' rmv(a,b,temp_var4,d,E)
#' from_named_list( nl )
#' Print(a,a1,b,temp_var4,d,E)
#'
#' # in a function:
#' f <- function(){
#'  nl=list(a=1,b=2)
#'  from_named_list( nl )
#'  Print("in function f:",a,b)
#' }
#' f()
#'
#' @export
#'

from_named_list <- function( nl ){
 # create a named list from ...
 # Shin-ichi Mayekawa
 # 20150803
 # bugfix: 20170513
 #

 name=names(nl)
 if( is.null(name) &&  is.vector(nl) ) return( nl )
 if( is.null(name) &&  is.list(nl) ){
  cat("\nerror1(from_named_list) The input list does not have name.\n")
  return( NULL )
 }

 nn=length(name)
 nam=paste("temp_var",1:nn, sep="")
 nam[is.varname(name)]=name[is.varname(name)]

 # ev=environment()
 # pev=parent.env(ev)
 pf=parent.frame()

 return( invisible( mapply( assign, nam, nl, MoreArgs=list(envir=pf) ) ) )

} # from_named_list

#' Joint Frequency Distribution of Discrete Variables
#'
#'
#' @param X Input multivariate discrete data
#' @param weight weight vector or 1
#' @param omitmiss = 0 to return the cartesian product of
#' unique values of each column of X.
#' @param sort = 1 to sort so that the last column changes first. \cr
#' = 2 to sort by frequency in ascending order \cr
#' = 3 to sort by frequency in descending order \cr
#'
#' @details
#' Calculates the frequency of the unique rows of X matrix. \cr
#' Note, however, the order of the rows are different from
#' the reslt of unique(X) which does not sort the result. \cr
#' If sort=0, the first column changes first, otherwise, the last
#' column changes first.
#'
#' @return
#' A list of: \cr
#' Xu An object consisting of the (sorted) unique rows of X \cr
#' f  Assosiated frequency vector
#'
#' @examples
#' # numeric varaibles
#' set.seed(1701)
#' X <- matrix(sample(0:1,50,replace=1),25)
#' X <- cbind(X,matrix(sample(1:2,50,replace=1),25))
#' X <- rbind(X,matrix(c(0,1,0,1),5,4,byrow=1))
#' colnames(X) <- paste("ccc",1:4,sep="")
#' res <- jfreq( X )
#' Print( res$Xu, res$f )
#'
#' # sort so that the last column changes first
#' res1 <- jfreq( X, sort=1 )
#' Print( res1$Xu, res1$f )
#'
#' # sort by frequency
#' res1 <- jfreq( X, sort=3 )
#' Print( res1$Xu, res1$f )
#'
#' # mixed variable types in a data.frame
#' Y <- as.data.frame(X)
#' Y[,2] <- letters[Y[,2]+1]
#' res2 <- jfreq( Y )
#' Print( res2$Xu, res2$f )
#'
#' @export
#'

jfreq <- function( X, weight=NULL, omitmiss=1, sort=0 ){
 # joint frequency
 # Shin-ichi Mayekawa
 # 20161106c
 # sort: 20161116
 # bugfix: 20161202
 # sort by freq: 20170917
 #

 # error
 if( length(dim(X)) > 2 ){
  cat("\nerror1(jfreq): Array object cannot be handled.\n\n")
  return()
 }

 # num or char
 vt=vartype(X)

 # const
 mode="df"
 if( is.matrix(X) | is.data.frame(X) ){
  cn=colnames(X)
  nvar=ncol(X)
  if( is.matrix(X) ){
   mode="mat"
   vt=rep(vt,nvar)
  }
 }
 else{
  cn=NULL
  nvar=1
  X=matrix(X,,1)
  mode="vec"
 }

 #  f=xtabs(formula = weights ~ X[,1] + X[,2] + ..., drop.unused.levels = 0)
 if( is.null(weight) ) wt=""
 else wt="weight"
 model=paste( wt," ~ X[,1]", sep="" )
 if( nvar > 1 ){
  for( j in 2:nvar ) model=paste(model,"+ X[,",j,"]", sep="")
 }
 code=paste("f=xtabs( ", model, ", drop.unused.levels=FALSE )", sep=" ")
 eval(parse(text=code))

 # analyze the result of xtabs
 namesf=dimnames(f) # This is the formatted value of X by xtabs.
 dimf=dim(f)
 midpindex=as.matrix( expand.grid(lapply(dimf,function(a) seq(1:a))) )
 dim(f)=prod(dimf)
 f=unclass(as.matrix(f)) # removing the table attribute

 if( omitmiss ){
  loc=which( f > 0 )
  f=f[loc]
  midpindex=midpindex[loc,,drop=0]
 }

 # Print(namesf)
 # mm=mapply( function(a,i){ a[midpindex[,i]] }, namesf, 1:nvar, SIMPLIFY=1 )
 # Print(mm)

 # values as character
 Xu=as.data.frame(
  mapply( function(a,i){ a[midpindex[,i]] }, namesf, 1:nvar, SIMPLIFY=1 )
  , stringsAsFactors=0  )
 colnames(Xu)=cn

 # convert back to the original variable type
 for( j in 1:nvar ){
  if( vt[j] == "numeric" ) Xu[,j]=as.numeric(Xu[,j])
 }

 # sort
 if( sort > 0 ){
  if( sort == 1 ){
   # sort so that the last col changes first
   Xu <- sortit( data.frame( Xu, f, stringsAsFactors=0 ), cols=1:nvar )
  }
  else if( sort == 2 ){
   Xu <- sortit( data.frame( Xu, f, stringsAsFactors=0 ), cols=nvar+1 )
  }
  else if( sort == 3 ){
   Xu <- sortit( data.frame( Xu, f, stringsAsFactors=0 ), cols=-(nvar+1) )
  }
  f=Xu[,nvar+1]
  Xu=Xu[,1:nvar]
 }

 # remove rownames
 rownames(Xu)=NULL

 # back to original mode
 if( mode == "vec" ){
  Xu=as.vector(Xu)
 }
 else if( mode == "mat" ){
  Xu=as.matrix(Xu)
 }

 return( list( Xu=Xu, f=f ) )

} # end of jfreq






#' Merge Tables or Frequency Distributions of the form (score, freq)
#'
#' @param ... tables, matrices or data.frames containing frequency distribution
#' @param omitmiss = 1 to remove zero-frequency cells
#'
#' @details
#' Each of ... can be a class table object created by native table or xtabs,
#' or a data.frame or matrix containing the level-index and frequency. \cr
#' Each of ... should have the same # of factors to be tabulated.
#'
#'
#' @examples
#'
#' set.seed(1701)
#' x <- sample(1:8,10,replace=1); x <- c(x,c(1,1,7)); xc <- paste("x",x,sep="")
#' y <- sample(0:9,10,replace=1); y <- c(y,c(5,5,0)); yc <- paste("y",y,sep="")
#'
#' # univariate example
#' rest11 <- table( x )
#' rest12 <- rest11[-c(1,3)]
#' resm <- merge_tables( rest11, rest12 )
#' plot_freqdist( resm )
#'
#' # bivariate example
#' rest2 <- table( x, y )
#' fd21 <- table2freqdist( rest2, omitmiss=1, index=1 )
#' fd22 <- fd21[-c(1,10),]
#' merge_tables( rest2, fd21, fd22, omitmiss=1 )
#'
#' @return
#' A frequency distribution of the form (x,freq)
#'
#' @export
#'

merge_tables <- function( ..., omitmiss=0 ){
 # merge several frequency distributions
 # Shin-ichi Mayekawa
 # 20160729,0801,02,03
 # pat=2 in is.number: 20171026
 # bugfix for the condition has length > 1: 20220612 
 #

 # analyse ...
 name=name <- as.character(substitute(list(...)))[-1L]
 val=list(...)
 locnull=which(unlist(lapply(val,is.null)))
 if( length(locnull) > 0 ){
  name=name[-locnull]
  val=val[-locnull]
 }
 narg=length(val)

 # convert the input to freq dist format
 nf0=numeric(narg)
 for( i in 1:narg ){
   # below if is fixed by adding [1] to class.  20220612
   if( class(val[[i]])[1] %in% c("table","xtab") ){
   # Print("*****",i,val[[i]])
   val[[i]]=table2freqdist( val[[i]], omitmiss=omitmiss, index=1 )
   # Print("**",i,val[[i]],colnames(val[[i]]))
  }
  nfi=ncol(val[[i]])
  if( is.matrix(val[[i]]) ){
   # Print(colnames(val[[i]]))
   val[[i]]=as.data.frame(val[[i]], stringsAsFactors=0)
   if( nfi == 1 ) colnames(val[[i]])[1:(nfi-1)]=rep("",nfi-1)
   # Print(colnames(val[[i]]))
  }
  nf0[i]=nfi
  colnames(val[[i]])[nf0[i]]="freq"
 }

 if( any(nf0 != nf0[1]) ){
  cat("\nerror1(merge_tables): The sizes of input do not conform.\n")
  Print(nf0)
  return()
 }

  if( narg >= 3 ){
  # recursive call to merge_tables
  res=Recall( val[[1]], val[[2]], omitmiss=omitmiss )
  for( i in 3:narg ){
   # Print(i, res, val[[i]])
   res=Recall( res, val[[i]], omitmiss=omitmiss )
  }

  return( res )
 }


 ########################################################################
 # simple case starts here.
 ########################################################################
 fd1=val[[1]]; fd2=val[[2]]

 # Print(fd1,fd2,val[[1]],val[[2]])
 if( ncol(fd1) != ncol(fd2) ){
  cat("\nerror1(merge_tables): The sizes of input do not conform.\n")
  return()
 }
 nf=ncol(fd1)-1


 if( nf == 1 ){
  fd12=rbind(fd1,fd2, stringsAsFactors=0)
  # Print(fd12)
  fd=xtabs(fd12[,2]~fd12[,1])
  nam=as.character(fd12[,1])
  if( is.number(nam, pat=2) ) nam=as.numeric(nam)
  fd=data.frame(unique(nam),unclass(fd), stringsAsFactors=0)
  colnames(fd)=c(colnames(fd1)[1],"freq")
  rownames(fd)=NULL
  od=fd[,]
 }
 else{
  fd=merge( fd1, fd2, by=1:nf, all=TRUE )
  fdm1=fd[,1:nf]
  fdm2=fd[,(nf+1):ncol(fd)]
  fdm2[is.na(fdm2)]=0
  fd=data.frame(fdm1,rowSums(fdm2), stringsAsFactors=0)
  colnames(fd)[nf+1]="freq"
 }

 return( fd )

} # end of merge_tables






#' Convert an output of table or xtabs to a simple frequency table format
#'
#' @param tab An output of native table or xtabs function.
#' @param omitmiss = 1 to exclude the zero-freq cells.
#' @param index = 0 to output no level indices
#' @param pat  argument to is.number function.
#' @param sort = 0 not to sort the result
#'
#' @return
#' When index == 1 and there are nf factors(variables) to be tabulated, \cr
#' the output is a matrix or data.frame: \cr
#' the first nf columns of which consist of the level indices,
#' and the nf+1st column contains the frequency count. \cr
#'
#' If the levels are numeric, a matrix will be returned. Otherwise,
#' a data.frame. \cr
#' In both cases
#'   \code{rownames(fd)} contains the combinations of the level names.
#'
#' @examples
#' set.seed(1701)
#' x <- sample(1:8,10,replace=1); x <- c(x,c(1,1,7)); xc <- paste("x",x,sep="")
#' y <- sample(0:9,10,replace=1); y <- c(y,c(5,5,0)); yc <- paste("y",y,sep="")
#' z <- sample(5:9,10,replace=1); z <- c(z,c(5,5,9)); zc <- paste("z",z,sep="")
#'
#' rest1 <- table( x )
#' rest2 <- table( x, y )
#' rest3 <- table( xc,y,z )
#' restx3 <- xtabs( ~xc+yc+z )
#'
#' table2freqdist( rest1 )
#' table2freqdist( rest2 )
#' table2freqdist( rest3, omitmiss=1 )
#' table2freqdist( restx3, omitmiss=1, index=0 )
#'
#' tab2 <- table( x=sample( letters[1:5], 20, replace=1)
#'              , sample(letters[6:8], 20, replace=1) )
#' table2freqdist( tab2 )
#'
#' @export
#'

table2freqdist <- function( tab, omitmiss=0, index=1, pat=2, sort=1 ){
 # convert the output of table or xtabs to simple format
 # Shin-ichi Mayekawa
 # 20160731,0801,02,03
 # bugfix: 20171026
 # pat=1 -> pat=2: 20171026
 # sort: 20171026
 # varnames: 20180812
 #

 if( !any( class(tab) %in% c("xtabs", "table") ) ){
  cat("\n Input must be of class 'xtabs' or 'table'. \n")
  return()
 }

 if( index != 1 ) index=0

 # main body
 fd=matrix(tab,,1)
 dimnames=dimnames(tab)
 nf=length(dimnames)
 cnames=names(dimnames)
 loc=which(cnames == "")
 cnames[loc]=paste("v_",1:nf,sep="")[loc]

 names2=expand.grid(dimnames, stringsAsFactors=0)
 rownames(fd)=apply( names2, 1, paste, collapse=":" )
 colnames(fd)="freq"
 colnames(names2)=cnames

 if( index > 0 ){
  if( all(is.number(dimnames, pat=pat)==1) ){
   names2=matrix(t( apply( names2, 1, as.numeric ) ),,nf)
   colnames(names2)=cnames
   fd=cbind( names2, fd )
   # fd is a matrix
  }
  else{
   fd=data.frame( names2, fd, stringsAsFactors=0 )
   # colnames(fd)[1:nf]=""           # 20180812
   # above fd is a data.frame
  }
 }

 if( omitmiss == 1 ){
  fd=fd[fd[,nf*index+1] > 0,,drop=0]
 }

 # sort
 if( sort == 1 ){
  if( index == 1 ) fd=sortit( fd, cols=1:nf )
  else{
   od=order(rownames(fd))
   rn=rownames(fd)[od]
   fd=fd[od,,drop=0]
  }
 }

 return( fd )

} # table2freqdist




#' Plot Univariate Tables or Frequency Distributions of the form (score, freq)
#'
#' @param fd tables, matrices or data.frames containing
#'  univariate frequency distribution (midpoint,freq).
#' @param weight A vector of weights associated with fd[,1] \cr
#' @param main A string to be used as the main title.
#' @param numeric = 0 to plot w/o filling in the missing value of fd[,1].
#' @param space an option to be passed to native barplot (default=0) \cr
#' the value of which specifies the width of the spaces between bars.
#' @param las an option to be passed to native to barplot (default=2) \cr
#' = 0 to rotate h, = 1 to rotate none, = 2 to rotate x, = 3 to rotate both.
#' @param title An alias for main.
#' @param ... additional parameters to barplot.
#'
#' @details
#' If tab is a class table object created by native table or xtabs function,
#' \code{plot_freqdist(tab)} and \code{plot(tab)} are similar. \cr
#' 
#' If weight is given, (fd, weight) or(fd[,1], weight) will be considered as
#' the frequency distribution table. \cr
#' Native barplot is used with  space=0 and las=2 and ... options as \cr
#' \code{barplot( height=fd[,2], names.arg=as.character(fd[,1])}
#' \code{, main=main, space=space, las=las, ... )}
#' \cr
#' When numeric=1 is given (default),  \code{ min(fd[,1]):max(fd[,1]) }
#' will be used as the horizontal names and 0 will be substituted for
#' the missing frequencis. \cr
#'
#' @return The object from native barplot.
#'
#' @examples
#'
#' set.seed(1701)
#'
#' score <- 0:10
#' freq <- c(0,5,10,20,30,40,30,20,10,5,0)
#' fd <- cbind(score,freq)
#' plot_freqdist( fd, main="no missing value" )
#'
#' fd1 <- fd[-c(3,7),]
#' plot_freqdist( fd1, main="score=2, 6 are missing." )
#' plot_freqdist( fd1, numeric=0
#' , main="score=2, 6 are missing and numeric=0." )
#'
#' tab=table(score)
#' tab=tab[-c(3,7)]
#' tab[]=freq[-c(3,7)]
#' plot( tab, main="using native plot function" )
#' plot_freqdist( tab, main="from class table object" )
#'
#'
#' @export
#'

plot_freqdist <- function( fd, weight=NULL, main="Frequency Distribution"
                         , numeric=1, space=0, las=2, title=NULL, ... ){
 # plot univariate frequency distributions
 # Shin-ichi Mayekawa
 # 20160808cot
 # weight: 20161202
 # numeric: 20180814dnc,17
 # bugfix for the condition has length > 1: 20220612
 #

 if( !is.null(title) ) main=title

 if( class(fd)[1] %in% c("table","xtabs") ){
  fd=table2freqdist( fd, omitmiss=0, index=1 )
 }
 else if( is.vector(fd) ) fd=matrix(fd,,1)

 nf=ncol(fd)-1
 if( nf > 1 ){
  cat("\nerror1(plot_freqdist): Input must be univariate.\n")
  stop()
 }

 if( !is.null(weight) ){
  fd=cbind(fd[,1],weight)
 }

 # horizontal value/name and hight
 x=fd[,1]
 h=fd[,2]

 # numeric
 if( numeric == 1  && is.numeric(x) ){
  locobs=which( !is.na(x) )
  x=x[locobs]; h=h[locobs]
  xx=min(x):max(x)
  hh=numeric(length(xx))
  hh[xx %in% x]=h
  x=xx; h=hh
 }

 # main body
 bp=barplot(height=h, names.arg=as.character(x)
         , main=main, space=space, las=las, ...)


 return( invisible(bp) )

} # end of plot_freqdist




#' Compress Frequency Distribution by Aggregating Duplicate Entries
#'
#' @param x Vector of values (domain of a scalar variable, numeric or string)
#' @param freq Vector of frecuencies:
#'   \code{(x, freq)} is the frequency distribution.
#' @param fill = 1 to fill the domain
#' @param domain Domain of \code{x} to be filled. (coalescing)  
#'
#' @details
#' The purpose of this function is to remove the duplicate values of \code{x}
#' and adjusting the associated frequency count in \code{freq}.
#'
#' The core part of this function is:
#' \preformatted{
#'  fnew <- xtabs( freq ~ x )
#'  xnew <- names(fnew)
#'  res <- data.frame(x=xnew, freq=fnew)
#' }
#' and if \code{domain} and \code{fill} are given, the result will be
#' coalesced. (coalescing)  
#' Using the frequency table below
#' \preformatted{
#'  x  freq
#'  1  1
#'  2  2
#'  2  1
#'  3  1
#' }
#' \preformatted{
#'   compress_freqdist( x, freq, fill=1, domain=0:4)
#' }
#' returns
#' \preformatted{
#'  x  freq
#'  0  0
#'  1  1
#'  2  3
#'  3  1
#'  4  0
#' }
#'
#' @return
#' A data frame consisting of \cr
#' \code{x} Vector of unique values \cr
#' \code{freq} Vector of frequencies
#'
#'
#' @examples
#' x <- c(1,2,4,5,6, 2,4,6, 9 )
#' f <- c(1,2,2,1,2, 1,2,1, 1 )
#'
#' ( res <- compress_freqdist( x, f ) )
#' ( res <- compress_freqdist( x, f, fill=1, domain=0:10 ) )
#'
#'
#' xc <- letters[c(1,2,4,5,6, 2,4,6, 9 )]
#' f <- c(1,2,2,1,2, 1,2,1, 1 )
#' ( res <- compress_freqdist( xc, f ) )
#' ( res <- compress_freqdist( xc, f, fill=1, domain=letters[0:10] ) )
#'
#' @export
#'

compress_freqdist <- function( x, freq, fill=0, domain=NULL ){
 # compress freqdist of the form (x, freq)
 # Shin-ichi Mayekawa
 # 20230812
 # bugfix: 20230815dnc
 # within: 20230822
 #

 fnew=xtabs( freq~x )
 if( is.numeric(x) ) xnew=as.numeric( names(fnew) )
 else xnew=names(fnew)
 fnew=unclass(fnew)
 # printm( "*", xnew, fnew )
 res=data.frame(x=xnew, freq=fnew)
 if( fill  &&  !is.null(domain) ){

  #  res=merge2( res, data.frame(x=domain, freq=0), by="x", all=TRUE )
  #  freq=ifelse( res$in.x == 1, res$freq.x, 0 )
  #  res=data.frame( x=res$x, freq=freq)
  #
  res=within( merge2( res, data.frame(x=domain, freq=0), by="x", all=TRUE )
              , freq <- ifelse( in.x == 1, freq.x, 0 ) )[,c("x","freq")]
  # Print(res)
 }

 rownames(res)=res$x
 # if( all( apply(res, 2, is.numeric)) ) res=as.matrix(res)
 # Print( res )

 return( res )

} # compress_freqdist

#' Return the locations of the matching parentheses
#'
#' @param expr Input string vector.
#' @param loc The starting or ending locations of the parentheses
#' in \code{expr}. (See the details and examples below.)
#' @param line line number
#' @param escape = 0 not to escape parens with double backslashes.
#' @param debug = 1 to print intermediate results
#'
#' @details
#' This is an expanded version of \code{lazy.tools::locbalpar}
#' which can handle a string vector input. \cr
#'
#' When \code{expr} is a scalar string, better use \code{lazy.tools::locbalpar}.
#' \code{loc} must be positive and less than \code{sum(nchar(exp))}.
#'
#' When \code{expr} is a string vector,
#' assuming that there are no length 0 elements of expr,
#' the location of the parens can be specified as follows:\cr
#' Method 1 (with \code{line} and \code{loc})\cr
#' Suppose that the location of the paren of interest is in location
#' (chracter) LOC at line LINE. Then the following code will find
#' the location of the matching paren. \cr
#' locbalpar2( expr, LOC, LINE ) \cr
#' Note that \code{loc} and \code{line} can be vectors of the same length.\cr
#'
#' Method 2 (with \code{loc})\cr
#' In this case, \code{loc} must be specified so that \cr
#' \code{unlist(strsplit(expr,""))[loc]} \cr
#' contains opening or closing parens.
#'
#' This function returns \code{line=0, locbp=0}.
#'
#' The following characters are treated as parentheses: \cr
#' \code{"(", "{", "[", "<", ")", "}", "]", ">"}
#'
#'
#' @examples
#'
#' # When counting characters, count the length 0 character, "", as 1.
#' # For example, consider the example below where
#' # expr1 has length 0 character, "", at lines 2 and 4 and
#' # expr2 has length 1 blank character, " ",  at lines 2 and 4.
#' # The locations of parens for expr1 must be counted from expr2 as: \cr
#' #  \code{unlist(strsplit(expr2,""))}\cr
#' # The two results below should be the same.
#' expr1 <- c( "1(", "", "4)(", "", "89)" )
#' expr2 <- c( "1(", " ", "4)(", " ", "89)" )
#' printm( expr1, expr2 )
#' exprc1 <- unlist(strsplit(expr1,""))
#' exprc2 <- unlist(strsplit(expr2,""))
#' printm( exprc1, exprc2 )
#' locbalpar2( expr2, c(2,3,2), c(1,3,3) )
#' locbalpar2( expr1, c(2,6,5) )
#' locbalpar2( expr2, c(2,6,5) )
#'
#' # escape or not
#' locbalpar2( "a(b\\(d)",2 )
#' locbalpar2( "a(b\\(d)",2, escape=0 )
#'
#'
#' # Find the matching closing brace.
#' expr <- c(
#'  "locbalpar2 <- function( expr, loc ){",
#'  " # find the location of the balanced paren",
#'  " # use standard version when expr is a string",
#'  "",
#'  " if( !is.vector(expr) ){",
#'  "  return( locbalpar( expr, loc ) )",
#'  " }",
#'  "",
#'  "} # end of locbalpar2"
#' )
#' locbalpar2( expr, regexpr("{", expr, fixed=1 )[1] )
#' 
#' # find all "(" and their matching ")".
#' locs <- gregexpr("(", expr,  fixed=TRUE )
#' LL <- NULL
#' for( i in 1:length(locs) ){
#'  if( all( locs[[i]] < 0 ) ) next
#'  LL <- rbind(LL,cbind(i,locs[[i]]))
#' }
#' locbalpar2(expr,LL[,2],LL[,1])
#'
#'
#'
#' @return
#' A list of \cr
#' \code{line}   the line number. \cr
#' \code{locbp}  the locations of matching parens in the line.
#'
#'
#' @export
#'

locbalpar2 <- function( expr, loc, line=1, escape=1, debug=0 ){
  # find the location of the balanced paren
  # Shin-ichi Mayekawa
  # vector exp: 20230806
  # new version: 20250304,05
  # escape: 20250822
  # old version recovered: 20250823
  # line option: 20250823
  #

 #
 # See locbalpar2_4.R for new version in which the 0-length character
 # is calculated as 0. #
 #

 markparens <- function( expr ){
  # mark escaped parens with the string of the same # of characters
  expr=gsub( "\\(", ".!p", expr, fixed=TRUE )
  expr=gsub( "\\)", "..p", expr, fixed=TRUE )
  expr=gsub( "\\{", ".!c", expr, fixed=TRUE ) # curly brackets
  expr=gsub( "\\}", "..c", expr, fixed=TRUE )
  expr=gsub( "\\[", ".!s", expr, fixed=TRUE ) # square brackets
  expr=gsub( "\\]", "..s", expr, fixed=TRUE )
  expr=gsub( "\\<", ".!i", expr, fixed=TRUE ) # inequality
  expr=gsub( "\\>", "..i", expr, fixed=TRUE )
  return( expr )
 } # end of markparens

 unmarkparens <- function( expr ){
  # mark escaped parens: Not Used
  expr=gsub( ".!p", "\\(", expr, fixed=TRUE )
  expr=gsub( "..p", "\\)", expr, fixed=TRUE )
  expr=gsub( ".!c", "\\{", expr, fixed=TRUE ) # curly brackets
  expr=gsub( "..c", "\\}", expr, fixed=TRUE )
  expr=gsub( ".!s", "\\[", expr, fixed=TRUE ) # square brackets
  expr=gsub( "..s", "\\]", expr, fixed=TRUE )
  expr=gsub( ".!i", "\\<", expr, fixed=TRUE ) # inequality
  expr=gsub( "..i", "\\>", expr, fixed=TRUE )
  return( expr )
 } # end of unmarkparens


 if( escape ) expr=markparens( expr )

 # use standard version when expr is a string
  if( !is.vector(expr) ){
    expr=locbalpar( expr, loc )
    if( escape ) expr=unmarkparens( expr )
    return( expr )
  }


  # const
  # # of rows of expr
  nexpr=length(expr)
  # # of characters per row:  nchar("")=0
  nc=nchar(expr)

  # replace "" by "&"
  loczero=which(nc==0)
  if( length(loczero) > 0 ){
   expr[loczero]="&"
  }
  exprs=unlist(strsplit(expr,""))
  nc1=length(exprs)

  if( any( is.na(loc) )  ||  any( loc > nc1) ){
   cat("\nerror1:(balpar2) loc out of range.\n")
   printm(loc,nc1)
   stop()
  }

  if( debug ){
   printm(expr, expr1,loc)
   printm(unlist(strsplit(expr,"")),unlist(strsplit(expr1,"")),index)
  }



  # # of characters after replacement
  nc[loczero]=1
  cnc=cumsum(nc)
  cnc0=c(0,cnc)
  brks=c(cnc0, 999999)

  # add cumulative # of characters in the lines to loc
   if( debug ) printm(cn0,loc,line)
   loc=loc+cnc0[line]
   if( debug ) printm("after line", loc,line)


  # concatenate all elements
  expr2=paste( expr, collapse="" )

  # use standard version
  locbp=locbalpar( expr2, loc )

  # recover the line info using breaks
  line=as.numeric( cut( locbp, brks, labels=1:(nexpr+1) ) )

  # relative locations in each line
  locbp=locbp-cnc0[line]
  line[is.na(locbp)]=0
  locbp[is.na(locbp)]=0

  # if( escape ) expr=unmarkparens( expr )

  res=named_list(line,locbp)
  return( res )

} # end of locbalpar2

#' Returns a permutation which rearranges its first argument in the order
#'  specified by the second argument.
#'
#' @param x a vector to be sorted according to order
#' @param order a vector containing the definition of the sort order \cr
#' If order is not given, this function returns the same as the native order.
#' @param descending = 1 to sort descendingly
#'
#' @details
#' The returning value, od, will be calculated as: \cr\cr
#' \code{  df1 <- data.frame( id1=1:length(x), x ) } \cr
#' \code{  df2 <- data.frame( id2=1:length(order), order ) } \cr
#' \code{  temp <- merge( df1, df2, by.x="x", by.y="order", all=1 ) } \cr
#' \code{  temp <- temp[order(temp$id2),] } \cr
#' \code{  od <- (temp$id1)[1:length(x)] } \cr
#'
#'
#' @examples
#' order <- c("B", "D", "A", "C", "E","X")
#' x <- c( "A", "D", "B", "E", "C", "D", "A" )
#' od <- order2( x, order )
#' Print( x, order, od, x[od] )
#'
#' @return
#' The permutation vector, \code{od}, to be used as:\cr
#' \code{ x_sorted <- x[od] }
#'
#' @export
#'

order2 <- function( x, order=NULL, descending=0 ){
 # sort x using the order given by order.
 # Shin-ichi Mayekawa
 # 20170926,27
 # trim order: 20190207rc
 #

 # error
 if( !is.vector(x) ){
  cat("\nerror1:(order2) x must be a vector.\n")
  stop()
 }

 if( any(is.na(x)) ){
  cat("\nerror1:(order2) x cannot have missing values.\n")
  stop()
 }

 if( is.null(order) ){
  # default order = same as native order
  order=sort(unique(x))
 }
 else if( !is.vector(order) ){
  cat("\nerror1:(order2) order must be a vector.\n")
  stop()
 }
 if( any(is.na(order)) ){
  cat("\nerror1:(order2)"
      , " order cannot have missing values.\n")
  stop()
 }


 # There are too many elements in x than order.
 if( !all( unique(x) %in% order ) ){
  cat("\nerror1:(order2) Given order is not complete.\n")
  Print(order)
  stop()
 }

 # trim too may orders
 order=order[order %in% unique(x)]


 # The core part
 df1=data.frame( id1=1:length(x), x )
 df2=data.frame( id2=1:length(order), order )
 temp=merge( df1, df2, by.x="x", by.y="order", all=1 )
 if( descending == 0 ) temp=temp[order(temp$id2),]
 else  temp=temp[order(-temp$id2),]
 od=(temp$id1)[1:length(x)]

 return( od )

} # end of order2

#' Generating Test Score Distribution
#'
#' This function generates a frequency distribution table
#'  of n discrete test scores in [minscore,maxscore].
#'
#' @param n # of observations.
#' @param minscore The minimum score.
#' @param maxscore The maximum score.
#' @param normal Vector consisting of mean and std for normal distribution.
#' @param beta Vector of alpha and beta parameters for beta distribution
#' in [minscore,maxscore].
#' @param beta_ms Vector of mean and std of the generalized beta distribution
#' in [minscore,maxscore].
#' @param dump = 1 to output the discrete random numbers.
#' @param print = 1 to print the result.
#' @param plot = 1 to plot the result.
#' @param title Title string to be used in plot.
#'
#' @details
#' This function generates a frequency distribution table
#'  of n discrete test scores in [minscore,maxscore].
#'
#' Continuous random numbers, either normal or generalized beta,
#' are first generated and then they are discretized by cut function.
#' Those values outside of [minscore,maxscore] will be set to minscore
#' or maxscore, resp.
#'
#' beta_ms parameter has the priority over beta and nomal parameter.
#'
#' @examples
#' seed <- 1701
#' set.seed(seed)
#'
#' minscore <- 0; maxscore <- 100
#' n <- 500
#'
#' s1 <- gen_test_score( n, minscore, maxscore, beta=c(4,8), plot=1 )
#' ( mands(s1[,1],s1[,2]) )
#' s2 <- gen_test_score( n, minscore, maxscore
#'                     , beta_ms=c(33.29, 13.07), plot=1 )
#' ( mands(s2[,1],s2[,2]) )
#'
#' # mixture
#' minscore <- 0; maxscore <- 50
#' scoredist1 <- gen_test_score( n, minscore, maxscore, beta=c(4,8), plot=1 )
#' scoredist2 <- gen_test_score( n, minscore, maxscore, normal=c(30,5), plot=1)
#' scoredist12 <- merge_tables( scoredist1,scoredist2 )
#' barplot(height=scoredist12[,2], names.arg=as.character(scoredist12[,1])
#'         , main="Score Dist", cex.names=.65, space=0, las=2)
#'
#' @return
#' If dump == 0, a matrix consisting of (score, frequency). \cr
#' otherwise, a list of rv, random numbers, and freqdist matrix.
#'
#' @export
#'

gen_test_score <- function( n, minscore=0, maxscore=100, normal=c(50,15)
                            , beta=NULL, beta_ms=NULL, dump=0, print=0, plot=0
                            , title="Score Distribution" ){
 # generate test score in [minscore,maxscore].
 # Shin-ichi Mayekawa
 # 20160722
 # bugfix score[x]: 20160821
 # dump option: 20160821
 # title: 20161202
 # roxygen commnets: 20180814dnc
 # beta_ms: 20230818
 #

 range=maxscore-minscore

 # random numbers
 if( !is.null(beta_ms) ){
  if( beta_ms[2] > range^2/4 ) beta_ms[2]=range^2/4
  my=(beta_ms[1]-minscore)/range
  sy2=( beta_ms[2]/range )^2
  beta=numeric(2)
  beta[1]=-my*(my^2-my+sy2)/sy2
  beta[2]=-(1-my)*(my^2-my+sy2)/sy2
  Print(my,sy2,beta)
  x0=rbeta(n,beta[1],beta[2])
  x0=x0*range+minscore
  param=paste( beta_ms, collapse=", ")
  dist="beta_ms"
 }
 else if( !is.null(beta) ){
    x0=rbeta(n,beta[1],beta[2])
    x0=x0*range+minscore
    param=paste( beta, collapse=", ")
    dist="beta"
 }
 else if( !is.null(normal) ){
  mean=normal[1]; std=normal[2]
  x0=mean+std*rnorm(n)
  param=paste( normal, collapse=", ")
  dist="normal"
 }

 x0[x0<minscore]=minscore
 x0[x0>maxscore]=maxscore

 # histogram
 score=seq(minscore,maxscore,length.out=maxscore-minscore+1)
 brk=c(score-0.5, maxscore)
 brk[length(brk)]=maxscore
 x=as.numeric( cut( x0, breaks=brk, right=1 ) )
 x=score[x]
 resh=hist( x, breaks=c(-0.5,score+.5), right=1, plot=0 )
 scoredist=cbind(score,resh$count)
 colnames(scoredist)=c("score","freq")

 if( plot ){
  title=paste( title, ":  n = ", n, sep="" )
  title=paste(title, ",  ", dist, " with param = (", param, " )", sep="")
  barplot(height=scoredist[,2], names.arg=as.character(scoredist[,1])
          , main=title, cex.names=.65, space=0, las=2)
 }

 if( print ){
  Print(scoredist)
 }

 res=scoredist
 if( dump == 1 ) res=list( rv=x, freqdist=scoredist )
 return( res )

} # gen_test_score

#' Running Average Smooth
#'
#' @param x a univariate time series.
#' @param bandwid # of the neighbouring elements to be averaged.
#' Must be a positive odd number.
#' @param maxiter # of repetitions.
#'
#' @details The following code will be repeated maxiter times: \cr
#'    \code{filter(x,rep(1/bandwid,bandwid), sides=2)} \cr
#' x[1:m] and x[(length(x)-m+1):length(x)] will remain unchanged.
#' where \code{m=(bandwid-1)/2}.
#'
#' @return a vector of the same length as x.
#'
#' @examples
#' x <- c(4, 1, 3, 6, 6, 4, 1, 6, 2, 4, 2)
#' y <- smoothra( x, bandwid=3, 2 )
#' Print(x,y)
#'
#' @export
#'

smoothra <- function( x, bandwid=3, maxiter=1 ){
 # running average smooth
 # Shin-ichi Mayekawa
 # 20160718
 #

 # error
 if( bandwid %% 2 == 0 ){
  cat("\nerror1(smoothra): bandwid must be a positive odd number.\n")
  stop()
 }
 lenx=length(x)
 if( bandwid >= lenx ){
  cat("\nerror1(smoothra): bandwid must be less than length(x).\n")
  stop()
 }

 m=(bandwid-1)/2
 left=x[1:m]; right=x[(lenx-m+1):lenx]
 for( i in 1:maxiter ){
  x=filter(x,rep(1/bandwid,bandwid), sides=2)
  x[1:m]=left; x[(lenx-m+1):lenx]=right
 }
 attributes(x)=NULL
 return( x )
} # smoothra

#' Preparation of Stack
#'
#' 
#' @details
#' This is a function  factory which makes  the following list of functions.\cr
#'   \code{push, pop,  inspect,  length} and \code{clear}
#'
#' In the usual image of a stack, the newest object will be placed
#' on top of the older objects.
#' However, in this R version, the newest object will be placed
#' at the end of a hidden list named \code{valueList} in the function factory
#' environment.
#'
#' It enables LIFO (Last-In First-Out) or  FILO (First-In Last-Out).
#'
#'
#' @return
#'  A list of functions \cr
#'   \code{push, pop,  inspect,  length} and \code{clear}
#'
#' @examples
#'  # generate functions
#'  stack <- makeStack()
#'
#'  stack$push(3)
#'  stack$push(5)
#'  stack$push(9)
#'  stack$length()
#'  stack$inspect(1:stack$length())
#'  stack$pop()
#'  stack$inspect(1:stack$length())
#'  stack$pop()
#'  stack$inspect(1:stack$length())
#'  stack$pop()
#'
#'  # Below causes error.
#'  # stack$inspect(1:stack$length())
#'
#'
#'  STACK=makeStack()
#'  PUSH <- STACK$push
#'  POP <- STACK$pop
#'  LEN <- STACK$length
#'  INSP <- STACK$inspect
#'  CLEAR <- STACK$clear
#'  PUSH( list(a=1,b=2,c="C"))
#'  PUSH( list(a=12,b=22,c="C2"))
#'  PUSH( list(a=13,b=23,c="C3"))
#'  INSP(1:LEN())
#'  POP()
#'  INSP(1:LEN())
#'  POP()
#'  INSP(1:LEN())
#'
#'
#' @references
#'  The original version is from the answer by Matt: \cr
#'   https://stackoverflow.com/questions/28687806
#'   /a-better-way-to-push-and-pop-to-from-lists-in-r
#'
#'
#' @export
#'

makeStack <- function() {
 # stack generator
 # Shin-ichi Mayekawa
 # inspect, length, clear: 20230908
 # list added in append: 20230908
 # in lazy.tools: 20230921
 #
 # The original version is from the answer by Matt:
 # https://stackoverflow.com/questions/28687806
 # /a-better-way-to-push-and-pop-to-from-lists-in-r
 #
 # This is a function factory which returns functions
 #  push, pop,  inspect,  length and clear.
 #
 # In the usual image of a stack, the newest object will be placed
 # on top of the older objects.
 # However, in this R version, the newest object will be placed
 # at the end of a list.
 # Anyway, the newly put object will be popped first.
 #
 #

 #
 # The objects in the stack will be stored in the following list.
 # This list can be accessed only from the functions generated in this
 # environment.
 #
 valueList <- list()

 # return the following list of functions.
 list(
  push = function(value) {
   valueList <<- append(valueList, list(value) )
  },
  pop = function() {
   value <- utils::tail(valueList, 1)[[1]]
   valueList <<- utils::head(valueList, -1)
   value
  },
  inspect = function(loc){
    if( any(loc <= 0) || any(loc > length(valueList)) ){
      cat("\nerror1:(makeStack:show) loc must be positive and less than"
          , length(valueList),".\n")
      stop()
    }
    return(valueList[loc])
  },
  length = function(){
    length(valueList)
  },
  clear=function(){
    valueList <<- list()
  }
 )
} # end of makeStack

#' Display Package Description
#' 
#' @param pname name of package
#' 
#' @examples
#' package_description("lazy.tools")
#' 
#' @export
#' 

package_description <- function( pname ){
 # Display Package Description
 # Shin-ichi Mayekawa
 # 20240331
 # 
 
 # exists?
 if( !(pname %in% installed.packages()) ){
  cat("error1:(packageDescription) Package", pname, "is not installed.\n")
  return()
 }
 
 # read DESCRIPTION
 ds=readLines(system.file("DESCRIPTION", package=pname))
 
 loc=which( regexpr("Built:", ds,fixed=TRUE) > 0 )
 dd=ds[loc]
 ymdhms=strsplit(dd,";")[[1]][3]

 # Print(strsplit(dd,";")[[1]])
 # Print(pname,loc, dd)
 
 ymdhmsP=as.POSIXct(ymdhms, tz="UTC", format="%Y-%m-%d %H:%M:%OS ")
 
 ymdhmsJST=paste( as.character( as.POSIXct(ymdhmsP, tz="Asia/Tokyo") ),"JST")
 
 #Print(ymdhms,"/",ymdhmsJST)
 
 cat("\n")
 writeLines(ds)
 cat("\nThe Package was built on ", ymdhmsJST, ".\n\n", sep="")
 
} # end of package_description

#' Use Friendly Grep/Gsub
#'
#' @param expr expression to be modified.
#' @param value = 1 to print the first matches for each of \code{...}.
#' @param rule replacement rule to be applied in terms of pattern names.
#' @param rep # of times that the pattern and the rule apply.
#' @param ... patterns of the form: name=pattern w/o enclosing parens. \cr
#' No positional arguments are allowed.
#' @param out_pattern = 1 to output pattern and replacement rule
#' to be used in \code{gsub}.
#' @param sep  A character which separates each element of the rule.
#' @param fixed fixed parameter to \code{gsub}.
#' @param debug = 1 to print useful info
#'
#' @details
#'
#'  When \code{value=1} this function interprets \code{...} and
#'  print the first match to each element of \code{...}.
#'
#'  When \code{rule} is given,
#'  this function interprets \code{...} and \code{rule} and generates
#'  the pattern and the replacement for \code{gsub} function.
#'
#'  You can use this function in your program code to perform the replacement,
#'  or, you can use this to generate the pattern and the replacement for
#'  \code{gsub} function to be used later with \code{out_pattern=1} option.
#'
#'  The names in ... can be anything as long as they are valid variable names
#'  except for  \code{rep,value,out_pattern, split, fixed, debug}.
#'  \code{name=".v"} and \code{name=".n"} can be used for a valid variable name
#'  and a number, resp.. \cr
#'  All of the named arguments in ...  will be concatenated in the order
#'  of appearance and will be used as the pattern to match.
#'
#'  \code{rule} can contain any string separated by \code{sep}.
#'  If the string matches one of \code{name} elements, the string
#'  will be replaced by the pattern assocated with that name.#'
#'
#'  In the original \code{gsub} function,
#'  the values in ... will be denoted as 1, 2, ... and
#'  these numbers must be used in the rule parameter as \\\\1, \\\\2, ... .
#'
#'  The following pattern names cannot be used \cr
#'  \code{ expr, rule, out_patten, sep, debug}
#'
#' @return
#'  The result of \code{gsub}, or when out_pattern=1, the pattern and the rule
#'  to be used in \code{gsub} in a character vector of length 2.
#'
#' @examples
#' # The third example below generates the following grep/gsub patterns:
#' # gsub(
#' # "(^|[-+\\*`~])([[:alpha:].][[:alnum:]_.]*)(\\*)([0-9]*\\.{0,1}[0-9]*)
#' # ($|([-+*/`~].+))",   "\\1\\4\\3\\2\\5",   "a123+a*17.01-b*34+9" )
#' #
#'
#' # varname*num -> num*varname
#' top <- "^|[-+\\*`~]"; varname <- "[[:alpha:].][[:alnum:]_.]*"; times <- "\\*"
#' num <- "[0-9]*\\.{0,1}[0-9]*"; tail <- "$|([-+*/`~].+)"
#' expr <- "a123+a*17.01-b*34+9"
#'
#' Grep( expr, value=1
#'           , top=top, varname=varname, times=times, number=num, tail=tail )
#' Grep( expr, value=1
#'           , top=top, varname=".v", times=times, number=".n", tail=tail )
#' ( temp <- Grep( expr, rule="top?num?times?varname?tail", out_pattern=1
#'           , top=top, varname=".v", times=times, num=".n", tail=tail ) )
#' Grep( temp, rule="top?num?times?varname?tail"
#'           , top=top, varname=".v", times=times, num=".n", tail=tail )
#' Grep( expr, rule="top?(?num?)?times?varname?tail", rep=2
#'           , top=top, varname==".v", times=times, num=".n", tail=tail )
#'
#' # transposition and trace of a scalar
#' expr <- "+t(09)-tr(123)"
#' Grep( expr, rule="top?sign?digits?tail", debug=1
#'               , top="^|[-+*`~]"
#'               , tparen="t\\(|tr\\("
#'               , sign="[-+]*"
#'               , digits="[0-9]+"
#'               , paren="\\)"
#'               , tail="$|[-+*/`~]", rep=2 )
#'
#'
#' # Cancel x/x etc with back reference
#' top <- "^|[-/+*=`~]"; varnam <- "[[:alpha:].]+[[:alnum:]_]*|[1-9]+[0-9]*"
#' div <- "/\\2"; tail <- "($|[-/+*=`~])"
#' Grep( "17*x12e/x12e*100+a/a+2/2", rule="top?1?tail"
#'     , top=top, varnam=varnam, div=div, tail=tail )
#' Grep( "17*x12e/x12e*100+a/a+2/2", rule="top?1?tail"
#'     , top=top, varnam=varnam, div=div, tail=tail, rep=2 )
#'
#' @export
#'

Grep <- function( expr, rule=NULL, ..., rep=1, value=0, out_pattern=0
                  , split="?", fixed=0, debug=0 ){
 # modify expr by gsub function
 # Shin-ichi Mayekawa
 # 20200529
 # rule by patter name: 20200602,03
 # rep: 20230505cot
 # roxygen2 examples: 20230505cot
 # matched pattern: 20230505cot,0506
 # value: 20230506
 # gsub -> sub in Code: 20230508
 # Gsub in lazy.symboloc\Gsub5.R renamed as Grep: 20230508
 # . added to varname: 20230509dnc
 # Exists abandoned: 20230510
 # varname and number: 20230510
 # lv() removed from examples for version 4.2: 20230625
 #

 # preset values (Not Used)
 varname="[[:alpha:].][[:alnum:]_.]*"
 number="[0-9]*\\.{0,1}[0-9]*"


 # analyze ...
 res=analyze_three_dots( ..., print_the_result=0 )
 pat=unlist(res$val)
 name=res$name
 npat=length(pat)

 # replace .n and .v
 loc=which(pat == ".v")
 pat[loc]=varname
 loc=which(pat == ".n")
 pat[loc]=number

 # enclose each pattern by parens
 pat=unlist( lapply( pat,function(x) paste("(",x,")",sep="") ) )

 # combine all the patterns
 pattern=paste(pat,collapse="")


 if( value == 1 ){

   # matched pattern
   values=numeric(npat)
   for( i in 1:npat ){
     Code=paste("values[",i
                  ,"]=sub(pattern,';\\\\",i,";',expr,fixed=fixed)", sep="")
     eval(parse(text=Code))
   }

   # leftovers before and after pattern
   LocLOL=regexpr(";", values[1])
   LeftOverL=substr(values[1],1,LocLOL-1)
   LocLOR=nchar(values[1])-regexpr(";", revchar(values[1]))+1
   LeftOverR=substring(values[1],LocLOR+1)
   for( i in 1:npat ){
    LocLOL=regexpr(";", values[i])
    LocLOR=nchar(values[i])-regexpr(";", revchar(values[i]))+1
    values[i]=substr(values[i],LocLOL+1,LocLOR-1)
   }

   # append LocLOL and LocLOR
   .name=c("PRE",name,"POST")
   .pat=c("NA",pat,"NA")
   .values=c(LeftOverL,values,LeftOverR)
   Print(npat)
   Print(pattern)
   Print(.name,.pat,.values)

 } # end of value


 if( !is.null(rule) ){

   # split rule by split char
   rule1=unlist(strsplit(rule,split, fixed=1))


   # replace the names in the rule by their values
   df1=data.frame( id1=1:length(name), name, stringsAsFactors=0 )
   df2=data.frame( id2=1:length(rule1), rule1, stringsAsFactors=0 )
   temp=merge( df1, df2, by.x="name", by.y="rule1", all=1 )
   temp1=temp[order(temp$id2),]

   # printm(which(name%in%rule1), which(rule1%in%name))
   # printm(loceq(name,rule1), loceq(rule1,name))
   # printm(df1,df2)
   # printm(temp,temp1)

   locobs=which( !is.na(temp1$id1)[1:length(rule1)] )

   rule2=rule1
   rule2[locobs]=paste("\\",temp1$id1[locobs],sep="")
   rule2[-locobs]=temp1$name[1:length(rule1)][-locobs]

   rule2=paste(rule2,collapse="")

   if( debug >= 2 ){
     Print(name, pat, rule1)
     Print(rule2)
   }

   if( debug ){
     cat("\nbefore=",expr,"\n")
   }


   # replace
   code="expr=gsub(pattern, rule2,  expr, fixed=fixed)"
   for( i in 1:rep ){
     eval(parse(text=code))
     if( debug ){
       cat(" after=",expr,"\n")
     }
   }

   if( out_pattern == 1 ){
     res=c(pattern,rule2)
     names(res)=c("pattern","rule")
     return( res )
   }
   else{
     return( expr )
   }

 } # end of rule


} # end of Grep

#' Check if the argument is a valid variable name
#'
#' @param A Input string vector or list
#'
#' @details
#' varname is a string starting with \code{[:alpha:]} or \code{.}
#'  and having no \code{[:punct:]} except \code{.} or \code{_} .
#'
#' @return
#' A logical vector
#'
#' @examples
#' is.varname( c("and2","age_1_x","a.1","a_1.","1.","_a",".a") )
#'
#' @export
#'

is.varname <- function( A ){
 # checking if the input is a valid variable name
 # Shin-ichi Mayekawa
 # 121228
 # _ and . added: 20150803
 # new method w/o trim: 20220927dnc
 #
 #
 # varname is a string starting with [:alpha:] or .
 #                                     and having no [:punct:] except _.
 #

 if( length(A) == 1 ){
  # single input
  cond=length( grep("[ ]*^[a-zA-Z\\.][a-zA-Z_\\.0-9]*[ ]*$",A) ) > 0
  return( cond )
 }
 else{
  # recursive call for vector input:  cannot use Recall here in lapply!!
  return( unlist(lapply(A, is.varname)) )
 }

} # end of is.varname

#' Print multiple objects together with their names \cr
#' (Compact and better version of Print)
#'
#' @param ... objects to be printed
#' @param .delim. delimiter string
#' @param .NAchar. a character to indicate NA
#' (One or two single characters, a double byte character, or "" for "na".)
#' @param .FILL. not used.
#' @param fmt a string containing format specification in the form: wid.dig
#' @param .fmt. same as fmt (high precedence)
#' @param .flag. flag options for formatC
#' @param .printrownum. = 0 not to print row numbers for a vector
#' @param .lj. = 1 to left justify character vectors
#' @param .debug. = 1 to print debugging info
#'
#' @details
#' This function can print data.frames and list objects together
#' with other objects.
#' In contrast, \code{lazy.tools::Print} prints data.frames or lists
#' separately.
#'
#' Object named \code{fmt} cannot be printed unless  \code{.fmt.} is given.
#'
#' List objects cannot be formatted. \cr
#' Data.frames can only be formatted by digits= option which
#' specifies the length of the field.
#' Better use \code{Hmisc::format.df} to format data.frames.
#'
#' When the actual arguments consist of string literals or numbers,
#' they will be printed without names.
#'
#' The string literals will be printed without names.
#'
#' The string literals "/" or "\\n" indicate new line when they are not
#' the first object of ... .
#'
#' The native vectors will be printed as column vectors.
#' If a vector has a name, it will be printed as row names.
#' If a vector does not have name, numeric numbers will be printed as
#' row names unless \code{.printrownum.=0} is given.
#'
#' If the objects contain UTF-8 characters, the result may be misleading
#' due to the bug of \code{print.data.frame} method.
#'
#' \code{fmt} or \code{.fmt.} contain, if any,
#'  formats to be used separated by blanks: \cr
#'  \code{fmt="Width.Digits"} will use the following formatC function \cr
#'  \code{formatC( object, format="f", width=Width, digits=Digits )} \cr
#'  to format the object.
#'
#'  The first element, separated by blanks, will be applied to the first
#'  object of ..., the 2nd element to the 2nd object, and so on. \cr
#'  However, when the first object of ... is a string literal, the first
#'  format element will be applied to the 2nd object in ... .
#'
#'  If there are more objects in ... than length(fmt),
#'  the last fmt value will be used to fill.
#'

#'
#'
#' This function can be used with pipe operator.\cr
#'
#' Use \code{capture.output} function, not the return value,
#' to utilize the printed output of this function.
#'
#' @examples
#' df1 <- data.frame(x=1:3, y=11:13, row.names=paste("rn",1:3,sep=""))
#' df1[1,1] <- NA
#' df2 <- data.frame(x=21:24, y=31:34)
#' df2[2,2] <- NA
#' df3 <- data.frame(aa=31:34+.12345, bb=51:54)
#' df3[2,3] <- NA
#' mat <- matrix(1:6,3,2, dimnames=list(NULL,c("c1","C2")))
#' mat[3,2] <- NA
#' array <- array(1:12, dim=c(3,2,2))
#' array[1,1,1] <- NA
#' list <- list(df1,df2)
#' df <- as.data.frame(rbind(df1,df2))
#' df[,2] <- as.character(df[,2])
#' let <- letters[1:4]
#' names(let) <- toupper(let)
#' let[2] <- NA
#' printm( "title",1:3, df1, df2, df3, "/", list, df2+df2, mat, array
#'         , "newline", "\n", let, letters[1:2], letters[2] )
#'
#' # vector, matrix and array with format
#' printm( mat[1,], mat, array, fmt="5.0 5.2 6.3")
#' # data frame with format: wid part of wid.dig will be used as digits=wid.
#' printm( df1+0.12345, df1+0.12345, df1+0.12345, fmt="3 4 7")
#'
#' # no blank line before but a blank line after.
#' cat("No blank lines.\n")
#' printm(mat)
#' cat("One blank line above.\n")
#' cat("To insert a blank line, use below.\n")
#' printm("","/", mat+mat)
#'
#' @export
#'

printm <- function( ...,   .delim.=" ", .NAchar.=".", .FILL.=".^"
                    , fmt="", .fmt.="", .flag.=" "
                    , .printrownum.=1,  .lj.=0, .debug.=0 ){
 # print many objects with name
 # Shin-ichi Mayekawa
 # 20220822
 # colname for df: 20220906dnc,08
 # FILL and NAchar: 20220906dnc
 # string literal: 20220906dnc,07
 # avoid printing list and NULL: 20220910cot
 # some stringsAsFactors missing: 20220914dnc
 # printm as an alias: 20240626
 # return the first element of ... invisibly: 20240626
 # print rownames: 20250121
 # new colnames: 20250122
 # format: 20250122
 # .fmt --> fmt: 20250122
 # examples: 20250131
 # print fmt: 20250131
 # flag: 20250131
 # printrownum: 20250201
 # use capture.output for all the ...: 20250202,05,11,12,13,14,17,18
 # "/" and "\n": 20250218,19,20
 # do not add rownames to a scalar: 20250221
 # printm(NA): 20250223
 # MASS:fractions: 20250301
 # justify left string vector and data frame: 20250306
 # .lj.: 20250307
 # locatt[1]: 20250308
 # when NULL is the first of ...: 20250415
 # \\\\\\\\ -> \\\\: 20250417toyota
 # format when the 1st obj is a string literal: 20250417toyota
 # function, expression etc: 20250722
 # do not remove attributes from list: 20250727
 # valtype as mode, not typeof: 20250820
 #

 # analyze ...
 name=as.character(substitute(list(...)))[-1L]
 val <- list(...)

 if( .debug.){
  cat("name\n")
  print(name)
  cat("val\n")
  print(val)
 }

 # remove NULL objects
 locnull=which(unlist(lapply(val,is.null)))
 if( length(locnull) > 0 ){
  name=name[-locnull]
  val=val[-locnull]
  .fmt.=.fmt.[-locnull]
  .flag.=.flag.[-locnull]
 }
 nvar=length(val)
 if( nvar == 0 ) return( NULL )

 valtype=sapply(val,mode)
 if( .debug. ) print(valtype)

 # functions and expressions
 is_str=numeric(nvar)
 for( i in 1:nvar ){
   if( valtype[i] == "call" ){
   val[[i]]=deparse(val[[i]], width.cutoff=500)
   # print(valtype[i])
  }
  else if( valtype[i] == "function" )
   val[[i]]=deparse(val[[i]], width.cutoff=500)

  if( valtype[i] == "character" ) is_str[i]=1
 }


 # string literals etc
 is_strlit=numeric(nvar)
 for( i in 1:nvar ){
  if( any(class(val[[i]]) == "fractions") )
   val[[i]]=attributes(val[[i]])$fracs
  if( valtype[i] != "function" &&
   all(!is.na(val[[i]]))  &&  all(name[i] == val[[i]]) ) is_strlit[i]=1
 }


 # choose fmt or .fmt.
 if( fmt != ""  && .fmt. != "" ){
  name=c(name,"fmt")
  val=append(val,list(fmt))
  nvar=length(val)
  is_strlit=c(is_strlit,0)
  # names(val)[nvar]="fmt"
 }
 else if( fmt != "" ) .fmt.=fmt

 # decompose format string into a character vector  # 20250415
 if( is.null(.fmt.) || identical(.fmt.,character(0)) || is.na(.fmt.) )
  .fmt.=rep("", nvar)
 else{
  if( length(.fmt.) == 1  && .fmt. != "" ){
   temp=strsplit(.fmt.,split="[\\, ]", fixed=0)[[1]]
   temp=temp[temp != ""]
   .fmt.=temp
  }
 }

 # fill
 if( length(.flag.) == 1 ) .flag.=rep(.flag., nvar)
 if( length(.fmt.) == 1 ) .fmt.=rep(.fmt., nvar)

 # fill the missing format
 if( length(.fmt.) < nvar )
  .fmt.[(length(.fmt.)+1):nvar]=.fmt.[length(.fmt.)]
 if( length(.flag.) < nvar )
  .flag.[(length(.flag.)+1):nvar]=.flag.[length(.flag.)]  # 20250415

 # when the first object is a string literal, add dummy format at the top.
 if( is_strlit[1] == 1 ){
  .fmt.=c("",.fmt.)
 }

 # decompose .fmt. vector into width and digits vectors
 fwid=numeric(nvar)
 fdig=numeric(nvar)
 for( i in 1:length(.fmt.) ){
  fi=.fmt.[i]
  if( fi == "" ){
   fwid[i]=NA
   fdig[i]=NA
  }
  else if( substr(fi,1,1) == "." ){
   fwid[i]=NA
   fdig[i]=as.numeric(substring(fi,2))
  }
  else if( substring(fi,nchar(fi)) == "." ){
   fwid[i]=as.numeric(strsplit(fi,".",fixed=TRUE)[[1]])
   fdig[i]=NA
  }
  else if( regexpr(".", fi, fixed=TRUE) > 0 ){
   fwid[i]=as.numeric(strsplit(fi,".",fixed=TRUE)[[1]][1])
   fdig[i]=as.numeric(strsplit(fi,".",fixed=TRUE)[[1]][2])
  }
  else{
   fwid[i]=fi
   fdig[i]=0
  }
 }


 if( .debug. ){
  print(cbind(.fmt.,fwid,fdig))
 }


 # new lines
 locnl=c(which( name %in% c("/","\n") ),length(name))
 if( length(locnl) > 1 )
  rangenl=range=cbind(c(1,locnl[-length(locnl)]+1)
                      , c(locnl[-length(locnl)]-1,locnl[length(locnl)]))
 else rangenl=matrix(c(1,length(name)),1,2)
 if( length(locnl) > 1  &&  locnl[1] == 1 ) rangenl=rangenl[-1,,drop=FALSE]

 if( .debug. ){
  cat("locnl")
  print(locnl)
  print(rangenl)
  cat("is.null\n")
  print(which(unlist(lapply(val,is.null))))
 }



 # location of the list objects
 loclist=which(unlist(lapply(val,function(x)is.list(x) && !is.data.frame(x))))
 nlist=length(loclist)

 # format each val[[i]] and mark numeric missing values as ","!.!" .
 for( i in 1:nvar ){
  if( !(i %in% loclist) ){
   if( is.numeric(val[[i]]) ){
    if( is.na(fwid[i]) & !is.na(fdig[i]) )
     val[[i]]=formatC(val[[i]], format="f", digits=fdig[i], flag=.flag.[i])
    else if( !is.na(fwid[i]) & is.na(fdig[i]) )
     val[[i]]=formatC(val[[i]], format="f", width=fwid[i], flag=.flag.[i])
    else if( !is.na(fwid[i]) & !is.na(fdig[i]) )
     val[[i]]=formatC(val[[i]], format="f", width=fwid[i], digits=fdig[i]
                      , flag=.flag.[i])
    val[[i]][]=gsub("^ *NA","!.!",val[[i]])
   }
   else if(  .lj. == 1  &&  is.character(val[[i]]) ){
    val[[i]]=format(val[[i]], justify="left")
   }
  }
 }

 if( .debug. ){
  cat("format\n")
  print(cbind(.fmt.,  fwid,  fdig))
  cat("\nafter format val\n")
  print(val)
  str(lazy.tools::was(val))
  print(val[[1]])
 }


 # capture the output of print function to string vector Lines.
 # Lines[[i]] contains a string vector of nrow(val[[i]]) x 1
 # which is the captured output of print(val[[i]]).
 #
 Lines=NULL
 for( i in 1:nvar ){

  if( is.vector(val[[i]]) && !is.matrix(val[[i]]) && !is.list(val[[i]]) ){
   if( .debug. ){
    cat("i=",i,"vec not mat or lst\n")
    print(val[[i]])
    }
   # vector: make it a column vector
   vi=t(t(val[[i]]))
   if( is.null(names(val[[i]])) ){
    if( .printrownum.==1  && is_strlit[i] == 0  && nrow(vi) > 1 )
     rownames(vi)=1:length(vi)
    else rownames(vi)=rep("",length(vi))
   }
   colnames(vi)=""
   Lines[[i]]=capture.output( cat(name[i],"\n")
                    , print(vi, quote=FALSE, right=TRUE, na.print="!.!") )
   if( .debug. ){
    cat(i,"vector\n"); print(val[[i]]); print(Lines[[i]])
   }
  }
  else if( is.data.frame(val[[i]]) ){
   # data.frame: format by ddig which may not work as expected.
   if( .debug. ){
    cat("i=",i,"df\n")
    print(val[[i]])
   }
   ddig=NULL
   if( !is.na(fwid[i]) ) ddig=fwid[i]
   Lines[[i]]=capture.output( cat(name[i],"\n")
        , print.data.frame(val[[i]]
                   , quote=FALSE, right=0, na.print="!.!", digits=ddig) )
   if( .debug. ){
    cat(i,"data.frame\n"); print(Lines[[i]])
   }
  }
  else{
   # other: matrix, arrays and lists
   if( .debug. ){
    cat("i=",i,"other\n")
    print(val[[i]])
   }
   Lines[[i]]=capture.output( cat(name[i],"\n")
         , print(val[[i]], quote=FALSE, right=TRUE, na.print="!.!") )
   if( .debug. ){
    cat(i,"other\n"); print(Lines[[i]])
   }
  }

  if( !is.list(val[[i]]) ){   # 20250727
  # remove attributes (arrays)
  locatt=which( regexpr("attr(", Lines[[i]], fixed=TRUE) > 0 )
  if( length(locatt) > 0 ) Lines[[i]]=Lines[[i]][1:(locatt[1]-1)]
  }

  # replace \\\\ by \\
  Lines[[i]]=gsub("\\\\\\\\", "\\\\", Lines[[i]])

  # missing values: 1st gsub for formatted val and character data frame vars,
  # 2nd gsub for numeric data frame vars.
  if( nchar(.NAchar., type="byte") == 1 ){
   Lines[[i]]=gsub("!.!", paste("  ", .NAchar., sep=""), Lines[[i]], fixed=1)
   Lines[[i]]=gsub("NA", paste(" ", .NAchar., sep=""), Lines[[i]], fixed=1)
  }
  else if( nchar(.NAchar., type="byte") >= 2 ){
   Lines[[i]]=gsub("!.!",  paste(" ", .NAchar., sep="") ,Lines[[i]], fixed=1)
   Lines[[i]]=gsub("NA",  paste("", .NAchar., sep="") ,Lines[[i]], fixed=1)
  }
  else{
   Lines[[i]]=gsub("!.!", " na", Lines[[i]], fixed=1)
   #  Lines[[i]]=gsub("NA", .NAchar. ,Lines[[i]], fixed=1)
  }

  # remove object name from string lietral
  if( is.character(val[[i]]) && length(val[[i]]) == 1
                             && is_strlit[i] == 1 ){
    Lines[[i]]=Lines[[i]][-(1:2)]
  }

 } # end of i loop

 if( .debug. ){
  cat("\nLLLLL\n")
  print(Lines)
 }
 # Horizontally stack (merge) the string vectors with the key and
 # with or w/o delimiter strings (root=".") to create Df data frame.
 # Each string will have the column name whose root is "._" .
 # These are the shortest column names.
 if( .delim. == "" ){
  Df=data.frame(key__=1:length(Lines[[1]]), Lines[[1]])
  colnames(Df)[2]="._1"
  if( nvar > 1 )
   for( i in 2:nvar ){
    temp=data.frame(key__=1:length(Lines[[i]]), Lines[[i]])
    colnames(temp)[2]=paste("._",i,sep="")
    Df=merge( Df, temp
              , by="key__", all=TRUE, no.dups=FALSE)
   }
 }
 else{
  Df=data.frame(key__=1:length(Lines[[1]]), Lines[[1]])
  colnames(Df)[2]="._1"
  if( nvar > 1 )
   for( i in 2:nvar ){
    temp=data.frame(key__=1:length(Lines[[i]]), .delim., Lines[[i]])
    colnames(temp)[2:3]=c(paste(".",i,sep="")
                          , paste("._",i,sep=""))
    Df=merge( Df, temp
              , by="key__", all=TRUE, no.dups=FALSE)
   }
 }

 # remove key
 Df=Df[,-1, drop=FALSE]

 if( .debug. ){
  cat("\nLines\n")
  print(Lines)
  cat("\nDf\n")
  print.data.frame(Df, row.names=FALSE)
  cat("\n\n***\n")
 }

 # for each newline group
 for( k in 1:nrow(rangenl)){

  # Capture the output of print(Df) in out.
  # Here, nchar(colnames(Df)) must be as small as possible so that
  # .delim. is meaningful.
  # Note that if there are too many objects to print in a line,
  # the out may have multiple vertical groups.

  f=2*(rangenl[k,1]-1)+1
  t=2*rangenl[k,2]-1

  if( .debug. ){
   cat("rangenl\n")
   print(rangenl)
   cat("Df[rangenl]\n")
   print(Df[,f:t])
  }

  out=capture.output( print.data.frame(Df[,f:t, drop=FALSE]
                                       , right=0, print.gap=1
                                       , na.print="NA", row.names=FALSE) )

  # Find the location of the column names of Df and replace them by blank.
  locdots=which( regexpr("._", out, fixed=TRUE) > 0 )
  out[locdots]=" "
  #
  out=out[-1]

  # location of the object names which follow the column names of Df.
  # Find how many blocks (created by print.data.frame function) are there,
  locnames=locdots
  nblock=length(locnames)
  range=matrix(0,nblock,2)
  range[1,]=c(1,length(out))
  if( nblock >= 2 ){
   for( i in 1:(nblock-1) ){
    range[i,]=c(locnames[i],locnames[i+1]-1)
   }
   range[nblock,]=c(locnames[nblock],length(out))
  }

  if( .debug. ){
   print(locnames)
   print(out[locnames])
   print(range)
   cat("\n\n+++\n")
  }

  # replace blank space
  # out=gsub("NA", paste(.NAchar.," ",sep=""), out, fixed=TRUE)
  out=gsub("NA", "  ", out, fixed=TRUE)


  # for each block
  for( i in 1:nblock ){
   for( j in range[i,1]:range[i,2]){
    # remove all blank lines
    if( regexpr("^ +$", out[j]) != 1 ) cat(out[j],"\n")
   }
   cat("\n")
  }

 } # end of k loop

 return( invisible( val[[1]] ) )

} # end of printm



#'
#' @rdname printm
#' @export
#'

Printm <- printm

# end of Printm

#' Plot Horizontal Line on top of Density Plot
#'
#' @param fromto vector of length 2 consisting of the x-coordinats of end points.
#' @param ypos y coodinate of the line.
#' @param type line tyoe
#' @param col  line color
#' @param tck  length of the tick mark at both ends of the line
#' @param ...  additional parameter to native line function
#'
#' @examples
#' alpha <- 10; beta <- 20
#' hdr <- c(0.173,0.500)
#' x <- seq(0,1,.01)
#' plot( x, dbeta(x,alpha,beta), type="l" )
#' ypos <- dbeta(hdr[1],alpha,beta)
#' plot_interval( hdr, ypos, tck=0.1 )
#'
#' @export
#'

plot_interval <- function( fromto, ypos=0, type="l", col="black", tck=0.03
                           , ... ){
 # drow horizontal line at fromto[1] to fromto[2] at y=ypos with ticks
 # at the both ends.
 # Shin-ichi Mayekawa
 # 20160716,17
 #

 lines( fromto, c(ypos,ypos), type=type, col=col, ... )
 lines( rep(fromto[1],2), c(ypos-tck, ypos+tck), type=type, col=col, ... )
 lines( rep(fromto[2],2), c(ypos-tck, ypos+tck), type=type, col=col, ... )
} # end of plot_interval

#' inspect the contents of RDS file
#'
#' This function inspects the contents of an RDS file.
#'
#' @param RDS The name of .RDS file
#' @param func The function to apply to the contents of the .RDS file
#' such as \code{str} or \code{print},
#' @param max.level the max.level option for str when func=str.
#' @param return = 1 to return the the result of func.
#' @param ... additional parameters to func
#'
#' @details
#' Note that \code{saveRDS(object,filename.RDS)} saves a single object in
#' an external file and that
#' \code{save(object1,object2,...,file="filename.RData")}
#' saves the multiple objects in an external file.
#'
#' @examples
#' a <- 1;  b <- 2;  C <- matrix(1:6,3,2);  E <- letters[1:5]
#' ds <- data.frame(a,b,C);  ds2 <- list(C,ds)
#' Big <- list(a=a, b=b,C=C,E=E,ds=ds,ds2=ds2)
#' saveRDS(Big, file="testRDS.RDS")
#' inspect_RDS("testRDS.RDS")
#' inspect_RDS("testRDS.RDS", func=print)
#'
#' # retrieve the contents from .RDS file
#' # loadRDS( RDSfilename )
#'
#'@export
#'

inspect_RDS <- function( RDname, func=str, max.level=1, return=0, ... ){
 # inspect .rds saved by saveRDS function
 # Shin-ichi Mayekawa
 # 20240122,20260219
 # load -->  readRDS: 20260411
 # comments: 20260516
 #

 funcname=deparse(substitute(func))

 tempEnv=new.env()
 RDS=readRDS( RDname )
 if( funcname == "str" ){
  if( !return ) str(RDS, max.level=max.level, ...)
  else{
   res=capture.output( str(RDS, max.level=max.level, ...) )
   print(res)
  }
 }
 else{
  res=func(RDS, ...)
  print(res)
 }
 rm(RDS)
 if( return ) return( res )

} # inspect_RData




#' inspect the contents of RData file
#'
#' This function inspects the contents of an RData file.
#'
#' @param RData The name of .RData file
#' @param str = 1 to apply str to each object
#' @param max.level the max.level option for str
#'
#' @details
#' Note that \code{saveRDS(object,filename.RDS)} saves a single object in
#' an external file and that
#' \code{save(object1,object2,..., file="filename.RData")}
#' saves the multiple objects in an external file.
#'
#' @examples
#' a <- 1;  b <- 2;  C <- matrix(1:6,3,2);  E <- letters[1:5]
#' ds <- data.frame(a,b,C);  ds2 <- list(C,ds)
#' save(a,b,C,ds,ds2,E, file="testRData.RData")
#' inspect_RData("testRData.RData")
#'
#' # retrieve the contents from .RData file
#' # loadRDS( RDatafilename )
#'
#' @export
#'

inspect_RData <- function( RData, str=0, max.level=0 ){
 # inspect .RData saved by save function
 # Shin-ichi Mayekawa
 # 20260219
 # new func: 20260415
 # new method: 20260416
 # inspect_RD renamed as inspect_RData: 20260516
 #

 tempEnv=new.env()
 load(RData, envir=tempEnv)
 olist=ls(envir=tempEnv)
 cat("\nObjects in",RData,"\n  ")
 olist2=paste(1:length(olist),olist,sep=":")
 cat(olist2, sep=", "); cat("\n")

 if( str ){
  temp=capture.output(
   for( i in 1:length(olist) ){
    cat(olist[i],": ")
    str(tempEnv[[olist[i]]], max.level=max.level)
   }
  )
  str=gsub("\t"," ", temp, fixed=TRUE)
  printm(str, .lj.=1)
 }

} # end of inspect_RData

#' Convert a string to matrix\cr
#'  For those who miss the luxury of using spaces as delimitters.
#'
#' @param a string which contains the body of the matrix with \\nr.
#' @param header = 1 if the string has row/col names
#' @param missing a string which indicates missing values: "NA" or "."
#' @param rnameroot root of the row names when header=0
#' @param cnameroot root of the column names when header=0
#' @param nummat = 1 to force the result to be a numeric matrix. \cr
#' This may cause an error!
#' @param debug = 1 to dump intermediate result
#' @return a matrix or data.frame
#'
#' @details
#' Use space ( blank characters ) or a comma as delimiters. \cr
#' Two or more commas indicate the existence of  missing values in between.\cr
#' The last comma indicates one element is missing at the end.\cr
#' Blank lines will be neglected.
#'
#' \preformatted{
#' CARDS <- " v1 v2 v3
#' s1 11 12 13
#' s2 21 22 23
#' s3 31 32 33"
#' A <- cards( CARDS, header=1 )
#' }
#' A will be a 3 x 3 matrix with row and column names.
#'
#'  This module converts the following types of string:
#' \preformatted{
#'      a1="         , a2="11 12, 13  ,  a3="C1 C2 C3  ,  a4="C0 C1 C2 C3
#'          11 12 13       21, 22  23     R1 11 12 13         R1 11 12 13
#'          21 22 23       31 32 33       R2 21 22 23         R2 21 22 23
#'          31 32 33       41,, 43"       R3 31 32 33         R3 31 32 33
#'          41 42 43                      R4 41 42 43"        R4 41 42 43"
#'         "}
#'  into a matrix or data.frame. \cr
#'
#'  When the string contains both row and column names
#'  as in a3 or a4 above, header=1 must be specified. \cr
#'  If, as a3 above,
#'  the first line ending with \\n has one less elements than the rest,
#'  it is assumed that both row and col names are given. \cr
#'  Otherwise, as a4 above, C0 will be treated as the first column name
#'  of the resulting matrix. \cr
#'   \cr
#'  The row/col names cannot have commas in it. (for csv compatibility) \cr
#'  When the string does not have row and column names, characters specified
#'  in rnameroot or cnameroot will be used to generate the names. \cr
#'  Character elements of the matrix cannot have blanks in them. \cr
#'  The quotes will be treated as characters. \cr
#'
#'  \cr
#'   When a column contains both numeric and character string
#'   a data frame, not a matrix, will be created. \cr
#'   Scientific notation like 1e-9 will be mistaken as character. \cr
#'
#' \cr
#'  To input from the clip board, \cr
#'  copy the body of the matrix with row and column names and use \cr
#'    \code{ read.table(file("clipboard"),header=1) }
#' \cr
#'   \code{ write.table( string-such-as-a1-etc-above, "clipboard",sep=" ")}
#'    \cr
#'   may be used to improve this module.
#'
#'
#' @examples
#' CARDS=" v1 v2 v3
#' s1 11 12 13
#' s2 21 22 23
#'
#' s3 NA 32 33"
#' cards( CARDS, header=1 )
#'
#' CARDS=" v1 v2 v3
#' s1, 11, 12, 13
#' s2, 21, 22, 23
#' s3,,    32, 33"
#' cards( CARDS, header=1 )
#'
#' CARDS="v0 v1 v2 v3
#' s1 11 12 13
#' s2 21 22 23
#' s3 .  32 33"
#' cards( CARDS, header=1, missing="." )
#'
#' CARDS="
#'  11 12 13
#'  21 22 23
#'  NA  32 33
#'  "
#' cards( CARDS  )
#'
#' # result is a character matrix.
#' CARDS <- " cos(theta) -sin(theta)
#'            sin(theta) cos(theta)"
#' T <- cards(CARDS)
#'
#'
#' # result is a data frame.
#' CARDS <- "
#' num char num
#' 11 aa 12
#' 21 bb 22
#' "
#' df <- cards( CARDS, header=1 )
#' str(df)
#'
#' CARDS <- "
#'      ncat      a         b1        b2
#'  r1  3.00e+00  1.25e+00 -2.06e+00 .
#'  r2  3.00e+00  1.36e+00 -1.44e-16 .
#'  r3  3.00e+00  1.40e+00  1.97e+00 .
#'  r4  3.00e+00  1.02e+00 -2.04e+00 .
#'  r5  3.00e+00  1.06e+00 -1.46e-16 .
#'  r6  3.00e+00  1.07e+00  1.98e+00 .
#'  r7  4.00e+00  1.10e+00 -2.13e+00  1.05e-15
#'  r8  4.00e+00  7.19e-01 -2.03e+00 -3.27e-16
#'  "
#'  Ad <- cards(CARDS, header=1, missing=".")
#'  Print("This is a character matrix.", Ad)
#'
#'  Ad <- cards(CARDS, header=1, missing=".", nummat=1)
#'  Print("This is a numeric matrix.", Ad)
#'
#' @export
#'

cards <- function( a, rnameroot="r", cnameroot="c", header=0
                   , missing="NA", nummat=0, debug=0 ){
 # convert a string containing a matrix to a numeric matrix
 # Shin-ichi Mayekawa
 # 111112c,
 # missing values: 13c,14
 # T -> 1, F -> 0: 111121,120127
 # new algorithm: 111125
 # data frame: 120217,18
 # nummat: 120304
 # remove tab: 120307
 # drop=F -> drop=0: 130110
 # rownamesin: 20150528,29
 # new algorithm: 20150529,30
 # lazy.matrix -> lazy_matrix: 20150602
 # renamed as cards: 20150603
 # return character mat if possible: 20151107
 # above version added to lazy.tools as it is: 20161124
 #
 #
 #
 #  This module converts the following types of string:
 #
 #      a1="           a2="11 12, 13     a3="C1 C2 C3     a4="C0 C1 C2 C3
 #          11 12 13       21, 22  23     R1 11 12 13         R1 11 12 13
 #          21 22 23       31 32 33       R2 21 22 23         R2 21 22 23
 #          31 32 33       41,, 43"       R3 31 32 33         R3 31 32 33
 #          41 42 43                      R4 41 42 43"        R4 41 42 43"
 #         "
 #  into a matrix or data.frame.
 #
 #
 #  Args:
 #   a          string containing the body of a matrix (with or w/o names)
 #   rnameroot  root of the row names to be generated
 #   cnameroot  root of the col names to be generated
 #   missing    "." or "NA" to identify missing values
 #   header     = 1 if the string contains both row and col names
 #   nummat     = 1 to convert data frame to numeric matrix
 #              This may fail.
 #
 #
 #  When the string contains both row and column names
 #  as in a3 or a4 above, header=1 must be specified.
 #  If, as a3 above,
 #  the first line ending with \n has one less elements than the rest
 #  it is assumed that both row and col names are given.
 #  Otherwise, as a4 above, C0 will be treated as the first column name
 #  of the resulting matrix.
 #
 #  The row/col names cannot have commas in it. (for csv compatibility)
 #
 #  When the string does not have row and column names, characters specified
 #  in rnameroot or cnameroot will be used to generate the names.
 #
 #  Character elements of the matrix cannot have blanks in them.
 #  The quotes will be treated as characters.
 #
 #
 # Note:
 #
 #   When a column contains both numeric and character string
 #   a data frame, not a matrix, will be created.
 #
 #   Scientific notation like 1e-9 will be mistaken as character.
 #
 #
 #  To input from the clip board,
 #  copy the body of the matrix with row and column names and use
 #    read.table(file("clipboard"),header=1)
 #
 #   write.table( string-such-as-a1-etc-above, "clipboard",sep=" ")
 #   may be used to improve this module.
 #
 #


 if ( debug > 0 ){
  Print(rnameroot, cnameroot, header, missing, debug, sep=" ")
  cat("\nInput string as it is\n")
  print(list(a=a))
 }

 # mark . for grep: needs spaces on both sides!!
 #if ( missing == "." ) missing=" \\. "
 missing1=paste(" ",missing," ",sep="")

 # compress multiple blanks and \n
 a=gsub(" {2,}"," ",a)
 a=gsub("\n{2,}","\n",a)

 # split by \n
 a=strsplit(a, "\n")[[1]]

 a=gsub("^ *","",a) # leading blanks
 a=gsub(" *$","",a) # trailing blanks

 a=a[which(a != "")]

 if( debug ){
  cat("\nInput string after strsplitting by \\n.\n")
  print(a)
 }

 # mark the missing elements by unusual number.

 a=gsub("NA","-999999",a)
 a=gsub(",,", ",-999999,", a)
 a=gsub("\t\t", ",-999999,", a)
 a=gsub(", ,", ",-999999,", a)
 a=gsub("\t \t", ",-999999,", a)
 a=gsub("^,","-999999,",a)
 a=gsub(",$",",-999999",a)
 a=gsub(missing1, " -999999 ", a, fixed=1)  # needs spaces on both sides!!
 a=gsub(" \\.$", " -999999", a)  # blank + . at the end

 # translate commas and tabs to blanks
 a=chartr(",", " ", a)   # translate ,   to space  (csv compatibility)
 a=chartr("\t", " ", a)  # translate TAB to space  (csv compatibility)
 a=gsub(" {2,}"," ",a) # remove multiple blanks

 # below 2 lines are to compact the default names like [1, ]
 a=gsub("\\[ ","\\[",a)
 a=gsub(" \\]","\\]",a)

 f=a[which(a != "")]
 lenf=unlist(sapply(f,function(x){length(gregexpr(" ",x)[[1]])}))+1
 #lenf=sapply(regmatches(f, gregexpr(" ", f)), length)+1
 names(lenf)=NULL

 if( debug > 0){
  cat("\nAfter interpretation\n")
  Print(a,f,lenf)
  #Print(c); Print(f);Print(lenf)
 }

 # with names or not
 if( header == 1 ){

  # check if there is rowname
  if( lenf[1] == lenf[2]  &  all(lenf[-1] == lenf[2]) ){
   rownamesin=0; nc=lenf[2]
  }
  else if( lenf[1]+1 == lenf[2]  &  all(lenf[-1] == lenf[2]) ){
   rownamesin=1; nc=lenf[1]
  }
  else{
   cat("\n\nerror1:(lazy_matri) The input string does not seem to be")
   cat(" the elements of a matrix. \n")
   cat(" Below is the # of elements per line.\n")
   cat(" Something is wrong with the first line:\n")
   print(lenf)
   cat(" Below is the input string interpreted so far:\n")
   a=gsub("-999999","NA",a)
   print(a)
   return()
  }
  # case where the input contains row/col names
  # first element is the column names
  colname=f[1]
  colname=strsplit(colname," ")[[1]]
  #nc=length(colname)
  # g contains the 2nd row and below with row names at column 1
  g=f[-1]                 # remove colnames
  nr=length(g)
  # separate all the elements and make the result a matrix
  h=unlist(strsplit(g," "))
  h=h[which(h != "")]
  # row names are located at first column
  if( rownamesin == 1 ){
   res=matrix(h,,nc+1,byrow=1)
   rowname=res[,1]
   # remove the first row
   res=res[,-1,drop=0]
  }
  else{
   res=matrix(h,,nc,byrow=1)
   rowname=paste(rnameroot,1:nr,sep="")
  }

  if( debug > 0 ){
   cat("\nDecomposed string\n")
   Print(h)
   Print(nr,nc,rowname,colname)
  }

 } # end of header=1
 else{

  # case where the input does not contains row/col names
  if( length(which(lenf==lenf[1])) == length(lenf) ){
   nc=lenf[1]
  }
  else{
   cat("\n\nerror1:(lazy_matri) The input string does not seem to be\n")
   cat(" the elements of a matrix without row/col names. \n")
   cat(" Below is the # of elements per line.\n")
   print(lenf)
   cat(" Below is the input string interpreted so far:\n")
   a=gsub("-999999","NA",a)
   print(a)
   return()
  }
  nr=length(f)
  h=unlist(strsplit(f," "))
  h=h[which(h != "")]
  # nc=length(h)/nr
  res=matrix(h,nr,byrow=1)
  if( debug > 0 ){
   Print(h)
   Print(nr,nc,h)
  }

 } # end of header=0

 if (debug > 0 ){
  Print(nr,nc)
  Print(res)
  Print(was(res))
 }

 # generate names when the names are not in.
 if ( header == 0 ){
  colname=paste(cnameroot,1:nc,sep="")
  rowname=paste(rnameroot,1:nr,sep="")
 }

 # Here, res is the desired matrix of character type.

 # character, numeric or data frame?
 if( debug ) Print(class(res))
 nr=nrow(res); nc=ncol(res)
 loc=grep("[[:alpha:][:blank:]!#$%&'()*,/]",res)

 if( debug ){
  cat("loc is the location of non-digit elements: ", loc, "\n")
 }

 # mat of df
 if( length(loc) == 0 ){
  # make it a numeric matrix
  res=as.numeric(res)
  res=matrix(res,nr,nc)
  res[which(res == -999999)]=NA  # mark the missing elements by NAs.
 }
 else if( length(loc) < length(res) ){
  # make it a data frame
  res1=as.data.frame(res)
  for( j in 1:nc ){
   loc=grep("[[:alpha:][:blank:]!#$%&'()*,/]",res[,j])
   if( length(loc) == 0 ){
    res1[,j]=as.numeric(unlist(res[,j]))
    if( length(which(res1[,j] == -999999)) > 0 )
     res1[which(res1[,j] == -999999),j]=NA
   }
   else{
    res1[,j]=res[,j]
    if( length(which(res1[,j] == "-999999")) > 0 )
     res1[which(res1[,j] == "-999999"),j]=NA
   }
  }
  res=res1
 }

 # result matrix to be returned
 dimnames(res)[[1]]=rowname
 dimnames(res)[[2]]=colname

 # remove the names with [ or ]
 # if( length(grep("\\[",dimnames(res))) > 0 )   dimnames(res)=NULL

 if( nummat == 1 )
  res=matrix( as.numeric(unlist(res)),nrow(res)
              , dimnames=list(rownames(res),colnames(res)) )


 return( res )

} # end of cards

#' Combine string arguments by a logical infix operator
#'
#' @param A The first string argument
#' @param B The second string argument
#' @param ... strings to be combined.
#' @param oper = "&" or "|" or  "&&" or "||" \cr
#' And uses "&" as the default, Or uses "|" as the default.
#' @param exp a string to be negated by Not.
#'
#' @details
#' \code{And( "a", "b" )} returns \code{"a & b"} \cr
#' \code{Or( "a", "b" )} returns \code{"a | b"} \cr
#' \code{Not( "exp" )} returns \code{"!exp"} \cr
#' \cr
#' \code{And( "a", "b" )} is equivalent to \code{ "a" \%&\% "b" } \cr
#' \code{And( "a", "b", oper="&&" )} is equivalent to \code{"a" \%&&\% "b"} \cr
#' \code{Or( "a", "b" )} is equivalent to \code{ "a" \%|\% "b" } \cr
#' \code{Or( "a", "b", oper="||" )} is equivalent to \code{ "a" \%||\% "b"} \cr
#' \cr
#' Use \code{eval( parse(text=cond) )} to evaluate the expression.
#'
#' @return
#' a string
#'
#' @examples
#' And( "a==b", "(c==d)" )
#' "a==b" %&% "(c==d)"
#' Or( "a==b", "(c==d)" )
#' "a==b" %||% "(c==d)"
#' Or( Not("a"), Not("(b)") )
#' Not( And("a==b", "(c==d)") )
#'
#' @export
#'

And <- function( ..., oper=" & " ){
 return( paste(add_paren( unlist(list(...)), check=1 ), collapse=oper) )
}

#' @rdname And
#' @export
#'
Or <- function( ..., oper=" | " ){
 return( paste(add_paren( unlist(list(...)), check=1 ), collapse=oper) )
}

#' @rdname And
#' @export
#'
Not <- function( exp, oper=" | " ){
 return( paste("!", add_paren( exp, check=1 ), sep="") )
}


#' @rdname And
#' @export
#'

`%&%` <- function( A, B=NULL ){
 # & operator
 # Shin-ichi Mayekawa
 # 20171006

 if( is.character(A) | is.character(B) ){
  if( is.null(B) ) return( And(TRUE,A))
  else return( And(A,B) )
 }
 else{
  if( is.null(B) )  return(  .Primitive("&")(TRUE,A) )
  else return(  .Primitive("&")(A,B) )
 }
} # end of &


#' @rdname And
#' @export
#'

`%|%` <- function( A, B=NULL ){
 # | operator
 # Shin-ichi Mayekawa
 # 20171006

 if( is.character(A) | is.character(B) ){
  if( is.null(B) ) return( Or(TRUE,A))
  else return( Or(A,B) )
 }
 else{
  if( is.null(B) )  return(  .Primitive("|")(FALSE,A) )
  else return(  .Primitive("|")(A,B) )
 }
} # end of |


#' @rdname And
#' @export
#'

`%&&%` <- function( A, B=NULL ){
 # & operator
 # Shin-ichi Mayekawa
 # 20171006

 if( is.character(A) | is.character(B) ){
  if( is.null(B) ) return( And(TRUE,A, oper=" && "))
  else return( And(A,B, oper=" && ") )
 }
 else{
  if( is.null(B) )  return(  .Primitive("&")(TRUE,A) )
  else return(  .Primitive("&")(A,B) )
 }
} # end of &&


#' @rdname And
#' @export
#'

`%||%` <- function( A, B=NULL ){
 # | operator
 # Shin-ichi Mayekawa
 # 20171006

 if( is.character(A) | is.character(B) ){
  if( is.null(B) ) return( Or(TRUE,A, oper=" || "))
  else return( Or(A,B, oper=" || ") )
 }
 else{
  if( is.null(B) )  return(  .Primitive("|")(FALSE,A) )
  else return(  .Primitive("|")(A,B) )
 }
} # end of ||

#' Print Super Matrix ( Partitioned Matrix, Block Matrix ) with Separators.
#'
#' @param smat   input super matrix with SMdim attributes
#' @param fmt    format specification of the form: name & width & . & dicimal
#' @param nrows  a vector consisting of the # of rows of each sub matrix
#' @param ncols  a vector consisting of the # of columns of each sub matrix
#' @param sep    separator character
#' @param debug  = 1 to debug
#'
#' @details
#'  This function prints the input partitioned matrix with separator spaces
#'  and returns it invisibly (via invisible(x)).  \cr
#'  Therefor, it can be used with a pipe operator. \cr
#'  For the partitioned matrices with supermatrix attributes,
#'  see the description of supermat, getSM, or modSM.
#'
#' @examples
#' SM <- demomat(c(2,3,4),c(1,2,4),supermat=1)
#' SMc <- demomat(c(2,3,4),c(1,2,4),supermat=1, root="a")
#'
#' Printb( SM )
#' Printb( SM, sep="." )
#' Printb( SMc, fmt="8.0" )
#'
#' attributes(SM)$SMdim=NULL
#' Printb( SMc, fmt="5", nrows=c(2,3,4), ncols=c(1,2,4))
#' Printb( SMc, fmt="5", nrows=c(3,3,3), ncols=c(2,2,2,1))
#'
#' # with pipe
#' bigS <- demomat(c(2,3,4),c(1,2,4),supermat=1, root="a")  |>  printb()
#' print(bigS)
#'
#' @export
#'

Printb <- function( smat, fmt=NULL, nrows=NULL, ncols=NULL, sep=""
                    , debug=0 ){
 # print block matrix with separation lines
 # Shin-ichi Mayekawa
 # 20200523,24
 # get rid of getSM: 20200526
 # simple Print with original name: 20200527
 # return invisible(smat): 20240626
 # bugfix when smat=demomat(2,2): 20240626
 # printb as an alias: 20240626
 # warning: 20250315
 #
 
 decomp_fmt <- function( fmt=NULL, debug=1 ){
  # decompose format string of type:  NameWidth.Digits
  # Shin-ichi Mayekawa
  # 121223c
  # 130103
  #
  # Assuming fmt has a structure: name + width + dot + digits,
  # return the vector consisting of name, width, and digits
  #
  # The default name is "f".
  #   If fmt = ".5",  name="f", width="", digits="5".
  #
  #  fmt is a scalar, not a vector.
  #
  # We could execute formatC from here if we check all the 2^3 possible
  # combinations of fmt[1] -- fmt[3]
  #
  
  digits=""
  format=""    # defailt format name when nothing is given.
  width=""
  fwd=c("","","")
  if( is.null(fmt) ){
   
   # remove digits, if any.
   res=strsplit( fmt, split="\\." )[[1]]
   if( debug > 0 )
    cat("\n decomp_fmt:1", "length(res)=", length(res), " res=", res, "\n")
   if( length(res) >= 1 ){
    digits=res[2]
    res=res[-2]
   }
   if( debug > 0 )
    cat(" decomp_fmt:1", "res=", res, "\n")
   
   # if there are no name, use "f" as default and return
   if( length(grep("[[:alpha:]]", res)) == 0 ){
    format="f"
    width=res
    if( debug > 0 )
     cat(1.5, res, format,width,digits,"\n")
    fwd[1]=format; fwd[2]=width; fwd[3]=digits
    return( fwd )
   }
   
   # If res contains "fg", return
   res=strsplit( res, split="fg", fixed=1 )[[1]]
   if( debug > 0 )
    cat(" decomp_fmt:2", "length(res)=", length(res), " res=", res, "\n")
   if( length(res) >= 1  & res[1] == "" ){
    # the first element of strsplit("fg5","fg")[[1]] is NA.
    format="fg"
    width=res[2]
    fwd[1]=format; fwd[2]=width; fwd[3]=digits
    if( debug > 0 )
     cat(" decomp_fmt:2", "res=", res, "fwd=", fwd, "\n")
    return( fwd )
   }
   
   # Other format type than "fg".
   format=substr(res[1],1,1)
   res=strsplit( res, split="[defgisEG]", fixed=0 )[[1]]
   if( debug > 0 )
    cat(" decomp_fmt:3", "length(res)=", length(res), " res=", res, "\n")
   if( length(res) >= 1 ){
    if( res[1] != "" ) format="f"
    width=res[2]
   }
  }
  
  # old fortran I format.
  if( toupper(format) == "I" ) format="d"
  if( toupper(format) == "A" ) format="s"
  fwd[1]=format; fwd[2]=width; fwd[3]=digits
  
  if( debug > 0 )
   cat(" decomp_fmt:final", "fwd=", fwd, "\n")
  return( fwd )
  
 } # end of decomp_fmt
 
 
 
 
 ################################################################
 
 # error
 if( is.matrix(fmt) ){
  cat("\nwarning1(Printb): Can print only one matrix.\n\n")
 }
 
 
 # name of the matrix to be printed
 name=as.character(substitute(smat))
 
 if( !is.null(fmt) ){
  # format specification
  ff=decomp_fmt( fmt, debug=0 )
  ff[is.na(ff)]=0
  ff[ff==""]=0
  fname=ff[1]
  fwidth=as.numeric(ff[2])
  fdigits=as.numeric(ff[3])
 }
 
 # Print(ff)
 
 # nrows and ncols
 if( !is.null(nrows) & !is.null(ncols) ){
  if( !(sum(nrows) == nrow(smat) & sum(ncols) == ncol(smat)) ){
   cat("\n\nerror1:(Printb) sum(nrows) and sum(ncols) must equal to"
       , " the nrow and ncol of the input matrix. \n\n")
   return( invisible(smat) )
  }
  attributes(smat)$SMdim=list(nrows=nrows,ncols=ncols)
 }
 else if( !is.null(attributes(smat)$SMdim) ){
  nrows=attributes(smat)$SMdim$nrows
  ncols=attributes(smat)$SMdim$ncols
  # Print(nrows,ncols,rr,rc)
  #
 }
 else{
  cat(name,"\n")
  print(smat)
  return( invisible(smat) )
 }
 
 # range of each submatrix
 rr=get_range(nrows)
 rc=get_range(ncols)
 
 # # of submatrices
 nsmr=length(nrows)
 nsmc=length(ncols)
 
 # row and colnames
 rnames=rownames(smat)
 if( is.null(rnames) ) rnames=paste("r",1:sum(nrows),sep="")
 cnames=colnames(smat)
 if( is.null(cnames) ) cnames=paste("c",1:sum(ncols),sep="")
 
 nr=nrow(smat)
 nc=ncol(smat)
 
 if( debug ){
  Print(nrows, ncols, rr, rc, nsmr, nsmc, rnames, cnames, nr, nc)
 }
 
 # format the input: convert from numeric to character
 if( is.null(fmt) ){
  smatf=format( smat )
 }
 else{
  smatf=formatC( smat,format=ff[1], width=fwidth, digits=fdigits)
 }
 # If we formatC a character matrix, we will loose its attributes
 attributes(smatf)$SMdim=list(nrows=nrows,ncols=ncols)
 
 if( debug ){
  Print(smatf)
 }
 
 # add separator
 M=matrix(sep,nr+nsmr-1,nc+nsmc-1)
 Rnames=character(nr+nsmr-1)
 Cnames=character(nc+nsmc-1)
 Rnames=rep(sep,nr+nsmr-1)
 Cnames=rep(sep,nc+nsmc-1)
 for(i in 1:nsmr ){
  Rnames[rr[i,1]:rr[i,2] + i-1]=rnames[rr[i,1]:rr[i,2]]
  for( j in 1:nsmc ){
   # Print(i,j,rr[i,1]:rr[i,2] + i-1,rc[j,1]:rc[j,2] + (j-1))
   # Print(getSM(smatf,i,j))
   M[rr[i,1]:rr[i,2] + i-1,rc[j,1]:rc[j,2] + (j-1)]=
    smatf[rr[i,1]:rr[i,2],rc[j,1]:rc[j,2], drop=0]
  }
 }
 for( j in 1:nsmc )
  Cnames[rc[j,1]:rc[j,2] + j-1]=cnames[rc[j,1]:rc[j,2]]
 
 # Print(Rnames,Cnames,dim(M))
 rownames(M)=Rnames
 colnames(M)=Cnames
 
 cat(name,"\n")
 print(M, quote=0)
 
 return( invisible(smat) )
 
} # end of Printb




#' 
#' @rdname Printb
#' @export
#' 

printb <- Printb

# end of printb

#' Returns the locations of the elements of the first argument whose
#' values are equal to one of the elements in the second argument
#'
#' @param a vector
#' @param b vector
#' @param keep = 1 not to sort the result
#'
#' @return vector consisting of the locations of the elements of
#' the first argument whose
#' values are equal to one of the elements in the second argument, or 0
#'
#' @details
#' \code{loceq( a, b, keep=0 )} is equivalent to \cr
#' \code{which( match(a,b, nomatch=0) > 0 ) } \cr
#' or \cr
#' \code{which( a \%in\% b )} .\cr
#'
#' loceq(letters, c("x","cc","a"))  returns c(24,1) \cr
#' With keep=0, the result will be sorted according to the order
#' of appearance in the first argument.\cr
#' loceq(letters, c("x","cc","a"), 0)  returns c(1,24) \cr\cr
#'  If subset below contains a subset of the variable names of
#'  a data frame,\cr
#'  loc=loceq( colnames(dataframe), subset ) \cr
#'  wii return the location of those variables.\cr\cr
#'  It will return 0 if no match.
#'
#' @examples
#' loceq(letters, c("x","cc","a"))
#' loceq(letters, c("x","cc","a"), 0)
#' loceq(c("x","cc","a"), letters)
#' loceq(c("x","cc","a"), letters, 0)
#'
#' @export
#'

loceq <- function( a, b, keep=1 ){
 # returns the locations of the elements of a whose
 # values are equal to one of the elements in b
 # Shin-ichi Mayekawa
 # 121018
 # keep: 20140528
 # comments added: 20140708,20150131
 # new algorithm and -1 -> 0: 20150530
 # roxgen comments: 20161124

 #
 # This function returns a vector of the locations of the elements of a whose
 # values are equal to one of the elements in b.
 #
 # For example,
 #   loceq(letters, c("a","cc","x"), 0)  returns c(1,24)
 # and
 #   loceq(c("a","cc","x"), letters, 0)   returns c(1,3)
 #
 # When keep = 1, a[loceq(a,b)] has the same order as b.
 #   loceq(letters, c("x","cc","a"))  returns c(24,1)
 # but
 #   loceq(letters, c("x","cc","a"), 0)  returns c(1,24)
 # and
 #   loceq(c("x","cc","a"), letters)   returns c(3,1)
 # but
 #   loceq(c("x","cc","a"), letters, 0)   returns c(1,3)
 #
 #
 # When there are no match, 0 will be returned.
 #
 #  In order to find the locations of the subset of the variables
 # in a dataframe, use
 #   loc=loceq( colnames(dataframe), subset )
 #
 # For example,
 #  cn=c("x1","y1","gender","a","b")
 #  loceq(cn,c("a","x1"))  returns c(4,1)
 #  loceq(cn,c("x1","a"))  returns c(1,4)
 #

 # native match function
 matchres=match(a,b, nomatch=0) # this contains the original location in b
 loc=which(matchres > 0)
 if( length(loc) == 0 ) loc=0
 if( keep != 1  |  length(loc) == 1 ) return( loc )

 # sort the result according to match result
 matchres=matchres[matchres > 0]
 od=order(matchres)
 loc=loc[od]

 return( loc )

} # end of loceq

#' Calculation of sample quantiles by interpolating
#' sample cumulative distribution function.
#'
#' @param x numeric vector
#' @param freq frequency associated with x \cr
#'  (x,freq) is the frequency table
#' @param probs  numeric vector of probabilities with values in [0,1]
#' @param method = "constant" | "linear" | "spline",  interpolation method
#' @param cdf cdf at x. This has priority over freq.
#'
#' @details
#'  If freq=1  and method="constant" (default),
#'  the result is equivalent to native quantile function with type=1.
#'
#'  Given the frequency table of data, (x, freq), cdf will be calculated as:
#'  \preformatted{
#'  cdf = sumsum(freq)/sum(freq)
#'  }
#'
#'
#' @return
#'  a vector of quantiles
#'
#' @examples
#'
#' # generate data
#' set.seed(1701)
#' n <- 100
#' x <- round(10*rnorm(n),0)
#'
#' # by native quantile func
#' probs <- c(0.25, 0.5, 0.75)
#' res1 <- quantile(x,probs, type=1)
#'
#' # by Quantile
#' res2 <- Quantile( x,1, probs=probs )
#'
#' # frequency table
#' freq <- table(x)
#' xc <- as.numeric(names(freq))
#'
#' # from frequency table
#' res3 <- Quantile( xc,freq, probs=probs )
#'
#' # comparison
#' Print(probs,res1,res2, res3)
#'
#'
#'
#' #
#' # Effect of the sample size
#' #
#' # When small, Quantile with linear method is equal to quantile with type=4.
#' #
#' #
#'
#' getQ <- function( x, prob ){
#'  res1 <- NULL;
#'  for( type in 1:9 ) res1 <- cbind(res1,quantile(x,probs, type=type))
#'  colnames(res1) <- paste("t",1:9,sep="")
#'  return( res1 )
#' }
#'
#' # generate data
#' set.seed(1701)
#' n <- 5
#' x <- round(10*rnorm(n),0)
#' probs <- c(0.10, 0.25, 0.5, 0.75, 0.90)
#'
#' # by native quantile func
#' res1 <- getQ(x,probs)
#' # by Quantile
#' res2 <- Quantile( x,1, probs=probs, method="linear" )
#' # comparison
#' Print(res1, fmt="8.4")
#' Print(probs,res2, fmt="8.4")
#'
#'
#' # generate data
#' set.seed(1701)
#' n <- 50
#' x <- round(10*rnorm(n),0)
#' probs <- c(0.10, 0.25, 0.5, 0.75, 0.90)
#'
#' # by native quantile func
#' res1 <- getQ(x,probs)
#' # by Quantile
#' res2 <- Quantile( x,1, probs=probs, method="linear" )
#' # comparison
#' Print(res1, fmt="8.4")
#' Print(probs,res2, fmt="8.4")
#'
#' # comparison
#' Print(res1)
#' Print(probs,res2, res3)
#'
#'
#' # generate data
#' set.seed(1701)
#' n <- 500
#' x <- round(10*rnorm(n),0)
#' probs <- c(0.10, 0.25, 0.5, 0.75, 0.90)
#'
#' # by native quantile func
#' res1 <- getQ(x,probs)
#' # by Quantile
#' res2 <- Quantile( x,1, probs=probs, method="linear" )
#' # comparison
#' Print(res1, fmt="8.4")
#' Print(probs,res2, fmt="8.4")
#'
#' # comparison
#' Print(res1)
#' Print(probs,res2, res3)
#'
#'
#' @export
#'

Quantile <- function( x, freq=NULL, probs=seq(0,1,0.25), method="constant"
                      , cdf=NULL ){
 # find the 100 p % quantiles from (x,freq) or (x,cdf)
 # by interpolating (cdf, x) at cdf=p.
 # Shin-ichi Mayekawa
 # 20211227
 # in lazy.tools: 20220620
 # comment and examples: 20230720
 #


 # Args:
 #  method = "constant" gives the lowest value as the quantile.
 #

 if( is.null(freq) & is.null(cdf) ){
  stop( "error1:(Quantile) (x,freq) or (x,cdf) must be given.")
 }

 # freq from cdf
 if( !is.null(cdf) ){
  freq=cumsuminv(cdf)
  vals=x
 }
 else{
  # take care of freq
  lx=length(x); lf=length(freq)
  if( lf == 1 ) freq=rep(freq,lx)
  else if( lf > lx ) freq=freq[1:lx]
  else if( lf < lx)  freq=c(freq,rep(freq[lf],lx-lf))
  freq[is.na(freq)]=0   # Better than specifying na.pass to xtabs.

  # use xtabs to collect duplicated x values
  temp=xtabs( freq~x )
  vals=as.numeric(names(temp))
  cdf=cumsum(temp)/sum(temp)
 }

 # here, (vals, freq, cdf) is ready.

 # find the value of x at cdf=probs by interpolating (cdf,vals).
 # method="constant" with f=1 corresponds to native quantile with type=1.
 if( method %in% c("linear", "constant") ){
  # linear
  qt=approx( cdf, vals, xout=probs, method=method
             , yleft=min(x), yright=max(x), f=1, ties="ordered")$y
 }
 else{
  # spline
  if( method == "spline" ) method="natural"
  qt=spline( cdf, vals, xout=probs, method=method
             , ties="mean")$y
 }

 return( qt )

} # end of Quantile

#' Generating Test Score Distribution according to Classical Test Theory
#'
#' This function generates a frequency distribution table
#'  of n discrete test scores in [minscore,maxscore].
#'
#' @param n # of observations.
#' @param minscore The minimum score.
#' @param maxscore The maximum score.
#' @param rho The reliability coefficient
#' @param meanX0 The mean of observed test score
#' @param varX0 The variance of observed test score
#' @param beta Vector of alpha and beta parameters for beta distribution
#' in [minscore,maxscore].
#' @param orth = 1 to orthogonalize T and E. (not yet available.)
#' @param discrete = 0 to use continuous random variables
#' @param seed The random number seed or NULL
#' @param print = 1 to print the result.
#' @param plot = 1 to plot the result.
#' @param title Title string to be used in plot.
#'
#' @details
#' This function generates a set of \cr
#'  (observed score X, true score T, error score E) \cr
#' of size n in [minscore <= X <= maxscore] according to
#' the Classical Test Theory.
#'
#' Continuous random numbers, either normal or generalized beta,
#' are first generated and then they are discretized by cut function.
#' Those values outside of [minscore,maxscore] will be set to minscore
#' or maxscore, resp.
#'
#' beta parameter has priority over nomal parameter.
#'
#' @examples
#'
#'
#' res=gen_test_score_CTT(n=10000, varX0=17^2, rho=0.60, seed=1701
#'                   , plot=1, print=1, minscore=0, maxscore=100)
#' seed=1701; set.seed(seed)
#' res=gen_test_score_CTT(n=10000, varX0=17^2, rho=0.60
#'                   , plot=1, print=1, minscore=0, maxscore=100)
#'
#' res=gen_test_score_CTT(n=10000, varX=18^2, rho=0.80, beta=c(5,2)
#'                   , plot=1, print=1, minscore=0, maxscore=100, seed=1701)
#' res=gen_test_score_CTT(n=10000, meanX0=60, varX=18^2, rho=0.80, beta=c(5,2)
#'                   , plot=1, print=1, minscore=0, maxscore=100, seed=1701)
#'
#' @return
#' A list of: \cr
#' rho The actual reliability \cr
#' meanX The actual mean of X \cr
#' stdX The actual standard deviation of X \cr
#' meanT The actual mean of T \cr
#' stdT The actual standard deviation of T \cr
#' meanE The actual mean of E \cr
#' stdE The actual standard deviation of E \cr
#' cor The correlation matrix of (X,T,E) \cr
#' XTE A matrix consisting of \code{X,T,E}. (n times 3 matrix) \cr
#' scoredist Score distribution of X \cr
#'
#' @export
#'

gen_test_score_CTT <- function( n, minscore=NULL, maxscore=NULL, rho0=0.9
                                , meanX0=NULL, varX0=15^2
                                , beta=NULL,  orth=0, discrete=1
                                , seed=NULL, print=0, plot=0, title=NULL ){

 # generation of test score according to CTT
 # Shin-ichi Mayekawa
 # 20190901,02,03,03dnc
 # comments: 20191029dnc
 # seed: 20191030,1101dnc
 # stanine removed: 20191106dnc
 # roxygen2 comment: 20200526
 #

 #
 # Args:
 #  n        = # of obs
 #  minscore = minimum score of X or NULL
 #  maxscore = maximum score of X or NULL
 #  rho0      = reliability of X:  Must be given.
 #  meanX0    = mean of X or NULL
 #  varX0     = variance of X:  Must be given.
 #  beta     = parameter vec of beta dist for T
 #  discrete = 1 to discretize X
 #

 # random number seed
 if( !is.null(seed) ) set.seed(seed)

 # mean of X
 # When beta is given, meanX must be specified carefully.
 if( is.null(meanX0) ){
  if( is.null(beta) ){
   meanX0=(minscore+maxscore)/2
  }
  else{
   meanX0=(maxscore-minscore)*beta[1]/sum(beta) + minscore
  }
 }

 # Given rho and varX, define stdE and stdT
 stdX0=sqrt(varX0)
 stdE0=sqrt( (1-rho0)*varX0 )
 stdT0=sqrt( rho0*varX0 )

 # generate true scores
 if( is.null(beta) ){
  T=rnorm(n,meanX0,stdT0)
 }
 else{
  T=rbeta(n,beta[1],beta[2])
  T=T*(maxscore-minscore)+minscore
  # T above is in [minscore,maxscore]
  stdt=sqrt((n-1)/n*var(T))
  T=T/stdt*stdT0
  # T above is in  [minscore,maxscore] / stdt*stdT
 }

 # error and obs score
 E=rnorm(n,0,stdE0)

 # orthogonalize T and E
 if( orth ){
 #########################
 }


 # obs score
 X=T+E

 # set mean and std of X exactly.
 meanx=mean(X)
 stdx=sqrt((n-1)/n*var(X))
 dd=stdX0/stdx
 X=(X-meanx)*dd+meanX0
 T=(T-meanx)*dd+meanX0
 E=dd*E
 T=T-mean(T)+meanx
 E=E+mean(T)-meanx


 # Print(mands(cbind(X,T,E)))

 # discretize
 if( discrete ){
  X=round(X)
 }


 # put all into [minscore,maxscore]
 if( !is.null(minscore) ){
  if( any( X < minscore ) ){
   loc=which(X<minscore)
   T[loc]=T[loc]+(minscore-X[loc])
   # E[loc]=E[loc]+(minscore-X[loc])
   X[loc]=minscore
  }
 } else minscore=min(X)
 if( !is.null(maxscore) ){
  if( any( X > maxscore ) ){
   loc=which(X > maxscore)
   T[loc]=T[loc]-(X[loc]-maxscore)
   # E[loc]=E[loc]-(X[loc]-maxscore)
   X[loc]=maxscore
  }
 } else maxscore=max(X)


 # sort X
 od=order(X)
 X=X[od]; T=T[od]; E=E[od]

 # stat
 XTE=cbind(X,T,E)
 ms=mands(XTE)

 if( print ){
  Print(rho0, meanX0, varX0, stdX0, discrete)
  Print(summary(XTE))
  Print(ms, fmt="10.3")
 }

 meanX=ms[1,1]; stdX=ms[2,1]; varX=ms[3,1]
 meanT=ms[1,2]; stdT=ms[2,2]; varT=ms[3,2]
 meanE=ms[1,3]; stdE=ms[2,3]; varE=ms[3,3]
 rho=varT/varX

 # corr
 cor=cor(XTE)


 # histogram
 score=seq(minscore,maxscore,length.out=maxscore-minscore+1)
 tt=table(X)
 # Print(score, names(tt), as.numeric(tt),as.numeric(names(tt))-minscore+1)
 freqx=numeric(length(score))
 freqx[as.numeric(names(tt))-minscore+1]=as.numeric(tt)
 scoredist=cbind(score,freqx)
 colnames(scoredist)=c("score","freq")

 if( print ){
  Print( rho0, meanX0, stdX0, stdT0, stdE0, fmt="6.3" )
  Print( rho, meanX, stdX, stdT, stdE, fmt="6.3" )
  if( print >= 2 ) Print(scoredist)
 }


 if( plot ){
  rhoc=formatC(rho,digits=2,format="f")
  meanXc=formatC(meanX,digits=2,format="f")
  stdXc=formatC(stdX,digits=2,format="f")
  stdEc=formatC(stdE,digits=2,format="f")
  title=paste( title, "  n = ", n, sep="" )
  title=paste(title, ",  mean =", meanXc, ", std =", stdXc, sep="")
  title=paste(title, ",  rho =", rhoc, ",  SEM =", stdEc, sep="")
  barplot(height=scoredist[,2], names.arg=as.character(scoredist[,1])
          , main=title, cex.names=.65, space=0, las=2)
 }


 if(0){
  ThatC=NULL; tabS=NULL; bpx=NULL; bpt=NULL; stat=NULL
 }


 return(
  named_list( rho, meanX, stdX, meanT, stdT, meanE, stdE
              , cor, XTE, scoredist )
  )


} # end of gen_test_score_CTT

#' Observed Score Equating
#'
#' This performs the observed score equating of
#' the test score 1 to the test score 2.
#'
#' @param score1 a vector consisting of the score of test1
#' @param freq1  a vector consisting of the frequency counts at score1
#' @param cdf1   a vector consisting of the cumulative frequencies at score1
#' @param score2 a vector consisting of the score of test2
#' @param freq2  a vector consisting of the frequency counts at score2
#' @param cdf2   a vector consisting of the cumulative frequencies at score2
#'
#' @param lim1 min and max score of test1
#' @param lim2 min and max score of test2
#'
#' @param smooth1 # of times to smooth cdf1
#' @param bandwid1 bandwidth for running average smooth of cdf1
#' @param smooth2 # of times to smooth cdf2
#' @param bandwid2 bandwidth for running average smooth of cdf2
#'
#'
#' @param method = 1 to use linear equating \cr
#'               = 2 to use equi-percentile equating
#'
#' @param interpol_method = "constant", "linear" or "spline"
#'
#' @param title  title string
#' @param nolow  = 1 to avoid lowering scores
#' @param round  = 0 not to round the result to integer. (not yet available)
#'
#' @param print = 1 to print the result
#' @param plot = 1 to plot the conversion table \cr
#'             = 2 to plot the cdf \cr
#'             = 3 to plot the smoothed cdf
#'
#' @details
#' Equipercentile equating of test1 score \code{x1} to test2 score \code{x2}
#'  is defined as\cr
#' \preformatted{
#'  x21 = invF2( F1(x1) )
#' }
#' where \code{F1} is the distribution function of \code{x1} and
#' \code{invF2} is the inverse of the distribution function of \code{x2}.
#' In this function, \code{invF2} is calculated by interpolating
#' \code{( F2(x2), x2 )} at \code{F1(x1)} with or without smoothing.
#'
#' Equipercentile equating is essentially the same as native \code{qqplot}. \cr
#' Therefore, the following two codes produce similar results: \cr
#' \preformatted{
#'  eq( score1=score1, freq1=freq1, score2=score2, freq2=freq2 )
#'  qqplot( expand_freqdist( score1, freq1 )
#'              , expand_freqdist( score2, freq2 ), type="l" )
#' }
#' Note that, since native \code{qqplot} cannot handle case weight,
#' \code{lazy.tools::expand_freqdist} is used to recover the raw data
#' from frequency table.
#'
#'
#' \code{cdf} has priority over \code{freq}.
#'
#'
#'
#' @return a list of the following:\cr
#' ctable: the conversion table consisting of (score, score21, freq1) \cr
#' mands:  the summary stat of the converted score dist: (score21,freq1) \cr
#' newfreq: the frequency distribution of the converted score (score21, freq21)
#' \cr
#' sdist1 and sdist2: input and smoothed score distributions \cr
#' cntr: a list of control parameters used.
#'
#' \code{ctable} shows that
#' test score score1[i] of test1 is equivalent to
#' test score score21[i] of test2.
#'
#' @examples
#'
#' seed <- 1701
#' set.seed(seed)
#'
#' scoredist1 <- gen_test_score( 500, 0,10, beta=c(2,4), plot=1 )
#' scoredist2 <- gen_test_score( 1000, 0,15, beta=c(4,2), plot=1 )
#' reseq <- eq( score1=scoredist1[,1], freq1=scoredist1[,2]
#'            , score2=scoredist2[,1], freq2=scoredist2[,2]
#'            , smooth1=3, smooth2=3, method=2, plot=3 )
#'
#' @export
#'

eq <- function( score1, freq1, cdf1=NULL, score2, freq2, cdf2=NULL
              , lim1=NULL, lim2=NULL
              , smooth1=0, bandwid1=3, smooth2=0, bandwid2=3
              , method=2, interpol_method="linear"
              , title="", nolow=0, round=1, print=1, plot=1 ){
 # Observed Score Equating of Test1 to Test2
 # Shin-ichi Mayekawa
 # 20160722,23,27
 # 20180810dnc,12,14dnc
 # cdf priority: 20230717
 # mandd -> mands: 20230717
 # no print mands: 20230721
 # roxygen2: 20231030
 #

 # limits
 if( is.null(lim1) ){
  lim1=c(min(score1),max(score1))
 }
 if( is.null(lim2) ){
  lim2=c(min(score2),max(score2))
 }
 minscore=min(lim1,lim2)
 maxscore=max(lim1,lim2)

 # calculate cdf if not given: when method==1 they are not used.
 if( is.null(cdf1) ){
   cdf1=cumsum(freq1)/sum(freq1)
 }
 else{
   freq1=cumsuminv(cdf1)
 }
 if( is.null(cdf2) ){
   cdf2=cumsum(freq2)/sum(freq2)
 }
 else{
   freq2=cumsuminv(cdf2)
 }
 cdf01=cdf1; cdf02=cdf2

 #
 len1=length(score1)
 n1=sum(freq1)
 len2=length(score2)
 n2=sum(freq2)

 # summary stat
 mands=rbind(t(mands(score1,freq1)[c(1,2,4)]),t(mands(score2,freq2)[c(1,2,4)]))
 rownames(mands)=c("test1","test2")
 colnames(mands)=c("mean","std","n")

 if( print >= 1 ){
  cat("\nTest1 Score Distribution\n")
  prmatrix( cbind(score1, freq1, cdf1), rowlab=rep("",len1) )
  cat("\nTest2 Score Distribution\n")
  prmatrix( cbind(score2, freq2, cdf2), rowlab=rep("",len2) )
 }


 if( method == 1 ){

  # linear
  m1=mands[1,1]; s1=mands[1,2]
  m2=mands[2,1]; s2=mands[2,2]

  score210=s2/s1*(score1-m1)+m2
  score21=round( score210 )

  # Print(score1,score21,freq1)

 }
 else if( method == 2  ){

  # equi-percentile

  if( plot >= 3 ){
   title1=paste(title, "\n", "CDF of two tests", sep="")
   plot( score1, cdf1, type="l", ylab=""
         , xlab="", xlim=c(minscore,maxscore), ylim=c(0,1), lty=1 )
   par( new=TRUE )
   plot( score2, cdf2, type="l", xlim=c(minscore,maxscore), ylim=c(0,1)
         , ylab="CDF", xlab="score", main=title1, lty=2 )
  }

  # smooth cdf if required
  if( smooth1 > 0 ){
   cdf1=smoothra( cdf1, bandwid=bandwid1, maxiter=smooth1 )
  }
  if( smooth2 > 0 ){
   cdf2=smoothra( cdf2, bandwid=bandwid2, maxiter=smooth2 )
  }

  if( smooth1 > 0 | smooth2 > 0 ){

   if( print >= 2 ){
    cat("\nTest1 Score Distribution after Smooth\n")
    prmatrix( cbind(score1, freq1, cdf01, cdf1), rowlab=rep("",n1) )
    cat("\nTest2 Score Distribution after Smooth\n")
    prmatrix( cbind(score2, freq2, cdf02, cdf2), rowlab=rep("",n2) )
   }

   if( plot >= 2 ){
    title1=paste(title, "\n", "Smoothed CDF of two tests", sep="")
    plot( score1, cdf1, type="l", ylab=""
          , xlab="", xlim=c(minscore,maxscore), ylim=c(0,1), lty=1 )
    par( new=TRUE )
    plot( score2, cdf2, type="l", xlim=c(minscore,maxscore), ylim=c(0,1)
          , ylab="CDF", xlab="score", main=title1, lty=2 )
   }
  }

  # Print(score1, freq1, cdf1, score2, freq2, cdf2 )

  # equipercentile equating
  score210=interpol( cdf2, score2, cdf1,  method=interpol_method )[,2]
  score21=round( score210 )

 } # end of method == 2


 # nolow
 if( nolow ){
  loc=which(score21 < score1)
  score21[loc]=score1[loc]
 }


 # conversion table
 min21=min(score2,score21); max21=max(score2,score21)
 s21=min21:max21
 f21=numeric(length(s21))
 ctable=cbind(score1,score21,freq1)
 ctable0=cbind(score1,score210,freq1)

 # pdf from cdf
 pdf1=cumsuminv(cdf1)
 pdf2=cumsuminv(cdf2)

 # freq dist of score21
 xt=xtabs( freq1~score21 )
 loc=loceq(min21:max21,score21)
 f21[loc]=xt
 newfreq=cbind(s21,f21)
 colnames(newfreq)=c("score21","freq21")

 # new summary stat
 ms21=unlist(mandd(s21,f21)[c(1,3,4)])
 mands=rbind(mands,ms21)
 rownames(mands)[3]="2from1"

 # Print(ms21)
 # Print(unlist(mandd(score21,pdf1)[c(1,3,4)]))



 if( print ){
  cat("\nSummary Statistics\n")
  print( mands)
  if( print >= 1 ){
   title1=paste(title, "\n", "The Conversion Table (rounded)", sep="")
   cat(title1,"\n")
   prmatrix( ctable, rowlab=rep("",nrow(ctable)) )
   title1=paste(title, "\n", "The Conversion Table", sep="")
   cat(title1,"\n")
   prmatrix( ctable0, rowlab=rep("",nrow(ctable)) )
  }
 }

 if( plot ){
  title1=paste(title, "\n", "Frequency Distribution of the Converted Score"
               , sep="")
  barplot( height=f21, names.arg=s21, space=0, main=title1 )
  title1=paste(title, "\nThe Conversion Table: from Test1 to Test2", sep="")
  plot( score1, score21, type="b", xlim=lim1, ylim=lim2
        , main=title1 )
 }

 cntr=named_list( method, smooth1, smooth2, bandwid1, bandwid2=3
                 , nolow, round )
 sdist1=cbind(score1,freq1, cdf01, cdf1, pdf1 )
 sdist2=cbind(score2,freq2, cdf02, cdf2, pdf2 )
 res=named_list( ctable, ctable0, mands, newfreq, sdist1, sdist2, method, cntr )
 return( res )

} # end of eq

#' Calculation of Weighted Distance Matrices
#'
#' @param X  n x r matrix
#' @param W  nG x r maatrix
#' @param array = 0 to make the result a list
#'
#' @details
#' This function calculates the weighted distance among the rows of
#' \code{X} matrix  by applying the k-th row of \code{W} matrix
#' as a set of dimensional weights as:
#' \deqn{ d_{ijk} = \sqrt( \sum_{a=1}^r   w[k,a] (x[i,a]-x[j,a])^2 ) }
#' or
#' \deqn{ d_{ijk} = \sqrt( (x[i,]-x[j,])' diag(W[k,]) (x[i,]-x[j,]) ) }
#' \cr
#' Note that \code{W} matrix will NOT be squared when calculating distance.
#' \cr
#'
#' @return
#' An array or a list of nG matrices of size n x n
#'
#' @examples
#' X <- matrix( c(0,0,  3,0,  0,4), 3,2,  byrow=1 )
#' distancew( X )
#' W <- matrix( c(2,1, 1,2, 1,1), 3,2, byrow=1 )
#' distancew( X, W )
#'
#' @export
#'

distancew <- function( X, W=matrix(1,1,ncol(X)), array=1 ){
 # weighted Euclidean distance between the rows of X matrices
 # Shin-ichi Mayekawa
 # 111122
 # This version allows negative elements for W
 # error check: 20150609
 # roxygen: 20210421
 #
 # Args:
 #  X      n x r matrix
 #  W      s x r matrix or subject weight
 #  array  = 0 to store the result in a list
 #
 # Value:
 #  D  array or list of s n x n matrices
 #
 # distance is calculated as follows using function distancew0:
 #   dijk = sqrt( sum_{a=1}^r   w[k,a] (x[i,a]-x[j,a])^2 )
 #   dijk = sqrt( (x[i,]-x[j,])' diag(W[k,]) (x[i,]-x[j,]) )
 #
 #
 # Needs:
 #  distancew0, Diag
 #

 # error
 if( !is.matrix(X) | !is.matrix(W) ){
  cat("\n\nerror1:(distancew) Input must be a matrix.\n")
  return()
 }

 n=nrow(X)
 nr=ncol(X)
 ns=nrow(W)

 rname=rownames(X)

 if ( length(rownames(W)) == 0 )
  rnw=paste("s",1:ns,sep="")
 else
  rnw=rownames(W)

 if ( array == 1 ){
  # as array
  D=array(0,c(n,n,ns))
  dimnames(D)[[1]]=rownames(X)
  dimnames(D)[[2]]=rownames(X)
  if( ns > 1 ) dimnames(D)[[3]]=rnw
  for ( k in 1:ns ){
   D[,,k]=distancew0(X,diag(W[k,]))
  }
 }
 else{
  # as list
  D=list(distancew0(X,diag(W[1,])))
  if ( ns >= 2 ) {
   for ( k in 2:ns ){
    D=append(D,list(distancew0(X,diag(W[k,]))))
   }
  }
  names(D)=rnw
 }

 return( D )

} #  end of distancew



#' Calculation of Weighted Distance Matrix
#'
#' @param X  n x r matrix
#' @param W  r x r diagonal maatrix
#'
#' @details
#' This function calculates the weighted distance among the rows of
#' \code{X} matrix  by applying the a-th diagonal element of \code{W} matrix
#' as the dimensional weights as:
#' \deqn{ d_{ij} = \sqrt( \sum_{a=1}^r   w[a,a] (x[i,a]-x[j,a])^2 ) }
#' or
#' \deqn{ d_{ij} = \sqrt( (x[i,]-x[j,])' W (x[i,]-x[j,]) ) }
#' \cr
#' Note that \code{W} matrix will NOT be squared when calculating distance.
#' \cr
#'
#' @return
#' A matrices of size n x n
#'
#' @examples
#' X <- matrix( c(0,0,  3,0,  0,4), 3,2,  byrow=1 )
#' distancew0( X )
#' W <- diag(c(2,1))
#' distancew0( X,W )
#'
#' @export
#'
distancew0 <- function( X, W=diag(ncol(X)) ){
 # Weighted Euclidean distance between the rows of X matrix
 # Shin-ichi Mayekawa
 # 111122,120130
 # error check: 20150609
 # roxygen: 20210421
 #
 # Args:
 #  X  n x r matrix
 #  W  r x r diagonal matrix of dimensional weight
 #
 # Value:
 #  D  n x n matrix
 #
 # distance is calculated as:
 #   dij = sqrt( sum_{k=1}^r w_{k,k}( x_{ik}-x_{jk} )^2 )
 #   dij = sqrt( (x[i,]-x[j,])' W (x[i,]-x[j,]) )
 #
 #

 # error
 if( !is.matrix(X) | !is.matrix(W) ){
  cat("\n\nerror1:(distancew0) Input must be a matrix.\n")
  return()
 }

 n=nrow(X)
 nr=ncol(X)

 XX=rowSums(X*X%*%W)
 XX11=XX%*%matrix(1,1,n); XX11=XX11+t(XX11)
 D=sqrt( abs( XX11-2*X%*%W%*%t(X) ) )

 rownames(D)=rownames(X)
 colnames(D)=rownames(X)

 return( D )

} # end of distancew0


#' Calculation of Weighted Distance Matrix between the rows of F and A
#'
#'
#' @param F  n x r matrix
#' @param A  p x r matrix
#' @param W  p x r maatrix
#' @param R  p x (r x r) array
#'
#' @details
#' This program calculates the weighted distance between the rows of F matrix
#' and the rows of A matrix as\cr
#' \deqn{ d_{ij} = \sqrt( (f[i,]-a[j,])' diag(W[j,]) (f[i,]-a[j,]) )  } \cr
#' \deqn{ d_{ij} = \sqrt( (f[i,]-a[j,])' R[,,j] (f[i,]-a[j,]) )  }
#'
#' \cr
#' Note that \code{W} matrix will NOT be squared when calculating distance.
#' \cr
#'
#' @return
#' A matrices of size n x p
#'
#' @examples
#' F <- matrix( c(0,0,  3,0,  0,4), 3,2,  byrow=1 )
#' A <- matrix( c(0,0,  3,4 ), 2,2, byrow=1 )
#' distrectw( F,A )
#' Wa <- matrix( c(1,1,  1,2 ), 2,2, byrow=1 )
#' distrectw( F,A, Wa )
#' R <- matrix(c(1,0, 0,1, 1,0.5, 0.5,3 ), ,2, byrow=TRUE)
#' # R <- t(R)
#' # dim(R)=c(2,2,2)
#' distrectw( F,A, R=R )
#'
#' @export
#'

distrectw <- function( F, A, W=matrix(1, nrow(A), ncol(A)), R=NULL
                        , outD2=0 ){
 # weighted distance between F and A
 # Shin-ichi Mayekawa
 # 111122,120130
 # error check: 20150609
 # roxygen: 20210421
 # R: 20251226
 # 20251227
 # distrectw renamed as distrectWR: 20260111
 # outD2: 20260214
 # nr --> ndim: 20260423
 # renamed as distrectw: 20260514
 #
 # Args:
 #  F  n x r matrix
 #  A  nv x ndim matrix
 #  W  nv x ndim matrix
 #  R  nv * ndim x ndim matrix
 #  outD2 = 1 to return squared distance as D
 #
 # Value:
 #  D  n x nv matrix
 #
 # distance is calculated as:
 #   dij =sqrt( (f[i,]-a[j,])' diag(W[j,]) (f[i,]-a[j,]) )
 #   dij =sqrt( (f[i,]-a[j,])' R[(j-1)*ndim*(1:ndim),](f[i,]-a[j,]) )
 #
 #

 # error
 if( !is.matrix(F) | !is.matrix(A)  | !is.matrix(W) ){
  cat("\n\nerror1:(distrectWR) Input must be a matrix.\n")
  return()
 }

 ns=nrow(F)
 ndim=ncol(F)
 nv=nrow(A)

 if( !is.null(R) ){
  if( nrow(R) != nv*ndim ){
   cat("\nwarning1:(distrectWR) R must be ncol(A)*nrow(A) x nrow(A).\n")
   stop()
  }
 }

 if( is.null(R) ){
  FF=F^2
  AA=apply((A^2)*W,1,sum)
  # Print(F,A,W,Ws,FF,AA)
  D=FF%*%t(W)-2*F%*%t(A*W)+matrix(1,ns,1)%*%t(AA)
 }
 else{
  D=matrix(0,ns,nv)
   for( j in 1:nv ){
    FmA=F-matrix(A[j,],ns,ndim, byrow=1)
    Rj=R[((j-1)*ndim+1):(j*ndim), , drop = FALSE]
    D[,j]=rowSums((FmA %*% Rj) * FmA)
   }
 }

 if( outD2 == 0 ) D=sqrt(D)
 rownames(D)=rownames(F)
 colnames(D)=rownames(A)

 return( D )

} # distrectw

#' Set up a matrix whose elements consist of row and col numbers.
#'
#' This can create either symbolic or numeric matrix.
#'
#' @param nr # of rows of the matrix to be generated
#'    or a vector consisting of the # of rows of each submatrix.
#' @param nc # of columns of the matrix to be generated
#'    or a vector consisting of the # of columns of each submatrix.
#' @param n3 # of the 3d dimension or 0, if any
#' @param root The root character of the elements of the symbolic matrix. \cr
#' If this is present, a character matrix/array will be created.
#' @param vec = 1 to drop the redundant subscript.
#' @param shape Shape of the resulting matrix: \cr
#'        "sym", "symU", "symL", "diag", "uppert", "lowert"
#' @param nodiag = 1 to set the diagonal elements to 0 when either
#'             "upper" or "lower" is specified
#' @param list = 1 to return the array as list when n3 > 2.
#' @param fullsymb = 1 to make the result a fully symbolic matrix
#' @param mul   See below.
#' @param mul3  See below.
#' @param rnroot root of the row names
#' @param cnroot root of the column names
#' @param anroot root of the third dimension names
#' @param supermat = 1 to create horizontally stacked array\cr
#' = 2 to create vertically stacked array\cr
#' = 0 othereize
#'
#' @details The i,j elements of a matrix will be named as mul*i + j\cr
#'  The i,j,k elements of an array will be named as mul3*(mul*i + j) + k\cr
#'  \cr
#'  When a supermatrix is generated, SMdim attributes will be set \cr
#'  where SMdim	is a list of the form \cr
#'  (nrows=nr, ncols=nc, rownames=rn, colnames=cn, anames=an) \cr
#'  where \cr
#'   nr is a vector consisting of the number of rows of each submatrix, \cr
#'   nc is a vector consisting of the number of columns of each submatrix \cr
#'   rn is a vector consisting of the supplemental row names \cr
#'   cn is a vector consisting of the supplemental column names \cr
#'   an is a vector consisting of the names associated with the 3rd dim.\cr
#'   '
#' @return A matrix or an array. \cr
#' Note that even with vec=1 option, thre result is a matrix. \cr
#' Use c() to convert it to a vector.
#'
#' @examples
#' demomat( 11,3 ) # numeric matrix
#' demomat( 11,3,root="a" ) # symbolic matrix
#' demomat( 4,1,root="a", vec=1 ) # symbolic matrix
#'
#' demomat( 4,3,2 ) # numeric array
#' demomat( 4,3,2, supermat=1 ) # numeric supermatrix (horizontally stacked)
#' demomat( 4,3,2, supermat=2 ) # numeric supermatrix (vertically stacked)
#'
#' # Below two creates almost identical super matrices.
#' demomat( 4,3,2, root="a", supermat=1 )
#' demomat( 4,c(3,3), root="a" )
#'
#' # 2 x 2 partitioned matrix
#' A=demomat( c(4,3),c(3,2), root="a" )
#' Printb(A)
#'
#' # various shape
#' demomat(3,3,root="a", shape="sym")
#' demomat(3,3,root="a", shape="diag")
#' demomat(3,3,root="a", shape="lowert")
#'
#' # fully symbolic matrix
#' demomat(4,4, root="S", fullsymb=1)
#' demomat(4,4, root="S", shape="sym", fullsymb=1)
#'
#' @export
#'

demomat <- function( nr, nc, n3=0, root=NULL, vec=0
                   , shape=NULL, nodiag=0, supermat=0, list=0, fullsymb=0
                   , mul=NULL, mul3=NULL
                   , rnroot="r", cnroot="c", anroot="a" ){
 # geneation of a demo matrix or array whose i,j  or  i,j,k element is
 #     mul*i + j   or   mul3*(mul*i + j) + k
 # Shin-ichi Mayekawa: 070701,02
 # dimnames: 111028
 # 3rd dimension: 120107
 # shape, supermat: 120124
 # row/col names for supermat: 120125
 # attributes$dim.array in comment: 120125
 # initialization of list: 120131
 # shape="sym" corrected: 130109
 # super mat with vector nc and nr: 20150531cot,0601,03
 # symbolic mat; 20150725
 # vec: 20150822
 # leading 0: 20150910JR,ShinagawaP
 # ceiling_n removed: 20150911ShinagawaP,11
 # lowert etc with root: 20150924
 # anname: 20151127
 # rowname, colname renamed as rnroot and cnroot: 20151127
 # list names: 20190206rc
 # names attributes of supermat: 20190207rc
 # roxygen2 error fixed: 20191129dnc
 # reverse the subscripts of supermatrix: 20200525,26
 # fullsymb: 20200527
 # sym with fullsymb: 20250513
 # printm(upptert) removed: 20250627
 #
 # Args:
 #   nr       # of rows (first dimension)
 #            or a vector consisting of the number of rows for each of
 #            vertically stacked matrices.
 #   nc       # of columns (second dimension)
 #            or a vector consisting of the number of columns for each of
 #            horizontally stacked matrices.
 #   n3       # of third dimension
 #   shape    = one of "sym", "symU", "symL", "diag", "uppert", "lowert"
 #   nodiag   = 1 to set the diagonal elements equal to 0 when
 #               shape equals "uppert" or "lowert"
 #   supermat = 2 to return the vertically stacked super matrix
 #            = 1 to return the horizontally stacked super matrix
 #            This option will not be used when vector nr or nc are given.
 #   list     = 1 to return the array as a list with anroot
 #                list has precedence to supermat
 #   mul      = a factor to be multiplied to the row index
 #   mul3     = a factor to be multiplied to the row/column index of an array
 #   rowname  the root of the row names
 #   colname  the root of the column names
 #
 #   root     A character string to be used as the root of symbolic mat.
 #
 #
 # Value:
 #  If i and j are scalar,
 #    res[i,j] = mul*i + j,  i=1,2,...,nr, j=1,2,...,nc
 #    res[i,j,k] = mul3*(mul*i + j) + k,  k=1,2,...,n3
 #   or
 #    if supermat = 2, mat[((k-1)*nr+1):(k*nr),] = res[,,k]
 #    if supermat = 1, mat[,((k-1)*nc+1):(k*nc),] = res[,,k]
 #   or
 #    if list = 1, a list of length n3 consisting of res[,,k]
 #
 #  If supermat= is given, SMdim attribute will be added.
 #  Also S3 class supermatrix is set to the result.  -> abandoned 20150602
 #
 #  If i or j are voctor,
 #   The ii-jj element of the i-j submatrix will be
 #    sub[ii,jj] =(10^wid3)*(mul*ii+jj) +  mul3*i+j
 #   where wid3 is the # of digits of mul3*i+j.
 #
 #
 # If nr and nc are both scalar and if n3 > 0,
 # this program first generates an array of the size c(nr,nc,n3)
 # and, if requested, it will convert the array to either
 # a vertically stacked super matrix, a horizontally stacked super matrix,
 # or a list.
 #
 # If nr or nc are vector, the program works recursively.
 #
 #
 #  Note on supermatrix 20150531
 #   If nr=c(2,3,4)  and nc=c(1,2,4), the following supermatrix SM will be
 #   generated with attributes(SM)$SMdim=list(nrows=c(2,3,4), ncols=c(1,2,4))
 #
 #     c1     c1  c2       c1   c2   c3   c4
 # r1 1111    1112 1212    1113 1213 1313 1413
 # r2 2111    2112 2212    2113 2213 2313 2413
 #
 # r3 1121    1122 1222    1123 1223 1323 1423
 # r4 2121    2122 2222    2123 2223 2323 2423
 # r5 3121    3122 3222    3123 3223 3323 3423
 #
 # r6 1131    1122 1222    1123 1233 1333 1433
 # r7 2131    2122 2222    2123 2233 2333 2433
 # r8 3131    3122 3222    3123 3233 3333 3433
 # r9 4131    4132 4232    4123 4233 4333 4433
 #
 #   If nr=5  and nc=c(1,2,4), the following supermatrix SM will be
 #   generated with attributes(SM)$SMdim=list(nrows=1, ncols=c(1,2,4))
 #   This is a horizontally stacked matrix.
 #
 #     c1    c1  c2    c1  c2  c3  c4
 # r1 111   112 122   113 123 133 143
 # r2 211   212 222   213 223 233 243
 # r3 311   312 322   313 323 333 343
 # r4 411   412 422   413 423 433 443
 # r5 511   512 522   513 523 533 543
 #

 # function to set shapes
 setshape <- function( mat, shape, nodiag ){
  square=0; if( nrow(mat) == ncol(mat) ) square=1
  # printm(mat,shape,nodiag)
  if( square && shape == "DIAG"){
   mat=diag(diag(mat))
  }
  else if( square && (shape == "SYM" || shape == "SYML") ){
   mat[upper.tri(mat,diag=0)]=t(mat)[upper.tri(mat,diag=0)]
  }
  else if( square && shape == "SYMU" ){
   mat[lower.tri(mat,diag=0)]=t(mat)[lower.tri(mat,diag=0)]
  }
  else if( shape == "UPPERT" ){
   mat[lower.tri(mat, diag=nodiag)]=0
  }
  else if( shape == "LOWERT" ){
   mat[upper.tri(mat, diag=nodiag)]=0
  }
  return( mat )
 } # end of setshape



 # list or supermat
 if( list > 0 & supermat > 0 ) supermat=0

 rowname=rnroot; colname=cnroot; aname=anroot
 shape=toupper(shape)
 if( length(shape) == 0 ) shape=""


 ############# recursive call for supermatrix #######################
 if( length(nr) > 1  |  length(nc) > 1 ){

  # how may super-rows and columns
  lnr=length(nr); lnc=length(nc)
  mul0=ceiling_n(max(nc))$val
  if( is.null(mul) ) mul=mul0
  mul0=ceiling_n(lnc)$val
  if( is.null(mul3) ) mul3=mul0
  # wid3=ceiling_n(lnr)$power+ceiling_n(lnc)$power

  res=matrix(0,sum(nr),sum(nc))
  rownames(res)=1:sum(nr); colnames(res)=1:sum(nc)

  for( i in 1:lnr ){
   nri=nr[i]  # This works even if nr is a sclar!
   rnam=paste(rowname,i,"-",1:nri,sep="")
   for( j in 1:lnc ){
    ncj=nc[j]
    cnam=paste(colname,j,"-",1:ncj,sep="")
    # basic row and col elements
    row=matrix(rep(1:nri,ncj), nri, ncj)
    col=t(matrix(rep(1:ncj,nri), ncj, nri))
    A=Recall( nri,ncj,n3=0
              , shape=shape, nodiag=nodiag, supermat=0, list=0
              , mul=mul, mul3=mul3, rnroot=rowname, cnroot=colname )
    dimnames(A)=list(rnam,cnam)

    # reverse the order: submatrix-index + element-index      # 20200525
    wid3=ceiling_n(max(A))$power
    if( lnr == 1 ){
     A=(10^wid3)*j+A        # 20200525
    }
    else if( lnc == 1 ){
     A=(10^wid3)*i+A        # 20200525
    }
    else{
     A=(10^(wid3))*(mul3*i+j)+A        # 20200525
    }

    toi=sum(nr[1:i]); fromi=sum(nr)-sum(nr[i:lnr])+1
    toj=sum(nc[1:j]); fromj=sum(nc)-sum(nc[j:lnc])+1
    res[fromi:toi,fromj:toj]=A
    rownames(res)[fromi:toi]=rnam; colnames(res)[fromj:toj]=cnam
    #Print(i,j,A)
   }
  }
  #class(res)="supermat"
  attributes(res)$SMdim=list(nrows=nr, ncols=nc)

  # add root if any
  if( !is.null(root) ){
   dd=attributes(res)
   wid=ceiling_n(max(res))$power
   locdash=wid-wid3
   # Print(wid3,wid,locdash)
   resf=formatC(res,width=wid,format="d",flag="0")
   # res=paste(root,resf,sep="")     # 20200525
   res=paste(root,substr(resf,1,locdash),"_",substr(resf,locdash+1,wid),sep="")
   attributes(res)=dd
  }

  # fullsymb
  if( fullsymb ) attributes(res)$fullsymb=1

  return(res)

 } # end of recursive call for supermatrix



 ########### standard call #########################
 # basic row and col elements
 row=matrix(rep(1:nr,nc), nr, nc)
 col=t(matrix(rep(1:nc,nr), nc, nr))

 mul0=ceiling_n(nc)$val
 if( is.null(mul) ) mul=mul0


 if( n3 == 0 ){

  # simple matrix
  mat=mul*row+col
  dimnames(mat)=list(paste(rowname,1:nr,sep=""), paste(colname,1:nc,sep=""))
  if( length(shape) > 0 ) mat=setshape( mat, shape, nodiag )

  if( vec == 1 ){
   if( nr == 1 ) mat=col
   if( nc == 1 ) mat=row
  }

  if( !is.null(root) ){
   loczero=which(mat==0)
   dd=attributes(mat)
   wid=ceiling_n(max(mat))$power
   resf=formatC(mat,width=wid,format="d",flag="0")
   mat=paste(root,resf,sep="")
   mat[loczero]="0"
   attributes(mat)=dd
  }

  # fullsymb
  if( fullsymb ){
   attributes(mat)$fullsymb=1

   # full symb symmetric matrix
   if( nrow(mat) == ncol(mat) ){
    if( nrow(mat) == ncol(mat) && shape %in% c("SYM","SYML") ){
     mat[upper.tri(mat,diag=0)]=unlist(
     Map( function(x){paste("t(",x,")", sep="")}, mat[lower.tri(mat, diag=0)] )
      )
    }
    else if( shape == "SYMU" ){
     mat[lower.tri(mat,diag=0)]=unlist(
     Map( function(x){paste("t(",x,")", sep="")}, mat[upper.tri(mat, diag=0)] )
     )
    }
   }

  } # end of fullsymb

  return(mat)

 }
 else{

  # 3D array

  mul0=ceiling_n(n3)$val
  if( is.null(mul3) ) mul3=mul0

  res=array( 0, c(nr,nc,n3)
             , dimnames=list(paste(rowname,1:nr,sep="")
                             , paste(colname,1:nc,sep="")
                             , paste(aname,1:n3,sep=""))
  )
  mat0=mul*row+col       # 20200525,26
  wid3=ceiling_n(max(mat0))$power
  # Print(nr,nc,n3,wid3,mul3)
  for( k in 1:n3 ){
   mat=(10^wid3)*k+mat0       # 20200525,26
   if( length(shape) > 0 ) mat=setshape( mat, shape, nodiag )
   res[,,k]=mat
  }
  an=paste(anroot,1:n3, sep="")
  if( supermat == 2 ){
   # array to vertically stacked super matrix conversion
   res=t(matrix(aperm(res,c(2,1,3)),c(nc,nr*n3)))
   colnames(res)=paste(colname,1:nc,sep="")
   rownames(res)=rep(paste(rowname,1:nr,sep=""),n3)
   attributes(res)$SMdim=
    list(nrows=rep(nr,n3), ncols=nc
         , rownames=rep(an,each=nr), colnames=NULL, anames=an )
   #class(res)="supermat"
  }
  else if( supermat == 1 ){
   # array to horizontally stacked super matrix conversion
   res=array(res,c(nr,nc*n3))
   rownames(res)=paste(rowname,1:nr,sep="")
   colnames(res)=rep(paste(colname,1:nc,sep=""),n3)
   attributes(res)$SMdim=
    list(nrows=nr, ncols=rep(nc,n3)
         , rownames=NULL, colnames=rep(an,each=nc), anames=an )
   #class(res)="supermat"
  }
  else if( list > 0 ){
   # convert array to list
   L=list(); length(L)=n3
   for( k in 1:n3 )
    L[[k]]=res[,,k]
   res=L
   names(res)=paste(aname,1:n3,sep="")
  }

  if( !is.null(root) ){
   loczero=which(res==0)
   dd=attributes(res)
   wid=ceiling_n(max(res))$power
   resf=formatC(res,width=wid,format="d",flag="0")
   if( n3 == 0 ){       # 20200525,26
    res=paste(root,resf,sep="")
   }
   else{
    locdash=wid-wid3
    res=paste(root,substr(resf,1,locdash),"_",substr(resf,locdash+1,wid),sep="")
   }
   if( length(loczero) > 0) res[loczero]="0"
   attributes(res)=dd
  }

  # fullsymb:  This may NOT be necessary!!
  if( fullsymb ) attributes(res)$fullsymb=1


  return(res)

  }

} # end of demomat

#' Get Values from a List Consisting of Data Frames
#'
#' @param L A list of data frames
#' @param s1 Vector consisting of the subscripts for the 1st dimension
#' @param s2 Vector consisting of the subscripts for the 2nd dimension
#' @param s3 Vector consisting of the subscripts for the 3rd dimension
#' @param simplify = 0 to leave the result of get_valL unsimplified (list)
#'
#' @param value The value to be assigned. Either a vector or a matrix.
#'
#' @details
#' It is assumed that \code{L} is a list consisting of several data frames.
#' and the third subscript, \code{s3} is for the list.\cr
#' Therefore, \code{get_valL(L,,,s3)} with scalar s3 is
#' equivalent to \code{L[[s3]]} \cr
#' and  \code{get_valL(L,,,2:3)} returns the 2nd and the 3rd data frames
#' in a list. \cr
#'
#' \code{s1,s2,s3} can be a set of vectors consisting of row/col/list names.
#'
#' \cr
#' When some of the subscripts are out of range, only the valid ones
#' will be used.
#'
#' \cr
#' The result will be simplified to a matrix or vector if possible. \cr
#' When \code{length(s1)==1}, the result will be
#' a \code{length(L)} x \code{ncol(L[[1]])} matrix. \cr
#' When \code{length(s2)==1}, the result will be
#' a \code{ncol(L[[1]])} x \code{length(L)} matrix. \cr
#'
#' @examples
#' L0 <- list( A=data.frame(demomat(3,2)*10+1)
#'           , B=data.frame(demomat(4,3)*10+2)
#'           , C=data.frame(demomat(5,4)*10+3)
#'          )
#' L <- L0
#' get_valL( L, ,,2 ) # should be equal to get_valL( L, ,,"B" )
#' get_valL( L, 2, )  # should be equal to get_valL( L, "r2",, )
#' get_valL( L, ,2 )  # should be equal to get_valL( L, , "c2", )
#' get_valL( L, c(1,3),,3:2 )
#' get_valL( L, c(1,3),, c("B","A") )
#'
#' L <- L0; put_valL(L, 1:2,1:2,3) <- 0;                  Print(L)
#' L <- L0; put_valL(L, 1:2,1:2,c("C","A")) <- 0;         Print(L)
#' L <- L0; put_valL(L, 3:5,2:4,) <- 0;                   Print(L)
#' L <- L0; put_valL(L, 3:5,2:4,2) <- 0;                  Print(L)
#' L <- L0; put_valL(L, 1:4,1,2:3) <- 1:4;                Print(L)
#' L <- L0; put_valL(L, 1:2,2:3,2:3) <- demomat(2,2)*100; Print(L)
#'
#' # effect of simplify
#' L0 <- list( A=data.frame(demomat(3,4)*10+1)
#'           , B=data.frame(demomat(3,4)*10+2)
#'           , C=data.frame(demomat(3,4)*10+3)
#'          )
#' L <- L0
#' get_valL( L, ,,2 )
#' get_valL( L, ,,2, simplify=0 )
#' get_valL( L, 2, )
#' get_valL( L, 2,, simplify=0 )
#' get_valL( L, ,2 )
#' get_valL( L, ,2, simplify=0 )
#'
#' # When the selected elements are scalars, the result is a column vector.
#' get_valL(L0,1:2,1) # matrix
#' get_valL(L0,1,1)   # col vector
#' get_valL(L0,1,1:2) # matrix
#'
#' # mixed type variables in a data frame
#' A <- data.frame(demomat(3,4)*10+1)
#' A[,2] <- paste("c",as.character(A[,2]),sep="")
#' B <- data.frame(demomat(3,4)*10+2)
#' B[,2] <- paste("c",as.character(B[,2]),sep="")
#' C <- data.frame(demomat(3,4)*10+3)
#' C[,2] <- paste("c",as.character(C[,2]),sep="")
#' L <- list(A=A,B=B,C=C)
#'
#' get_valL( L, 2, )
#' get_valL( L, , 2 )
#'
#'
#' @return
#'  \code{get_valL(L, s1,s2,s3)} returns
#'  \code{(s1,s2)} element(s) of the s3-th data frame(s). \cr
#'  \code{put_valL(L, s1,s2,s3) <- value} replaces the \code{s1,s2}
#'  element(s) of the data frame(s) specified by \code{s3}
#'  with \code{value}.
#'
#' @export
#'

get_valL <- function( L, s1=NULL, s2=NULL, s3=NULL, simplify=1 ){
 # list version of get_val
 # Shin-ichi Mayekawa
 # 20171030,31
 # roxgen example added: 20171111
 # when #s of obs are not equal: 20201008
 # A removed from  @param: 20230328cato
 #
 # s3 is the list subscript
 # L is a list of K data frames
 #

 if( !is.list(L) ){
  cat("\nerror1:(get_valL) The first argument must be a list.\n")
  stop()
 }

 lenL=length(L)
 dims=lapply(L,dim)
 namL=names(L)

 # Print(s1,s2,s3,lenL,dims)

 # s3 given as names
 if( is.null(s3) ) s3=1:lenL
 else if( is.character(s3) ){
  s3=loceq(names(L),s3)
 }

 L=L[s3]
 lenL=length(L)
 dims=lapply(L,dim)

 # Print(s1,s2,s3,lenL,dims)

 for( k in 1:lenL ){

  Lk=L[[k]]
  if( is.null(s1) ){
   s01=1:nrow(Lk)
  }
  else if( is.character(s1) ){
   s01=loceq(rownames(Lk),s1)
  }
  else s01=s1

  if( is.null(s2) ){
   s02=1:ncol(Lk)
  }
  else if( is.character(s2) ){
   s02=loceq(colnames(Lk),s2)
  }
  else s02=s2
  ss1=s01[which(s01 %in% 1:nrow(Lk))]
  ss2=s02[which(s02 %in% 1:ncol(Lk))]
  # Print(s1,s01,s2,s02)
  # Print(k,ss1,ss2)
  if( length(ss1)*length(ss2) == 0 ){
   L[[k]]="____NULL____"     # 20201008
  }
  else{
   # Print(k,dim(Lk),s1,ss1,s2,ss2)
   L[[k]]=Lk[ss1,ss2,drop=0]
  }

 }

 # print(L)
 locnull=which(unlist(lapply(L,function(x){x[[1]]=="____NULL____"})))
 # Print(locnull)
 if(length(locnull) == length(s3) ){
  cat("\nerror1:(get_valL) Subscript out of range.\n")
  return( NULL )
 }
 else if( length(locnull) > 0 ){
  L=L[-locnull]
  namL=namL[-locnull]
 }

 if( simplify ){

  if( length(L) == 1 ){
   L=L[[1]]
  }
  else if(  all( sapply(L,nrow) == 1 )
            && all( sapply(L,ncol) == ncol(L[[1]])) ){
   cn=colnames(L[[1]])
   L=do.call( rbind, L )
   #L=matrix(unlist(L),length(L), byrow=1)
   rownames(L)=namL
   colnames(L)=cn
  }
  else if( all( sapply(L,ncol) == 1 )
           && all( sapply(L,nrow) == nrow(L[[1]])) ){
   rn=rownames(L[[1]])
   L=matrix(unlist(L),,length(L))
   colnames(L)=namL
   rownames(L)=rn
  }
 }
 return( L )

} # end of get_valL







#' @rdname get_valL
#' @export
#'

`put_valL<-` <- function( L, s1=NULL, s2=NULL, s3=NULL, value ){
 # modify the value of L at subs
 # Shin-ichi Mayekawa
 # 20171030
 # bugfix A in arg list -> L: 20180203
 #
 
 if( !is.list(L) ){
  cat("\nerror1:(put_valL) The first argument must be a list.\n")
  stop()
 }
 
 if( is.list(value) ){
  cat("\nerror1:(put_valL) The value cannot be a list.\n")
  stop()
 }
 
 lenL=length(L)
 dims=lapply(L,dim)
 
 
 if( is.null(s3) ) s3=1:lenL
 else if( is.character(s3) ){
  s3=loceq(names(L),s3)
 }
 
 # Print(s1,s2,s3)
 
 for( k in s3 ){
  if( is.null(s1) ){
   s01=1:nrow(Lk)
  }
  else if( is.character(s1) ){
   s01=loceq(rownames(Lk),s1)
  }
  else s01=s1
  
  if( is.null(s2) ){
   s02=1:ncol(Lk)
  }
  else if( is.character(s2) ){
   s02=loceq(colnames(Lk),s2)
  }
  else s02=s2
  
  ss1=s01[which(s01 %in% 1:nrow(L[[k]]))]
  ss2=s02[which(s02 %in% 1:ncol(L[[k]]))]
  
  # Print(ss1,ss2)
  if( length(ss1)*length(ss2) == 0 ){
   cat("\nerror1:(put_valL) Subscript out of range.\n")
   return( L )
  }
  # Print(k,s1,s01,ss1,s2,s02,ss2)
  if( is.matrix(value) && (length(ss1) != nrow(value) |
                           length(ss2) != ncol(value) ) ){
   cat("\nerror1:(put_valL) The replacement does not comform.\n")
   stop()
  }
  L[[k]][ss1,ss2]=value
 }
 
 return( L )
 
 
} # endof `put_valL<-`

#' Calculates mean vector and dispersion matrix with case weight
#'
#' @param data data matrix or data frame.
#' @param weight case weight vector of length nrow(data) or 1.
#' @param by a factor or a list of factors, each of length nrow(data) \cr
#' to be used as the INDICES of by function.
#' @param byvarname vector containing the names of by varialbes to be used.
#'  (Not Yet Implemented.)
#' @param vardef = "n-1" to use n-1 as denominator for variance.
#' @param nocov = 1 to skip the calculation of the variance/covariance matrix.
#' \cr \code{mands} function will be used.
#' @param cor = 1 to return the correlation matrix as well.
#' @param print = 1 to print the result
#' @param simplify = 1 to simplify the rewult when by= is given
#'
#' @details
#' When the dispersion matrix is not needed (nocov=1) use \code{mands} function.
#' \cr
#' When simplified=0 is given, the result is a list consisting of \cr
#' vectors: mean, var, std, nobs \cr
#' matrix: cov, nobspair \cr
#' scalars: n, vardef \cr
#' where nobspair is a matrix of the same size as cov containing
#' the weighted # of observations used to calculate cov. \cr
#' When cor=1 is given, correlation matrix will also be returned.
#'
#' When by is specified, the result is a class by list if simplify=0,
#' Otherwise, it is a list consisting of \cr
#' matrices: mean, var, std, nobs \cr
#' lists: cov, nobspair \cr
#' scalars: n, vardef \cr
#' \cr
#' To convert the list of covariance matrices to an array or stacked matrix,
#'  use:
#'  \cr
#'  \code{ library(lazy.mat) } \cr
#'  \code{ cov_array=l2a(res1$cov) } \cr
#'  \code{ cov_mat=a2m(cov_array,method=2) } \cr
#'
#' @return list of: \cr
#' mean vector, var vector, std vector, nobs vector, \cr
#' where nobs is sum(weights) for nonmissing obs and n=nrow(data). \cr
#' cov matrix,  nobspair matrix which contains # of obs. per pair
#' , or NULL, \cr
#' n (# of rows), and vardef.
#'
#' @examples
#' ##################################
#' # data matrix with by group
#' ##################################
#' nobs <- 1000
#' ndims <- 2
#'
#' # covariance matrix
#' cov <- 0.2
#' C1 <- matrix(cov,ndims,ndims)+(1-cov)*diag(ndims)
#' C1[1,1] <- .5
#' C2 <- -C1; diag(C2) <- diag(C1)
#' C3 <- diag(ndims)
#' Printm(C1,C2,C3)
#'
#' # random numbers
#' seed=1701
#' set.seed(seed)
#'
#' # X1 ~ N(0,C1),  X2 ~ N(0,C2),   X3 ~ N(0,C3)
#' X1=matrix(rnorm(nobs*ndims),nobs)
#' ev <- eigen(C1); eva <- diag(ev$values); eve <- ev$vectors
#' X1 <- X1%*%diag(sqrt(ev$values))%*%t(ev$vectors)
#' X2 <- matrix(rnorm(nobs*ndims),nobs)
#' ev <- eigen(C2); eva <- diag(ev$values); eve <- ev$vectors
#' X2 <- X2%*%diag(sqrt(ev$values))%*%t(ev$vectors)
#' X3 <- matrix(rnorm(nobs*ndims),nobs)
#'
#' X <- rbind(X1,X2,X3)
#' byvar <- rep(c("pos","neg","zero"), each=nobs)
#'
#' # missing values
#' locmiss <- sample(1:length(X), 0.2*length(X))
#' X[locmiss]=NA
#'
#' # statistics w/o by-group
#' res0 <- mandd( X )
#' # statistics by by-group
#' res1 <- mandd( X, 1, byvar, simplify=1 )
#' Printm(res1$mean, res1$std, res1$cov)
#'
#'
#' #############################
#' # grid points and weight
#' #############################
#' # C1, C2, C3 are defined above.
#' npoints <- 11; thmin <- -3; thmax <- 3; nobs2 <- 10000
#' theta <- seq(thmin,thmax,length.out=npoints)
#' theta <- cprod(theta,theta)
#' if( ndims > 2 )
#'  for( i in 2:ndims )
#'   theta <- cprod(theta,theta)
#'
#' # weight:  pdf of N(0,C1)
#' thdist1 <- exp( -0.5*diag( theta%*%solve(C1)%*%t(theta) ) )
#' thdist1 <- thdist1/sum(thdist1)
#' n1 <- round(nobs2*thdist1+.5)
#'
#' # statistics
#' res20 <- mandd( theta, n1 )
#' res200 <- mandd( theta )
#' Printm(res20$mean, res200$mean)
#' Printm(res20$var, res200$var)
#' Printm(C1, res20$cov, res200$cov)
#'
#'
#' # missing values
#' seed <- 1701
#' set.seed(seed)
#' locmiss1 <- sample( 1:(npoints^ndims), max(1,(npoints^ndims)/10) )
#' theta[locmiss1] <- NA
#'
#' # statistics
#' res2 <- mandd( theta, n1 )
#' Printm(res2$mean, res2$std, res2$cov)
#'
#'
#' # grid points and weight with by group
#' # theta, nobs2, C1, C2, C3 are defined above.
#' # weight:  pdf of N(0,C2)
#' thdist2 <- exp( -0.5*diag( theta%*%solve(C2)%*%t(theta) ) )
#' thdist2[is.na(thdist2)] <- 0
#' thdist2 <- thdist2/sum(thdist2)
#' n2 <- round(nobs2*thdist2+.5)
#' # weight:  pdf of N(0,C3)
#' thdist3 <- exp( -0.5*diag( theta%*%solve(C3)%*%t(theta) ) )
#' thdist3[is.na(thdist3)] <- 0
#' thdist3 <- thdist3/sum(thdist3)
#' n3 <- round(nobs2*thdist3+.5)
#'
#' Theta=rbind(theta,theta,theta)
#' n=c(n1,n2,n3)
#' byvar2=rep(c("pos","neg","zero"),each=nrow(theta))
#'
#' # statistics
#' res3 <- mandd( Theta, n, byvar2, simplify=1 )
#' Printm(res3$mean, res3$std, res3$cov)
#'
#' @export
#'

mandd <- function( data, weight=1, by=NULL, byvarname=NULL
                   , vardef="n", nocov=0, cor=0, print=0, simplify=0 ){
 # weighted statistics: mean and dispersion matrix
 # Shin-ichi Mayekawa
 # 20140725 at Madison
 # n added: 20150605
 # nocov: 20171024dnc, 20180227dnc
 # nobs and pick up numeric vars: 20180227dnc
 # missing values: 20180227dnc,28
 # nobspair: 20190603
 # by: 20190603,04
 # cor: 20190604,05dnc
 # NaN -> NA: 20190606dn
 # nrow(data) == 1: 20200830cot ???????????????????????????????
 # n=rep(nrow(data),ncol(data)) <--- stack_df side effect: 20200922
 # all NA: 20210209
 # weight=rep(1,n) removed: 20250115
 # Print --> Printm in @examples: 20250115
 #

 #
 # Args:
 #   data     data frame or matrix or vector
 #   weight   weight vector or 1
 #   vardef   "n" or "n-1"
 #
 #
 # Value:
 #   list of mean, var, std, nobs, cov, n, vardef
 #  where
 #   nobs is the sum of weights
 #

 # use mands if cov is not requested
 if( nocov ){
  return( mands( data=data, weight=weight, by=by, byvarname=byvarname
                 , vardef=vardef, print=print, simplify=simplify ) )
 }


 # # of by variables
 nby=0
 if( is.null(by) ) nby=0
 else if( is.list(by) ) nby=length(by)
 else if( is.vector(by) ) nby=1
 else if( is.matrix(by) && ncol(by) == 1 ){
  nby=1
  by=c(by)
 }
 else if( is.matrix(by) && ncol(by) == 1 ){
  nby=1
  by=as.vector(by)
 }
 else{
  cat("\nerror1:(mands) by must be a vector or list of variables.\n")
  return(NULL)
 }

 # make data a data frame so that, when by is cbinded, the name "by"
 # is used instead of "INDICES".
 if( is.vector(data) ){
  vname=as.character(substitute(data))
  data=matrix(data,,1)
  if( all(regexpr("^[[:alpha:]][[:alnum:]]*$",vname) > 0 ) )
   colnames(data)=vname
 }

 if( is.matrix(data) ) data=as.data.frame(data, stringsAsFactors=0 )
 vname=colnames(data)
 n=nrow(data)
 if( length(weight) == 1 ) weight=rep(1,n)

 # numeric only
 locallna=which(apply(data,2,function(x) all(is.na(x))))
 locnonnum=which(vartype(data) != "numeric")
 locdel=union(locallna,locnonnum)
 # Print(locallna,locnonnum,locdel)
 if( length(locdel) > 0 ) data=data[,-locdel, drop=FALSE]
 nvar=ncol(data)
 if( nvar <= 0 ){
  cat("\nerror1:(mands) Must have numeric variables.\n")
  retun(NULL)
 }


 ##################################
 # analysis of byvar
 #
 # See matReplace.R in lazy.symbolic.
 # See also test_parent.R in lay.tools directory
 #####################################


 ############## recursive call when by is present #############
 if( !is.null(by) ){

  byvalues=unique(by)
  byvarname=deparse(substitute(by))

  res=by( cbind(data, weight), by
          , FUN=function(x){
           mandd( x[,-ncol(x)],  x[,ncol(x)], by=NULL, vardef=vardef
                  , cor=cor, print=print-8 ) } )

  if( nby == 1 ){
   if( simplify ){
    names=dimnames(res[[1]])
    mean=get_valLL(res,"mean")
    std=get_valLL(res,"std")
    var=get_valLL(res,"var")
    nobs=get_valLL(res,"nobs")
    n=get_valLL(res,"n")
    cov=get_valLL(res,"cov")
    nobspair=get_valLL(res,"nobspair")
    if( cor ) cor=get_valLL(res,"cor")
    else cor=NULL

    res=list( mean=mean, std=std, var=var, nobs=nobs, n=n, cov=cov
              , nobspair=nobspair, cor=cor, n=n, vardef=vardef )

   }
  }

  if( print ) print(res)

  return( res )

 }



 ############# simple call when by is  not present ############

 Rmat=NULL
 if( is.vector(data) ) data=matrix(data,,1)
 data=data[, vartype(data)=="numeric", drop=FALSE]
 nvar=ncol(data)
 if( nvar == 0 ){
  cat("\nerror1:(mandd) Must have numeric variables.\n")
  retun(NULL)
 }
 vname=colnames(data)
 n=rep(nrow(data), ncol(data))

 # Print(n,ncol(data),colnames(data))
 # Print(length(weight))
 # stop()

 # if( length(weight) == 1 ) weight=rep(1,n)   # 20250115

 if( tolower(vardef) == "n" ) method="ML"
 else method="unbiased"

 if( all(!is.na(data)) ){
  # No Missing Elements
  # use native cov.wt function
  res=cov.wt( data, weight, cor=cor, method=method )
  if( cor ){
   Rmat=res$cor
  }
  cov=res$cov
  mean=res$center
  var=diag(cov)
  std=sqrt(var)
  nobs=rep(sum(weight),nvar)
  nobspair=matrix(sum(weight),nvar,nvar)
  dimnames(nobspair)=dimnames(cov)
  rm(res)
 }
 else{
  # With Missing Elements
  cov=matrix(NA,nvar,nvar)
  if( cor ) Rmat=cov
  nobspair=cov
  for( i in 1:nvar ){
   locobsi=!is.na(data[,i])
   if( nrow(data[,i,drop=0]) == 1 ){       # 20200830
    locobsi=!is.na(data[,i])
   }
   for( j in 1:i ){
    locobs=locobsi & !is.na(data[,j])
    if( sum(locobs) > 0 ){
     nobspair[i,j]=sum(weight[locobs])
     temp=cov.wt( data[locobs,c(i,j)], weight[locobs], cor=cor
                  , method=method )
     cov[i,j]=temp$cov[1,2]
     if( cor ){
      Rmat[i,j]=temp$cor[1,2]
     }
    }
    else{
     nobspair[i,j]=0
    }
    nobspair[j,i]=nobspair[i,j]
    cov[j,i]=cov[i,j]
    if( cor ) Rmat[j,i]=Rmat[i,j]
   }
  }
  res=mands( data, weight=weight, vardef=vardef )
  mean=res[1,]
  std=res[2,]
  var=res[3,]
  nobs=res[4,]
 }
 # var=diag(cov)
 # std=sqrt(var)
 names(mean)=vname
 names(var)=vname
 names(std)=vname
 names(n)=vname
 dimnames(cov)=list(vname,vname)
 dimnames(nobspair)=dimnames(cov)
 if( cor ) dimnames(Rmat)=dimnames(cov)

 if( print> 0 )
  Print( n,nobs,mean,var, std, cov )

 # NaN to NA
 mean[is.nan(mean)]=NA
 std[is.nan(std)]=NA
 var[is.nan(var)]=NA

 res=list( mean=mean, var=var, std=std, nobs=nobs, cov=cov, nobspair=nobspair
           ,  cor=Rmat, n=n, vardef=vardef)
 return(res)

} # end of mandd

#' Generate all m-permutations of n without repetition
#'
#' @param n total # of elements
#' @param m # of elements to choose
#' @param vals a vector of values to be permuted
#' @param noreverse = 1 to remove reverse-equivalent rows \cr
#' = 2 to check the result
#'
#' @details
#' The recursive algorithm below is used.
#' \preformatted{
#'   res=NULL
#'   for( i in 1:n )\{
#'     res <- rbind(res, cbind(vals[i], Recall(n - 1, m - 1, vals[-i])))
#'   \}
#' }
#'
#' The method of removal of reverse-equivalent rows comes from
#' the answer to stack overflow question: \cr
#' "How to remove reverse rows in a permutation matrix?" \cr
#' by obchardon at Sep 15, 2020 at 7:46 \cr
#' https://stackoverflow.com/questions/63896177/how-to-remove-reverse-rows
#' -in-a-permutation-matrix
#'
#'
#'
#' @return
#' A matrix of the size P(n,m) x m.
#'
#' @examples
#' n <- 5
#' for( m in 1:n )
#'  Print( m, perm(n,m), comb(n,m) )
#'  
#'  Print( perm(3,3), perm(3,3,,1) )
#'
#' @export
#'

perm <- function(n, m, vals=1:n, noreverse=0 ) {
 # permutations
 # Shin-ichi Mayekawa
 # 20220621
 # noreverse: 20231216
 # examples added: 20240415
 #

 if (m == 1) return( matrix(vals, n, 1) )
 else if (n == 1) return( matrix(vals, 1, m) )
 else{
  res=NULL
  for( i in 1:n ){
   res=rbind(res, cbind(vals[i], Recall(n - 1, m - 1, vals[-i])))
  }

  if( noreverse == 1 ){
   loc=which( res[,1] < res[,m] )
   res=res[loc,]
  }
  else if( noreverse == 2 ){
   res0=res
   loc=which( res[,1] < res[,m] )
   res=res[loc,]

   # reverse-equivalent part
   res2=res0[-loc,]
   # reverse the order
   res2=t(apply(res2,1,rev))
   # sort
   res2=res2[do.call(order,as.data.frame(res2)),]
   if( any(res-res2 != 0) ){
     cat("\nerror1:(perm) Removal of reverse-equivalent rows failed!!\n")
     stop()
   }
  }
  return( res )
 }

} # end of perm

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/demomat.R
\name{demomat}
\alias{demomat}
\title{Set up a matrix whose elements consist of row and col numbers.}
\usage{
demomat(
  nr,
  nc,
  n3 = 0,
  root = NULL,
  vec = 0,
  shape = NULL,
  nodiag = 0,
  supermat = 0,
  list = 0,
  fullsymb = 0,
  mul = NULL,
  mul3 = NULL,
  rnroot = "r",
  cnroot = "c",
  anroot = "a"
)
}
\arguments{
\item{nr}{# of rows of the matrix to be generated
or a vector consisting of the # of rows of each submatrix.}

\item{nc}{# of columns of the matrix to be generated
or a vector consisting of the # of columns of each submatrix.}

\item{n3}{# of the 3d dimension or 0, if any}

\item{root}{The root character of the elements of the symbolic matrix. \cr
If this is present, a character matrix/array will be created.}

\item{vec}{= 1 to drop the redundant subscript.}

\item{shape}{Shape of the resulting matrix: \cr
"sym", "symU", "symL", "diag", "uppert", "lowert"}

\item{nodiag}{= 1 to set the diagonal elements to 0 when either
"upper" or "lower" is specified}

\item{supermat}{= 1 to create horizontally stacked array\cr
= 2 to create vertically stacked array\cr
= 0 othereize}

\item{list}{= 1 to return the array as list when n3 > 2.}

\item{fullsymb}{= 1 to make the result a fully symbolic matrix}

\item{mul}{See below.}

\item{mul3}{See below.}

\item{rnroot}{root of the row names}

\item{cnroot}{root of the column names}

\item{anroot}{root of the third dimension names}
}
\value{
A matrix or an array. \cr
Note that even with vec=1 option, thre result is a matrix. \cr
Use c() to convert it to a vector.
}
\description{
This can create either symbolic or numeric matrix.
}
\details{
The i,j elements of a matrix will be named as mul*i + j\cr
 The i,j,k elements of an array will be named as mul3*(mul*i + j) + k\cr
 \cr
 When a supermatrix is generated, SMdim attributes will be set \cr
 where SMdim	is a list of the form \cr
 (nrows=nr, ncols=nc, rownames=rn, colnames=cn, anames=an) \cr
 where \cr
  nr is a vector consisting of the number of rows of each submatrix, \cr
  nc is a vector consisting of the number of columns of each submatrix \cr
  rn is a vector consisting of the supplemental row names \cr
  cn is a vector consisting of the supplemental column names \cr
  an is a vector consisting of the names associated with the 3rd dim.\cr
  '
}
\examples{
demomat( 11,3 ) # numeric matrix
demomat( 11,3,root="a" ) # symbolic matrix
demomat( 4,1,root="a", vec=1 ) # symbolic matrix

demomat( 4,3,2 ) # numeric array
demomat( 4,3,2, supermat=1 ) # numeric supermatrix (horizontally stacked)
demomat( 4,3,2, supermat=2 ) # numeric supermatrix (vertically stacked)

# Below two creates almost identical super matrices.
demomat( 4,3,2, root="a", supermat=1 )
demomat( 4,c(3,3), root="a" )

# 2 x 2 partitioned matrix
A=demomat( c(4,3),c(3,2), root="a" )
Printb(A)

# various shape
demomat(3,3,root="a", shape="sym")
demomat(3,3,root="a", shape="diag")
demomat(3,3,root="a", shape="lowert")

# fully symbolic matrix
demomat(4,4, root="S", fullsymb=1)
demomat(4,4, root="S", shape="sym", fullsymb=1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_info.R
\name{package_info}
\alias{package_info}
\title{Creates html files from the help files of an R package}
\usage{
package_info(pkgname, outdir = "d:/Rpgm/packages/HTML_help", debug = 0)
}
\arguments{
\item{pkgname}{Package name}

\item{outdir}{The name of the directory to which the html files 
are written.}

\item{debug}{= 1 to print details.}
}
\description{
Creates html files from the help files of an R package
}
\examples{
#  res1=package_info("lazy.tools",      "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.mat",        "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.symbolic",   "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.stat",       "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.sasef",      "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.mdpref",     "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.procrustes", "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.irt",        "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.fa",         "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.cluster",    "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.mirt",       "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.mds",        "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.accel",      "d:/Rpgm/packages/HTML_help")
#  res1=package_info("lazy.Bayes",      "d:/Rpgm/packages/HTML_help")
cat("none\n")

}
\references{
Part of the code is from http://m884.hateblo.jp/entry/20100214/1266135349 by
  masaha03 (id:phosphor_m).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smoothra.R
\name{smoothra}
\alias{smoothra}
\title{Running Average Smooth}
\usage{
smoothra(x, bandwid = 3, maxiter = 1)
}
\arguments{
\item{x}{a univariate time series.}

\item{bandwid}{# of the neighbouring elements to be averaged.
Must be a positive odd number.}

\item{maxiter}{# of repetitions.}
}
\value{
a vector of the same length as x.
}
\description{
Running Average Smooth
}
\details{
The following code will be repeated maxiter times: \cr
   \code{filter(x,rep(1/bandwid,bandwid), sides=2)} \cr
x[1:m] and x[(length(x)-m+1):length(x)] will remain unchanged.
where \code{m=(bandwid-1)/2}.
}
\examples{
x <- c(4, 1, 3, 6, 6, 4, 1, 6, 2, 4, 2)
y <- smoothra( x, bandwid=3, 2 )
Print(x,y)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midp2brk.R
\name{brk2midp}
\alias{brk2midp}
\title{Define Midpoints from Break Points}
\usage{
brk2midp(brk, min = NULL)
}
\arguments{
\item{brk}{a vector of break points.}

\item{min}{The value of the lowest midpoint.
This has priority over margin.}
}
\value{
A vector of midpoints of length length(brk)-1.
}
\description{
Define Midpoints from Break Points
}
\details{
Break values from midpoint values: \cr
 \code{brk[1]=midp[1] - margin}\cr
 \code{brk[2]=midp[1] + (midp[2]-midp[1])/2} \cr
 \code{brk[3]=midp[2] + (midp[3]-midp[2])/2} \cr
 \code{brk[length(midpoints)+1]=midp[length(midpoints)] + margin} \cr
 where \code{margin=(midp[2]-midp[1])/2}. \cr
 In general, the break point is located at the center of two midpoints:\cr
 \code{brk[k]=0.5*(midp[k-1]+midp[k])}
 \cr

Midpoint values from break values: \cr
 \code{midp[1]=0.5*(brk[1]+brk[2])} or {midp[1]=min}\cr
 \code{midp[2]=0.5*(brk[2]+brk[3])} \cr
 \code{midp[3]=2*brk[3]-midp[2]} \cr
 \code{brk[length(brk)-1]=2*brk[length(brk)-1]-midp[length(brk)-2]} \cr
\cr
Note that midp[1] is treated separately and it is assumed that
the break point is located at the center of two midpoints.
\cr\cr
Try below to see how the obs on the border are classified. \cr
   \code{ xx <- c(1,2,3,4,5,6,7) } \cr
   \code{ brks <- c(0,2,4,6,8) } \cr
   \code{ cc1 <- cut(xx,brks) } \cr
   \code{ cc2 <- cut(xx,brks, right=FALSE) } \cr
   \code{ Print(xx,as.character(cc1),as.character(cc2)) }\cr
}
\examples{
midp <- c(0,1,2,4,6,9,10,11)
brk <- midp2brk( midp )
Print( midp, brk, brk2midp(brk) )
brk[1] <- -4
Print( midp, brk, brk2midp(brk) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Exists.R
\name{Exists}
\alias{Exists}
\title{Check If the Objects Listed Exist in the Parent Frame}
\usage{
Exists(
  varname,
  ntogoback = 1,
  mode = c("numeric", "character", "logical"),
  get = 0,
  debug = 0
)
}
\arguments{
\item{varname}{A string vector containing object names}

\item{ntogoback}{n for \code{parent.frame}}

\item{mode}{A string vector containing what to look for.}

\item{get}{= 1 to retrieve the value of those objects.}

\item{debug}{= 1 to print intermediate results}
}
\value{
If \code{get=0}: \cr
  \code{ex} A vector indicating the existence of each object. \cr
 If \code{get=1}: \cr
  A list of \code{ex} and a list, \code{value}, which contains
  the values of the objects found.
}
\description{
Check If the Objects Listed Exist in the Parent Frame
}
\examples{
a=0; b=0; c=0; d=0; e=0
rm(a,b,c,d,e)

a="A0"
b="B0"
d=function(a){print(a)}
Exists( varname=c("a","b","c","d","e") )
Exists( varname=c("a","b","c","d","e"), get=1 )
Exists( varname=c("a","b","c","d","e"), get=1, mode="function" )

func1 <- function(){
 a="A1"
 b="B1"
 res=Exists( varname=c("a","b"), get=1 )
 Print(res)
 res=Exists( varname=c("a","b"), get=1, ntogoback=2 )
 Print(res)
}
func1()


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_range.R
\name{get_range}
\alias{get_range}
\title{Convert levels, starting locations, or ending locations to range matrix}
\usage{
get_range(levels = NULL, start = NULL, end = NULL, delim = NULL)
}
\arguments{
\item{levels}{vector of levels.}

\item{start}{vector of starting location. \cr
The last element must be the ending location.}

\item{end}{vector of ending locations. \cr
The last element must be the ending location.}

\item{delim}{vector of delimiter locations.}
}
\value{
range: a \code{length(levels) x 2} matrix, \cr
where \code{length(levels) = length(start)-1 = length(end)}.
}
\description{
Convert levels, starting locations, or ending locations to range matrix
}
\details{
If \code{levels} is given: \cr
\code{ range <- cbind(cumsum(levels)-levels+1,cumsum(levels)) } \cr

If \code{start} is given: \cr
 \code{range <-  cbind(start[-length(start)]} \cr
    \code{, c(start[2:(length(start)-1)]-1,start[length(start)]) ) } \cr

If \code{end} is given, \cr
 \code{range <- cbind(c(1,end[1:(length(end)-1)]+1),end) } \cr

If \code{delim} is given, \cr
 \code{range <-  cbind(c(1,delim[-length(delim)]+1)} \cr
    \code{, c(delim[-length(delim)]-1,delim[length(delim)])) } \cr

 Note that \code{start[-length(start)], end} and \code{delim}
 must be strictly increasing sequence of numbers.
}
\examples{
get_range( c(3,4,2) )
get_range( start=c(1,4,8,9) )
get_range( end=c(3,7,9) )
get_range( delim=c(3,7,9) )

levels <- c(1,2,3,1)
range <- get_range( levels )
levels2 <- range[,2]-range[,1]+1
identical(levels,levels2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen_test_score.R
\name{gen_test_score}
\alias{gen_test_score}
\title{Generating Test Score Distribution}
\usage{
gen_test_score(
  n,
  minscore = 0,
  maxscore = 100,
  normal = c(50, 15),
  beta = NULL,
  beta_ms = NULL,
  dump = 0,
  print = 0,
  plot = 0,
  title = "Score Distribution"
)
}
\arguments{
\item{n}{# of observations.}

\item{minscore}{The minimum score.}

\item{maxscore}{The maximum score.}

\item{normal}{Vector consisting of mean and std for normal distribution.}

\item{beta}{Vector of alpha and beta parameters for beta distribution
in [minscore,maxscore].}

\item{beta_ms}{Vector of mean and std of the generalized beta distribution
in [minscore,maxscore].}

\item{dump}{= 1 to output the discrete random numbers.}

\item{print}{= 1 to print the result.}

\item{plot}{= 1 to plot the result.}

\item{title}{Title string to be used in plot.}
}
\value{
If dump == 0, a matrix consisting of (score, frequency). \cr
otherwise, a list of rv, random numbers, and freqdist matrix.
}
\description{
This function generates a frequency distribution table
 of n discrete test scores in [minscore,maxscore].
}
\details{
This function generates a frequency distribution table
 of n discrete test scores in [minscore,maxscore].

Continuous random numbers, either normal or generalized beta,
are first generated and then they are discretized by cut function.
Those values outside of [minscore,maxscore] will be set to minscore
or maxscore, resp.

beta_ms parameter has the priority over beta and nomal parameter.
}
\examples{
seed <- 1701
set.seed(seed)

minscore <- 0; maxscore <- 100
n <- 500

s1 <- gen_test_score( n, minscore, maxscore, beta=c(4,8), plot=1 )
( mands(s1[,1],s1[,2]) )
s2 <- gen_test_score( n, minscore, maxscore
                    , beta_ms=c(33.29, 13.07), plot=1 )
( mands(s2[,1],s2[,2]) )

# mixture
minscore <- 0; maxscore <- 50
scoredist1 <- gen_test_score( n, minscore, maxscore, beta=c(4,8), plot=1 )
scoredist2 <- gen_test_score( n, minscore, maxscore, normal=c(30,5), plot=1)
scoredist12 <- merge_tables( scoredist1,scoredist2 )
barplot(height=scoredist12[,2], names.arg=as.character(scoredist12[,1])
        , main="Score Dist", cex.names=.65, space=0, las=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{str_olay}
\alias{str_olay}
\title{Overlay strings}
\usage{
str_olay(..., bottom = 1, blanks = " ", BLANK = " ")
}
\arguments{
\item{...}{Strings to be overlayed. Eeach string must be a scalar.}

\item{bottom}{= 0 to overlay on top of the last string}

\item{blanks}{Characters which will be treated as blank.}

\item{BLANK}{Blank character to be used in the final result.}
}
\description{
Overlay strings
}
\details{
When bottom=1, the i th string will be overlayed on top of i-1 th. \cr
}
\examples{
c1="12345 78 01"
c2="a c:efg x"
str_olay( c1,c2 )
str_olay( c1,c2, bottom=0 )
str_olay( c1,c2, blanks=": " )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{ceiling_n}
\alias{ceiling_n}
\title{Return the Ceiling and Power of the Input using Radix (Base) of n}
\usage{
ceiling_n(x, n = 10)
}
\arguments{
\item{x}{input vector, x > 0}

\item{n}{Radix or Base}
}
\value{
A list of: \cr
power The power \cr
val   The value of ceiling = n^power
}
\description{
Return the Ceiling and Power of the Input using Radix (Base) of n
}
\details{
When n=10 \cr
    0   <= x <    10,    ceiling_n(x)$val=10,  ceiling_n(x)$power=1 \cr
   10   <= x <   100,    ceiling_n(x)$val=100,  ceiling_n(x)$power=2 \cr
  100   <= x <  1000,    ceiling_n(x)$val=1000,  ceiling_n(x)$power=3 \cr
 When n=2 \cr
    0   <= x <     2,  ceiling_n(x,2)$val=2,  ceiling_n(x,2)$power=1 \cr
    2   <= x <     4,  ceiling_n(x,2)$val=4,  ceiling_n(x,2)$power=2 \cr
    4   <= x <     8,  ceiling_n(x,2)$val=8,  ceiling_n(x,2)$power=3 \cr
}
\examples{
ceiling_n( c(1,9,10,99,100) )
ceiling_n( c(1,2,3,4,7,8,15,16), n=2 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loceq.R
\name{loceq}
\alias{loceq}
\title{Returns the locations of the elements of the first argument whose
values are equal to one of the elements in the second argument}
\usage{
loceq(a, b, keep = 1)
}
\arguments{
\item{a}{vector}

\item{b}{vector}

\item{keep}{= 1 not to sort the result}
}
\value{
vector consisting of the locations of the elements of
the first argument whose
values are equal to one of the elements in the second argument, or 0
}
\description{
Returns the locations of the elements of the first argument whose
values are equal to one of the elements in the second argument
}
\details{
\code{loceq( a, b, keep=0 )} is equivalent to \cr
\code{which( match(a,b, nomatch=0) > 0 ) } \cr
or \cr
\code{which( a \%in\% b )} .\cr

loceq(letters, c("x","cc","a"))  returns c(24,1) \cr
With keep=0, the result will be sorted according to the order
of appearance in the first argument.\cr
loceq(letters, c("x","cc","a"), 0)  returns c(1,24) \cr\cr
 If subset below contains a subset of the variable names of
 a data frame,\cr
 loc=loceq( colnames(dataframe), subset ) \cr
 wii return the location of those variables.\cr\cr
 It will return 0 if no match.
}
\examples{
loceq(letters, c("x","cc","a"))
loceq(letters, c("x","cc","a"), 0)
loceq(c("x","cc","a"), letters)
loceq(c("x","cc","a"), letters, 0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance.R
\name{distrectw}
\alias{distrectw}
\title{Calculation of Weighted Distance Matrix between the rows of F and A}
\usage{
distrectw(F, A, W = matrix(1, nrow(A), ncol(A)), R = NULL, outD2 = 0)
}
\arguments{
\item{F}{n x r matrix}

\item{A}{p x r matrix}

\item{W}{p x r maatrix}

\item{R}{p x (r x r) array}
}
\value{
A matrices of size n x p
}
\description{
Calculation of Weighted Distance Matrix between the rows of F and A
}
\details{
This program calculates the weighted distance between the rows of F matrix
and the rows of A matrix as\cr
\deqn{ d_{ij} = \sqrt( (f[i,]-a[j,])' diag(W[j,]) (f[i,]-a[j,]) )  } \cr
\deqn{ d_{ij} = \sqrt( (f[i,]-a[j,])' R[,,j] (f[i,]-a[j,]) )  }

\cr
Note that \code{W} matrix will NOT be squared when calculating distance.
\cr
}
\examples{
F <- matrix( c(0,0,  3,0,  0,4), 3,2,  byrow=1 )
A <- matrix( c(0,0,  3,4 ), 2,2, byrow=1 )
distrectw( F,A )
Wa <- matrix( c(1,1,  1,2 ), 2,2, byrow=1 )
distrectw( F,A, Wa )
R <- matrix(c(1,0, 0,1, 1,0.5, 0.5,3 ), ,2, byrow=TRUE)
# R <- t(R)
# dim(R)=c(2,2,2)
distrectw( F,A, R=R )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqc.R
\name{freqc}
\alias{freqc}
\title{Frequency Distribution of Many Discrete Variables}
\usage{
freqc(..., useNA = "no", sortrow = 1, sortcol = 0, debug = 0)
}
\arguments{
\item{...}{Discrete/categorical variables to be tabulated}

\item{useNA}{= "ifany" if the frequency of NA is needed.}

\item{sortrow}{= 0  not to sort the rows of the result (category values)}

\item{sortcol}{= 1  to sort the column of the result (variable names)}

\item{debug}{= 1 to print intermediate result}
}
\value{
A values x variables matrix containing the frequencies. \cr
\code{F <- freqc( a, b )} returns a matrix \code{F}
whose i-j element contains the frequency of value \code{rownames(F)[i]}
in the variable \code{colnames(F)[j]}.
}
\description{
Frequency Distribution of Many Discrete Variables
}
\details{
If \code{X} is a matrix consisting of discrete variables, \cr
\code{freqc(X)} is equivaleet to
\preformatted{
  freq=lapply(as.list(X),table)
  res2=mapply( function(x,name){
    y=matrix(x,1); colnames(y)=names(x); rownames(y)=name;
    return(as.data.frame(y))  }
    ,  freq, names(freq) )
}

Space or Blank values will be denoted as V1.
}
\examples{
set.seed(1701)
a <- sample(1:9,20,replace=1)
b <- letters[sample(1:9,20,replace=1)]
c <- sample(3:5,20,replace=1)
d <- letters[sample(5:9,20,replace=1)]

a[9] <- NA
b[3:4] <- NA

data <- data.frame( a, b )

res1 <- freqc(a,b,c,d)
res2 <- freqc(data,c,d)
res3 <- freqc(c,d,a,b)
Print(res1,res2,res3)
res4 <- freqc(c,d,a,b, sortrow=0)
res5 <- freqc(c,d,a,b, sortcol=1)
Print(res1,res4,res5)

res11 <- freqc(a,b,c,d, useNA="ifany")
res21 <- freqc(data,c,d, useNA="ifany")
Print(res11,res21)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package_description.R
\name{package_description}
\alias{package_description}
\title{Display Package Description}
\usage{
package_description(pname)
}
\arguments{
\item{pname}{name of package}
}
\description{
Display Package Description
}
\examples{
package_description("lazy.tools")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Seq.R
\name{Seq}
\alias{Seq}
\title{Smart Sequence Generation}
\usage{
Seq(from, to, decreasing = 0)
}
\arguments{
\item{from}{the starting value of the sequence}

\item{to}{the ending value of the sequence}

\item{decreasing}{= 1 to generate the decreasing sequence}
}
\description{
Smart Sequence Generation
}
\details{
This function is defined as
\preformatted{
  from-1 + seq_len(max(0,to-from+1))
}
so that
when \code{from > to} \code{Seq( from, to )} returns \code{numeric(0)}.

Basically, same as
\preformatted{
Seq <- function(from, to) {
  if (from <= to) {
     return(from:to)
  } else {
     return(numeric(0))
  }
}
}
}
\examples{
cat("Seq(3,5)\n")
for( i in Seq(3,5) ) print(i)
cat("Seq(5,3)\n")
for( i in Seq(5,3) ) print(i)
cat("seq(5,3)\n")
for( i in seq(5,3) ) print(i)
cat("\nSeq(3,5, decreasing=1)\n")
for( i in Seq(3,5, decreasing=1) ) print(i)
cat("Seq(5,3, decreasing=1)\n")
for( i in Seq(5,3, decreasing=1) ) print(i)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locbalpar.R
\name{locbalpar}
\alias{locbalpar}
\title{Return the locations of the matching parentheses in a string.}
\usage{
locbalpar(exp, loc)
}
\arguments{
\item{exp}{Input string: a single (vector of length 1).}

\item{loc}{The starting or ending locations of the parentheses
whose matching pair will be found.}
}
\value{
The location vecotr of length(loc) of the matching parens  or  0.
}
\description{
Return the locations of the matching parentheses in a string.
}
\details{
\code{loc} must be positive and less than \code{nchar(exp)}.\cr
\code{substring(exp,loc,loc)} must be opening or closing parens.\cr
Otherwise, this function returns 0.
}
\examples{
# Input string
#                1111111111222222222233333333
#       1234567890123456789012345678901234567
exp <- "a+([a+b]*(c*{d+e}))-(f-g) +(((<()>)) "

locbalpar( exp, 3 )
locbalpar( exp, 19 )
locbalpar( exp, c(4,8) )
locbalpar( exp, c(13,17) )
locbalpar( exp, c(31,34) )
locbalpar( exp, c(34,31, 3,19) )

locbalpar( exp, regexpr( "\\\\(", exp ) )

# No matching paren
locbalpar( exp, 28 )


# As long as substr(exp,n,1) is a paren, below should always be true.
n <- 10; n == locbalpar( exp, locbalpar( exp, n ) )
n <- 36; n == locbalpar( exp, locbalpar( exp, n ) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_valLL.R
\name{get_valLL}
\alias{get_valLL}
\title{Retrieve the Elements Contained in the Lists in a List}
\usage{
get_valLL(
  L,
  name,
  removenull = 0,
  simplify = 1,
  removeNull = 0,
  method = 1,
  debug = 0
)
}
\arguments{
\item{L}{A list of lists or data frames}

\item{name}{The name of the list element, or the element number. \cr
Can be a vector.}

\item{removenull}{= 1 to remove NULL elements.}

\item{simplify}{= 0 not to simplify the result. \cr
= 1 to simplify the result if possible \cr
= 2 to simplify the result if possible when name is a vector. \cr
Only works with \code{method=1}.}

\item{removeNull}{= 1 to remove all NULL columns.}

\item{method}{= 2 to use new method with sapply}

\item{debug}{= 1 to print the details of \code{method=2}}
}
\value{
A list, data frame, matrix or vector consisting of the specified elements.
}
\description{
This version is very slow!!
}
\details{
It is assumed that \code{L} is a list consisting of several lists
which contain the same objects.

Specifying \code{method=2} is faster but works only when: \cr
1) \code{name} is a scalar and the elements to be retrieved are
scalars or vectors with no missing elements, not matrices or lists. \cr
The result will be a vector or matrix.
2) \code{name} is a vector and the elements to be retrieved are
scalars. \cr
The result will be a matrix.

If the above conditions are not met, \code{method=1} will be used.

Note that, Specifying \code{method=2} is equivalent to \cr
 \code{ res <- sapply( L, function(x) x[[name]] ) } \cr


When \code{method=1}: \cr

If \code{simplify=1} is given, the result will be converted to a data frame
or matrix if possible.

If the \code{L} contains lists or arrays, or when \code{name} is a vector,
the result will not be simplified.

Try \code{l2a( lapply( "get_valLL result", a2m, method=2 ) )} .

If \code{removenull=1} is given, and \code{name} is not valid,
 NULL or named list() may be returned.

If the \code{L} consists of data frames only, better use \code{get_valL}.

\code{removeNull=1} may be needed when not all the lists in \code{L}
have the same set of elements.
It can always be set equal to 1 but not thoroughly tested yet.
}
\examples{
# case where all the lists consists of the objects of the same size.
set.seed(1701)
n <- 100; nvar <- 3
ngroup <- 3
X <- matrix(rnorm(n*nvar)/100,n,nvar)
group <- sample(1:ngroup,n,replace=1)
X <- group+X
group <- letters[group]
md <- mandd(X,by=group)

get_valLL(md,"mean")
get_valLL(md,"mean", simplify=0)
get_valLL(md,c("mean","cov"))


# univariate case with simplify=2
md2 <- lapply(md, function(x){lapply(x,"[",1)})
get_valLL(md2,"mean")
get_valLL(md2,"mean", simplify=0)
get_valLL(md2,c("mean","std"))
get_valLL(md2,c("mean","std"), simplify=2)


# general case
L0 <- list( A=(demomat(3,2)*10+1)
            , B=(1:3)*10+2
            , C=data.frame(demomat(5,4)*10+3)
)
L00 <- list( A=(demomat(3,2)*100+1)
             , B=(1:3)*100+2
             , C=(demomat(5,4, root="c"))
             , D=(demomat(5,4, root="d"))
)
L000 <- list( A=(demomat(3,2)*1000+1)
              , B=(1:3)*1000+2
              , C2=(demomat(5,4)*1000+3)
              , D=(demomat(4,5)*1000+4)
)
L <- list(L0,L00,L000)
names(L) <- c("a","b","c")

get_valLL( L, "A" ) #   This is a list of matrices.
get_valLL( L, "B" ) #   This is a matrix.
get_valLL( L, "B", simplify=0 ) #   This is a list of vectors.
get_valLL( L, "C", simplify=0 ) # NULL indicates missing elements.
get_valLL( L, "D" )  # same as get_valLL( L, 4 )
get_valLL( L, 4, removenull=1  ) # missing elements were removed.


# example of removeNull
L21 <- list( a=1, b=1, c=1 )
L22 <- list( a=2, c=2 )
L23 <- list( a=3, b=3, c=3 )
L2 <- list( x=L21, y=L22, z=L23 )
( get_valLL( L2, "b" ) )
( get_valLL( L2, "b", removeNull=1 ) )




#' # Example of \code{method=2}

# mean and std are vectors
get_valLL(md,"mean", method=2, debug=1)
get_valLL(md,"cov", method=2, debug=1) # --> method=1
get_valLL(md,c("mean","std"), method=2, debug=1, simplify=2) # -> method=1


# mean and std are scalar
md2 <- lapply(md, function(x){lapply(x,"[",1)})
get_valLL(md2,"mean", method=2, debug=1)
get_valLL(md2,c("mean","std"), method=2, debug=1)
get_valLL(md2,c("mean","std"), method=1, debug=1, simplify=2)


# general case: B is a scalar
L0 <- list( A=(demomat(3,2)*10+1)
            , B=(1:3)*10+2
            , C=data.frame(demomat(5,4)*10+3)
)
L00 <- list( A=(demomat(3,2)*100+1)
             , B=(1:3)*100+2
             , C=(demomat(5,4, root="c"))
             , D=(demomat(5,4, root="d"))
)
L000 <- list( A=(demomat(3,2)*1000+1)
              , B=(1:3)*1000+2
              , C2=(demomat(5,4)*1000+3)   # C is missing.
              , D=(demomat(4,5)*1000+4)
)
L <- list(L0,L00,L000)
names(L) <- c("a1","b2","c3")

get_valLL( L, "A", method=2, debug=1 ) # --> method=1
get_valLL( L, "B" , method=2, debug=1)
get_valLL( L, "C", simplify=0, method=2, debug=1 ) # C is NULL at the 3rd list
get_valLL( L, 2, removenull=1, method=2, debug=1  ) # same as "B"


# example of missing list elements
L21 <- list( A=1, B=1, C=1 )
L22 <- list( A=2, C=2 )        # B is missing.
L23 <- list( A=3, B=3, C=3 )
L2 <- list( x=L21, y=L22, z=L23 )
get_valLL( L2, "B", method=2, debug=1 )
get_valLL( L2, c("A","B"), method=2, simplify=2, debug=1 )


# difference between method=1 and 2
get_valLL(md2,"mean", method=1)
get_valLL(md2,"mean", method=2)

get_valLL(L2,"B", method=1, removeNull=1)
get_valLL(L2,"B", method=2)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{was}
\alias{was}
\title{Displays the attribuets and the structure of the input.}
\usage{
was(X, opt = 0)
}
\arguments{
\item{X}{an object whose info we would like to have.}

\item{opt}{= 1 to print structure only.\cr
=2 to print attributes only.}
}
\value{
A list
}
\description{
Displays the attribuets and the structure of the input.
}
\examples{
  bool <- TRUE
  num <- 1
  cpl <- 1+1i
  strng <- "Hello."
  vec <- 1:10
  mat <- matrix(c(1, 2, 3), 3, 2)
  array <- array(1,c(2,2,2))
  df <- data.frame(a=1,b=letters[1:2])
  lis <- list(a=1:3, b="abc",NA)

  was(bool)
  was(num)
  was(cpl)
  was(strng)
  was(vec)
  was(mat)
  was(array)
  was(df)
  was(c)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locbalpar2.R
\name{locbalpar2}
\alias{locbalpar2}
\title{Return the locations of the matching parentheses}
\usage{
locbalpar2(expr, loc, line = 1, escape = 1, debug = 0)
}
\arguments{
\item{expr}{Input string vector.}

\item{loc}{The starting or ending locations of the parentheses
in \code{expr}. (See the details and examples below.)}

\item{line}{line number}

\item{escape}{= 0 not to escape parens with double backslashes.}

\item{debug}{= 1 to print intermediate results}
}
\value{
A list of \cr
\code{line}   the line number. \cr
\code{locbp}  the locations of matching parens in the line.
}
\description{
Return the locations of the matching parentheses
}
\details{
This is an expanded version of \code{lazy.tools::locbalpar}
which can handle a string vector input. \cr

When \code{expr} is a scalar string, better use \code{lazy.tools::locbalpar}.
\code{loc} must be positive and less than \code{sum(nchar(exp))}.

When \code{expr} is a string vector,
assuming that there are no length 0 elements of expr,
the location of the parens can be specified as follows:\cr
Method 1 (with \code{line} and \code{loc})\cr
Suppose that the location of the paren of interest is in location
(chracter) LOC at line LINE. Then the following code will find
the location of the matching paren. \cr
locbalpar2( expr, LOC, LINE ) \cr
Note that \code{loc} and \code{line} can be vectors of the same length.\cr

Method 2 (with \code{loc})\cr
In this case, \code{loc} must be specified so that \cr
\code{unlist(strsplit(expr,""))[loc]} \cr
contains opening or closing parens.

This function returns \code{line=0, locbp=0}.

The following characters are treated as parentheses: \cr
\code{"(", "{", "[", "<", ")", "}", "]", ">"}
}
\examples{

# When counting characters, count the length 0 character, "", as 1.
# For example, consider the example below where
# expr1 has length 0 character, "", at lines 2 and 4 and
# expr2 has length 1 blank character, " ",  at lines 2 and 4.
# The locations of parens for expr1 must be counted from expr2 as: \cr
#  \code{unlist(strsplit(expr2,""))}\cr
# The two results below should be the same.
expr1 <- c( "1(", "", "4)(", "", "89)" )
expr2 <- c( "1(", " ", "4)(", " ", "89)" )
printm( expr1, expr2 )
exprc1 <- unlist(strsplit(expr1,""))
exprc2 <- unlist(strsplit(expr2,""))
printm( exprc1, exprc2 )
locbalpar2( expr2, c(2,3,2), c(1,3,3) )
locbalpar2( expr1, c(2,6,5) )
locbalpar2( expr2, c(2,6,5) )

# escape or not
locbalpar2( "a(b\\\\(d)",2 )
locbalpar2( "a(b\\\\(d)",2, escape=0 )


# Find the matching closing brace.
expr <- c(
 "locbalpar2 <- function( expr, loc ){",
 " # find the location of the balanced paren",
 " # use standard version when expr is a string",
 "",
 " if( !is.vector(expr) ){",
 "  return( locbalpar( expr, loc ) )",
 " }",
 "",
 "} # end of locbalpar2"
)
locbalpar2( expr, regexpr("{", expr, fixed=1 )[1] )

# find all "(" and their matching ")".
locs <- gregexpr("(", expr,  fixed=TRUE )
LL <- NULL
for( i in 1:length(locs) ){
 if( all( locs[[i]] < 0 ) ) next
 LL <- rbind(LL,cbind(i,locs[[i]]))
}
locbalpar2(expr,LL[,2],LL[,1])



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pplot.R
\name{radarchart}
\alias{radarchart}
\title{Radar Chart}
\usage{
radarchart(
  df,
  var = NULL,
  axistype = 4,
  seg = 4,
  pty = 16,
  pcol = 1:8,
  plty = 1:8,
  plwd = 1,
  pdensity = NULL,
  pangle = 45,
  pfcol = NA,
  cglty = 3,
  cglwd = 1,
  cglcol = "navy",
  axislabcol = "blue",
  title = "",
  centerzero = FALSE,
  vlabels = NULL,
  caxislabels = NULL,
  paxislabels = NULL,
  frame = 1,
  clockwise = 1,
  caxisoffset = -0.1,
  caxiscex = 0.9,
  legend = 1,
  lpos = c(1.1, -0.1),
  lcex = 0.8,
  llwd = 1,
  standard = 0,
  nobspc = 0,
  down = 1,
  across = 1,
  mar = c(1, 1, 1, 1),
  ...
)
}
\arguments{
\item{df}{data frame}

\item{var}{sas-like abbreviated list of variables to be used.}

\item{axistype}{The type of axes, specified by any of 0:5. \cr
0 means no axis label. \cr
1 means center axis label only. 2 means around-the-chart label only. \cr
3 means both center and around-the-chart (peripheral) labels. \cr
4 is *.** format of 1, 5 is *.** format of 3. Default is 0. \cr}

\item{seg}{The number of segments for each axis (default 4).}

\item{pty}{A vector to specify point symbol: \cr
Default 16 (closed circle), if you don't plot data points, \cr
  it should be 32. This is repeatedly used for data series. \cr}

\item{pcol}{A vector of color codes for plot data: \cr
Default 1:8, which are repeatedly used.}

\item{plty}{A vector of line types for plot data: \cr
Default 1:6, which are repeatedly used.}

\item{plwd}{A vector of line widths for plot data: \cr
Default 1, which is repeatedly used.}

\item{pdensity}{A vector of filling density of polygons: \cr
Default NULL, which is repeatedly used.}

\item{pangle}{angle}

\item{pfcol}{A vector of color codes for filling polygons: \cr
Default NA, which is repeatedly usd.}

\item{cglty}{Line type for radar grids: Default 3, which means dotted line.}

\item{cglwd}{Line width for radar grids: Default 1,
which means thinnest line.}

\item{cglcol}{Line color for radar grids: Default "navy"}

\item{axislabcol}{Color of axis label and numbers: Default "blue"}

\item{title}{plot title string.}

\item{centerzero}{Logical. If true, this function draws charts with scaling
originated from (0,0). \cr
If false, charts originated from (1/segments). Default FALSE.}

\item{vlabels}{Character vector for the names for variables. \cr
If NULL, the names of the variables as colnames(df) are used.}

\item{caxislabels}{Character vector for center axis labels, \cr
overwriting values specified in axistype option. \cr
If NULL, the values specified by axistype option are used.}

\item{paxislabels}{Character vector for around-the-chart (peripheral)
labels, \cr
    overwriting values specified in axistype option. \cr
    If NULL, the values specified by axistype option are used.}

\item{frame}{= 1 to draw frame around the graph}

\item{clockwise}{= 1 if the graph places the variables in clockwise
from the top}

\item{caxisoffset}{offset}

\item{caxiscex}{cex}

\item{legend}{= 1 to display legend.}

\item{lpos}{upper left corner of the legend.}

\item{lcex}{font size for the legend.}

\item{llwd}{line width for the legend.}

\item{standard}{= 1 to standardize the variables}

\item{nobspc}{# of observations in one chart.}

\item{down}{# of graphs in a column.}

\item{across}{# of graphs in a row.}

\item{mar}{margins for multiple graphs per page.}

\item{...}{additional parameters for plot}
}
\value{
None
}
\description{
This is a modified version of \code{radarchart} in \code{fmsb} package.
}
\examples{
data=data.frame(a=c(1,2,3,4),b=c(0,1,0,1),c=c(4,3,2,1))
radarchart( data, nobspc=2, across=2, lpos=c(.9,.1) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/revchar.R
\name{revchar}
\alias{revchar}
\title{Reverse string}
\usage{
revchar(x)
}
\arguments{
\item{x}{Input character vector}
}
\value{
A vector containing the reversed elements.
}
\description{
Reverse string
}
\details{
revstr is the alias for this function.
}
\examples{
revchar(c("ncc","1701","a"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{comments}
\alias{comments}
\title{Comment out multiple lines}
\usage{
comments(...)
}
\arguments{
\item{...}{lines to be commented out in matched quotes}
}
\value{
None
}
\description{
Comment out multiple lines
}
\examples{
comments(' whatever you want except for single quotes ')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/remove_paren.R
\name{remove_paren}
\alias{remove_paren}
\title{Remove Redundant Parentheses
(very very primitive version)}
\usage{
remove_paren(exp, method = 1:9, loc = 0, debug = 0)
}
\arguments{
\item{exp}{expression vector or matrix}

\item{method}{1, 2 or 3 \cr
If exp contains any logical operator, method=1 will be used.\cr}

\item{loc}{the location of the paren to be removed. \cr
Use with caution when the paren is preceded by - sign,
or exp is a vector or matrix.}

\item{debug}{= 1 to print intermediate result.}
}
\value{
A vector or matrix of the same size as the input
}
\description{
Remove Redundant Parentheses
(very very primitive version)
}
\details{
When loc > 0 is given, those parentheses located at the loc-th position
of the elements of exp will be removed.\cr
No signs will be distributed: \cr
\code{remove_paren("-(a+b)",loc=2)} will result in \code{-a+b}.
\cr

First,  (+....)  will be replaced by  (...) .\cr
When method = 1, a pair of parentheses which encloses a single term
w/o divisions will be removed. \cr
When method = 2, a pair of parentheses which is not preceded by [*/-]
and not trailed by [*/] will be removed. \cr
When method = 3, patterns like  -(-a) or -(-(a)) will be processed.
\cr\cr
Use Expand to distribute multiplicative factors.

\cr\cr
\code{method=2} may return wrong result: \cr
\code{remove_paren("-((a+b))", method=2)} becomes \code{-a+b}.
\cr
\cr
}
\examples{
remove_paren( c("(x)", "-(x)", "(x+y)", "-(x+y)") )
remove_paren( c("a*(x)*b", "a*(x+y)*b", "(-x)*a", "-(x+y)*a") )
remove_paren( c("a\%*\%(x)\%*\%b", "a\%*\%(x+y)\%*\%b", "(-x)\%*\%a", "-(x+y)\%*\%a") )
remove_paren( c("a\%@\%(x)\%*\%b", "a\%@\%(x+y)\%@\%b", "(-x)\%@\%a", "-(x+y)\%@\%a") )

exp <- "((a)) - ((a)+f(b)) + (a-b) - (x-gg(y)) + (((z+1)))"
remove_paren( exp )

exp2 <- c(exp, "(a+b)*(c)-d+1")
remove_paren( exp2 )

( temp <- And(Or("a==A","b>0"),"c") )
remove_paren(temp)

remove_paren("!(!(a))")
remove_paren("!(!(a))",loc=4)
remove_paren("!( !(a) )",loc=9)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sortit.R
\name{sortit}
\alias{sortit}
\title{Sort an Object according to the Keys}
\usage{
sortit(x, ..., descending = 0, cols = NULL)
}
\arguments{
\item{x}{an object to be sorted.\cr
If x is a matrix, all the column will be sorted.}

\item{...}{vectors to be uses as keys.\cr
Must be the same length as x.}

\item{descending}{= 1 to sort descendingly when ... is present.\cr}

\item{cols}{vector consisting of the column numbers of x
which will be used as keys.\cr
This has priority over ... .}
}
\value{
Object of the same size as x.
}
\description{
Sort an Object according to the Keys
}
\details{
In order to sort descendingly using cols argument,
 specify negative column number in cols= \cr
or use - sign in numeric ... ariguments. \cr
Or, use descending=1 option.
}
\examples{
# sort the rows of x according to the order of ... .
sortit( demomat(4,2), c(4,3,1,2) )
sortit( demomat(4,2), c(4,3,1,2), c("a","a","b","b") )

# sort the rows of x using the colums specified in cols= as keys.
sortit( diag(4), cols=1:4 )

# sorting the rows of a matrix
X <- as.matrix( expand.grid( 1:2, 3:5 ) )
colnames(X)=c("xx1","xx2")
sortit( X, cols=c(1,2) )
Print( sortit( X, cols=c(1,-2) ), sortit( X, X[,1], -X[,2] ) )

# the last column first or the first column first.
Print( sortit( X, cols=1:2 ), sortit( X, cols=2:1 ) )

# sort the result of unique function
set.seed(1701)
Y <- matrix( sample(letters[1:3],99, replace=1),,3 )
Yu <- unique(Y)
Print( Yu, sortit(Yu, cols=1:3) )
Print( sortit(Yu, Yu[, 1], Yu[, 2], Yu[, 3]) )
Print( Yu, sortit(Yu, cols=-(1:3)) )
Print( sortit(Yu, Yu[, 1], Yu[, 2], Yu[, 3], descending=1) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locbalq.R
\name{locbalq}
\alias{locbalq}
\title{Find the Location of the Quote to the Right or Left of the One at loc.}
\usage{
locbalq(exp, loc, right = 1)
}
\arguments{
\item{exp}{A string.}

\item{loc}{The location of a quote.}

\item{right}{= 0 to find the one to the left.}
}
\value{
The location of the quote found.
}
\description{
Find the Location of the Quote to the Right or Left of the One at loc.
}
\details{
If exp contains double quotes, exp must be defined by using single quotes
or must be escaped by the use of \.
}
\examples{
exp='regexp("a", "12""a56")'
loc=regexpr('"',exp)
locn=locbalq(exp, loc)
Print(substr(exp,loc,locn))
loc=13
locn=locbalq(exp, loc)
Print(substr(exp,loc,locn))

# location of the non-matching quote
loc=10
locn=locbalq(exp, loc)
Print(substr(exp,loc,locn))

exp="regexp('a', '12''a56')"
locbalq(exp, regexpr("'",exp))
loc=16
locn=locbalq(exp, loc, right=0)
Print(substr(exp,locn,loc))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Quantile.R
\name{Quantile}
\alias{Quantile}
\title{Calculation of sample quantiles by interpolating
sample cumulative distribution function.}
\usage{
Quantile(
  x,
  freq = NULL,
  probs = seq(0, 1, 0.25),
  method = "constant",
  cdf = NULL
)
}
\arguments{
\item{x}{numeric vector}

\item{freq}{frequency associated with x \cr
(x,freq) is the frequency table}

\item{probs}{numeric vector of probabilities with values in [0,1]}

\item{method}{= "constant" | "linear" | "spline",  interpolation method}

\item{cdf}{cdf at x. This has priority over freq.}
}
\value{
a vector of quantiles
}
\description{
Calculation of sample quantiles by interpolating
sample cumulative distribution function.
}
\details{
If freq=1  and method="constant" (default),
 the result is equivalent to native quantile function with type=1.

 Given the frequency table of data, (x, freq), cdf will be calculated as:
 \preformatted{
 cdf = sumsum(freq)/sum(freq)
 }
}
\examples{

# generate data
set.seed(1701)
n <- 100
x <- round(10*rnorm(n),0)

# by native quantile func
probs <- c(0.25, 0.5, 0.75)
res1 <- quantile(x,probs, type=1)

# by Quantile
res2 <- Quantile( x,1, probs=probs )

# frequency table
freq <- table(x)
xc <- as.numeric(names(freq))

# from frequency table
res3 <- Quantile( xc,freq, probs=probs )

# comparison
Print(probs,res1,res2, res3)



#
# Effect of the sample size
#
# When small, Quantile with linear method is equal to quantile with type=4.
#
#

getQ <- function( x, prob ){
 res1 <- NULL;
 for( type in 1:9 ) res1 <- cbind(res1,quantile(x,probs, type=type))
 colnames(res1) <- paste("t",1:9,sep="")
 return( res1 )
}

# generate data
set.seed(1701)
n <- 5
x <- round(10*rnorm(n),0)
probs <- c(0.10, 0.25, 0.5, 0.75, 0.90)

# by native quantile func
res1 <- getQ(x,probs)
# by Quantile
res2 <- Quantile( x,1, probs=probs, method="linear" )
# comparison
Print(res1, fmt="8.4")
Print(probs,res2, fmt="8.4")


# generate data
set.seed(1701)
n <- 50
x <- round(10*rnorm(n),0)
probs <- c(0.10, 0.25, 0.5, 0.75, 0.90)

# by native quantile func
res1 <- getQ(x,probs)
# by Quantile
res2 <- Quantile( x,1, probs=probs, method="linear" )
# comparison
Print(res1, fmt="8.4")
Print(probs,res2, fmt="8.4")

# comparison
Print(res1)
Print(probs,res2, res3)


# generate data
set.seed(1701)
n <- 500
x <- round(10*rnorm(n),0)
probs <- c(0.10, 0.25, 0.5, 0.75, 0.90)

# by native quantile func
res1 <- getQ(x,probs)
# by Quantile
res2 <- Quantile( x,1, probs=probs, method="linear" )
# comparison
Print(res1, fmt="8.4")
Print(probs,res2, fmt="8.4")

# comparison
Print(res1)
Print(probs,res2, res3)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand_freqdist.R
\name{expand_freqdist}
\alias{expand_freqdist}
\title{Expand Frequency Distribution to Raw Data}
\usage{
expand_freqdist(fd, weight = NULL)
}
\arguments{
\item{fd}{tables, matrices or data.frames containing frequency
distribution (midpoints,freq), or \cr
a class table or xtabs object.}

\item{weight}{Frequency count of each row of fd. \cr
If null, fd[,ncol(fd)] will be used as the weight.}
}
\value{
A vector of length( sum(weight) ) or
matrix or data frame with nrow=sum(weight). \cr
}
\description{
Expand Frequency Distribution to Raw Data
}
\details{
In univariate case, this function returns \cr
 \code{ unlist( mapply( function(x,y) rep(x,y), fd[,1], fd[,2] ) ) } \cr
 or \cr
 \code{ rep( fd[,1], fd[,2] ) }
}
\examples{
seed <- 1701
set.seed(seed)
# univariate
tab1 <- table( sample(letters[1:5], 20, replace=1) )
res1 <- expand_freqdist( tab1 )
tab11 <- table(res1)
Print(tab1,res1, tab11)
# convert a table object to a freqdist format
res11 <- expand_freqdist( table2freqdist(tab1) )
Print(res11)

# multivariate
tab2 <- table( x=sample(letters[1:5], 20, replace=1)
   , sample(6:8, 20, replace=1) )
res2 <- expand_freqdist( tab2 )
tab22 <- table(res2)
Print(tab2,res2,tab22)
# convert a table object to a freqdist format
res22 <- expand_freqdist( table2freqdist(tab2) )
Print(res22)

# multivariate
temp <- c(
 1, -3.0, 1, 0, 2, -2.6, 2, 0, 3, -2.3, 3, 0, 4, -1.9, 4, 0, 5, -1.5, 5, 1
 , 6, -1.1, 6, 1, 7, -0.8, 7, 2, 8, -0.4, 8, 1, 9, 0.0, 9, 7, 10, 0.4, 8, 7
 , 11, 0.8, 7, 5, 12, 1.1, 6, 6, 13, 1.5, 5, 4, 14, 1.9, 4, 4, 15, 2.3, 3, 2
 , 16, 2.6, 2, 2, 17, 3.0, 1, 1
)
data_freq <- as.data.frame(matrix(temp,,4, byrow=1))
colnames(data_freq) <- c("obs","x","f","r")

res <- expand_freqdist( data_freq, weight=data_freq$f)


# equivalent to rep
Print( rep( c(1:4),c(1:4) ), expand_freqdist( c(1:4),c(1:4) ) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_AndOr.R
\name{generate_AndOr}
\alias{generate_AndOr}
\title{Generation of character infix operators: `&`, `&&`, `|`, `||`}
\usage{
generate_AndOr()
}
\value{
Generates four infix operators in the global environment.
}
\description{
Generation of character infix operators: `&`, `&&`, `|`, `||`
}
\details{
This function generates infix operators: `&`, `&&`, `|`, `||`
in the global environment, \cr
which are almost equivalent to, resp, `\%&\%`, `\%&&\%`, `\%|\%`, `\%||\%` .
\cr
The difference is that `&` has precedence over `|`  but `\%&\%` does NOT.
\cr For example: \cr\cr
\code{"a" | "b" & "c"} produces \code{"(a) | ((b) & (c))"} \cr
but \cr
\code{"a" \%|\% "b" \%&\% "c"} produces \code{"((a) | (b)) & (c)"} .
\cr\cr
See the examples below.
\cr\cr
\code{ "a" & "b" } is equivalent to  \code{And( "a", "b" )}
and generates \code{"(a) & (b)"}.\cr
\code{"a" && "b"} is equivalent to  \code{And( "a", "b", oper="&&" )}
and generates \code{"(a) && (b)"}.\cr
\code{ "a" | "b" } is equivalent to \code{Or( "a", "b" )}
and generates \code{"(a) | (b)"}.\cr
\code{ "a" || "b"} is equivalent to  \code{Or( "a", "b", oper="||" )}
and generates \code{"(a) || (b)"}.\cr
\cr
Use \code{eval( parse(text=condition) )} to evaluate the expression.
\cr
\cr
To remove these operators: \cr
 \code{ rm(  `&`, `&&`, `|`, `||` ) }
}
\examples{
generate_AndOr()

"a" & "b"
"a" \%&\% "b"
And( "a", "b" )
"a" | "b"
"a" \%|\% "b"
Or( "a", "b" )

# precedences
"a" | "b" & "c"
"a" \%|\% "b" \%&\% "c"

"a" & "b" | "c"
"a" \%&\% "b" \%|\% "c"


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postfix.R
\name{\%//\%}
\alias{\%//\%}
\title{Mathematica like postfix function application}
\usage{
exp \%//\% func
}
\arguments{
\item{exp}{argument of the function}

\item{func}{function name}
}
\description{
Mathematica like postfix function application
}
\details{
Equivalent to func( exp ). \cr
\cr
Implemented as:
\code{if( mode(func) == "function" ) return( func( exp ) ) }
}
\examples{
temp <- demomat(4,3)
temp \%//\% t

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{cumsuminv}
\alias{cumsuminv}
\title{Inverse of Cummurative Sum}
\usage{
cumsuminv(x)
}
\arguments{
\item{x}{A numeric vector}
}
\value{
A vector of the inverse of cumsum.
}
\description{
Inverse of Cummurative Sum
}
\details{
This function returns lagged difference of lagg=1 and x[1] as: \cr
 c( x[1], diff(x,1) )
}
\examples{
set.seed(1701)
x=sample(1:9,9,replace=1)
y=cumsum(x)
Print(x,y,cumsuminv(y))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_functions.R
\name{find_functions}
\alias{find_functions}
\title{Find Functions used in a Function or Functions in a List of Function Names}
\usage{
find_functions(
  func,
  nonative = 1,
  noglobal = 0,
  sort = 1,
  packages = NULL,
  unlist = 1
)
}
\arguments{
\item{func}{A function object (no quote) or
a list/vector of the function object names.}

\item{nonative}{= 0 to print the R native functions.}

\item{noglobal}{= 1 not to print the functions defined in the global env.}

\item{sort}{= 0 not to sort the result}

\item{packages}{A character vector containing the names of the packages.
\cr The functions included in those packages will not be printed.}

\item{unlist}{= 0 to output the result in a list when func is a
list or vector.}
}
\value{
A character vector containing the function names.
}
\description{
Find Functions used in a Function or Functions in a List of Function Names
}
\details{
\cr
This function  requires "codetools" package.
\cr
}
\examples{
\dontrun{
library(codetools)
library(lazy.tools)

find_functions( is.varname )

res1 <- find_functions( ls("package:lazy.tools") )
res2 <- find_functions( ls("package:lazy.tools"), unlist=0 )
res3 <- find_functions(
         ls("package:lazy.tools"), unlist=0, packages="lazy.tools" )
Print(res1,res2,res3)

# funclist <- ls("package:lazy.tools")
# res <- sort( unique( unlist( lapply( funclist, find_functions ) ) ) )
# res <- unique( unlist( lapply(
            funclist, find_functions, packages="lazy.tools" )
             ) )
}

}
\references{
The answer by kohske (answered Aug 9 '12 at 7:51) at: \cr\cr
  http://stackoverflow.com/questions/11872879/
   finding-out-which-functions-are-called-within-a-given-function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_interval.R
\name{plot_interval}
\alias{plot_interval}
\title{Plot Horizontal Line on top of Density Plot}
\usage{
plot_interval(fromto, ypos = 0, type = "l", col = "black", tck = 0.03, ...)
}
\arguments{
\item{fromto}{vector of length 2 consisting of the x-coordinats of end points.}

\item{ypos}{y coodinate of the line.}

\item{type}{line tyoe}

\item{col}{line color}

\item{tck}{length of the tick mark at both ends of the line}

\item{...}{additional parameter to native line function}
}
\description{
Plot Horizontal Line on top of Density Plot
}
\examples{
alpha <- 10; beta <- 20
hdr <- c(0.173,0.500)
x <- seq(0,1,.01)
plot( x, dbeta(x,alpha,beta), type="l" )
ypos <- dbeta(hdr[1],alpha,beta)
plot_interval( hdr, ypos, tck=0.1 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/new_name.R
\name{new_name}
\alias{new_name}
\title{Rename the elements of the input by the new set of names
given in the table n2o or by the consecutive numbers.}
\usage{
new_name(old, n2o = NULL, sort = 0, print = 0)
}
\arguments{
\item{old}{An object whose elements are to be replaced by numbers.}

\item{n2o}{Conversion table (data frame) from old to new or NULL. \cr
Must have two columns named "new" and "old" and "new" must be 1:nrow(n2o).}

\item{sort}{= 1 to sort the new name when n2o is NULL. \cr
= 2 to sort  the new name when n2o is NULL by naturalsort package.}

\item{print}{= 1 to print the conversion summary.}
}
\value{
A list of the following: \cr
  new     renamed version of old:  same shape and same order as old \cr
  n2o     conversion table from new to old \cr
  o2n     sorted version of n2o by old \cr
  freqnew frequency of new names \cr
          cbind( n2o,freqnew) will give you the frequency table. \cr
  od      order to be used to sort an object
}
\description{
Rename the elements of the input by the new set of names
given in the table n2o or by the consecutive numbers.
}
\details{
By default, the unique names in old will be renamed by the
consecutive numbers in the order of appearance. \cr
When old is a data frame, new may be a matrix.
\cr\cr
naturalsort package is required to use sort=2 option.
\cr\cr
}
\examples{
# add consecutive numbers in the order of appearance.
( new_name( matrix(c("IA","NY","WY","AR","NY","IA"),3) ) )

# renaming old
seed <- 1701; set.seed(seed)
old <- sample( letters[1:6], 10, replace=TRUE )
# order of appearance
res1 <- new_name( old, print=1 )
# sorted order
res2 <- new_name( old, print=1, sort=1)
# using renaming table
old2 <- sample( old, 10, replace=TRUE )
res <- new_name( old2, res2$n2o, print=1 )

# Rename "a" as "Z", "b" as "Y", "c" as "X".
new <- c( "Z", "Y", "X" )
old <- c( "a", "b", "c" )
n2o <- data.frame( new=new, old=old, stringsAsFactors=0 )
temp <- new_name( old, n2o )
Print( old, temp$new )
old2 <- sample( old,10,replace=1 )
temp2 <- new_name( old2, n2o )
Print( old2, temp2$new  )

# sorting vectors
# Suppose that you hava a vector
#   vec = c( "A", "D", "B", "E", "C", "D", "A" )
# and you wish to sort it as
#  vec_sorted <- vec[od,]
# so that
#  vec_sorted = c("B", "D", "D", "A", "A", "C", "E") .
# This can be accomplished by using order2 function as:
# vec <- c( "A", "D", "B", "E", "C", "D", "A" )
# od <- order2( vec, order=c("B", "D", "A", "C", "E") )
# vec_sorted <- vec[od]
#
# The same result can be obtained as:#'
vec <- c( "A", "D", "B", "E", "C", "D", "A" )
n2o=data.frame(new=1:5, old=c("B", "D", "A", "C", "E"))
od <- new_name( vec, n2o )$od
vec_sorted <- vec[od]



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_RD.R
\name{inspect_RData}
\alias{inspect_RData}
\title{inspect the contents of RData file}
\usage{
inspect_RData(RData, str = 0, max.level = 0)
}
\arguments{
\item{RData}{The name of .RData file}

\item{str}{= 1 to apply str to each object}

\item{max.level}{the max.level option for str}
}
\description{
This function inspects the contents of an RData file.
}
\details{
Note that \code{saveRDS(object,filename.RDS)} saves a single object in
an external file and that
\code{save(object1,object2,..., file="filename.RData")}
saves the multiple objects in an external file.
}
\examples{
a <- 1;  b <- 2;  C <- matrix(1:6,3,2);  E <- letters[1:5]
ds <- data.frame(a,b,C);  ds2 <- list(C,ds)
save(a,b,C,ds,ds2,E, file="testRData.RData")
inspect_RData("testRData.RData")

# retrieve the contents from .RData file
# loadRDS( RDatafilename )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cards.R
\name{cards}
\alias{cards}
\title{Convert a string to matrix\cr
 For those who miss the luxury of using spaces as delimitters.}
\usage{
cards(
  a,
  rnameroot = "r",
  cnameroot = "c",
  header = 0,
  missing = "NA",
  nummat = 0,
  debug = 0
)
}
\arguments{
\item{a}{string which contains the body of the matrix with \\nr.}

\item{rnameroot}{root of the row names when header=0}

\item{cnameroot}{root of the column names when header=0}

\item{header}{= 1 if the string has row/col names}

\item{missing}{a string which indicates missing values: "NA" or "."}

\item{nummat}{= 1 to force the result to be a numeric matrix. \cr
This may cause an error!}

\item{debug}{= 1 to dump intermediate result}
}
\value{
a matrix or data.frame
}
\description{
Convert a string to matrix\cr
 For those who miss the luxury of using spaces as delimitters.
}
\details{
Use space ( blank characters ) or a comma as delimiters. \cr
Two or more commas indicate the existence of  missing values in between.\cr
The last comma indicates one element is missing at the end.\cr
Blank lines will be neglected.

\preformatted{
CARDS <- " v1 v2 v3
s1 11 12 13
s2 21 22 23
s3 31 32 33"
A <- cards( CARDS, header=1 )
}
A will be a 3 x 3 matrix with row and column names.

 This module converts the following types of string:
\preformatted{
     a1="         , a2="11 12, 13  ,  a3="C1 C2 C3  ,  a4="C0 C1 C2 C3
         11 12 13       21, 22  23     R1 11 12 13         R1 11 12 13
         21 22 23       31 32 33       R2 21 22 23         R2 21 22 23
         31 32 33       41,, 43"       R3 31 32 33         R3 31 32 33
         41 42 43                      R4 41 42 43"        R4 41 42 43"
        "}
 into a matrix or data.frame. \cr

 When the string contains both row and column names
 as in a3 or a4 above, header=1 must be specified. \cr
 If, as a3 above,
 the first line ending with \\n has one less elements than the rest,
 it is assumed that both row and col names are given. \cr
 Otherwise, as a4 above, C0 will be treated as the first column name
 of the resulting matrix. \cr
  \cr
 The row/col names cannot have commas in it. (for csv compatibility) \cr
 When the string does not have row and column names, characters specified
 in rnameroot or cnameroot will be used to generate the names. \cr
 Character elements of the matrix cannot have blanks in them. \cr
 The quotes will be treated as characters. \cr

 \cr
  When a column contains both numeric and character string
  a data frame, not a matrix, will be created. \cr
  Scientific notation like 1e-9 will be mistaken as character. \cr

\cr
 To input from the clip board, \cr
 copy the body of the matrix with row and column names and use \cr
   \code{ read.table(file("clipboard"),header=1) }
\cr
  \code{ write.table( string-such-as-a1-etc-above, "clipboard",sep=" ")}
   \cr
  may be used to improve this module.
}
\examples{
CARDS=" v1 v2 v3
s1 11 12 13
s2 21 22 23

s3 NA 32 33"
cards( CARDS, header=1 )

CARDS=" v1 v2 v3
s1, 11, 12, 13
s2, 21, 22, 23
s3,,    32, 33"
cards( CARDS, header=1 )

CARDS="v0 v1 v2 v3
s1 11 12 13
s2 21 22 23
s3 .  32 33"
cards( CARDS, header=1, missing="." )

CARDS="
 11 12 13
 21 22 23
 NA  32 33
 "
cards( CARDS  )

# result is a character matrix.
CARDS <- " cos(theta) -sin(theta)
           sin(theta) cos(theta)"
T <- cards(CARDS)


# result is a data frame.
CARDS <- "
num char num
11 aa 12
21 bb 22
"
df <- cards( CARDS, header=1 )
str(df)

CARDS <- "
     ncat      a         b1        b2
 r1  3.00e+00  1.25e+00 -2.06e+00 .
 r2  3.00e+00  1.36e+00 -1.44e-16 .
 r3  3.00e+00  1.40e+00  1.97e+00 .
 r4  3.00e+00  1.02e+00 -2.04e+00 .
 r5  3.00e+00  1.06e+00 -1.46e-16 .
 r6  3.00e+00  1.07e+00  1.98e+00 .
 r7  4.00e+00  1.10e+00 -2.13e+00  1.05e-15
 r8  4.00e+00  7.19e-01 -2.03e+00 -3.27e-16
 "
 Ad <- cards(CARDS, header=1, missing=".")
 Print("This is a character matrix.", Ad)

 Ad <- cards(CARDS, header=1, missing=".", nummat=1)
 Print("This is a numeric matrix.", Ad)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.R
\name{convert}
\alias{convert}
\title{Convert the columns of a data frame to a specific type/class.}
\usage{
convert(X, what = "factor", func = as.character)
}
\arguments{
\item{X}{A Data Frame or a Matrix}

\item{what}{A character vector of class names, or "ANY" to match any class
\cr where class names can be "factor", "character", "numeric" etc.}

\item{func}{The function to be used for the conversion.}
}
\value{
X with the converted values.
}
\description{
Convert the columns of a data frame to a specific type/class.
}
\details{
This function returns \cr
\code{ rapply( X, func, classes=what, how="replace" ) }
\cr\cr
The same can be obtained by:\cr
\code{ loc <- which( sapply(vartype(X), "\%in\%", what ) ) }\cr
\code{ X[loc] <- lapply( X[loc], func ) }\cr
}
\examples{
A <- demomat(5,3,root="")
Af <- as.data.frame(A)
Af[,2]=as.numeric(Af[,2])
str(Af)

Ac <- convert( Af )
str(Ac)

Ac <- convert( Af, what=c("factor","numeric") )
str(Ac)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_val.R
\name{get_val}
\alias{get_val}
\alias{put_val<-}
\title{Get Values from an Array}
\usage{
get_val(A, i = NULL, j = NULL, k = NULL, l = NULL, subs = NULL)

put_val(A, i = NULL, j = NULL, k = NULL, l = NULL, subs = NULL) <- value
}
\arguments{
\item{A}{An array object}

\item{i}{Vector consisting of the subscripts for the 1st dimension}

\item{j}{Vector consisting of the subscripts for the 2nd dimension}

\item{k}{Vector consisting of the subscripts for the 3rd dimension}

\item{l}{Vector consisting of the subscripts for the 4th dimension}

\item{subs}{string specifying the subscripts such as "1:2, , 4, 2:2" \cr
or a numeric vector consisting of the scalar subscript for each dim.}

\item{value}{The value to be assigned.}
}
\value{
\code{get_val(A, i,j.,k,l)} returns
 \code{A[i,j,k,l]}. \cr
 \code{put_val(A, i,j,k,l) <- value} replaces
 \code{A[i,j,k,l]} with \code{value}.
}
\description{
Get Values from an Array
}
\details{
It is assumed that \code{A} is an array. \cr
When \code{A} is a list of data frames, use \code{get_valL} instead.\cr
When numeric \code{subs} is given it must be a vector of length
\code{length(dim(A))} specifying a scalar element of \code{A}. \cr
\cr
\code{get_val(A,i,j,k,l)} returns \code{A[i,j,k,l]}. \cr
\code{put_val(A,i,j,k,l) <- value} puts the value at \code{A[i,j,k,l]}.
\cr
Therefore, when \code{A} is an array, better use \code{A[i,j,k,l]}.
\cr

\code{get_val(A,i,j,k,l)} is equivalent to \code{get_val(A,subs=subs)}
\cr where \code{subs} contains the values of \code{i,j,k,l} as a string.
\cr For example, \cr
\code{get_val(A, 1:2, , 3)} is equivalent to
\code{get_val(A, subs="1:2, , 3")}. \cr
\cr
Use \code{subs=} when the # of dimensions is more than 5.
\cr

\code{subs=} has priority over \code{i,j,k,l}.

\cr
The result will be simplified to a matrix or vector if possible.
}
\examples{
# set up an array B
dims=c(5,4,3,2)
B <- array(0,dims)
B[,,,1] <- demomat(5,4,3)*10+1
B[,,,2] <- demomat(5,4,3)*10+2
dimnames(B) <-
 mapply( function(x,y){paste(x,1:y,sep="")}, letters[1:length(dims)],dims)

# get B[5,4,3,2]
res1=get_val(B,5,4,3,2)
res2=get_val(B,subs=c(5,4,3,2))
res3=get_val(B,subs="5,4,3,2")
Print(res1,res2,res3)

# get B[2,,3,]:  cannot use numeric subs here.
res1=get_val(B,2,,3)
res3=get_val(B,subs="2,,3,")
Print(res1, res3 )

# modify B[,1,2:3,1]
( temp <- B[,1,2:3,1] )
put_val(B,,1,2:3,1) <- -9999
B[,1,2:3,1]
put_val(B,subs=",1,2:3,1") <- temp
B[,1,2:3,1]
put_val(B,subs=c(1,1,1,2)) <- -9999
B[1,1,1,2]


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.R
\name{logistic1}
\alias{logistic1}
\title{logistic function with D and minimum value}
\usage{
logistic1(x, D = 1.7, minp = 1e-08)
}
\arguments{
\item{x}{vector}

\item{D}{= 1.7}

\item{minp}{minimum value of the function}
}
\value{
vector of the size of x
}
\description{
logistic function with D and minimum value
}
\details{
This function calculates \cr
  p=1/(1+exp(-D*x)) \cr
and applies minp as \cr
  p[p<minp]=minp; p[p>1-minp]=1-minp
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/order2.R
\name{order2}
\alias{order2}
\title{Returns a permutation which rearranges its first argument in the order
 specified by the second argument.}
\usage{
order2(x, order = NULL, descending = 0)
}
\arguments{
\item{x}{a vector to be sorted according to order}

\item{order}{a vector containing the definition of the sort order \cr
If order is not given, this function returns the same as the native order.}

\item{descending}{= 1 to sort descendingly}
}
\value{
The permutation vector, \code{od}, to be used as:\cr
\code{ x_sorted <- x[od] }
}
\description{
Returns a permutation which rearranges its first argument in the order
 specified by the second argument.
}
\details{
The returning value, od, will be calculated as: \cr\cr
\code{  df1 <- data.frame( id1=1:length(x), x ) } \cr
\code{  df2 <- data.frame( id2=1:length(order), order ) } \cr
\code{  temp <- merge( df1, df2, by.x="x", by.y="order", all=1 ) } \cr
\code{  temp <- temp[order(temp$id2),] } \cr
\code{  od <- (temp$id1)[1:length(x)] } \cr
}
\examples{
order <- c("B", "D", "A", "C", "E","X")
x <- c( "A", "D", "B", "E", "C", "D", "A" )
od <- order2( x, order )
Print( x, order, od, x[od] )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalize_config.R
\name{normalize_config}
\alias{normalize_config}
\title{Normalize Configuration Matrix}
\usage{
normalize_config(F, center = 1, rotate = 0, A = NULL)
}
\arguments{
\item{F}{n x ndim matrix to be normalized}

\item{A}{nvar x ndim additional matrix to be normalized}

\item{center=0}{not to column center F}

\item{rotate=1}{to rotate F so that F'F = diagonal.}
}
\value{
Normalized \code{F} or a list consisting of \code{F} and \code{A}.
}
\description{
This function normalizes the input matrix.
}
\details{
When \code{center==1}, \code{F} will be column centered. \cr
If \code{rotate==1}, \code{F} will be rotated
 so that \code{t(F)\%*\%F}=diagonal.\cr
When \code{A} is present, and \code{rotate==1} \cr
\preformatted{
 t(F)\%*\%F = nrow(F)*I  and  t(A)\%*\%A = diagonal
}

Note that, given \code{F} and \code{A}, 
the above normalization with center=0
does not change the value of \code{F \%*\%t(A)}.
}
\examples{

n=10; nvar=5; ndim=3
set.seed(1701)
F=matrix(rnorm(n*ndim),n,ndim)
A=matrix(runif(nvar*ndim),n,ndim)
FA=F\%*\%t(A)
meanF=colMeans(F)
Print(F,A,meanF)

F1=normalize_config( F )
temp=mandd(F1)
Print(round(t(F1)\%*\%F1,4), colMeans(F1))

F11=normalize_config( F, rotate=1 )
temp=mandd(F11)
Print(round(t(F11)\%*\%F11,4), colMeans(F11))

F2=normalize_config( F, center=0, rotate=1 )
Print(round(t(F2)\%*\%F2,4), colMeans(F2))

temp=normalize_config( F, center=1, rotate=1, A=A )
F3=temp$F; A3=temp$A
Print(round(t(F3)\%*\%F3,4), colMeans(F3))
Print(round(t(A3)\%*\%A3,4), max(abs(FA-F3\%*\%t(A3))))

temp=normalize_config( F, center=0, rotate=1, A=A )
F4=temp$F; A4=temp$A
Print(round(t(F4)\%*\%F4,4), colMeans(F4))
Print(round(t(A4)\%*\%A4,4), max(abs(FA-F4\%*\%t(A4))))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midp2brk.R
\name{gen_midp}
\alias{gen_midp}
\title{Generate n Equally Spaced Midpoints in [min,max]}
\usage{
gen_midp(min, max, n)
}
\arguments{
\item{min}{The minimum value.}

\item{max}{Them maximum value.}

\item{n}{# of midpoints including min and max}
}
\value{
A vector of length n.
}
\description{
Generate n Equally Spaced Midpoints in [min,max]
}
\details{
\code{gen_midp( min, max, n )} is equivalent to 
\code{seq( min, max, len=n )}.
}
\examples{
gen_midp( 0, 10, 6 )
gen_midp( 1, 11, 6 )
gen_midp( 0, 100, 11 )
gen_midp( 0, 100, 11 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/eq.R
\name{eq}
\alias{eq}
\title{Observed Score Equating}
\usage{
eq(
  score1,
  freq1,
  cdf1 = NULL,
  score2,
  freq2,
  cdf2 = NULL,
  lim1 = NULL,
  lim2 = NULL,
  smooth1 = 0,
  bandwid1 = 3,
  smooth2 = 0,
  bandwid2 = 3,
  method = 2,
  interpol_method = "linear",
  title = "",
  nolow = 0,
  round = 1,
  print = 1,
  plot = 1
)
}
\arguments{
\item{score1}{a vector consisting of the score of test1}

\item{freq1}{a vector consisting of the frequency counts at score1}

\item{cdf1}{a vector consisting of the cumulative frequencies at score1}

\item{score2}{a vector consisting of the score of test2}

\item{freq2}{a vector consisting of the frequency counts at score2}

\item{cdf2}{a vector consisting of the cumulative frequencies at score2}

\item{lim1}{min and max score of test1}

\item{lim2}{min and max score of test2}

\item{smooth1}{# of times to smooth cdf1}

\item{bandwid1}{bandwidth for running average smooth of cdf1}

\item{smooth2}{# of times to smooth cdf2}

\item{bandwid2}{bandwidth for running average smooth of cdf2}

\item{method}{= 1 to use linear equating \cr
= 2 to use equi-percentile equating}

\item{interpol_method}{= "constant", "linear" or "spline"}

\item{title}{title string}

\item{nolow}{= 1 to avoid lowering scores}

\item{round}{= 0 not to round the result to integer. (not yet available)}

\item{print}{= 1 to print the result}

\item{plot}{= 1 to plot the conversion table \cr
= 2 to plot the cdf \cr
= 3 to plot the smoothed cdf}
}
\value{
a list of the following:\cr
ctable: the conversion table consisting of (score, score21, freq1) \cr
mands:  the summary stat of the converted score dist: (score21,freq1) \cr
newfreq: the frequency distribution of the converted score (score21, freq21)
\cr
sdist1 and sdist2: input and smoothed score distributions \cr
cntr: a list of control parameters used.

\code{ctable} shows that
test score score1[i] of test1 is equivalent to
test score score21[i] of test2.
}
\description{
This performs the observed score equating of
the test score 1 to the test score 2.
}
\details{
Equipercentile equating of test1 score \code{x1} to test2 score \code{x2}
 is defined as\cr
\preformatted{
 x21 = invF2( F1(x1) )
}
where \code{F1} is the distribution function of \code{x1} and
\code{invF2} is the inverse of the distribution function of \code{x2}.
In this function, \code{invF2} is calculated by interpolating
\code{( F2(x2), x2 )} at \code{F1(x1)} with or without smoothing.

Equipercentile equating is essentially the same as native \code{qqplot}. \cr
Therefore, the following two codes produce similar results: \cr
\preformatted{
 eq( score1=score1, freq1=freq1, score2=score2, freq2=freq2 )
 qqplot( expand_freqdist( score1, freq1 )
             , expand_freqdist( score2, freq2 ), type="l" )
}
Note that, since native \code{qqplot} cannot handle case weight,
\code{lazy.tools::expand_freqdist} is used to recover the raw data
from frequency table.


\code{cdf} has priority over \code{freq}.
}
\examples{

seed <- 1701
set.seed(seed)

scoredist1 <- gen_test_score( 500, 0,10, beta=c(2,4), plot=1 )
scoredist2 <- gen_test_score( 1000, 0,15, beta=c(4,2), plot=1 )
reseq <- eq( score1=scoredist1[,1], freq1=scoredist1[,2]
           , score2=scoredist2[,1], freq2=scoredist2[,2]
           , smooth1=3, smooth2=3, method=2, plot=3 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mandd.R
\name{mandd}
\alias{mandd}
\title{Calculates mean vector and dispersion matrix with case weight}
\usage{
mandd(
  data,
  weight = 1,
  by = NULL,
  byvarname = NULL,
  vardef = "n",
  nocov = 0,
  cor = 0,
  print = 0,
  simplify = 0
)
}
\arguments{
\item{data}{data matrix or data frame.}

\item{weight}{case weight vector of length nrow(data) or 1.}

\item{by}{a factor or a list of factors, each of length nrow(data) \cr
to be used as the INDICES of by function.}

\item{byvarname}{vector containing the names of by varialbes to be used.
(Not Yet Implemented.)}

\item{vardef}{= "n-1" to use n-1 as denominator for variance.}

\item{nocov}{= 1 to skip the calculation of the variance/covariance matrix.
\cr \code{mands} function will be used.}

\item{cor}{= 1 to return the correlation matrix as well.}

\item{print}{= 1 to print the result}

\item{simplify}{= 1 to simplify the rewult when by= is given}
}
\value{
list of: \cr
mean vector, var vector, std vector, nobs vector, \cr
where nobs is sum(weights) for nonmissing obs and n=nrow(data). \cr
cov matrix,  nobspair matrix which contains # of obs. per pair
, or NULL, \cr
n (# of rows), and vardef.
}
\description{
Calculates mean vector and dispersion matrix with case weight
}
\details{
When the dispersion matrix is not needed (nocov=1) use \code{mands} function.
\cr
When simplified=0 is given, the result is a list consisting of \cr
vectors: mean, var, std, nobs \cr
matrix: cov, nobspair \cr
scalars: n, vardef \cr
where nobspair is a matrix of the same size as cov containing
the weighted # of observations used to calculate cov. \cr
When cor=1 is given, correlation matrix will also be returned.

When by is specified, the result is a class by list if simplify=0,
Otherwise, it is a list consisting of \cr
matrices: mean, var, std, nobs \cr
lists: cov, nobspair \cr
scalars: n, vardef \cr
\cr
To convert the list of covariance matrices to an array or stacked matrix,
 use:
 \cr
 \code{ library(lazy.mat) } \cr
 \code{ cov_array=l2a(res1$cov) } \cr
 \code{ cov_mat=a2m(cov_array,method=2) } \cr
}
\examples{
##################################
# data matrix with by group
##################################
nobs <- 1000
ndims <- 2

# covariance matrix
cov <- 0.2
C1 <- matrix(cov,ndims,ndims)+(1-cov)*diag(ndims)
C1[1,1] <- .5
C2 <- -C1; diag(C2) <- diag(C1)
C3 <- diag(ndims)
Printm(C1,C2,C3)

# random numbers
seed=1701
set.seed(seed)

# X1 ~ N(0,C1),  X2 ~ N(0,C2),   X3 ~ N(0,C3)
X1=matrix(rnorm(nobs*ndims),nobs)
ev <- eigen(C1); eva <- diag(ev$values); eve <- ev$vectors
X1 <- X1\%*\%diag(sqrt(ev$values))\%*\%t(ev$vectors)
X2 <- matrix(rnorm(nobs*ndims),nobs)
ev <- eigen(C2); eva <- diag(ev$values); eve <- ev$vectors
X2 <- X2\%*\%diag(sqrt(ev$values))\%*\%t(ev$vectors)
X3 <- matrix(rnorm(nobs*ndims),nobs)

X <- rbind(X1,X2,X3)
byvar <- rep(c("pos","neg","zero"), each=nobs)

# missing values
locmiss <- sample(1:length(X), 0.2*length(X))
X[locmiss]=NA

# statistics w/o by-group
res0 <- mandd( X )
# statistics by by-group
res1 <- mandd( X, 1, byvar, simplify=1 )
Printm(res1$mean, res1$std, res1$cov)


#############################
# grid points and weight
#############################
# C1, C2, C3 are defined above.
npoints <- 11; thmin <- -3; thmax <- 3; nobs2 <- 10000
theta <- seq(thmin,thmax,length.out=npoints)
theta <- cprod(theta,theta)
if( ndims > 2 )
 for( i in 2:ndims )
  theta <- cprod(theta,theta)

# weight:  pdf of N(0,C1)
thdist1 <- exp( -0.5*diag( theta\%*\%solve(C1)\%*\%t(theta) ) )
thdist1 <- thdist1/sum(thdist1)
n1 <- round(nobs2*thdist1+.5)

# statistics
res20 <- mandd( theta, n1 )
res200 <- mandd( theta )
Printm(res20$mean, res200$mean)
Printm(res20$var, res200$var)
Printm(C1, res20$cov, res200$cov)


# missing values
seed <- 1701
set.seed(seed)
locmiss1 <- sample( 1:(npoints^ndims), max(1,(npoints^ndims)/10) )
theta[locmiss1] <- NA

# statistics
res2 <- mandd( theta, n1 )
Printm(res2$mean, res2$std, res2$cov)


# grid points and weight with by group
# theta, nobs2, C1, C2, C3 are defined above.
# weight:  pdf of N(0,C2)
thdist2 <- exp( -0.5*diag( theta\%*\%solve(C2)\%*\%t(theta) ) )
thdist2[is.na(thdist2)] <- 0
thdist2 <- thdist2/sum(thdist2)
n2 <- round(nobs2*thdist2+.5)
# weight:  pdf of N(0,C3)
thdist3 <- exp( -0.5*diag( theta\%*\%solve(C3)\%*\%t(theta) ) )
thdist3[is.na(thdist3)] <- 0
thdist3 <- thdist3/sum(thdist3)
n3 <- round(nobs2*thdist3+.5)

Theta=rbind(theta,theta,theta)
n=c(n1,n2,n3)
byvar2=rep(c("pos","neg","zero"),each=nrow(theta))

# statistics
res3 <- mandd( Theta, n, byvar2, simplify=1 )
Printm(res3$mean, res3$std, res3$cov)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UTF8pgm.R
\name{nbyte}
\alias{nbyte}
\title{Returns the number of bytes used to represent each character
in each of the string.}
\usage{
nbyte(x)
}
\arguments{
\item{x}{String vector}
}
\value{
a list consisting of the number of bytes per character
}
\description{
Returns the number of bytes used to represent each character
in each of the string.
}
\details{
Let x = "abc". The core of this function is \cr
\code{lapply(
  strsplit(x,""), function(y) unlist(lapply(lapply(y,charToRaw),length))
  ) } \cr
It counts the length of the Raw Expression of each character.

About rawToChar and charToRaw functions: \cr
To get the raw expression of character, use\cr
\code{ raw <- charToRaw(paste(c(0:9," ",letters,LETTERS),collapse="")) }\cr
To get the numeric equivalent of raw, use\cr
\code{ num <- as.numeric(raw) }.\cr
To recover the original characters, use\cr
\code{ char <- rawToChar( as.raw(num) ) }.\cr
}
\examples{
# Example of one, two, three amd four byte characters on UTF-8 machines
if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 ){
 num=c(97, 206, 177, 227, 129, 130, 240, 160, 174, 183)
 char=num2char(num)
 nbyte(char)
}

# A string consisting of single and multi byte characters
# The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
{if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
 nza <- c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
else
 nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
za <- rawToChar(as.raw(nza))

# location of the multi byte characters
nchar(za)
is.DBCS(za)
printm( nchar(za), length(charToRaw(za)), nchar2(za) )
sza=unlist(strsplit(za,""))
printm(sza,nbyte(za),is.DBCS(za))

cat("When using Shif-JIS encoding, ",
    "it seems that 'strsplit' makes the result 3 byte characters.\n")
printm(sza,unlist(nbyte(sza)),unlist(is.DBCS(sza)))
psza <- paste(sza,collapse="")
printm(sza,nbyte(psza),is.DBCS(za))

identical(za,psza)
identical(nbyte(za),nbyte(psza))

# Instead of strsplit, use below
sza2 <- substring(za,1:nchar(za),1:nchar(za))
printm( sza, unlist( nbyte( sza2 ) ) )
psza2 <- paste(sza2,collapse="")
nbyte(psza2)

identical(za,psza2)
identical(nbyte(za),nbyte(psza2))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/etime.R
\name{etime}
\alias{etime}
\title{Elapsed Time}
\usage{
etime(opt = 0, title = NULL)
}
\arguments{
\item{opt}{= 0 to reset the clock \cr
= 1  to print the elapsed time. \cr
= 2  to print the elapsed time and reset the clock.}

\item{title}{a title string}
}
\value{
etime in seconds.
}
\description{
This program calculates the elapsed time between concecutive calls.
}
\details{
A global variable ..start.. is used.
}
\examples{
etime(0)
x=rnorm(10000000)
etime(1)
x=rnorm(10000000)
etime(2)
x=rnorm(10000000)
etime(1, "This is the title string.")
rm(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_zenkaku_sets.R
\name{generate_zenkaku_sets}
\alias{generate_zenkaku_sets}
\title{Generate Zenkaku Character Sets:}
\usage{
generate_zenkaku_sets(where = "global")
}
\arguments{
\item{where}{Name of the environment in which the character set
are generated.}
}
\description{
Generate Zenkaku Character Sets:
}
\details{
This function generates the following objects in the specifed environment:\cr
\code{ zalpha, zAlpha, zdigit, zhkana, zhkana, Prefectures }\cr
\cr
When \code{where != "global"} the character set will be generated
in the calling environment: \code{envir=parent.frame()}
}
\examples{
generate_zenkaku_sets()

paste( paste(zAlpha[c(14,3,3)], collapse="")
 , paste(zdigit[c(1,7,10,1)],collapse=""), sep="-" )
cat("\nMy favorite prefectures: ", Prefectures[c(26,40,13)], "\n")

# clear the ones in the global env.
zdigit=NULL; zalpha=NULL

# generate in the global env.
generate_zenkaku_sets(  )
Print(zdigit[1:5],zalpha[1:5])


# Run below by copy and paste.
\dontrun{
# a function to create and print zdigit and zalpha
testZ <- function( where="global" ){
 cat("\nIn test\n")
 generate_zenkaku_sets( where=where )
 Print(zdigit[1:5],zalpha[1:5])
}

# clear the ones in the global env.
zdigit=NULL; zalpha=NULL
# generate in the function
testZ( "here" )

# clear the ones in the global env.
zdigit=NULL; zalpha=NULL

# generete in the global env. from a function
testZ()
Print(zdigit[1:5],zalpha[1:5])

rm(  zalpha, zAlpha, zdigit, zhkana, zkkana, Prefectures )
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/procote.R
\name{procote}
\alias{procote}
\title{Extened Orthogonal Procrustes Rotation}
\usage{
procote(A, C, estm = 0, estd = 0, maxiter = 200, eps = 1e-06, print = 2)
}
\arguments{
\item{A}{The input matrix to be rotated}

\item{C}{The target matrix}

\item{estm}{= 1 to estimate column means}

\item{estd}{= 1 to estimate scaling constant}

\item{maxiter}{max # of iterations}

\item{eps}{convergence criterion for rmse}

\item{print}{= 0 to surpress the output}
}
\value{
A list of: \cr
B=d * A \%*\% T + matrix(1,nv,1)\%*\%t(m), \cr
 T, m, d, Cm the updated target matrix, rmse=sqrt(RSS)
}
\description{
Extened Orthogonal Procrustes Rotation
}
\details{
This program finds those A, m, d  and orthonormal T which minimize \cr
  \deqn{RSS = tr( ( C  - Chat )' ( C - Chat ) ) }\cr
  where \eqn{Chat = d * A \%*\% T + matrix(1,nv) \%*\% t(m)}, \cr
  \eqn{d} is a scalar, \eqn{T} is an orthonormal rotation matrix, \cr
  and \eqn{m} is the column mean vector. \cr
\cr
Note that when \code{estd=0} and \code{estm=0}
, and the target matrix does not
have missing elements, \cr
this is equivalent to the usual orthogonal Procrustes rotation.
}
\examples{
 set.seed(1701)
 nv <- 10; nr <- 2
 pmiss <- 0.3

 A <- matrix(rnorm(nv*nr),nv,nr)
 dum <- matrix(rnorm(nv*nr),nv,nr)
 T <- eigen(t(dum)\%*\%dum)[[2]]
 m <- matrix(rnorm(nr),nr,1)
 d <- 10*runif(1)
 C <- d*A\%*\%T + matrix(rep(1,nv),nv,1)\%*\%t(m)

 res <- procote( A, C, estm=0, estd=0, print=1 )
 res2 <- procote( A, C, estm=1, estd=1, print=1 )

 # missing elements in the target
 dum <- matrix(runif(nv*nr),nv,nr)
 C[which(dum < pmiss)] <- NA

 res3 <- procote( A, C, estm=0, estd=0, print=1 )
 res4 <- procote( A, C, estm=1, estd=1, print=1 )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.R
\name{logit}
\alias{logit}
\title{logit function with D=1.7}
\usage{
logit(p, D = 1.7)
}
\arguments{
\item{p}{vector}

\item{D}{= 1.7}
}
\value{
vector of the size of p
}
\description{
logit function with D=1.7
}
\details{
This function calculates \cr
  x=log(p/(1-p))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand.grid1.R
\name{expand.grid1}
\alias{expand.grid1}
\title{expand.grid with rev and colnames option}
\usage{
expand.grid1(
  ...,
  rev = 1,
  colnames = 0,
  KEEP.OUT.ATTRS = TRUE,
  stringsAsFactors = FALSE
)
}
\arguments{
\item{...}{vectors, factors or a list containing these.}

\item{rev}{= 1 to make the last elements of ... chage first.}

\item{colnames}{= 1 to use  ... as the column names.}

\item{KEEP.OUT.ATTRS}{a logical indicating the "out.attrs" attribute (see below) should be
computed and returned.}

\item{stringsAsFactors}{logical specifying if character vectors are converted to factors.}
}
\value{
A data frame.
}
\description{
expand.grid with rev and colnames option
}
\details{
See comments of native expand.grid.\cr
  Compare \code{expand.grid( 1:2, 2:3 )} and
  \code{expand.grid1( 1:2, 2:3 )}.\cr
  \code{expand.grid1( 1:2, 2:3, rev=0 )} should be equal to
   native \code{expand.grid( 1:2, 2:3 )}. \cr
 \cr
 When the first element of ... is a list, expand.grid1 works
 as if it is called only with the first element.
 See the example below.\cr

 \cr
 When ... contains a matrix, better use \code{cprod}.
}
\examples{

Print( expand.grid( 1:2, 2:3 ), expand.grid1( 1:2, 2:3 ) )

res=expand.grid( 1:2, c("a","b") )
res1=expand.grid1( 1:2, c("a","b") )
Print(res,res1)


ll=list(aaa=c("a","b"),bbb=1:3)
expand.grid1(ll, colnames=0)
expand.grid1(ll, colnames=1)
expand.grid1(ll, colnames=0, rev=0)
expand.grid1(ll, colnames=1, rev=0)

expand.grid(ll,ll)
expand.grid1(ll,ll)


x=c("a","b"); y=1:3
expand.grid1(x,y, colnames=0)
expand.grid1(x,y, colnames=1)
expand.grid1(x,y, colnames=0, rev=0)
expand.grid1(x,y, colnames=1, rev=0)



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/revchar.R
\name{revstr}
\alias{revstr}
\title{Reverse string}
\usage{
revstr(x)
}
\arguments{
\item{x}{Input character vector}
}
\value{
A vector containing the reversed elements.
}
\description{
Reverse string
}
\details{
revchar is the alias for this function.
}
\examples{
revstr(c("ncc","1701","a"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pplot.R
\name{iotable}
\alias{iotable}
\title{Inside-Out table}
\usage{
iotable(
  x,
  id = NULL,
  var = NULL,
  rank = 0,
  abs = 0,
  descending = 1,
  dec = NULL,
  mark = "-",
  print = 1
)
}
\arguments{
\item{x}{Input vector matrix}

\item{id}{id to be used instead of rownames(x).}

\item{var}{sas-like abbreviated list of variables to be used.}

\item{rank}{= 1 to retrun the rank in stead of the row name.}

\item{abs}{= 1 to use the absolute value of x to sort.}

\item{descending}{= 0 to sort ascendingly.}

\item{dec}{a vector indicating if the sort is descending order
for each varaible.}

\item{mark}{A character to mark the negative numbers.}

\item{print}{= 1 to print the result.}
}
\value{
A vector or matrix of the same size as x.
}
\description{
Inside-Out table
}
\details{
This function sort the id according to the magnitude of each variable.\cr
 When rank = 0 and abs = 1, those names corresponding to\cr
   the negative values  will be marked by - sign. \cr
}
\examples{
data <- matrix(c(c(2,1,4,5,3),1:5),,2)
rownames(data) <- paste("r",1:5,sep="")
colnames(data) <- paste("v",1:2,sep="")
data[4,2]=-9
iotable( data, descending=1, print=0 )
iotable( data, descending=1, abs=1, print=0 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{decp}
\alias{decp}
\title{Returns the (approximate) # of Digits on the Both Sides of the Decimal Point}
\usage{
decp(num, maxpower = 10)
}
\arguments{
\item{num}{A numeric vector}

\item{maxpower}{10 or higher}
}
\value{
A vector or matrix containing the # of digits
}
\description{
Returns the (approximate) # of Digits on the Both Sides of the Decimal Point
}
\examples{
decp(c(-5.2,-3.99, 0, 1, 1.2, 5.23, 9999.8888, 99999999.8888888888))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UTF8pgm.R
\name{cchar}
\alias{cchar}
\title{Starting and Ending Columns of Each Character in a Text File}
\usage{
cchar(x, from = NULL, to = NULL, last = TRUE)
}
\arguments{
\item{x}{A string vector}
}
\value{
When \code{from} is not given, a list of matrices
\code{FT[[i]]} where \code{FTT[[i]][j,1]} and \code{FTT[[i]][j,2]},
resp, are the starting and ending columns of the \code{j}-th character
of the \code{i}-th string.
When the input is a string, \code{FT[[1]]} will be returned.
\cr
When \code{from} is given, a list of matrices
\code{FTc[[i]]} corresponding to each of the strings where
\code{FTc[[i]][j,1], FTc[[i]][j,2]} are, resp,
the starting and ending characters corresponding to
\code{from[j]} and \code{to[j]} of the i-th string.
}
\description{
Starting and Ending Columns of Each Character in a Text File
}
\details{
This function can be used to get the substring of a string
by specifying the starting and the ending columns. \cr
It is assumed that the double or triple byte characters takes two columns
of a text file.  (Zenkaku Character)

If \code{x} contains a string consisting of single and multibyte characters,
\preformatted{
 ft <- cchar(x)
 substr( x, which(ft[,1] == start), To=which(ft[,2] == stop) )
}
will return the characters between columns \code{start} and \code{stop}.
}
\examples{
# A string consisting of single and multi byte characters
# The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
{if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
 nza <- c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
else
 nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
za <- rawToChar(as.raw(nza))
# za <- num2char(nza)

# location of the multi byte characters
nchar(za)
is.DBCS(za)
printm( nchar(za), nchar2(za), nchar(za)+sum(is.DBCS(za)) )
printm(1:nchar(za), unlist(strsplit(za,"")),nbyte(za),cchar(za))

# When input is a string vector
cchar(c(za,za))

# convert column position to character position
# from and to as column position
from <- c(1,2,3,5,6,7,9,11)
to <- c(1,2,4,5,6,8,10,11)
# character position corresponding to (from,to) to be used in substr(ing)
FTc <- cchar(za,from,to)[[1]]
# substrings
ssg <- substring(za,FTc[,1],FTc[,2])
# substrings using substr2 which can hadle column (from,to)
ss2 <- substr2(za,from,to)
za1 <- strsplit(za,"")[[1]]   # to print each character
printm(za1, from, to, FTc, ssg,ss2)

# use of last option
cchar( za, 1:11, 1:11 )
cchar( za, 1:11, 1:11, last=0 )

cchar( za, 3,4 )
cchar( za, 3,5 )
cchar( za, 3,7 )
cchar( za, 3,7, last=0 )

# When input is a string vector
cchar( c(za,za), 1:11, 1:11, last=0 )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stack_df.R
\name{stack_df}
\alias{stack_df}
\title{Vertically Stack Data Frames /  Matrices with Different Sets of Variables}
\usage{
stack_df(..., sort = 0)
}
\arguments{
\item{...}{data frames or matrices}

\item{sort}{= 1 to alphabetically sort the columns of the result \cr
Otherwise, the order of the original data frames will be kept.}
}
\value{
A data frame or a matrix
}
\description{
Vertically Stack Data Frames /  Matrices with Different Sets of Variables
}
\details{
Conceptually, \code{stack_df( df1, df2 )} is equivalent to
\preformatted{
 t( merge( cbind(t(df1),cn=colnames(df1)), cbind(t(df2),cn=colnames(df2))
         , by="cn", all=1 ) )
}.

Using the data frames in the example section, the core part is:\cr
\preformatted{
res <- merge( data.frame(df1m,id=1:nrow(df1m))
            , data.frame(df2m,id=(1:nrow(df2m))+nrow(df1m)), all=1 )
 }
which vertically stacks data frames because the values of
the common variable id are unique.
\cr
Note that, since check.names=0 and optional=1 are used
when creating data frames,
the resulting data frame may not have valid column names.
\cr
\cr
When all the row names are unique, it will be used as the row names
of the resulting data frame. Otherwise, native \code{make.unique} will be
used to mark the duplicate row names which appends ".n" to the original names.
}
\examples{
A <- demomat(5,4,3,cnroot="")
A1 <- A[,,1]; A2 <- A[,,2]; A3 <- A[1:2,,3]
colnames(A1) <- sub("4","v4",colnames(A1))
colnames(A2) <- sub("4","v4",colnames(A2))
colnames(A3) <- sub("4","v4",colnames(A3))
rownames(A1) <- paste("a1_",1:nrow(A1),sep="")
rownames(A2) <- paste("a2_",1:nrow(A2),sep="")
rownames(A3) <- paste("a3_",1:nrow(A3),sep="")
rownames(A3)[2] <- "a1_2"
A1m <- A1[,c(1,4)]; A2m <- A2[,c(1,3), drop=0]; A3m <- A3[,c(4,2,3)]
rownames(A2m) <- NULL

print(A1m)
print(A2m)
print(A3m)

res <- stack_df( A1m, A2m, A3m )
Print(res)

# no factors allowed.
df1m <- as.data.frame(A1m, stringsAsFactors=0)
df2m <- as.data.frame(A2m, stringsAsFactors=0)
df2m[,2] <- paste("c",df2m[,2],sep="")
df3m <- as.data.frame(A3m, stringsAsFactors=0)

res <- stack_df( df1m, df2m, df3m )
Print(df1m, df2m, df3m)
Print(res)
res2 <- stack_df( df2m, df3m, df1m )
Print(res2)
res3 <- stack_df( df1m, df2m, df3m, sort=1 )
Print(res3)

# use of native Reduce function
res0 <- Reduce( stack_df, list(df1m, df2m, df3m) )

res4 <- stack_df( A=demomat(3,2), b=letters[1:3] )
Print(res4)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.R
\name{logit1}
\alias{logit1}
\title{logit function with D and minimum value}
\usage{
logit1(p, D = 1.7, minp = 1e-09)
}
\arguments{
\item{p}{vector}

\item{D}{= 1.7}

\item{minp}{minimum value of the function}
}
\value{
vector of the size of p
}
\description{
logit function with D and minimum value
}
\details{
This function calculates \cr
  x=log(p/(1-p))
at the following p values \cr
  p[p<minp]=minp; p[p>1-minp]=1-minp
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pplot.R
\name{pplot}
\alias{pplot}
\title{Profile Plot}
\usage{
pplot(
  df,
  var = NULL,
  pty = 16,
  pcol = 1:8,
  pch = 2,
  lwd = 2,
  plty = 1:8,
  plwd = 2,
  axislabcol = "blue",
  title = "",
  vlabels = NULL,
  caxislabels = NULL,
  paxislabels = NULL,
  legend = 1,
  lpos = c(1.5, -0.2),
  lcex = 1,
  llwd = 1,
  lbty = "n",
  standard = 0,
  nobspc = 0,
  down = 1,
  across = 1,
  mar = c(8, 2, 1, 2),
  ...
)
}
\arguments{
\item{df}{data frame}

\item{var}{sas-like abbreviated list of variables to be used.}

\item{pty}{A vector to specify point symbol: \cr
Default 16 (closed circle), if you don't plot data points, \cr
  it should be 32. This is repeatedly used for data series. \cr}

\item{pcol}{A vector of color codes for plot data: \cr
Default 1:8, which are repeatedly used.}

\item{pch}{plot character}

\item{lwd}{line width}

\item{plty}{A vector of line types for plot data: \cr
Default 1:6, which are repeatedly used.}

\item{plwd}{A vector of line widths for plot data: \cr
Default 1, which is repeatedly used.}

\item{axislabcol}{not used.}

\item{title}{plot title string.}

\item{vlabels}{not used.}

\item{caxislabels}{not used.}

\item{paxislabels}{not used.}

\item{legend}{= 1 to display legend.}

\item{lpos}{upper left corner of the legend.}

\item{lcex}{font size for the legend.}

\item{llwd}{line width for the legend.}

\item{lbty}{the type of box to be drawn around the legend.}

\item{standard}{= 1 to standardize the variables \cr
= 2 to scale the variables using min and max}

\item{nobspc}{# of observations in one chart.}

\item{down}{# of graphs in a column.}

\item{across}{# of graphs in a row.}

\item{mar}{margins for multiple graphs per page.}

\item{...}{Miscellaneous arguments to be given for plot.}
}
\value{
None
}
\description{
Profile Plot
}
\examples{
data=data.frame(a=c(1,2,3,4),b=c(0,1,0,1),c=c(4,3,2,1))
pplot( data)
pplot( data, nobspc=2 )
pplot( data, nobspc=2, across=2 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.varname.R
\name{is.varname}
\alias{is.varname}
\title{Check if the argument is a valid variable name}
\usage{
is.varname(A)
}
\arguments{
\item{A}{Input string vector or list}
}
\value{
A logical vector
}
\description{
Check if the argument is a valid variable name
}
\details{
varname is a string starting with \code{[:alpha:]} or \code{.}
 and having no \code{[:punct:]} except \code{.} or \code{_} .
}
\examples{
is.varname( c("and2","age_1_x","a.1","a_1.","1.","_a",".a") )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yearJAD.R
\name{yearJAD}
\alias{yearJAD}
\title{Conversion between Japanese Era and AD}
\usage{
yearJAD(yearJ = 1952, yearAD = NULL, EraName = "")
}
\arguments{
\item{yearJ}{a vector of Japanese years
with Japanese era name,  e.g.  M01, T15, S27, H1, R04}

\item{yearAD}{a vector of years in AD \cr
This has priority over yearJ.}

\item{EraName}{a Japanese Era Name to be used for the conversion from AD}
}
\description{
Conversion between Japanese Era and AD
}
\details{
\code{ yearJAD( "R1" ) } returns 2019. \cr
 \code{ yearJAD( yearAD=2019 ) } returns "R01". \cr
 \code{ yearJAD( yearAD=2019, EraName="S" ) } returns "S94".
}
\examples{
# conversion table
yJ <- c("M1","M2","M44","M45","T1","T2","T14","T15","S1","S2","S63","S64"
     , "H01","H02","H30","H31","R1","R2","R4")
yAD <- yearJAD( yJ )
yJfromAD <- yearJAD( yearAD=yAD )
Print(yJ,yAD,yJfromAD)

# using out-of-range Era Name
yearJAD(yearAD=2017, EraName="R")  # negative number
yearJAD(yearAD=2018, EraName="R")  # year 0 means a year before year 1.
yearJAD(yearAD=2019, EraName="R")

yearJAD(yearAD=1952, EraName="T")
yearJAD(yearAD=1952, EraName="S")
yearJAD(yearAD=1952, EraName="H")  # negative number

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_paren.R
\name{add_paren}
\alias{add_paren}
\title{Add Enclosing Parentheses}
\usage{
add_paren(exp, check = 0, trim = 0)
}
\arguments{
\item{exp}{a string vector consisting of symbolic expressions..}

\item{check}{= 1 to avoid adding redundant parens.}

\item{trim}{= 1 to remove leading and trailing blanks from exp.}
}
\value{
an object of the same size as exp
}
\description{
Add Enclosing Parentheses
}
\examples{
exp=c("a","(a+b)","(a+b)-(c-d)")
add_paren( exp )
add_paren( exp, check=1 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqdist1.R
\name{freqdist1}
\alias{freqdist1}
\title{Frequency distribution of univariate numeric data with case weight}
\usage{
freqdist1(
  x,
  weights = 1,
  npoints = 7,
  min = NULL,
  max = NULL,
  prob = 0,
  midpoints = NULL,
  wid = 6,
  dig = 1,
  maxf = NULL,
  title = NULL,
  digits = 3,
  print = 0,
  plot = 0
)
}
\arguments{
\item{x}{numeric vector or column vector}

\item{weights}{case weight
Can be a scalar, vector of size length(x) or matrix of length(x) x nf.\cr
If nf > 1, frequency for each column of weights will be calculated.}

\item{npoints}{# of mid-points}

\item{min}{minimum value of mid-points}

\item{max}{maximum value of mid-points}

\item{prob}{= 1 to return relative frequency in stead of frequency}

\item{midpoints}{vector of predefied mid-point values}

\item{wid}{Format of mid-point value (wid.dig)}

\item{dig}{Format of mid-point value (wid.dig)}

\item{maxf}{max value of frequency for plot}

\item{title}{tile to be used in plot}

\item{digits}{Format for mid-points}

\item{print}{= 1 to print result}

\item{plot}{= 1 to plot historgam}
}
\value{
A list consisting of\cr
  midpoints =  midpoints \cr
  freq =       column vector of frequency at midpoints \cr
  breaks =     breaks associated with the midpoints \cr
  class =      classification of each obs into the midpoints \cr
  nobs =       sum of case weights \cr
  error =      error flag
}
\description{
Frequency distribution of univariate numeric data with case weight
}
\details{
(npoints, min, max)  defines the mid-points. \cr
   If min or max is NULL, min(x) and max(x) will be used. \cr
   midpoints has priority over (npoints, min, max).\cr
The out of range observations,  that is, those x satisfying \cr
 x < midpoints[1]     or   midpoints[npoints] < x \cr
 will be be included in midpoints[1] and midpoints[npoints], resp. \cr
\cr
Try this to see how the obs on the border are classified. \cr
   xx=c(1,2,3,4,5,6,7) \cr
   brks=c(0,2,4,6,8) \cr
   cc=cut(xx,brks) \cr
}
\examples{
set.seed(1701)
n <- 10000
npoints <- 21
x <- rnorm(n)
res <- freqdist1( x, npoints=npoints,min=-3,max=3, print=1, plot=1 )
res1 <- freqdist1( x, midpoints=res$midpoints, print=1 )
weight=c(0,1,3,4,2,2,1)
freqdist1(seq(-3,3,1),weight, min=-3,max=3,npoints=7, print=1,plot=1)
rm(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance.R
\name{distancew0}
\alias{distancew0}
\title{Calculation of Weighted Distance Matrix}
\usage{
distancew0(X, W = diag(ncol(X)))
}
\arguments{
\item{X}{n x r matrix}

\item{W}{r x r diagonal maatrix}
}
\value{
A matrices of size n x n
}
\description{
Calculation of Weighted Distance Matrix
}
\details{
This function calculates the weighted distance among the rows of
\code{X} matrix  by applying the a-th diagonal element of \code{W} matrix
as the dimensional weights as:
\deqn{ d_{ij} = \sqrt( \sum_{a=1}^r   w[a,a] (x[i,a]-x[j,a])^2 ) }
or
\deqn{ d_{ij} = \sqrt( (x[i,]-x[j,])' W (x[i,]-x[j,]) ) }
\cr
Note that \code{W} matrix will NOT be squared when calculating distance.
\cr
}
\examples{
X <- matrix( c(0,0,  3,0,  0,4), 3,2,  byrow=1 )
distancew0( X )
W <- diag(c(2,1))
distancew0( X,W )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recode.R
\name{recode}
\alias{recode}
\title{Recode the Values of obj from "from" to "to".}
\usage{
recode(
  obj,
  from = NULL,
  to = NULL,
  ft = NULL,
  method = 2,
  useNA = 0,
  debug = 0
)
}
\arguments{
\item{obj}{a vector, matrix or list of those.}

\item{from}{a vector of old values}

\item{to}{a vector or new values}

\item{ft}{the conversion table data frame}

\item{method}{= 1 to use \code{merge}, \cr
= 2 to use R style teble look-up using names attribute of a vector.}

\item{useNA}{= 1 to mark out-of-range elements}

\item{debug}{= 1 to print intermediate result.}
}
\value{
A matrix or vector depending on the \code{obj}.
 When \code{obj} is a list, a list will be returned.
}
\description{
Recode the Values of obj from "from" to "to".
}
\details{
When \code{ft} is not given,
 \code{cbind(from,to)}  makes the conversion table \code{ft}. \cr
 \code{ft} has priority over from and to.

 The core of this function is:\cr\cr
 If \code{method==1} \cr
 \code{New <- merge( cbind(from,to), data.frame(from=c(obj))
 , by="from", sort=FALSE, all.y=TRUE )} \cr
 If \code{method==2} \cr
 \code{names(to) <- from; New <- to[as.character(c(obj))]} \cr

 Method 2 is faster than method 1. \cr

 If the conversion table is not large enough,
 if \code{method=2} and \code{useNA=1},
 out-of-range elements will be set to NAs,
 otherwise, the original values will not  be changed.

 When the types of \code{obj} and \code{from} are different,
 they will be converted to character.

 \code{lazy.tools::new_name} will be deprecated.
}
\examples{
n <- 10
n1 <- n-2
obj <- cbind( letters[1:n], rev( letters[1:n] ) )
obj[1,1] <- NA; obj[3,2] <- NA
rownames(obj) <- paste("RR",1:n,sep="")
colnames(obj) <- paste("CC",1:ncol(obj),sep="")
from <- letters[1:n1]
to <- LETTERS[1:n1]
# to <- 1:n1
res <- recode( obj, from, to )
Print(obj,res,from,to)
res2 <- recode( obj, from, to, useNA=1 )
Print(obj,res2,from,to)


n=10
n1=n-2
obj=cbind( 1:n, n:1 )
rownames(obj)=paste("RR",1:n,sep="")
colnames(obj)=paste("CC",1:ncol(obj),sep="")
from=1:n1
to=from^2
res=recode( obj, from, to, method=2 )
Print(obj,res, from, to)


n <- 10
n1 <- n-2
obj <- cbind( 1:n, n:1 )        # This is numeric.
rownames(obj) <- paste("RR",1:n,sep="")
colnames(obj) <- paste("CC",1:ncol(obj),sep="")
from <- as.character(1:n1)      # This is character.
to <- LETTERS[1:n1]
res <- recode( obj, from, to )
Print(obj,res, from, to)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqfuncs.R
\name{merge_tables}
\alias{merge_tables}
\title{Merge Tables or Frequency Distributions of the form (score, freq)}
\usage{
merge_tables(..., omitmiss = 0)
}
\arguments{
\item{...}{tables, matrices or data.frames containing frequency distribution}

\item{omitmiss}{= 1 to remove zero-frequency cells}
}
\value{
A frequency distribution of the form (x,freq)
}
\description{
Merge Tables or Frequency Distributions of the form (score, freq)
}
\details{
Each of ... can be a class table object created by native table or xtabs,
or a data.frame or matrix containing the level-index and frequency. \cr
Each of ... should have the same # of factors to be tabulated.
}
\examples{

set.seed(1701)
x <- sample(1:8,10,replace=1); x <- c(x,c(1,1,7)); xc <- paste("x",x,sep="")
y <- sample(0:9,10,replace=1); y <- c(y,c(5,5,0)); yc <- paste("y",y,sep="")

# univariate example
rest11 <- table( x )
rest12 <- rest11[-c(1,3)]
resm <- merge_tables( rest11, rest12 )
plot_freqdist( resm )

# bivariate example
rest2 <- table( x, y )
fd21 <- table2freqdist( rest2, omitmiss=1, index=1 )
fd22 <- fd21[-c(1,10),]
merge_tables( rest2, fd21, fd22, omitmiss=1 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UTF8pgm.R
\name{nchar2}
\alias{nchar2}
\title{Count the number of characters using bytes, not characters, as the unit.}
\usage{
nchar2(x)
}
\arguments{
\item{x}{A string}
}
\value{
A number or a numeric vector.
}
\description{
Count the number of characters using bytes, not characters, as the unit.
}
\details{
This function is equivalent to native nchar function with type="byte"
option.


About rawToChar and charToRaw functions: \cr
To get the raw expression of character, use\cr
\code{ raw <- charToRaw(paste(c(0:9," ",letters,LETTERS),collapse="")) }\cr
To get the numeric equivalent of raw, use\cr
\code{ num <- as.numeric(raw) }.\cr
To recover the original characters, use\cr
\code{ char <- rawToChar( as.raw(num) ) }.\cr
}
\examples{
# A string consisting of single and multi byte characters
# The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
{if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
 nza <- c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
else
 nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
za <- rawToChar(as.raw(nza))
# za <- num2char(nza)

# location of the multi byte characters
nchar(za)
is.DBCS(za)
printm( nchar(za), length(charToRaw(za)), nchar2(za) )
printm(unlist(strsplit(za,"")),nbyte(za),is.DBCS(za))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{complex_index}
\alias{complex_index}
\title{Find the Complexity Index of a 0-1 String
Definded as the Number of Switches from "01" to "10" to Make It Simple.}
\usage{
complex_index(pat01, rev = 0, print = 0)
}
\arguments{
\item{pat01}{A matrix consisting of 0s or 1s \cr
or a vector whose elements are strings consisting of 0s and 1s.}

\item{rev}{= 1 to count the # of switches from "10" to "01".}

\item{print}{= 1 to print the intermediate result.}
}
\value{
A vector consisting of the # of switches for each string.
}
\description{
Find the Complexity Index of a 0-1 String
Definded as the Number of Switches from "01" to "10" to Make It Simple.
}
\details{
When rev == 0, if a string contains no 1's to the left of 0's
it is assumed to simple. \cr
This function returns the # of switches from "01" to "10"
in order to make the string simple. \cr
\cr
When rev == 0, the roles of 0 and 1 will be interchanged.
\cr\cr
When the input is a matrix or data frame, complexity index of each row will
be calculated.
}
\examples{

complex_index("0101", print=1 )
pat01=matrix(c(0,1,1,0,0,1,0,1,0,0,1,1,0,0),2)
res=complex_index( pat01 )
Print(pat01,res)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/princ.R
\name{princ}
\alias{princ}
\title{Principal Component Analysis of Y with possible missing elements}
\usage{
princ(Y, ndim = 1, estmu = 1, maxiter = 1000, eps = 1e-08, init = 1, print = 1)
}
\arguments{
\item{Y}{matrix of n x p}

\item{ndim}{# of dimensions: Must be less than or equal to p.}

\item{estmu}{= 0 not to estimate mu  (No standardization of Y)}

\item{maxiter}{Maximum # of iterations for missing elements}

\item{eps}{criterion for convergence}

\item{init}{= 0 to use random number as the first estimate of missing\cr
= 1 to use the gramd mean \cr
= 2 to use the column mean}

\item{print}{= 1 to print the result}
}
\value{
A list of:\cr
Yhat=1mu'-FA', F, A, mu, iter, eps, rmse=sqrt(RSS)
}
\description{
Principal Component Analysis of Y with possible missing elements
}
\details{
This function minimizes the following RSS \cr
     RSS = tr( ( Y - 1 mu' - F A')'( Y - 1 mu' - F A') ) \cr
where mu is the p x 1 vector, \cr
F is the n x ndim orthonormal score matrix, \cr
and A is the p x ndim matrix of loadings.
}
\examples{
n <- 20; np <- 5;  pmiss <- 0.1
Y <- matrix( rnorm(n*np),n )
Y[which(matrix(runif(n*np),n,np) < pmiss)]=NA
princ(Y,2, print=10, estmu=1, init=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqdist.R
\name{freqdist}
\alias{freqdist}
\title{Frequency distribution of multivariate numeric data with case weight}
\usage{
freqdist(
  x,
  weights = 1,
  npoints = NULL,
  min = NULL,
  max = NULL,
  prob = 0,
  wid = 6,
  dig = 1,
  maxf = NULL,
  rev = 0,
  class = 0,
  omitmiss = 0,
  midpoints = NULL,
  title = NULL,
  digits = 3,
  print = 0,
  plot = 0
)
}
\arguments{
\item{x}{numeric vector or column vector}

\item{weights}{case weight}

\item{npoints}{# of mid-points for each variable}

\item{min}{minimum value of mid-points}

\item{max}{maximum value of mid-points \cr
For the multivariate x, (npoints, min, max) can be vectors \cr
specifying different numbers for each variable.}

\item{prob}{= 1 to return relative frequency in stead of frequency}

\item{wid}{Format of mid-point value (wid.dig)}

\item{dig}{Format of mid-point value (wid.dig)}

\item{maxf}{max value of frequency for plot}

\item{rev}{= 1 to change the last element of x first.}

\item{class}{= 1 to  output the classification of each observation
to the mid-point.}

\item{omitmiss}{= 1 to omit the missing grids}

\item{midpoints}{a list of the vectors of predefied mid-point values \cr
For the multivariate x, midpoints defines the midpoints of
each variable,  not their Cartesian products.}

\item{title}{tile to be used in plot}

\item{digits}{Format for mid-points}

\item{print}{= 1 to print result}

\item{plot}{= 1 to plot historgam}
}
\value{
A list consisting of \cr
  midpoints=  list of midpoints \cr
  midpoints1= list of midpoints with lost values \cr
  midpindex   cartesian product of midpoints number \cr
  grid=       cartesian product of midpoints \cr
              grid[,j]=midpoints[[j]][midpindex[,j]], j=1,2,...,nvar \cr
  freq=       column vector of frequency at midpoints \cr
  breaks=     breaks associated with the midpoints \cr
  class=      classification of each obs into the midpoints if class=1. \cr
  classc=     classification of each obs into the midpoints class
   if class=1 . \cr
  nobs= sum of weights\cr
}
\description{
Frequency distribution of multivariate numeric data with case weight
}
\details{
(npoints, min, max)  defines the mid-points. \cr
   If min or max is NULL, min(x) and max(x) will be used. \cr
   midpoints has priority over (npoints, min, max).\cr
The out of range observations,  that is, those x satisfying \cr
 x < midpoints[1]     or   midpoints[npoints] < x \cr
 will be be included in midpoints[1] and midpoints[npoints], resp. \cr
\cr
Try this to see how the obs on the border are classified. \cr
   xx=c(1,2,3,4,5,6,7) \cr
   brks=c(0,2,4,6,8) \cr
   cc=cut(xx,brks) \cr
}
\examples{
set.seed(1701)
n <- 1000
X <- matrix(rnorm(2*n),n,2)
res <- freqdist( X, npoints=7,min=-3,max=3, print=1, weights=1, plot=1)

X <- expand.grid(seq(-3,3),seq(-3,3))
weight <- round(n*exp(-0.5*rowSums(X^2)))
res1 <- freqdist( X, weight, npoints=c(7,5),min=-3,max=3, print=1, plot=1)
res2 <- freqdist( X, weight, npoints=c(7,5),min=-3,max=3, print=1, class=1)
str(res2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/midp2brk.R
\name{midp2brk}
\alias{midp2brk}
\title{Define Break Points from Midpoints}
\usage{
midp2brk(midpoints, min = NULL, max = NULL, margin = NULL)
}
\arguments{
\item{midpoints}{a vector of midpoints.}

\item{min}{value of the lowest break.}

\item{max}{value of the highest break.}

\item{margin}{value to be subtracted/added to
the lowest and highest midpoints. \cr
default margin = ( midpoints[2]-midpoints[1] )/2}
}
\value{
A vector of break points of length length(midpoints)+1.
}
\description{
Define Break Points from Midpoints
}
\details{
Break values from midpoint values: \cr
 \code{brk[1]=midp[1] - margin}\cr
 \code{brk[2]=midp[1] + (midp[2]-midp[1])/2} \cr
 \code{brk[3]=midp[2] + (midp[3]-midp[2])/2} \cr
 \code{brk[length(midpoints)+1]=midp[length(midpoints)] + margin} \cr
 where \code{margin=(midp[2]-midp[1])/2}. \cr
 In general, the break point is located at the center of two midpoints:\cr
 \code{brk[k]=0.5*(midp[k-1]+midp[k])}
 \cr

Midpoint values from break values: \cr
 \code{midp[1]=0.5*(brk[1]+brk[2])} or {midp[1]=min}\cr
 \code{midp[2]=0.5*(brk[2]+brk[3])} \cr
 \code{midp[3]=2*brk[3]-midp[2]} \cr
 \code{brk[length(brk)-1]=2*brk[length(brk)-1]-midp[length(brk)-2]} \cr
\cr
Note that midp[1] is treated separately and it is assumed that
the break point is located at the center of two midpoints.
\cr\cr
Try below to see how the obs on the border are classified. \cr
   \code{ xx <- c(1,2,3,4,5,6,7) } \cr
   \code{ brks <- c(0,2,4,6,8) } \cr
   \code{ cc1 <- cut(xx,brks) } \cr
   \code{ cc2 <- cut(xx,brks, right=FALSE) } \cr
   \code{ Print(xx,as.character(cc1),as.character(cc2)) }\cr
}
\examples{
midp <- seq(0,5,length.out=11)
brk <- midp2brk( midp, margin=1 )
Print(midp,brk)

midp <- seq(0,5,length.out=11)
brk <- midp2brk( midp, margin=1 )
Print(midp,brk)

set.seed(1701)
x <- rnorm(100)+2.5

# histogram by native hist
resh <- hist( x, breaks=brk )

# histogram by native barplot
# freq <- resh$counts
# names(freq) <- midp
# barplot( freq, space=0 )
barplot( resh$count,names.arg=midp, space=0, xlab="X", col="white"
        , sub="top and bottom bars contain outliers." )

# Below causes error because 'brk' does not span the range of 'x'.
# brk <- midp2brk( midp )
# resh <- hist( x, breaks=brk )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interpol.R
\name{interpol}
\alias{interpol}
\title{Interpolation of data}
\usage{
interpol(
  x,
  y,
  xout = NULL,
  method = "natural",
  n = 3 * length(x),
  print = 0,
  plot = 0
)
}
\arguments{
\item{x}{input vector}

\item{y}{input vector}

\item{xout}{vector at which y is calculated,  or NULL.}

\item{method}{= "constant" or "linear" or "natural" \cr
"spline" can be uses as an alias for "natural".}

\item{n}{# of points in (min(x), max(x)) at which y is calculated
when xout is NULL.}

\item{print}{= 1 to print the result}

\item{plot}{= 1 to plot the result}
}
\value{
A matrix: cbind( xout, yout )
}
\description{
Interpolation of data
}
\examples{
x <- c(1,2,3,4,5,6)
y <- c(1,4,9,7,6,8)
xout <- seq(1, 7, length=10)
sp=interpol( x, y, xout, method="constant", plot=1 )
sp=interpol( x, y, xout, method="linear", plot=1 )
sp=interpol( x, y, xout, method="spline", plot=1 )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Printb.R
\name{Printb}
\alias{Printb}
\alias{printb}
\title{Print Super Matrix ( Partitioned Matrix, Block Matrix ) with Separators.}
\usage{
Printb(smat, fmt = NULL, nrows = NULL, ncols = NULL, sep = "", debug = 0)

printb(smat, fmt = NULL, nrows = NULL, ncols = NULL, sep = "", debug = 0)
}
\arguments{
\item{smat}{input super matrix with SMdim attributes}

\item{fmt}{format specification of the form: name & width & . & dicimal}

\item{nrows}{a vector consisting of the # of rows of each sub matrix}

\item{ncols}{a vector consisting of the # of columns of each sub matrix}

\item{sep}{separator character}

\item{debug}{= 1 to debug}
}
\description{
Print Super Matrix ( Partitioned Matrix, Block Matrix ) with Separators.
}
\details{
This function prints the input partitioned matrix with separator spaces
 and returns it invisibly (via invisible(x)).  \cr
 Therefor, it can be used with a pipe operator. \cr
 For the partitioned matrices with supermatrix attributes,
 see the description of supermat, getSM, or modSM.
}
\examples{
SM <- demomat(c(2,3,4),c(1,2,4),supermat=1)
SMc <- demomat(c(2,3,4),c(1,2,4),supermat=1, root="a")

Printb( SM )
Printb( SM, sep="." )
Printb( SMc, fmt="8.0" )

attributes(SM)$SMdim=NULL
Printb( SMc, fmt="5", nrows=c(2,3,4), ncols=c(1,2,4))
Printb( SMc, fmt="5", nrows=c(3,3,3), ncols=c(2,2,2,1))

# with pipe
bigS <- demomat(c(2,3,4),c(1,2,4),supermat=1, root="a")  |>  printb()
print(bigS)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analyze_three_dots.R
\name{analyze_three_dots}
\alias{analyze_three_dots}
\title{Analyze Three Dots and Return the Names and the Values}
\usage{
analyze_three_dots(..., omit_NULL = 1, print_the_result = 0, debug_Debug = 0)
}
\arguments{
\item{...}{The three dots to be analyzed.}

\item{omit_NULL}{= 0 to include the NULL arguments.}

\item{print_the_result}{= 1 to print the result}

\item{debug_Debug}{= 1 to print intermediate info}
}
\value{
A list of: \cr
name  A vector consisting of the names of each of ... \cr
val   A list consisting of the values of each of ... or NULL \cr
name2  For the named arguments, name2 contains the righthand side of
the equal signs which appear in ... . \cr
For the positional arguments, name2 contains the actual argument itself.
}
\description{
Analyze Three Dots and Return the Names and the Values
}
\details{
... will be analyzed and the pair of ( name, val ) and name2
 will be returned.\cr
 See the first example below.

 Note that, when ... is used, partial match of formal and actual arguments
 may cause a confusing error. \cr
 See the last example below.
}
\examples{
cc="cccc"
analyze_three_dots( a=1, b=c(2,3), cc, list(aa=1,2), diag(2) )
analyze_three_dots( a=1, b=c(2,3), cc=NULL, d=0, NULL )
analyze_three_dots( a=1, b=c(2,3), cc=NULL, d=0, NULL, omit_NULL=0 )

# a function to print all of ... with names
mprint <- function( ... ){
 atd=analyze_three_dots( ... )
 for( i in 1:length(atd$name) ){
  cat(atd$name[i], ":", atd$name2[i],"\n")
  print(atd$val[[i]])
 }
}
mprint( 1:5, b=7:11 )
#'
# About partial match of the arguments
# "e" will be missing
test <- function( exp, ..., debug_Debug=0 ){
	res=analyze_three_dots( ..., debug_Debug=debug_Debug  )
	Print(res)
}
expr="aaa"
test( expr, a=1, b=2, c=3, d=4, e=5, f=6, g=7 )



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqfuncs.R
\name{plot_freqdist}
\alias{plot_freqdist}
\title{Plot Univariate Tables or Frequency Distributions of the form (score, freq)}
\usage{
plot_freqdist(
  fd,
  weight = NULL,
  main = "Frequency Distribution",
  numeric = 1,
  space = 0,
  las = 2,
  title = NULL,
  ...
)
}
\arguments{
\item{fd}{tables, matrices or data.frames containing
univariate frequency distribution (midpoint,freq).}

\item{weight}{A vector of weights associated with fd[,1] \cr}

\item{main}{A string to be used as the main title.}

\item{numeric}{= 0 to plot w/o filling in the missing value of fd[,1].}

\item{space}{an option to be passed to native barplot (default=0) \cr
the value of which specifies the width of the spaces between bars.}

\item{las}{an option to be passed to native to barplot (default=2) \cr
= 0 to rotate h, = 1 to rotate none, = 2 to rotate x, = 3 to rotate both.}

\item{title}{An alias for main.}

\item{...}{additional parameters to barplot.}
}
\value{
The object from native barplot.
}
\description{
Plot Univariate Tables or Frequency Distributions of the form (score, freq)
}
\details{
If tab is a class table object created by native table or xtabs function,
\code{plot_freqdist(tab)} and \code{plot(tab)} are similar. \cr

If weight is given, (fd, weight) or(fd[,1], weight) will be considered as
the frequency distribution table. \cr
Native barplot is used with  space=0 and las=2 and ... options as \cr
\code{barplot( height=fd[,2], names.arg=as.character(fd[,1])}
\code{, main=main, space=space, las=las, ... )}
\cr
When numeric=1 is given (default),  \code{ min(fd[,1]):max(fd[,1]) }
will be used as the horizontal names and 0 will be substituted for
the missing frequencis. \cr
}
\examples{

set.seed(1701)

score <- 0:10
freq <- c(0,5,10,20,30,40,30,20,10,5,0)
fd <- cbind(score,freq)
plot_freqdist( fd, main="no missing value" )

fd1 <- fd[-c(3,7),]
plot_freqdist( fd1, main="score=2, 6 are missing." )
plot_freqdist( fd1, numeric=0
, main="score=2, 6 are missing and numeric=0." )

tab=table(score)
tab=tab[-c(3,7)]
tab[]=freq[-c(3,7)]
plot( tab, main="using native plot function" )
plot_freqdist( tab, main="from class table object" )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{standard}
\alias{standard}
\title{Column standardization of a data frame, matrix, or vector}
\usage{
standard(X, center = 0, vardef = "n")
}
\arguments{
\item{X}{data frame, matrix, or vector \cr
If X contains non-mumeric variable, they will be left as they are.}

\item{center}{= 1 to perform column centering only.}

\item{vardef}{= "n-1" to use unbiased variance}
}
\value{
The standardized input.
}
\description{
Column standardization of a data frame, matrix, or vector
}
\details{
When X is a data frame or matrix, the colums of which
 will be standardized. \cr
 NaN will be returned for those variables with 0 variance.
}
\examples{
standard( data.frame(id=paste("id",1:5,sep=""), diag(5), a="a") )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{lv}
\alias{lv}
\title{Retruns the Last Value}
\usage{
lv(...)
}
\arguments{
\item{...}{Not Used}
}
\description{
Retruns the Last Value
}
\examples{
\dontrun{
set.seed(1701)
rnorm(10)
lv()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/named_list.R
\name{named_list}
\alias{named_list}
\title{Create a named list}
\usage{
named_list(...)
}
\arguments{
\item{...}{elements of the list to be created}
}
\value{
A list.
}
\description{
Create a named list
}
\details{
When ... contains objects with valid variable names,
those names will be used as the names of the objcts in the list.\cr
}
\examples{
a=1; b=1:2
named_list( a, a1=a, b, 3:4, d=5:6, "E" )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fxtabs.R
\name{fxtabs}
\alias{fxtabs}
\title{Perfectly Flat Cross Table by native xtabs}
\usage{
fxtabs(
  formula = ~.,
  data = parent.frame(),
  subset,
  sparse = FALSE,
  na.action,
  na.rm = FALSE,
  addNA = FALSE,
  exclude = if (!addNA) c(NA, NaN),
  drop.unused.levels = FALSE,
  remove_zeros = 0,
  print = 0,
  debug = 0
)
}
\arguments{
\item{formula}{a formula object of the form lhs~rhs}

\item{data}{an optional data frame containing the variables in
the formula formula.}

\item{subset}{xtabs parameter}

\item{sparse}{xtabs parameter}

\item{na.action}{xtabs parameter}

\item{na.rm}{xtabs parameter}

\item{addNA}{xtabs parameter}

\item{exclude}{xtabs parameter}

\item{drop.unused.levels}{xtabs parameter}

\item{remove_zeros}{= 1 to remove zero frequency cells}

\item{print}{= 1 to print the result}

\item{debug}{= 1 to print the intermediate results}
}
\value{
a matrix of flat contingency table.
}
\description{
This function makes the result of xtabs perfectly flat.
}
\details{
\code{formula} and \code{data} parameters function the same way
as native xtab function. \cr
In addtion, those parameters marked as "xtabs parameter",
see \link[stats:xtabs]{stats::xtabs}.

The core part of thus function is as follows:
\preformatted{
 res <- ftable( xtabs( lhs~rhs,  data=data ) )
}
The above returns a matrix when \code{lhs} is a matrix.
However, as shown in the examples,
when \code{lhs} is a vector or missing, the result is a cross table. \cr
Note that, by adjusting \code{row.vars} and \code{col.vars} options of
 \code{ftable}, it is possible to produce a completely flat table.
}
\examples{

seed=1701
set.seed(seed)
n <- 50
x1 <- sample(1:3,n,replace=TRUE)
x2 <- sample(LETTERS[1:2],n,replace=TRUE)
x3 <- sample(letters[1:2],n,replace=TRUE)
c1 <- sample(c(0:3),n,replace=TRUE)
c2 <- sample(c(0:5),n,replace=TRUE)
data=data.frame(x1,x2,c1,c2)

# 3 by variables
fxtabs( cbind(c1,c2)~x1+x2+x3, data=data)
ftable(xtabs( cbind(c1,c2)~x1+x2+x3, data=data))
fxtabs( c1~x1+x2+x3, data=data)
ftable(xtabs( c1~x1+x2+x3, data=data))
fxtabs( ~x1+x2+x3, data=data)
ftable(xtabs( ~x1+x2+x3, data=data))

# 2 by variables
fxtabs( cbind(c1,c2)~x1+x2, data=data)
ftable(xtabs( cbind(c1,c2)~x1+x2, data=data))
fxtabs( c1~x1+x2, data=data)
ftable(xtabs( c1~x1+x2, data=data))
fxtabs( ~x1+x2, data=data)
ftable(xtabs( ~x1+x2, data=data))

# single by variable
fxtabs( cbind(c1,c2)~x1, data=data)
ftable(xtabs( cbind(c1,c2)~x1, data=data))
fxtabs( c1~x1, data=data)
ftable(xtabs( c1~x1, data=data))
fxtabs( ~x1, data=data)
ftable(xtabs( ~x1, data=data))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmv.R
\name{rmv}
\alias{rmv}
\title{Remove Objects If They Exists in the Parent Frame}
\usage{
rmv(...)
}
\arguments{
\item{...}{The objects to be removed, as names (unquoted)}
}
\description{
Remove Objects If They Exists in the Parent Frame
}
\examples{
\dontrun{
a=1; b=1
rm( a, b, c )
rmv( a, b, c )
print(a)
print(b)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.number.R
\name{is.number}
\alias{is.number}
\title{Check if a string vector contains numbers only.}
\usage{
is.number(arg, pat = 1)
}
\arguments{
\item{arg}{a vector or a list}

\item{pat}{= 1 to match non-negative integers \cr
= 2 to match signed real numbers \cr
= 3 to match integers}
}
\value{
If arg is a vector or matrix, a scalar TRUE or FALSE. \cr
If arg is a list, a vector of the same length as the arg consisting of
TRUEs or FALSEs.
}
\description{
Check if a string vector contains numbers only.
}
\details{
The leading and trailing blank characters will be ignored.
}
\examples{
vec=c("a","1","1.4","b.","-b","-2","2-"," 12 ","5 5","0.4",".4","-0.","-0.8")
is.number(vec)
printm(vec,is.number(as.list(vec)),is.number(as.list(vec), 2))
is.number( list( c("1","2","a"), c(1,2) ) )
is.number( list( 2, -2,"-2","-2." ), 3 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqfuncs.R
\name{table2freqdist}
\alias{table2freqdist}
\title{Convert an output of table or xtabs to a simple frequency table format}
\usage{
table2freqdist(tab, omitmiss = 0, index = 1, pat = 2, sort = 1)
}
\arguments{
\item{tab}{An output of native table or xtabs function.}

\item{omitmiss}{= 1 to exclude the zero-freq cells.}

\item{index}{= 0 to output no level indices}

\item{pat}{argument to is.number function.}

\item{sort}{= 0 not to sort the result}
}
\value{
When index == 1 and there are nf factors(variables) to be tabulated, \cr
the output is a matrix or data.frame: \cr
the first nf columns of which consist of the level indices,
and the nf+1st column contains the frequency count. \cr

If the levels are numeric, a matrix will be returned. Otherwise,
a data.frame. \cr
In both cases
  \code{rownames(fd)} contains the combinations of the level names.
}
\description{
Convert an output of table or xtabs to a simple frequency table format
}
\examples{
set.seed(1701)
x <- sample(1:8,10,replace=1); x <- c(x,c(1,1,7)); xc <- paste("x",x,sep="")
y <- sample(0:9,10,replace=1); y <- c(y,c(5,5,0)); yc <- paste("y",y,sep="")
z <- sample(5:9,10,replace=1); z <- c(z,c(5,5,9)); zc <- paste("z",z,sep="")

rest1 <- table( x )
rest2 <- table( x, y )
rest3 <- table( xc,y,z )
restx3 <- xtabs( ~xc+yc+z )

table2freqdist( rest1 )
table2freqdist( rest2 )
table2freqdist( rest3, omitmiss=1 )
table2freqdist( restx3, omitmiss=1, index=0 )

tab2 <- table( x=sample( letters[1:5], 20, replace=1)
             , sample(letters[6:8], 20, replace=1) )
table2freqdist( tab2 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/varlist.R
\name{varlist}
\alias{varlist}
\title{Interprete sas-like abbreviated variable list}
\usage{
varlist(var = NULL, varlistf = NULL, location = 0, regexpr = NULL)
}
\arguments{
\item{var}{string containing sas-like abbreviated variable list such as\cr
     "q5-q7  x1  height--gender"\cr
Comma can be a denominator.}

\item{varlistf}{full list of variables on the basis of which double - is
interpreted. \cr
Use varlistf=colnames(data)\cr
This can be NULL when double - is not used.}

\item{location}{= 1 to output the locatin of var in varlistf \cr
If location == 1, varlistf must be given.}

\item{regexpr}{a string containing a regular expression to be used
to select the variables. \cr
This has priority over var.}
}
\value{
vector or list
}
\description{
Interprete sas-like abbreviated variable list
}
\details{
varlist( "q2-q5 a c--d", varlistf=c("a", "bb","c","d", "e", "gender") ) \cr
will return c( "q2", "q3", "q4", "q5", "a",  "c",  "d" )\cr
  When location == 1, in addition to the vector above, \cr
   the location of the variable names contained in varlistf, if any,
   will be output as the 2nd element of the list, which is\cr
   c(1,3,4)
   \cr
  Expression such as abc: can be used to designate all the variables
  whose names start with abc.
}
\examples{
varlist( "x3-x1, y5, x8-x9" )
varlistf=c("a", "b","c","d", "e", paste("Q", 1:5, sep=""), "fff" )
varlist( "q2-q5 a e--c", varlistf=varlistf )
varlist( "q2-q5 a e--c Q:", varlistf=varlistf )
varlist( regexpr="^Q[123]", varlistf=varlistf )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqfuncs.R
\name{compress_freqdist}
\alias{compress_freqdist}
\title{Compress Frequency Distribution by Aggregating Duplicate Entries}
\usage{
compress_freqdist(x, freq, fill = 0, domain = NULL)
}
\arguments{
\item{x}{Vector of values (domain of a scalar variable, numeric or string)}

\item{freq}{Vector of frecuencies:
\code{(x, freq)} is the frequency distribution.}

\item{fill}{= 1 to fill the domain}

\item{domain}{Domain of \code{x} to be filled. (coalescing)}
}
\value{
A data frame consisting of \cr
\code{x} Vector of unique values \cr
\code{freq} Vector of frequencies
}
\description{
Compress Frequency Distribution by Aggregating Duplicate Entries
}
\details{
The purpose of this function is to remove the duplicate values of \code{x}
and adjusting the associated frequency count in \code{freq}.

The core part of this function is:
\preformatted{
 fnew <- xtabs( freq ~ x )
 xnew <- names(fnew)
 res <- data.frame(x=xnew, freq=fnew)
}
and if \code{domain} and \code{fill} are given, the result will be
coalesced. (coalescing)  
Using the frequency table below
\preformatted{
 x  freq
 1  1
 2  2
 2  1
 3  1
}
\preformatted{
  compress_freqdist( x, freq, fill=1, domain=0:4)
}
returns
\preformatted{
 x  freq
 0  0
 1  1
 2  3
 3  1
 4  0
}
}
\examples{
x <- c(1,2,4,5,6, 2,4,6, 9 )
f <- c(1,2,2,1,2, 1,2,1, 1 )

( res <- compress_freqdist( x, f ) )
( res <- compress_freqdist( x, f, fill=1, domain=0:10 ) )


xc <- letters[c(1,2,4,5,6, 2,4,6, 9 )]
f <- c(1,2,2,1,2, 1,2,1, 1 )
( res <- compress_freqdist( xc, f ) )
( res <- compress_freqdist( xc, f, fill=1, domain=letters[0:10] ) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeStack.R
\name{makeStack}
\alias{makeStack}
\title{Preparation of Stack}
\usage{
makeStack()
}
\value{
A list of functions \cr
  \code{push, pop,  inspect,  length} and \code{clear}
}
\description{
Preparation of Stack
}
\details{
This is a function  factory which makes  the following list of functions.\cr
  \code{push, pop,  inspect,  length} and \code{clear}

In the usual image of a stack, the newest object will be placed
on top of the older objects.
However, in this R version, the newest object will be placed
at the end of a hidden list named \code{valueList} in the function factory
environment.

It enables LIFO (Last-In First-Out) or  FILO (First-In Last-Out).
}
\examples{
 # generate functions
 stack <- makeStack()

 stack$push(3)
 stack$push(5)
 stack$push(9)
 stack$length()
 stack$inspect(1:stack$length())
 stack$pop()
 stack$inspect(1:stack$length())
 stack$pop()
 stack$inspect(1:stack$length())
 stack$pop()

 # Below causes error.
 # stack$inspect(1:stack$length())


 STACK=makeStack()
 PUSH <- STACK$push
 POP <- STACK$pop
 LEN <- STACK$length
 INSP <- STACK$inspect
 CLEAR <- STACK$clear
 PUSH( list(a=1,b=2,c="C"))
 PUSH( list(a=12,b=22,c="C2"))
 PUSH( list(a=13,b=23,c="C3"))
 INSP(1:LEN())
 POP()
 INSP(1:LEN())
 POP()
 INSP(1:LEN())


}
\references{
The original version is from the answer by Matt: \cr
  https://stackoverflow.com/questions/28687806
  /a-better-way-to-push-and-pop-to-from-lists-in-r
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distance.R
\name{distancew}
\alias{distancew}
\title{Calculation of Weighted Distance Matrices}
\usage{
distancew(X, W = matrix(1, 1, ncol(X)), array = 1)
}
\arguments{
\item{X}{n x r matrix}

\item{W}{nG x r maatrix}

\item{array}{= 0 to make the result a list}
}
\value{
An array or a list of nG matrices of size n x n
}
\description{
Calculation of Weighted Distance Matrices
}
\details{
This function calculates the weighted distance among the rows of
\code{X} matrix  by applying the k-th row of \code{W} matrix
as a set of dimensional weights as:
\deqn{ d_{ijk} = \sqrt( \sum_{a=1}^r   w[k,a] (x[i,a]-x[j,a])^2 ) }
or
\deqn{ d_{ijk} = \sqrt( (x[i,]-x[j,])' diag(W[k,]) (x[i,]-x[j,]) ) }
\cr
Note that \code{W} matrix will NOT be squared when calculating distance.
\cr
}
\examples{
X <- matrix( c(0,0,  3,0,  0,4), 3,2,  byrow=1 )
distancew( X )
W <- matrix( c(2,1, 1,2, 1,1), 3,2, byrow=1 )
distancew( X, W )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/named_list.R
\name{from_named_list}
\alias{from_named_list}
\title{Retrieve Objects from a Named List}
\usage{
from_named_list(nl)
}
\arguments{
\item{nl}{A named list}
}
\description{
Retrieve Objects from a Named List
}
\details{
This function defines or replaces the objects in the parent frame. \cr

mapply( assign, names(nl), nl, MoreArgs=list(envir=parent.frame()) )
}
\examples{

# in global env.:
a=1; a1=2; b=1:2
nl=named_list( a, a1=a+3, b, 3:4, d=5:6, "E" )
rmv(a,b,temp_var4,d,E)
from_named_list( nl )
Print(a,a1,b,temp_var4,d,E)

# in a function:
f <- function(){
 nl=list(a=1,b=2)
 from_named_list( nl )
 Print("in function f:",a,b)
}
f()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reorder_fa.R
\name{reorder_fa}
\alias{reorder_fa}
\title{Reorder the rows and columns of factor pattern matrix}
\usage{
reorder_fa(A, unclass = 0, sortcols = 1, print = 0)
}
\arguments{
\item{A}{Factor Pattern matrix or an object with $loadings.}

\item{unclass}{= 1 to unclass the result}

\item{sortcols}{= 0 not to sort factors (columns of A) according to
factor contributions.}

\item{print}{= 1 to print the result}
}
\value{
A list of \cr
loadings = reordered loadings matrix \cr
row_order = sort order of the rows \cr
col_order = sort order of the cols
}
\description{
Reorder the rows and columns of factor pattern matrix
}
\details{
First, if sortcols == 1, the columns of \code{A} are reordered
according to the magnitude of \code{diag(t(A)\%*\%A)}. \cr
Then, for each row, the column number of the maximum elements of
\code{abs(A)}
is recorded (as \code{max.col(abs(A))}) and the rows of \code{A}
are sorted according
to these column numbers. \cr
Finally, within each max.col group, the rows are again sorted
according to the magnitude of \code{A}.
}
\examples{
set.seed(1701)
n <- 20; ndim <- 4
A0 <- matrix(runif(n*ndim),n,ndim)
dimnames(A0)=list( paste("v",1:n,sep=""),paste("f",1:ndim,sep=""))
A0v <- varimax(A0)$loadings
A0v <- A0v*matrix(sample(c(-1,1,1),n*ndim,replace=TRUE),n,ndim)
A0v[,3:4] <- -A0v[,3:4]

resrc <- reorder_fa(A0v)
A0vrc <- resrc$loadings
Print(A0v,A0vrc, fmt="6.3", fuzz=0.7)

resr <- reorder_fa(A0v, sortcols=0)
A0vr <- resr$loadings
Print(A0v,A0vr, fmt="6.3", fuzz=0.7)

# reordered matrix from original
dif1 <- A0vrc-A0v[resrc$row_order,resrc$col_order]

# original from reordered matrix
A0v2 <- A0vrc; A0v2[resrc$row_order,resrc$col_order] <- A0vrc
rownames(A0v2)[resrc$row_order] <- rownames(A0vrc)
Print(A0v,A0v2, fmt="6.3")
dif2 <- A0v2-A0v
Print(dif1, dif2)

# reducing the number of negative signs
A0vrcs <- reflect_fa( A0vrc )$loadings
Print(A0vrc, A0vrcs, fmt="6.3")

# attributes
print(reorder_fa(A0v)$loadings)
print(reorder_fa(A0v, unclass=1)$loadings)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{fuzz}
\alias{fuzz}
\title{Print an Object with Print function with fuzz= option}
\usage{
fuzz(obj, fuzz = 1e-04, ntogoback = 1)
}
\arguments{
\item{obj}{An object to be printed}

\item{fuzz}{fuzz parameter of Print}

\item{ntogoback}{n parameter of sys.parent function}
}
\description{
Print an Object with Print function with fuzz= option
}
\details{
When used with postfix function application operator of
\strong{lazy.symbolic} package as
\code{ lv() \%//\% fuzz }, \cr
.fuzz. global variable will be referenced instead of fuzz parameter.
}
\examples{
Ap <- c(1e-2,1e-5,1e-7)
Ap
fuzz(Ap)
# Above is equivalent to: Ap \%//\% fuzz
.fuzz. <- 1e-6
fuzz(Ap)
#Ap \%//\% fuzz

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_valL.R
\name{get_valL}
\alias{get_valL}
\alias{put_valL<-}
\title{Get Values from a List Consisting of Data Frames}
\usage{
get_valL(L, s1 = NULL, s2 = NULL, s3 = NULL, simplify = 1)

put_valL(L, s1 = NULL, s2 = NULL, s3 = NULL) <- value
}
\arguments{
\item{L}{A list of data frames}

\item{s1}{Vector consisting of the subscripts for the 1st dimension}

\item{s2}{Vector consisting of the subscripts for the 2nd dimension}

\item{s3}{Vector consisting of the subscripts for the 3rd dimension}

\item{simplify}{= 0 to leave the result of get_valL unsimplified (list)}

\item{value}{The value to be assigned. Either a vector or a matrix.}
}
\value{
\code{get_valL(L, s1,s2,s3)} returns
 \code{(s1,s2)} element(s) of the s3-th data frame(s). \cr
 \code{put_valL(L, s1,s2,s3) <- value} replaces the \code{s1,s2}
 element(s) of the data frame(s) specified by \code{s3}
 with \code{value}.
}
\description{
Get Values from a List Consisting of Data Frames
}
\details{
It is assumed that \code{L} is a list consisting of several data frames.
and the third subscript, \code{s3} is for the list.\cr
Therefore, \code{get_valL(L,,,s3)} with scalar s3 is
equivalent to \code{L[[s3]]} \cr
and  \code{get_valL(L,,,2:3)} returns the 2nd and the 3rd data frames
in a list. \cr

\code{s1,s2,s3} can be a set of vectors consisting of row/col/list names.

\cr
When some of the subscripts are out of range, only the valid ones
will be used.

\cr
The result will be simplified to a matrix or vector if possible. \cr
When \code{length(s1)==1}, the result will be
a \code{length(L)} x \code{ncol(L[[1]])} matrix. \cr
When \code{length(s2)==1}, the result will be
a \code{ncol(L[[1]])} x \code{length(L)} matrix. \cr
}
\examples{
L0 <- list( A=data.frame(demomat(3,2)*10+1)
          , B=data.frame(demomat(4,3)*10+2)
          , C=data.frame(demomat(5,4)*10+3)
         )
L <- L0
get_valL( L, ,,2 ) # should be equal to get_valL( L, ,,"B" )
get_valL( L, 2, )  # should be equal to get_valL( L, "r2",, )
get_valL( L, ,2 )  # should be equal to get_valL( L, , "c2", )
get_valL( L, c(1,3),,3:2 )
get_valL( L, c(1,3),, c("B","A") )

L <- L0; put_valL(L, 1:2,1:2,3) <- 0;                  Print(L)
L <- L0; put_valL(L, 1:2,1:2,c("C","A")) <- 0;         Print(L)
L <- L0; put_valL(L, 3:5,2:4,) <- 0;                   Print(L)
L <- L0; put_valL(L, 3:5,2:4,2) <- 0;                  Print(L)
L <- L0; put_valL(L, 1:4,1,2:3) <- 1:4;                Print(L)
L <- L0; put_valL(L, 1:2,2:3,2:3) <- demomat(2,2)*100; Print(L)

# effect of simplify
L0 <- list( A=data.frame(demomat(3,4)*10+1)
          , B=data.frame(demomat(3,4)*10+2)
          , C=data.frame(demomat(3,4)*10+3)
         )
L <- L0
get_valL( L, ,,2 )
get_valL( L, ,,2, simplify=0 )
get_valL( L, 2, )
get_valL( L, 2,, simplify=0 )
get_valL( L, ,2 )
get_valL( L, ,2, simplify=0 )

# When the selected elements are scalars, the result is a column vector.
get_valL(L0,1:2,1) # matrix
get_valL(L0,1,1)   # col vector
get_valL(L0,1,1:2) # matrix

# mixed type variables in a data frame
A <- data.frame(demomat(3,4)*10+1)
A[,2] <- paste("c",as.character(A[,2]),sep="")
B <- data.frame(demomat(3,4)*10+2)
B[,2] <- paste("c",as.character(B[,2]),sep="")
C <- data.frame(demomat(3,4)*10+3)
C[,2] <- paste("c",as.character(C[,2]),sep="")
L <- list(A=A,B=B,C=C)

get_valL( L, 2, )
get_valL( L, , 2 )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/idplot.R
\name{idplot}
\alias{idplot}
\title{plot X[,1:2] with id}
\usage{
idplot(X, id = NULL, type = "n", asp = 1, pos = NULL, cex = 1, col = NULL, ...)
}
\arguments{
\item{X}{matrix whose rows will be plotted}

\item{id}{id of the rows of X}

\item{type}{type of the plot}

\item{asp}{aspect ratio}

\item{pos}{position of the text}

\item{cex}{size of the text}

\item{col}{color of the text}

\item{...}{additional graphical paramaters to text function}
}
\value{
none
}
\description{
plot X[,1:2] with id
}
\details{
Equivalent to:
\preformatted{
plot( X[,1:2], type="n", asp=1 )
text( X[,1],X[,2], labels=id, ... )
}
}
\examples{
 X=matrix(c(
-1,  1,  0,  1,  1,    1, -1,   0, 0,  0, 1, 0, -1, -1, 0, -1, 1, -1
, -.5,  .5, .5,  .5, -.5,  -.5, .5,  -.5, -2,  0, 2,  0
),,2,byrow=1)
idplot(X, id=paste("s",1:nrow(X),sep=""))
idplot(X, id=paste("s",1:nrow(X),sep=""), pos=1)
idplot(X, id=paste("s",1:nrow(X),sep=""), pos=2, cex=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_df.R
\name{merge_df}
\alias{merge_df}
\title{Horizontally Stack Data Frames with Different # of Observations}
\usage{
merge_df(..., simplify = 1)
}
\arguments{
\item{...}{vectors, matrices or data frames}

\item{simplify}{= 0 not to convert the result to a matrix if possible.}
}
\value{
A data frame or matrix
}
\description{
Horizontally Stack Data Frames with Different # of Observations
}
\details{
Using the data frames in the example section, the core part is:\cr
\code{ res <- merge( data.frame(A1m,id=1:nrow(df1m))
               , data.frame(A2m,id=(1:nrow(df2m))), by="id" ,all=1) }\cr

\cr
If simplify=1 is give, if possible, the result will be converted to
a matrix.
}
\examples{
# no overlapping col names
merge_df( x=1:2, y=1:5, z=1 )

# overlapping col names
A <- demomat(5,4,3,cnroot="V")
A1 <- A[,,1]; A2 <- A[,,2]; A3 <- A[,,3]
A1m <- A1[1:3,1,drop=0]; A2m <- A2[1:2,1:2]; A3m <- A3[1:3,1:3];
merge_df( A1m, A2m, A3m )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AndOr.R
\name{And}
\alias{And}
\alias{Or}
\alias{Not}
\alias{\%&\%}
\alias{\%|\%}
\alias{\%&&\%}
\alias{\%||\%}
\title{Combine string arguments by a logical infix operator}
\usage{
And(..., oper = " & ")

Or(..., oper = " | ")

Not(exp, oper = " | ")

A \%&\% B

A \%|\% B

A \%&&\% B

A \%||\% B
}
\arguments{
\item{...}{strings to be combined.}

\item{oper}{= "&" or "|" or  "&&" or "||" \cr
And uses "&" as the default, Or uses "|" as the default.}

\item{exp}{a string to be negated by Not.}

\item{A}{The first string argument}

\item{B}{The second string argument}
}
\value{
a string
}
\description{
Combine string arguments by a logical infix operator
}
\details{
\code{And( "a", "b" )} returns \code{"a & b"} \cr
\code{Or( "a", "b" )} returns \code{"a | b"} \cr
\code{Not( "exp" )} returns \code{"!exp"} \cr
\cr
\code{And( "a", "b" )} is equivalent to \code{ "a" \%&\% "b" } \cr
\code{And( "a", "b", oper="&&" )} is equivalent to \code{"a" \%&&\% "b"} \cr
\code{Or( "a", "b" )} is equivalent to \code{ "a" \%|\% "b" } \cr
\code{Or( "a", "b", oper="||" )} is equivalent to \code{ "a" \%||\% "b"} \cr
\cr
Use \code{eval( parse(text=cond) )} to evaluate the expression.
}
\examples{
And( "a==b", "(c==d)" )
"a==b" \%&\% "(c==d)"
Or( "a==b", "(c==d)" )
"a==b" \%||\% "(c==d)"
Or( Not("a"), Not("(b)") )
Not( And("a==b", "(c==d)") )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{vartype}
\alias{vartype}
\title{Find the mode/typeof of the columns of data}
\usage{
vartype(data, typeof = 0)
}
\arguments{
\item{data}{a data frame to be checked.}

\item{typeof}{= 1 to return typeof(var) instead of mode(var)}
}
\value{
a vector consisting of typeof or mode.
}
\description{
Find the mode/typeof of the columns of data
}
\details{
mode   -> "numeric", "character", etc \cr
   typeof -> "integer", "double", "character", etc \cr\cr
When data contains factors, "factor" will be returned for those varaibles.
}
\examples{
 df=data.frame(a=1,b="a",c=as.factor(1), stringsAsFactors=0)
 vartype(df)
 vartype(df, typeof=1)
 ls=list(a=1:2,b=2:4,c=c("a","b"),A=diag(2))
 vartype(ls)
 vartype(ls, typeof=1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Print_all.R
\name{Print}
\alias{Print}
\title{Print arguments with their names and formats}
\usage{
Print(
  ...,
  fmt = NULL,
  NAchar = ".",
  fuzz = -1,
  fuzzchar = "..",
  sep = ",",
  removeatt = 0,
  printrowname = 0,
  printrownames = 0,
  debug = 0
)
}
\arguments{
\item{...}{objects to be printed with name.}

\item{fmt}{A vector of formatC to be used}

\item{NAchar}{a character to represent NA.}

\item{fuzz}{Those elements which satisfy abs(X) < fuzz will be printed.}

\item{fuzzchar}{Replacement character(s) for those abs(X) < fuzz.}

\item{sep}{character to be used as the delimiter.}

\item{removeatt}{= 1 to remove the attributes from each of ... .}

\item{printrowname}{= 0 not to print rownames when printing vector.}

\item{printrownames}{Same as printrowname.}

\item{debug}{= 1 to dump intermediate result.}
}
\value{
The first element of ... invisibly.
}
\description{
Print arguments with their names and formats
}
\details{
If ... contains lists or data.frames, they will be printed separetely.#'
\cr
  Except for the case when ... consists of a single vector,
  vectors will be printed as column vectors without row/col names. \cr
  To add rownames to vectors, specify printrowname=1. \cr
  "/" or "\\n" can be used as the line break. \cr
\cr
  If any of the input matrices does not have row/col names
  they will be generated using "r" and "c" as the roots.
\cr
  If the first argument is a number or a quoted string, it will be printed
  as it is (without name.) \cr
\cr
  When there are only two objects to be printed,
  and the first one is a constant and the second one is not a matrix,
  there will be no line breaks  between the first and the second objects
  unless the first one has "\\n". \cr
   compare:  \cr
\preformatted{
    Print("first", pi)
    Print("first\n", pi )
    Print("first", pi, pi )
}
\cr

fmt  contains formats to be used separated by commas or blanks: \cr
 \code{fmt="FNameWid.Digits"} will use the following formatC function \cr
 \code{formatC( object, format="FName", width=Wid, digits=Digits )} \cr
 to format the object. \cr
 If there are more objects in ... than length(fmt),
 the last fmt value will be used to fill.
 \cr

When fmt is given, the attributes of the object may be printed together.\cr
Use removeatt=1 to remove all the attributes except dimnames and dim.


The first element of ... will be returned invisibly so that
this function can be used with pipe operator. \cr
Use \code{capture.output} function, not the return value,
to utilize the printed output of this function.
}
\examples{
bool <- TRUE; num <- 1; cpl <- 1+1i; str <- "Hello."
vec <- 1:10; mat <- matrix(c(1, 2, 3), 3, 2)
lis <- list(a=1:3, b="abc",NA); df <- data.frame(a=1:2,b=2:3,c="cc")
Print(bool, num, cpl, str, lis, vec, df, mat, plot, sin(0))

A=demomat(3,2)
Print("This is what I need.", A)
Print(A/22)
Print(A/22, A/22, fmt="6.2, 8.4")
Print(A/22, fuzz=0.6)

Print(diag(5),1:3)
Print(diag(5), diag(2))
Print(diag(5), "/", diag(5))
char_vec <- "This is a character vector."
Print("string", char_vec, t(1:2))

B <- demomat(3,3,2,supermat=2)
Print(B,fmt="6.3")
Print(B,fmt="6.3", removeatt=1)

# with pipe
C <- demomat(2,2, root="c")  |> Print(A)
print(C)
C <- demomat(2,2, root="c")  |> Print(A, .=_)
print(C)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen01pat.R
\name{gen01pat}
\alias{gen01pat}
\title{Generate all possible 0-1 patterns of given length,
or binary partition index matrix}
\usage{
gen01pat(nitems, sort = 0, partition = 0, print = 0)
}
\arguments{
\item{nitems}{The length of the pattern}

\item{sort}{=  1 to sort the result from the left \cr
= -1 to sort the result from the right \cr
=  2 to sort the result by rowSums and from the left \cr
= -2 to sort the result by rowSums and from the right \cr}

\item{partition}{= 1 to generate binary partition index matrix}

\item{print}{= 1 to print the result}
}
\value{
\code{2^nitems x nitems} matrix \\
or when \code{partition=1} is given \\
\code{2^{nitems-1}-1 x nitems} matrix
}
\description{
Generate all possible 0-1 patterns of given length,
or binary partition index matrix
}
\details{
Schatzoff et al method is used. \cr
nitems must be less than log(.Machine$integer.max)/log(2). \cr
\cr
Essentially the same  as \cr
  cprod( rep(list(0:1),nitems) ) \cr
but gen01pat is much faster.
}
\examples{
gen01pat( 4 )
gen01pat( 4, partition=1, sort=-2 )

pindex=gen01pat( 4, partition=1 )
left=apply(pindex,1,function(x) letters[1:4][as.logical(x)])
right=lapply(left,function(x){setdiff(letters[1:4],x)})
res=mapply( function(x,y){list(left=x,right=y)}, left,right, SIMPLIFY=FALSE)
Print(res)


}
\references{
Schatzoff, M., Tsao, R.  and Fienberg, S. (1968)
Efficient Calculation of All Possible Regressions.  Technometrics.
Vol. 10, No. 4, pp. 769-779
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Grep.R
\name{Grep}
\alias{Grep}
\title{Use Friendly Grep/Gsub}
\usage{
Grep(
  expr,
  rule = NULL,
  ...,
  rep = 1,
  value = 0,
  out_pattern = 0,
  split = "?",
  fixed = 0,
  debug = 0
)
}
\arguments{
\item{expr}{expression to be modified.}

\item{rule}{replacement rule to be applied in terms of pattern names.}

\item{...}{patterns of the form: name=pattern w/o enclosing parens. \cr
No positional arguments are allowed.}

\item{rep}{# of times that the pattern and the rule apply.}

\item{value}{= 1 to print the first matches for each of \code{...}.}

\item{out_pattern}{= 1 to output pattern and replacement rule
to be used in \code{gsub}.}

\item{fixed}{fixed parameter to \code{gsub}.}

\item{debug}{= 1 to print useful info}

\item{sep}{A character which separates each element of the rule.}
}
\value{
The result of \code{gsub}, or when out_pattern=1, the pattern and the rule
 to be used in \code{gsub} in a character vector of length 2.
}
\description{
Use Friendly Grep/Gsub
}
\details{
When \code{value=1} this function interprets \code{...} and
 print the first match to each element of \code{...}.

 When \code{rule} is given,
 this function interprets \code{...} and \code{rule} and generates
 the pattern and the replacement for \code{gsub} function.

 You can use this function in your program code to perform the replacement,
 or, you can use this to generate the pattern and the replacement for
 \code{gsub} function to be used later with \code{out_pattern=1} option.

 The names in ... can be anything as long as they are valid variable names
 except for  \code{rep,value,out_pattern, split, fixed, debug}.
 \code{name=".v"} and \code{name=".n"} can be used for a valid variable name
 and a number, resp.. \cr
 All of the named arguments in ...  will be concatenated in the order
 of appearance and will be used as the pattern to match.

 \code{rule} can contain any string separated by \code{sep}.
 If the string matches one of \code{name} elements, the string
 will be replaced by the pattern assocated with that name.#'

 In the original \code{gsub} function,
 the values in ... will be denoted as 1, 2, ... and
 these numbers must be used in the rule parameter as \\\\1, \\\\2, ... .

 The following pattern names cannot be used \cr
 \code{ expr, rule, out_patten, sep, debug}
}
\examples{
# The third example below generates the following grep/gsub patterns:
# gsub(
# "(^|[-+\\*`~])([[:alpha:].][[:alnum:]_.]*)(\\*)([0-9]*\\.{0,1}[0-9]*)
# ($|([-+*/`~].+))",   "\\1\\4\\3\\2\\5",   "a123+a*17.01-b*34+9" )
#

# varname*num -> num*varname
top <- "^|[-+\\\\*`~]"; varname <- "[[:alpha:].][[:alnum:]_.]*"; times <- "\\\\*"
num <- "[0-9]*\\\\.{0,1}[0-9]*"; tail <- "$|([-+*/`~].+)"
expr <- "a123+a*17.01-b*34+9"

Grep( expr, value=1
          , top=top, varname=varname, times=times, number=num, tail=tail )
Grep( expr, value=1
          , top=top, varname=".v", times=times, number=".n", tail=tail )
( temp <- Grep( expr, rule="top?num?times?varname?tail", out_pattern=1
          , top=top, varname=".v", times=times, num=".n", tail=tail ) )
Grep( temp, rule="top?num?times?varname?tail"
          , top=top, varname=".v", times=times, num=".n", tail=tail )
Grep( expr, rule="top?(?num?)?times?varname?tail", rep=2
          , top=top, varname==".v", times=times, num=".n", tail=tail )

# transposition and trace of a scalar
expr <- "+t(09)-tr(123)"
Grep( expr, rule="top?sign?digits?tail", debug=1
              , top="^|[-+*`~]"
              , tparen="t\\\\(|tr\\\\("
              , sign="[-+]*"
              , digits="[0-9]+"
              , paren="\\\\)"
              , tail="$|[-+*/`~]", rep=2 )


# Cancel x/x etc with back reference
top <- "^|[-/+*=`~]"; varnam <- "[[:alpha:].]+[[:alnum:]_]*|[1-9]+[0-9]*"
div <- "/\\\\2"; tail <- "($|[-/+*=`~])"
Grep( "17*x12e/x12e*100+a/a+2/2", rule="top?1?tail"
    , top=top, varnam=varnam, div=div, tail=tail )
Grep( "17*x12e/x12e*100+a/a+2/2", rule="top?1?tail"
    , top=top, varnam=varnam, div=div, tail=tail, rep=2 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_calling_functions.R
\name{find_calling_functions}
\alias{find_calling_functions}
\title{Find Calling Functions}
\usage{
find_calling_functions(
  func = NULL,
  resff = NULL,
  from = NULL,
  nonative = 1,
  noglobal = 0,
  packages = NULL,
  print = 1
)
}
\arguments{
\item{func}{A function  object (no quote) or
a list/vector of the function object names to be searched.}

\item{resff}{The result of find_functions from which func will be searched.}

\item{from}{A name of a function or a list of functions from which
func will be searched. (resff has priority over from.)}

\item{nonative}{: argument of find_functions}

\item{noglobal}{: argument of find_functions}

\item{packages}{: argument of find_functions}

\item{print}{= 0 not to print the result.}
}
\value{
A list of: \cr
res The list of function names which contains each of func. \cr
resff The reslt of find_functions.
}
\description{
Find Calling Functions
}
\details{
\cr
When resff is not given, this function  requires "codetools" package.
\cr

This function matches each element of func to the elements of resff.
\cr
resff can be created as \cr
\code{ resff <- find_functions( from, nonative=nonative, unlist=0
, noglobal=noglobal, packages=packages ) }
\cr
}
\examples{
\dontrun{
library(codetools)
library(lazy.tools)

res=find_calling_functions( vartype, from=ls("package:lazy.tools") )
res2=find_calling_functions( c("vartype","varlist"), resff=res$resff )$res
# res2 can be obtained by
#  find_calling_functions( c("vartype","varlist")
#                        , from=ls("package:lazy.tools") )$res
# but find_function takes long time.
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UTF8pgm.R
\name{is.DBCS}
\alias{is.DBCS}
\alias{is.MBCS}
\title{Checks if the Characters in a String are Multi Byte Characters.}
\usage{
is.DBCS(x)

is.MBCS(x)
}
\arguments{
\item{x}{String vector}
}
\value{
A logical vector or list of logical vectors indicating if
each character in the input string vector is a multi byte chractor or not.
}
\description{
Checks if the Characters in a String are Multi Byte Characters.
}
\details{
Locale of your Windows machine can be obtained by the following:
\preformatted{
shell("ver")
shell("PowerShell  Get-WinSystemLocale")
shell("chcp > chcp.txt")  # chcp.txt will be created somewhere.
strsplit(readLines("chcp.txt")," ")[[1]][3]
}
Code 923 means Shift-JIS.

Version and locale of R can be obtained as follows;
\preformatted{
version
Sys.getlocale()
}
\cr
T check the character encoding, use: \code{Encoding(char)}.
The result will be UTF-8 or unknown meaning system encoding.
\cr

About rawToChar and charToRaw functions: \cr
To get the raw expression of character, use\cr
 \code{ char <- paste(c(0:9," ",letters,LETTERS),collapse="")}
 \code{ raw <- charToRaw(char) }\cr
To get the numeric equivalent of raw, use\cr
 \code{ num <- as.numeric(raw) }.\cr
To recover the original characters, use\cr
 \code{ char <- rawToChar( as.raw(num) ) }.\cr
\cr
Hex numbers used to express raw expression can be bypassed by using \cr
 \code{num <- char2num(char)} \cr
 \code{char <- num2char(num)} \cr
}
\examples{

# A string consisting of single and multi byte characters
# The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
{if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
 nza=c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
else
 nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
za <- rawToChar(as.raw(nza))
# za <- num2char(nza)

# location of the multi byte characters in a string
is.DBCS(za)

# Pick up the single byte characters.
strsplit(za,"")[[1]][!is.DBCS(za)]
# Pick up the multi byte characters.
strsplit(za,"")[[1]][is.DBCS(za)]

# strsplit changes the encoding from Shift-JIS (double byte) to UTF-8.
Encoding(strsplit(za,"")[[1]])
Encoding(substring(za,1:nchar(za),1:nchar(za)))
unlist(nbyte(strsplit(za,"")[[1]]))
unlist(nbyte(substring(za,1:nchar(za),1:nchar(za))))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inspect_RD.R
\name{inspect_RDS}
\alias{inspect_RDS}
\title{inspect the contents of RDS file}
\usage{
inspect_RDS(RDname, func = str, max.level = 1, return = 0, ...)
}
\arguments{
\item{func}{The function to apply to the contents of the .RDS file
such as \code{str} or \code{print},}

\item{max.level}{the max.level option for str when func=str.}

\item{return}{= 1 to return the the result of func.}

\item{...}{additional parameters to func}

\item{RDS}{The name of .RDS file}
}
\description{
This function inspects the contents of an RDS file.
}
\details{
Note that \code{saveRDS(object,filename.RDS)} saves a single object in
an external file and that
\code{save(object1,object2,...,file="filename.RData")}
saves the multiple objects in an external file.
}
\examples{
a <- 1;  b <- 2;  C <- matrix(1:6,3,2);  E <- letters[1:5]
ds <- data.frame(a,b,C);  ds2 <- list(C,ds)
Big <- list(a=a, b=b,C=C,E=E,ds=ds,ds2=ds2)
saveRDS(Big, file="testRDS.RDS")
inspect_RDS("testRDS.RDS")
inspect_RDS("testRDS.RDS", func=print)

# retrieve the contents from .RDS file
# loadRDS( RDSfilename )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trim.R
\name{trim}
\alias{trim}
\title{trim leading and trailing blanks}
\usage{
trim(
  x,
  leading = FALSE,
  trailing = TRUE,
  middle = FALSE,
  all = FALSE,
  blank = " "
)
}
\arguments{
\item{x}{string vector or matrix}

\item{leading}{= 1 to remove the leading blanks}

\item{trailing}{= 1 to remove the trailing blanks}

\item{middle}{= 1 to replace consecutive blanks by one (compress)}

\item{all}{= 1 to remove the leading and trailing blanks and compress\cr
= 2 to remove all the blanks}

\item{blank}{= a character to define the blank}
}
\value{
trimed string vector
}
\description{
trim leading and trailing blanks
}
\examples{
testc="   c a  p r i    ci  ous   "
trim( testc )
trim( testc, leading=1, trailing=0 )
trim( testc, middle=1, trailing=0 )
trim( trim( testc, middle=1 ), leading=1 )
trim( testc , all=1 ) # same as above.
trim( testc, all=2 )
A=demomat(3,3,root="a"); A[1,1]="  a1 1"; A[2,2]="   a2 2 "
trim(A,all=1)
trim(A,all=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lazy.tools.R
\docType{_PACKAGE}
\name{lazy.tools}
\alias{lazy.tools}
\title{lazy.tools: Collection of useful tools for lazy boys and girls}
\description{
This package provides the following categories of functions:
}
\section{IO and Data Frame Functions}{

\itemize{
 \item cards: read inline data.
 \item Print: Print objects with their names and formats.
 \item Printb: Print super matrix (partitioned or block matrix) with
 separators.   alias: printb
 \item printm: A better version of Print which can print data frames
 together with matrices, arrays and lists.  alias: Printm
 \item fuzz: Print with fuzz=1e-4 option.
 \cr\cr
 
 \item inspect_RDS: Inspects the contents of an RDS file.
 \item inspect_RData: Inspects the contents of an RData file.
 \cr\cr
 
 \item demomat: Set up a matrix whose elements consist of row and col
  numbers.
 \cr\cr
 \item get_val: Get values from an array at the location specified by subs.
 \item `put_val: Modify the values of an array.
 \item get_valL: Get values from a list of data frames.
 \item `put_valL: Modify the values of a list of data frames.
 \cr
 \item get_valLL: Get values from a list of lists.
 \cr\cr
 \item merge2: Returns the merged data frames with contribution indices.
 \item stack_df: Vertically stack several data frames.
 \item merge_df: Horizontally stack (merge) several data frames with
 unequal # of obs.
 \item varlist: Interprete sas-like abbreviated variable list.
 \item vartype: find the mode/type of the columns of data.
 \item keepdrop: Keep or Drop the variables.
 \item convert: Converts the type/class of the columns of a data frame
 or a matrix.
 \cr\cr
 \item named_list: Create a named list.
 \item from_named_list: Retrieve Objects from a Named List.
 \cr\cr
 \item graph2pdf: Output Graphs to PDF Device.
 }
}

\section{Utility Functions}{

\itemize{
 \item analyze_three_dots: Analyze the Three Dots and
Return the Names and the Values.
 \item Exists: Check if the Objects Exists.
 \item ceiling_n: Return the Ceiling and Power of the Input using Radix
  (Base) of n.
 \item perm: Generate all m-permutations of n elements.
 \item comb: Generate all combinations of n elements taken m at a time.
 \item comments: Comment out multiple lines.
 \item complex_index: Finds the complexity index of a 0-1 pattern.
 \item cprod: Cartesian product of many vectors and matrices.
 \item cumsuminv: Inverse of Cummurative Sum.
 \item decp: Returns the (approximate) # of Digits on the Both Sides of
 the Decimal Point.
 \item etime: Elapsed Time.
 \item expand_grid1: Expand.grid with rev and name option.
 \item gen01pat: Generate all possible 0-1 patterns of given length, 
 or binary partition index matrix
 \item is.varname: Check if the argument is a valid variable name.
 \item is.number: Check if a vector consists of numbers.
 \item loceq: Returns the locations of the elements of the first argument
  whose values are equal to one of the elements in the second argument
 \item lv: Retruns the Last Value.
 \item rmv:  Remove Objects If They Exists in the Parent Frame.
 \item Seq:  Smart Sequence Generation.
 \item Unique: Unique values and their locations
 \item was: Displays the attribuets and the structure of the input.
 \item yearJAD: Conversion between Japanese era year and AD.
 \cr\cr
 \item makeStack: Generation of a set of stack operation functions.
 }
}

\section{Character Functions}{

\itemize{
 \item And: Combines the string arguments by &.
 Infix version is \code{\%&\%} and \code{\%&&\%}
 \item Or: Combines the string arguments by |.
 Infix version is \code{\%|\%} and \code{\%||\%}
 \item Not: Negate the expression as !expression.
 \item add_paren: Adds enclosing parentheses to an expression.
 \item remove_paren: removes enclosing parentheses from an expression.
 \item locbalpar: Returns the location of the matching parentheis 
in a string.
 \item locbalpar2: Returns the location of the matching parentheis.
 \item locbalq: Returns the location of the next quote.
 \item replacec: Replace Characters by New Ones.
 \item revchar: Reverse string.
 \item revstr: Reverse string.
 \item str_olay: Overlay strings.
 \item trim: trim leading and trailing blanks.
 \cr\cr
 \item testregex: Test Regular Expression
 \item Grep: User Friendly grep/gsub
}
}

\section{Functions for Multi Byte Characters}{
 
(lazy.utf8 package consists of the functions in this section + printm.)
\itemize{
 \item is.DBCS: Checks if the Characters in a String are Multi Byte
  Characters.
 \item is.MBCS: Checks if the Characters in a String are Multi Byte
  Characters.
 \cr\cr
 \item ReadLines: Read Lines with or without Encoding Option.
 \item WriteLines: Write Lines with or without Encoding Option.
 \cr\cr
 Examples of Shif-JIS and UTF-8( with BOM or w/o BOM)
 text files are included. \cr
 Use below to find out where these files are stored:
 \preformatted{
  system.file("TestData1.txt",package="lazy.tools")
  system.file("TestData2.txt",package="lazy.tools")
  system.file("TestData3.txt",package="lazy.tools")
 }
 
 \item nbyte: Returns the number of bytes used to represent each character.
 \item nchar2: nchar using byte as the unit.
 \item cchar: Starting and Ending Columns of Each Character in a Text File.
 \item substr2: substr using column location as the unit.
 \item substrDBCS: substr using column location as the unit for
strictly Double byte characters (including ASCII characters.) on R 4.1.3. 
 (deprecated)
 \cr\cr
 \item char2num: Convert a string  to a numeric vector
 \item num2char: Convert a numeric vector to a string
 \cr\cr
 \item generate_zenkaku_sets: Generate Zenkaku Character Set
 \item pref2dist: Convert Japanese prefectures to districts
}
}

\section{Infix and Postfix Operators}{

\itemize{
 \item \%&\%, \%&&\%, \%|\%, \%||\% for \code{And} and \code{Or}.
 \cr
 \item generate_AndOr: Generates string infix operators &, &&, |, ||
 which are almost equivalent to: \%&\%, \%&&\%, \%|\%, \%||\% .
 \cr
 \item \%//\% Mathematical-like postfix application of a function
}
}

\section{Special Utility Functions}{

\itemize{
  \item package_info: Creates html files from the help files of
  an R package.
  \item package_description: Displays the package description.
  \item find_functions: Find the functions used in a function.
  \cr This function uses "codetools" package.
  \item find_calling_functions: Find the functions which use a function.
  \cr This function uses "codetools" package.
  \cr
  \item check_partial_match: Checks if the partial match is correct.
}
}

\section{Sort Functions}{

\itemize{
 \item new_name: Renames the elements according to a table. (deprecated?)
 \cr This function uses "naturalsort" package.
 \item recode: Recode the values of an object from "from" to "to".
 \item sortit: Sort the first argument by ... .
 \item order2: Returns a permutation which rearranges its first argument
  in the order specified by the second argument.
 \item sort_missing_data: Sort Matrix/Data Frame
 According to the Pattern of the Missing Elements.
}
}

\section{Frequency Distribution Functions}{

\itemize{
 \item expand_freqdist: Expand Frequency Distribution to Raw Data
 \item freqdist1: Frequency distribution of univariate numeric data
 with case weight and midpoints.
 \item freqdist: Frequency distribution of multivariate numeric data
 with case weight and midpoints.
 \item freqc: Frequency distribution of many discrete variables
 \item jfreq: Joint frequency distribution of discrete variables
 \item merge_tables: Merge two or more tables or frequency distributions.
 \item compress_freqdist: Compress frequency distributions by aggregating
 duplicate entries.(deprecated)
 \item fxtabs: Perfelcty Flat Cross Table from native xtabs function.
 \item table2freqdist: Convert the output of native table or xtabs
 to (levels, frequency) format.
 \item plot_freqdist: Plot univariate frequency distribution.
 \cr\cr
 \item gen_midp: Generate equally spaced n midpoints in [min, max].
 \item get_range: Convert levels, start etc to range.
 \item brk2midp: Define midpoints from break points.
 \item midp2brk: Define break points from midpoints.
 \item Quantile: Sample quantiles by reverse interpolating sample cdf.
}
}

\section{Statistical Functions}{

\itemize{
 \item design_expand: Dummy Expantion of a Vector
 \item interpol: Interpolation of data
 \item logistic: logistic function with D=1.7
 \item logistic1: logistic function with D and minimum value
 \item logit: logit function with D=1.7
 \item logit1: logit function with D and minimum value
 \item mands: Calculates mean, std, var, and nobs  with case weight
 \item mandd: Calculates mean vector and dispersion matrix with case weight
 \item princ: Principal Component Analysis of Y with possible missing
  elements
 \item normalize_config: Normalize the input matrix
 \item procote: Extended Orthogonal Procrustes Rotation
 \item reorder_fa: Reorder the rows and columns of factor pattern matrix
 \item reflect_fa: Reflect the signs of the columns of
 factor pattern matrix
 \item smoothra: Running average smoothing
 \item standard: Column standardization of a data frame, matrix, or vector
 \item wquantile: Weighted Quantile
 }
}

\section{Plot Functions}{

\itemize{
 \item iotable: Inside-Out Plot
 \item pplot: Profile Display (horizontal)
 \item plot_interval: Plot Horizontal Line on top of Density Plot
 \item radarchart: Radar Chart
 }
}

\section{Distance Functions}{

\itemize{
 \item distancew: Calculation of Weighted Distance Matrices
 \item distancew0: Calculation of Weighted Distance Matrix
 \item distrectw: Calculation of Weighted Distance Matrix between
 the rows of F and A
 }
}

\section{Functions for Test Scores}{

\itemize{
 \item gen_test_score: Generate Discrete Test Score Distribution.
 \item gen_test_score_CTT: Generate Discrete Test Score Distribution
  according to Classical Test Theory.
 \item eq: Observed Test Score Equating.
 \cr\cr
 \item dummy_expand: dummy_expand the compressed item response data.
 \item dummy_compress: compress the uncompressed item response data.
 }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UTF8pgm.R
\name{substrDBCS}
\alias{substrDBCS}
\title{Substr Using Column Location as the Unit.}
\usage{
substrDBCS(x, start, stop = max(nchar(x, type = "byte")))
}
\arguments{
\item{x}{A string vector}

\item{start}{The first location}

\item{stop}{The last location}
}
\value{
A vector consisting of the substrings of the input vector.
}
\description{
Substr Using Column Location as the Unit.
}
\details{
This is for strictly DUOBLE byte characters. NOT for UTF-8 characters.
For strictly double byte characters, this is faster than susbtr2.

About rawToChar and charToRaw functions: \cr
To get the raw expression of a string, use\cr
\code{ raw <- charToRaw(paste(c(0:9," ",letters,LETTERS),collapse="")) }\cr
To get the numeric equivalent of a raw (vector),  use\cr
\code{ num <- as.numeric(raw) }.\cr
To recover the original string, use\cr
\code{ char <- rawToChar( as.raw(num) ) }.\cr

\cr
If the n-th byte is the beginning or the ending location of \cr
a double byte character, \code{substrDBCS(x,n,n)} is invalid.\cr
Consult \code{is.DBCS(x)} and use with caution.
}
\examples{
# The exa,ple below causes error on R 4.2.0 and above.
\dontrun{
# A string consisting of single and double byte characters
nza <- c(97,98,130,131,100,101,130,134,130,135,104)
za <- rawToChar(as.raw(nza))
Print(za)
# location of the double byte characters
is.DBCS(za)
# The first to the fourth characters
substr(za,1,4)
# The characters stored in the first to the fourth bytes of za.
substrDBCS(za,1,4)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dummy_expand.R
\name{dummy_expand}
\alias{dummy_expand}
\title{Dummmy Expantion of Categorical Data}
\usage{
dummy_expand(Uc, ncat = NULL, type = NULL, zero = 1)
}
\arguments{
\item{Uc}{Item Response matrix in Compressed Format}

\item{ncat}{a vector consisting of # of categories of each item}

\item{type}{NOT used.}

\item{zero}{= 0 to exclude the zero-th category from output}
}
\value{
A list of \cr
U, ncat, type, zero, fromP, toP
}
\description{
Dummmy Expantion of Categorical Data
}
\details{
In compressed format, Uc[i,j]=k means that subject i answers to item j
 by selecting option k, k=0,1,...,ncat[j]. \cr
 This function expands this compressed format to uncompressed
 dummy expanded format: \cr
 That is, each item takes ncat[j] colums of U[, fromP[j]:toP[j]], \cr
 and the k+fromP[j]-th colum will be 1, and 0 elsewhere.
}
\examples{
# generate item response
# in uncompressed format
# set.seed(1701)
# res <- gendataIRT( 1, paramS1, npoints=7, thmin=-3, thmax=3 )
# U=res$U
# in compressed format
# set.seed(1701)
# res1 <- gendataIRT( 1, paramS1, npoints=7, thmin=-3, thmax=3, compress=1 )
# Uc=res1$U
#
# uncompressed data: ncat=c(2,2,4)
U <- "
Q1_0 Q1_1 Q2_0 Q2_1 Q3_0 Q3_1 Q3_2 Q3_3
1    0    1    0    1    0    0    0   
1    0    1    0    0    1    0    0   
1    0    1    0    1    0    0    0   
1    0    1    0    0    1    0    0   
1    0    0    1    0    0    1    0   
0    1    0    1    0    0    0    1   
0    1    0    1    0    0    0    1   
"; U=cards( U, header=1 )
#
# compressed data: ncat=c(2,2,4)
Uc <- "
 Q1 Q2 Q3
 0  0  0 
 0  0  1 
 0  0  0 
 0  0  1 
 0  1  2 
 1  1  3 
 1  1  3 
"; Uc=cards( Uc, header=1 )


# from compressed to uncompressed
U1=dummy_expand(Uc)$U
# back to compressed
Uc1=dummy_compress(U)$Uc
Print(U-U1)
Print(Uc-Uc1)

# zero=0 option
U2=dummy_expand(Uc, zero=0)$U
Uc2=dummy_compress(U2, zero=0, ncat=c(2,2,4))$Uc
Print(Uc-Uc2)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge2.R
\name{merge2}
\alias{merge2}
\title{Merge two data frames and return the result with the contribution indices}
\usage{
merge2(x, y, ..., .inx. = "in.x", .iny. = "in.y")
}
\arguments{
\item{x}{First data frame.}

\item{y}{Second data frame.}

\item{...}{parameters to the native merge frunction.}

\item{.inx.}{The name of the contribution index for x.}

\item{.iny.}{The name of the contribution index for y.}
}
\description{
Merge two data frames and return the result with the contribution indices
}
\details{
The SAS like contribution indices will be returned as the variables
of the merged data frame.

In the followint example, those observations with \code{in.x=1} are
contributed from data frame x, and those with \code{in.y=1}, from y.
}
\examples{
d1 <- data.frame(x=c(11:15, 17:20),y=c(111:115, 117:120))
d1[2,2] <- NA
d2 <- data.frame(x=c(11:13, 16, 18:21),y=c(211:213, 216, 218:221))
d2[5,2] <- NA
d12 <- merge2(d1,d2, by="x", all=TRUE)
printm( d1, d2, d12 )

D12 <- merge2(d1,d2, by="x", all=TRUE, .inx.="IN1")
printm( d1, d2, D12 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keepdrop.R
\name{keepdrop}
\alias{keepdrop}
\title{Keep or Drop the variables in the data frame}
\usage{
keepdrop(df, keep = NULL, drop = NULL)
}
\arguments{
\item{df}{The data frame whose variables are to be kept/dropped.}

\item{keep}{A vector consisting of the names of the variables to keep\cr
or sas-like abbreviated list of variables to keep as a string.}

\item{drop}{A vector consisting of the names of the variables to drop\cr
or sas-like abbreviated list of variables to drop as a string.}
}
\value{
A data frame
}
\description{
Keep or Drop the variables in the data frame
}
\details{
This functin first expands the abbreviated variable list by varlst function.
\cr
Then, it creats a data frame with the variables in the keep list. \cr
Finally, it drops those variable in the drop list.
}
\examples{
df <- as.data.frame(demomat(5,7))
names(df) <- c("x1","x2","x3","a","s","m","i")

keepdrop( df, keep=c("x1", "a", "x3") )
keepdrop( df, keep="x1 a x3" )
keepdrop( df, keep="x1-x3 m" )
keepdrop( df, keep="x1--m" )

keepdrop( df, drop=c("x1", "a", "x3") )
keepdrop( df, drop="x1 a x3" )
keepdrop( df, drop="x1-x3 m" )
keepdrop( df, drop="x1--m" )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wquantile.R
\name{wquantile}
\alias{wquantile}
\title{Weighted Quantile}
\usage{
wquantile(
  x,
  freq = 1,
  probs = c(0, 0.25, 0.5, 0.75, 1),
  interpol = approx,
  ...
)
}
\arguments{
\item{x}{A vector of the random variable}

\item{freq}{A vector of probability or frequency associated with x}

\item{probs}{A vector of percents/100}

\item{interpol}{Name of interpolation function: approx or spline}

\item{...}{additional parameters to interpol function.}
}
\value{
A vector containing quantile values with name.
}
\description{
This program calculates the cdf of x from the frequency distribution
 (x, freq) and reverse interpolates it to calculate quantiles.
}
\details{
When \code{length(unique(x)) <= 10 }, \code{quantile(x,probs=probs)}
will be returned.

Quantile and wquantile differs in the way the interpolate (cdf,x). \cr
wquantile modifies (cdf,x) to (cdf, xb ) where xb is the mid point of
neighboring unique values of x.
}
\examples{
seed <- 1701
set.seed(seed)

# original real random variable
n <- 100
x0 <- 100+10*rnorm(n)
# round to integers
x <- round(x0)
# (xc, f) summarizes the dist of x
f <- table(x)
xc <- as.numeric(names(f))
# expand (xc, f) to (xxf, 1) : These two contains the same info.
xxf <- unlist(mapply( rep,xc,f ))

# quantiles
qtx0 <- quantile(x0)
qtx <- quantile(x)
qtxxf <- quantile(xxf)
qtxc <- quantile(xc)
wqtxc <- wquantile( xc, f )

# comparison
qt <- cbind(qtx0,qtx,qtxxf,qtxc,wqtxc)
Print(qt)
# difference from the real quantiles
dif <- abs(qt-qt[,1])
mad <- matrix(colMeans(dif),,1); rownames(mad)=colnames(dif)
Print(dif,mad)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_partial_match.R
\name{check_partial_match}
\alias{check_partial_match}
\title{Check if the Partial Match is correct.}
\usage{
check_partial_match(fargs1, fargs2, print = 0)
}
\arguments{
\item{fargs1}{as sys.call() formal arguments}

\item{fargs2}{as match.call()  result of the matching}

\item{print}{= 1 or 2 to print the details.}
}
\description{
Check if the Partial Match is correct.
}
\details{
This function checks if the arguments listed in ... are correctly
matched to the formal arguments of teh called function. \cr
The error may occur when ... contains single-lettered arguments.
\cr
This function should be called at the beginning of any function
which has ... as its formal argument.
}
\examples{
test1 <- function( alpha=0, beta=0, gamma=0, ... ){
 #
 cat("\nTop of test1\n")

 check_partial_match( sys.call(), match.call() )

 cat("\n Back to test1\n")
 Print(alpha,beta,gamma)
 val=list(...)
 cat("\n ... contains below:\n")
 print(val)
 cat("\nBottom of test1\n")

} # end of test1


test2 <- function(a){
  res=test1( alpha=1, b=rep(9,3), d=2,xx=4, g=8, print=1 )
} # end of test2


test3 <- function(a){
  res=test1( alpha=1, beta=2, gamma=3, b=rep(9,3), d=2,xx=4, g=8, print=1 )
} # end of test2


test3()

# If run, below caues error.
# test2()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic.R
\name{logistic}
\alias{logistic}
\title{logistic function with D=1.7}
\usage{
logistic(x, D = 1.7)
}
\arguments{
\item{x}{vector}

\item{D}{= 1.7}
}
\value{
vector of the size of x
}
\description{
logistic function with D=1.7
}
\details{
This function calculates \cr
  p=1/(1+exp(-D*x)) \cr
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cprod.R
\name{cprod}
\alias{cprod}
\title{Cartesian product of many vectors and matrices}
\usage{
cprod(..., rev = 1, as.data.frame = 0, rownames = 0, colnames = 0)
}
\arguments{
\item{...}{vectors, factors or matrices \cr
Or  ... can be a list containing the levels vectors as its elements.}

\item{rev}{= 0 to make the fist elements of ... change first}

\item{as.data.frame}{= 1 to make the result a data frame.}

\item{rownames}{= 1 to return the compressed version as the rownames.}

\item{colnames}{= 1 to use the names of ... as the colnames.}
}
\value{
A matrix or data frame.
}
\description{
Cartesian product of many vectors and matrices
}
\details{
When all the inputs are vectors, \code{cprod} is equivalent to
\code{expand.grid1}.\cr

When both \code{A} and \code{B} are matrices, 
\code{cprod( A, B )} is equivalent to
\preformatted{
  Reduce( rbind, lapply(as.data.frame(t(A))
       ,  function(x) cbind(repmat(matrix(x,1,ncol(A)),nrow(B)),B)) )
}
}
\examples{
cprod(1:2,3:5)
cprod(1:2,3:4, rev=0)
cprod(matrix(1:6,3,2, byrow=1),1:2)
cprod(matrix(1:6,3,2, byrow=1),matrix(1:4,2,2,byrow=1))

# when ... is a list
ll <- list(a=c("a","b"),bbb=1:3)
cprod(ll, colnames=0)
cprod(ll, colnames=1)

# ANOVA data
levels <- c(2,3)
ll <- lapply(levels,function(x)seq(1:x))
names(ll) <- c("A","B")
data <- cprod(ll, colnames=1)

# generate 3D grid of x0
x0 <- 1:3
ndim <- 3
x <- cprod( rep(list(x0),ndim) )
Print(x)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen_test_score_CTT.R
\name{gen_test_score_CTT}
\alias{gen_test_score_CTT}
\title{Generating Test Score Distribution according to Classical Test Theory}
\usage{
gen_test_score_CTT(
  n,
  minscore = NULL,
  maxscore = NULL,
  rho0 = 0.9,
  meanX0 = NULL,
  varX0 = 15^2,
  beta = NULL,
  orth = 0,
  discrete = 1,
  seed = NULL,
  print = 0,
  plot = 0,
  title = NULL
)
}
\arguments{
\item{n}{# of observations.}

\item{minscore}{The minimum score.}

\item{maxscore}{The maximum score.}

\item{meanX0}{The mean of observed test score}

\item{varX0}{The variance of observed test score}

\item{beta}{Vector of alpha and beta parameters for beta distribution
in [minscore,maxscore].}

\item{orth}{= 1 to orthogonalize T and E. (not yet available.)}

\item{discrete}{= 0 to use continuous random variables}

\item{seed}{The random number seed or NULL}

\item{print}{= 1 to print the result.}

\item{plot}{= 1 to plot the result.}

\item{title}{Title string to be used in plot.}

\item{rho}{The reliability coefficient}
}
\value{
A list of: \cr
rho The actual reliability \cr
meanX The actual mean of X \cr
stdX The actual standard deviation of X \cr
meanT The actual mean of T \cr
stdT The actual standard deviation of T \cr
meanE The actual mean of E \cr
stdE The actual standard deviation of E \cr
cor The correlation matrix of (X,T,E) \cr
XTE A matrix consisting of \code{X,T,E}. (n times 3 matrix) \cr
scoredist Score distribution of X \cr
}
\description{
This function generates a frequency distribution table
 of n discrete test scores in [minscore,maxscore].
}
\details{
This function generates a set of \cr
 (observed score X, true score T, error score E) \cr
of size n in [minscore <= X <= maxscore] according to
the Classical Test Theory.

Continuous random numbers, either normal or generalized beta,
are first generated and then they are discretized by cut function.
Those values outside of [minscore,maxscore] will be set to minscore
or maxscore, resp.

beta parameter has priority over nomal parameter.
}
\examples{


res=gen_test_score_CTT(n=10000, varX0=17^2, rho=0.60, seed=1701
                  , plot=1, print=1, minscore=0, maxscore=100)
seed=1701; set.seed(seed)
res=gen_test_score_CTT(n=10000, varX0=17^2, rho=0.60
                  , plot=1, print=1, minscore=0, maxscore=100)

res=gen_test_score_CTT(n=10000, varX=18^2, rho=0.80, beta=c(5,2)
                  , plot=1, print=1, minscore=0, maxscore=100, seed=1701)
res=gen_test_score_CTT(n=10000, meanX0=60, varX=18^2, rho=0.80, beta=c(5,2)
                  , plot=1, print=1, minscore=0, maxscore=100, seed=1701)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/printm.R
\name{printm}
\alias{printm}
\alias{Printm}
\title{Print multiple objects together with their names \cr
(Compact and better version of Print)}
\usage{
printm(
  ...,
  .delim. = " ",
  .NAchar. = ".",
  .FILL. = ".^",
  fmt = "",
  .fmt. = "",
  .flag. = " ",
  .printrownum. = 1,
  .lj. = 0,
  .debug. = 0
)

Printm(
  ...,
  .delim. = " ",
  .NAchar. = ".",
  .FILL. = ".^",
  fmt = "",
  .fmt. = "",
  .flag. = " ",
  .printrownum. = 1,
  .lj. = 0,
  .debug. = 0
)
}
\arguments{
\item{...}{objects to be printed}

\item{.delim.}{delimiter string}

\item{.NAchar.}{a character to indicate NA
(One or two single characters, a double byte character, or "" for "na".)}

\item{.FILL.}{not used.}

\item{fmt}{a string containing format specification in the form: wid.dig}

\item{.fmt.}{same as fmt (high precedence)}

\item{.flag.}{flag options for formatC}

\item{.printrownum.}{= 0 not to print row numbers for a vector}

\item{.lj.}{= 1 to left justify character vectors}

\item{.debug.}{= 1 to print debugging info}
}
\description{
Print multiple objects together with their names \cr
(Compact and better version of Print)
}
\details{
This function can print data.frames and list objects together
with other objects.
In contrast, \code{lazy.tools::Print} prints data.frames or lists
separately.

Object named \code{fmt} cannot be printed unless  \code{.fmt.} is given.

List objects cannot be formatted. \cr
Data.frames can only be formatted by digits= option which
specifies the length of the field.
Better use \code{Hmisc::format.df} to format data.frames.

When the actual arguments consist of string literals or numbers,
they will be printed without names.

The string literals will be printed without names.

The string literals "/" or "\\n" indicate new line when they are not
the first object of ... .

The native vectors will be printed as column vectors.
If a vector has a name, it will be printed as row names.
If a vector does not have name, numeric numbers will be printed as
row names unless \code{.printrownum.=0} is given.

If the objects contain UTF-8 characters, the result may be misleading
due to the bug of \code{print.data.frame} method.

\code{fmt} or \code{.fmt.} contain, if any,
 formats to be used separated by blanks: \cr
 \code{fmt="Width.Digits"} will use the following formatC function \cr
 \code{formatC( object, format="f", width=Width, digits=Digits )} \cr
 to format the object.

 The first element, separated by blanks, will be applied to the first
 object of ..., the 2nd element to the 2nd object, and so on. \cr
 However, when the first object of ... is a string literal, the first
 format element will be applied to the 2nd object in ... .

 If there are more objects in ... than length(fmt),
 the last fmt value will be used to fill.



This function can be used with pipe operator.\cr

Use \code{capture.output} function, not the return value,
to utilize the printed output of this function.
}
\examples{
df1 <- data.frame(x=1:3, y=11:13, row.names=paste("rn",1:3,sep=""))
df1[1,1] <- NA
df2 <- data.frame(x=21:24, y=31:34)
df2[2,2] <- NA
df3 <- data.frame(aa=31:34+.12345, bb=51:54)
df3[2,3] <- NA
mat <- matrix(1:6,3,2, dimnames=list(NULL,c("c1","C2")))
mat[3,2] <- NA
array <- array(1:12, dim=c(3,2,2))
array[1,1,1] <- NA
list <- list(df1,df2)
df <- as.data.frame(rbind(df1,df2))
df[,2] <- as.character(df[,2])
let <- letters[1:4]
names(let) <- toupper(let)
let[2] <- NA
printm( "title",1:3, df1, df2, df3, "/", list, df2+df2, mat, array
        , "newline", "\n", let, letters[1:2], letters[2] )

# vector, matrix and array with format
printm( mat[1,], mat, array, fmt="5.0 5.2 6.3")
# data frame with format: wid part of wid.dig will be used as digits=wid.
printm( df1+0.12345, df1+0.12345, df1+0.12345, fmt="3 4 7")

# no blank line before but a blank line after.
cat("No blank lines.\n")
printm(mat)
cat("One blank line above.\n")
cat("To insert a blank line, use below.\n")
printm("","/", mat+mat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/freqfuncs.R
\name{jfreq}
\alias{jfreq}
\title{Joint Frequency Distribution of Discrete Variables}
\usage{
jfreq(X, weight = NULL, omitmiss = 1, sort = 0)
}
\arguments{
\item{X}{Input multivariate discrete data}

\item{weight}{weight vector or 1}

\item{omitmiss}{= 0 to return the cartesian product of
unique values of each column of X.}

\item{sort}{= 1 to sort so that the last column changes first. \cr
= 2 to sort by frequency in ascending order \cr
= 3 to sort by frequency in descending order \cr}
}
\value{
A list of: \cr
Xu An object consisting of the (sorted) unique rows of X \cr
f  Assosiated frequency vector
}
\description{
Joint Frequency Distribution of Discrete Variables
}
\details{
Calculates the frequency of the unique rows of X matrix. \cr
Note, however, the order of the rows are different from
the reslt of unique(X) which does not sort the result. \cr
If sort=0, the first column changes first, otherwise, the last
column changes first.
}
\examples{
# numeric varaibles
set.seed(1701)
X <- matrix(sample(0:1,50,replace=1),25)
X <- cbind(X,matrix(sample(1:2,50,replace=1),25))
X <- rbind(X,matrix(c(0,1,0,1),5,4,byrow=1))
colnames(X) <- paste("ccc",1:4,sep="")
res <- jfreq( X )
Print( res$Xu, res$f )

# sort so that the last column changes first
res1 <- jfreq( X, sort=1 )
Print( res1$Xu, res1$f )

# sort by frequency
res1 <- jfreq( X, sort=3 )
Print( res1$Xu, res1$f )

# mixed variable types in a data.frame
Y <- as.data.frame(X)
Y[,2] <- letters[Y[,2]+1]
res2 <- jfreq( Y )
Print( res2$Xu, res2$f )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perm.R
\name{perm}
\alias{perm}
\title{Generate all m-permutations of n without repetition}
\usage{
perm(n, m, vals = 1:n, noreverse = 0)
}
\arguments{
\item{n}{total # of elements}

\item{m}{# of elements to choose}

\item{vals}{a vector of values to be permuted}

\item{noreverse}{= 1 to remove reverse-equivalent rows \cr
= 2 to check the result}
}
\value{
A matrix of the size P(n,m) x m.
}
\description{
Generate all m-permutations of n without repetition
}
\details{
The recursive algorithm below is used.
\preformatted{
  res=NULL
  for( i in 1:n )\{
    res <- rbind(res, cbind(vals[i], Recall(n - 1, m - 1, vals[-i])))
  \}
}

The method of removal of reverse-equivalent rows comes from
the answer to stack overflow question: \cr
"How to remove reverse rows in a permutation matrix?" \cr
by obchardon at Sep 15, 2020 at 7:46 \cr
https://stackoverflow.com/questions/63896177/how-to-remove-reverse-rows
-in-a-permutation-matrix
}
\examples{
n <- 5
for( m in 1:n )
 Print( m, perm(n,m), comb(n,m) )
 
 Print( perm(3,3), perm(3,3,,1) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testregex.R
\name{testregex}
\alias{testregex}
\title{Remove Brackets and Double \\s from R type Regular Expression}
\usage{
testregex(pattern, expr = "")
}
\arguments{
\item{pattern}{R type regular expression}

\item{expr}{A string vector containing expressions, \cr
or a string which contains several expressions separated by a comma, \cr
or "".}
}
\value{
NONE
}
\description{
Remove Brackets and Double \\s from R type Regular Expression
}
\details{
A pattern w/o brackets will be printed which can be pasted to \cr
 \code{https://jex.im/regulex/#!flags=&re=\%5E(a\%7Cb)*\%3F\%24}.
}
\examples{
testregex( "^[[:alpha:].][[:alnum:]_.]*$" )
testregex( "^[[:alpha:].][[:alnum:]_.]*$"
        , c( "a1", "b", ".c1.", "@d_", "1_2", "._" ) )
testregex( "^((([[:alpha:]]|[.][._[:alpha:]])[._[:alnum:]]*)|[.])$"
        , "a1, b, .c1., @d_, 1_2, ._" )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph2pdf.R
\name{graph2pdf}
\alias{graph2pdf}
\title{Output the Graphics to PDF Device}
\usage{
graph2pdf(
  outpdf,
  close = 0,
  paper = "a4r",
  height = 0,
  width = 0,
  onefile = TRUE,
  ...
)
}
\arguments{
\item{outpdf}{The output pdf file name.
When onefie=FALSE, give a C integer format such as "Rplot\%03d.pdf".}

\item{close}{= 1 to close the pdf device.}

\item{paper}{Size of the pdf paper: "a4", "letter", "legal"
and "executive", or "a4r" and "USr" for rotated (landscape.)}

\item{height}{Height of the pdf paper in mm (not inches)}

\item{width}{Width of the pdf paper  in mm (not inches)}

\item{onefile}{= FLASE to create many files.}

\item{...}{Other arguments to native pdf function.}
}
\description{
Output the Graphics to PDF Device
}
\details{
This function calls native pdf function as:\cr
\code{pdf(outpdf, paper=paper, height=height, width=width}
\code{, onefile=onefile, ... )} \cr
When paper=NULL or "special", the paper size will be determined by
width and hight.
\cr
The defalt paper size is "a4r" which is equivalent to: \cr
\code{paper="special", height=210, width=297}
}
\examples{
\dontrun{
plot(-5:5,(-5:5)^2, type="l", main="square")
plot(-5:5,(-5:5)^3, type="l", main="cubic")

# put everything in one pdf file
graph2pdf("testestestest.pdf")
plot(-5:5,(-5:5)^2, type="l", main="square")
plot(-5:5,(-5:5)^3, type="l", main="cubic")
graph2pdf(close=1)

# create one pdf file for each graph with automatic naming
graph2pdf("testestestest\%03d.pdf", onefile=FALSE)
plot(-5:5,(-5:5)^2, type="l", main="square")
plot(-5:5,(-5:5)^3, type="l", main="cubic")
graph2pdf(close=1)

# create one pdf file for each graph with specific name
for( i in 1:3 ){
 fname=paste("f", i, ".pdf", sep="")
 graph2pdf(fname)
 plot(-5:5,(-5:5)^i, type="l", main=i)
}
graph2pdf(close=1)

# default paper size: paper="a4r"
graph2pdf("test0.pdf")
plot(-5:5,(-5:5)^2, type="l", main="test")
graph2pdf(close=1)

# ??
graph2pdf("test00.pdf", paper="a4")
plot(-5:5,(-5:5)^2, type="l", main="test")
graph2pdf(close=1)


# paper size with width and height
graph2pdf("test1.pdf", paper="a4r", width=197, height=120)
plot(-5:5,(-5:5)^2, type="l", main="test")
graph2pdf(close=1)

# Below two produces the same pdfs.
graph2pdf("test2.pdf", paper=NULL, width=197, height=120)
plot(-5:5,(-5:5)^2, type="l", main="test")
graph2pdf(close=1)

graph2pdf("test3.pdf", paper="special", width=197, height=120)
plot(-5:5,(-5:5)^2, type="l", main="test")
graph2pdf(close=1)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replacec.R
\name{replacec}
\alias{replacec}
\title{Replace Characters by New Ones}
\usage{
replacec(from, to, char, n = 1, start = 1, fixed = 1, ...)
}
\arguments{
\item{from}{A character string to be replaced by "to".
By default, there are no need to escape special characters.}

\item{to}{A character string to replace "from".}

\item{char}{A character vector where replacement occur.}

\item{n}{# of times to replace}

\item{start}{The location from which the replacement starts.}

\item{fixed}{A parameter to be passed to gregexpr and gsub.}

\item{...}{Additional parameters to gregexpr and gsub.}
}
\value{
A string with replacement.
}
\description{
Replace Characters by New Ones
}
\details{
Do not use regular expressions in from such as [a-b] etc. \cr
When from does not contain regular expression, \cr
fixed=0 and ignoe.case=1 \cr
should work.
}
\examples{
char <- paste( paste(letters,collapse=""),"0123456789",'"'
          , "'`!#$\%&'()[]{}<>*+-,.;:/^~=_|", sep="", collapse="" )
lchar <- nchar(char)
char <- paste(char,char,char,sep=" ",collapse="")


# one to one
replacec( "b", "B", char )
replacec( "b", "B", char, 2 )
replacec( "b", "B", char, 1, 135)

# many to one
replacec( "bc", "B", char )
replacec( "bc", "B", char, 2 )
replacec( "bc", "B", char, 1, 135 )

# one to many
replacec( "b", "BC", char )
replacec( "b", "BC", char, 2 )
replacec( "b", "BC", char, 1, 135 )

# to null string
replacec( "b", "", char, 99 )

# special characters
replacec( "(", "[paren]", char )
replacec( "^", "[hat]", char )
replacec( "!#$", "[esd]", char )

# ignore.case
replacec("B","[B]",char, fixed=0, ignore.case=1 )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pgm.R
\name{comb}
\alias{comb}
\title{Generate all combinations of n elements taken m at a time.}
\usage{
comb(n, m)
}
\arguments{
\item{n}{total # of elements}

\item{m}{# of elements to choose}
}
\value{
A matrix of the size choose(n,m) x m.
}
\description{
This is very slow when n is large.
}
\details{
This function uses cprod function to calculate combinations. \cr
For example,
 n choose 3 will be calculated as: \cr
 x <- 1:n \cr
 y <- cprod(x,x[-1]) \cr
 y <- y[y[,1]<y[,2],] \cr
 y <- cprod(y,x[-(1:2)]) \cr
 y <- y[y[,2]<y[,3],] \cr
}
\examples{
n <- 5
for( m in 1:n )
 Print( m,choose(n,m), comb(n,m) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/char2num.R
\name{char2num}
\alias{char2num}
\alias{num2char}
\title{Convert a Character Vector to a List of Numeric Vectors
 or vice versa via raw expression.}
\usage{
char2num(char)

num2char(num)
}
\arguments{
\item{char}{a vector of strings}

\item{num}{a list of numeric vectors or a numeric vector
which can be converted to string via raw conversion.}
}
\value{
char2num \cr
If char is a string ( a vector of length 1 ), num is a vector. \cr
If char is a vector of strings, num is a list of length(char) \cr
\cr
num2char \cr
If num is a vector, char is a string. \cr
If num is a list, char is a vector of strings of length(num) \cr
}
\description{
Convert a Character Vector to a List of Numeric Vectors
 or vice versa via raw expression.
}
\details{
Since these functions can handle multi byte characters, \cr
they can be used to store the strings with multi byte characters
as a list of numeric vectors. \cr

The core part of these functions are: \cr
char to num
\preformatted{
 num <- as.numeric( charToRaw(char) )
}
num to char
\preformatted{
char <- rawToChar( as.raw( num ) )
}

The numbers and Raw expression are related as
\preformatted{
 num <- char2num( char )
 ( hex <- as.hexmode( num ) )
 print( as.raw(num) )
}
}
\examples{
# char is a string
( char <- "This is an example." )
( num <- char2num( char ) )
( is.numeric(num) )
( char2 <- num2char( num ) )

# char is a string vector
( char2 <- c("This is an example.", "2nd line") )
( num2 <- char2num( char2 ) )
( sapply( num2, is.numeric ) )
( char22 <- num2char( num2 ) )

# storing multi byte characters in a package: \cr
# See the description of 'generate_zenkaku_sets'.
{if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
 nn <- c(87, 101,  32, 104,  97, 118, 101,  32, 228, 184, 141, 232, 184
         , 176, 231, 159, 169,  32, 121, 101, 116,  33)
else
 nn <- c(87, 101,  32, 104,  97, 118, 101,  32, 149, 115, 230, 250, 139
         , 233, 32, 121, 101, 116,  33)}
( zz <- num2char(nn) )
( zzh <- as.hexmode(nn) )
charToRaw(zz)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Unique.R
\name{Unique}
\alias{Unique}
\title{Find unique values of a vector or unique rows of a matrix and their location}
\usage{
Unique(x, sort = 0, decreasing = 0)
}
\arguments{
\item{x}{a vector or a matrix}

\item{sort}{= 1 to sort the result}
}
\value{
A list of \cr
 \code{value} The unique values \cr
 \code{loc} The list of length \code{length(value)} containing
 the locations of the original elements.
}
\description{
Find unique values of a vector or unique rows of a matrix and their location
}
\examples{
( temp <- Unique( c(1,2,1,5,6,3,5,4,6) ) )
sum( unlist(lapply(temp$loc,length)) )
( temp <- Unique( c(1,2,1,5,6,3,5,4,6), sort=1 ) )

( temp2 <- Unique( rbind(demomat(5,4),demomat(3,4)) ) )
( temp2 <- Unique( rbind(demomat(5,4),demomat(3,4)), sort=1, decreasing=1 ) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mands.R
\name{mands}
\alias{mands}
\title{Calculates mean, std, variance and n with case weight}
\usage{
mands(
  data,
  weight = 1,
  sandk = 0,
  probs = NULL,
  by = NULL,
  byvarname = NULL,
  vardef = "n",
  print = 0,
  simplify = 1
)
}
\arguments{
\item{data}{data vector, matrix or data frame.}

\item{weight}{case weight vector of length nrow(data) or 1.}

\item{sandk}{= 1 to calculate skewness and kurtosis}

\item{probs}{vector of probabilities to calculate quantiles}

\item{by}{a factor or a list of factors, each of length nrow(data) \cr
to be used as the INDICES of by function.}

\item{byvarname}{vector containing the names of by varialbes to be used.
(Not Yet Implemented.)}

\item{vardef}{= "n-1" to use n-1 as denominator for variance.}

\item{print}{= 1 to print the result}

\item{simplify}{= 0 to return class by object when by= is given.}
}
\value{
A matrix of statistics(mean, std, var, nobs, n)  x  variables \cr
where nobs is sum(weights) for nonmissing obs, and n is nrow(data). \cr
In addition, if \code{probs} is given, quantiles will be appended to
the above. \cr
\cr
When by is present, this function returns a class by object
consisting of above for each subgroup if simplify=0. \cr
If by is not a list and simplify=1,
the result is a statistics x variables x subgroups array. \cr
If by is a list of several factors, simplify=1 does not affet the result.
\cr
When X is univariate, the result is a statistics x subgroup matrix.
\cr
\cr
When data is NULL, NULL will be returned.
}
\description{
Calculates mean, std, variance and n with case weight
}
\details{
The means and vars will be calculated using native weighted.mean function.
\cr
The kurtosis = weigted.mean( scale(data, scale=FALSE)^4 ) / std^4 - 3
The quantiles will be calculated using wquantile function. \cr
}
\examples{
# univariate example
y <- 1:9
pdf <- exp( -0.5*(y-5)^2/(1.3^2) )
pdf <- pdf/sum(pdf)
group <- c(1,1,1,1,1,2,2,2,2)
mands( y )
mands( y, pdf, sandk=1 )
mands( y, by=group, sandk=1 )

# bivariate example
ndims <- 2; npoints <- 11; xmin <- -3; xmax <- 3; corrmat <- 0.2
nobs <- 100000

# dispersion matrix
Dmat <- matrix(corrmat,ndims,ndims)+(1-corrmat)*diag(ndims)
Dmat[1,1] <- .5

# grid X
X <- seq(xmin,xmax,length.out=npoints)
if( ndims >= 2 ){
 for( i in 2:ndims )
  X <- cprod(X,seq(xmin,xmax,length.out=npoints))
}

# weight
pdf <- exp( -0.5*diag( X\%*\%solve(Dmat)\%*\%t(X)) )
pdf <- pdf/sum(pdf)
freq <- round(nobs*pdf+.5)

# all data
mands( X )
mands( X, freq )
mands( X, freq, probs=c(0.25,0.5,0.75,0.975) )

# grouping variable
set.seed(1701)
if( is.matrix(X) ) n <- nrow(X) else n <- length(X)
byvar <- sample(1:3, n, replace=1)
byvar <- sort(byvar)

# for each subgroup
mands( X, by=byvar )
mands( X, freq, by=byvar )
mands( X, freq, by=byvar, probs=c(0.25,0.75) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UTF8pgm.R
\name{substr2}
\alias{substr2}
\title{Substring Using Column Location as the Unit.}
\usage{
substr2(x, from, to = 0, strict = 0, trim = 0, ...)
}
\arguments{
\item{x}{A string vector}

\item{from}{The first column (location in a line of a text file)}

\item{to}{The last column or 0 meaning the end of string}

\item{strict}{= 1 to check if start and stop are valid.}

\item{trim}{= 1 to trim the result by lazy.tools::trim}

\item{...}{options for lazy.tools::trim}
}
\value{
A vector consisting of the substrings of the input vector.
The result will be a list if the input is a string vector.
}
\description{
Substring Using Column Location as the Unit.
}
\details{
It is assumed that the double or triple byte character takes two colums
of a text file.  (Zenkaku Character) \cr
See the description of \code{cchar}.

When scrict=1, if the start or stop value is invalid, NA will be returned.\cr
When strict=0, if the start value is valid and the stop value is invalid,
stop will be set to the location of the last character.

When trim is used, the result will be re-encoded to UTF-8.
}
\examples{

# A string consisting of single and multi byte characters
# The 3rd, 6th and 7th characters are uppercase Zenkaku Romans.
{if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
 nza <- c(97, 98, 239, 188, 163, 100, 101, 239, 188, 166, 239, 188, 167, 104)
else
 nza <- c(97,98,130,98,100,101,130,101,130,102,104)}
za <- rawToChar(as.raw(nza))
# za <- num2char(nza)

printm(za)
# location of the multi byte characters
is.DBCS(za)
printm( nchar(za), nchar2(za), nchar(za)+sum(is.DBCS(za)) )
printm(unlist(strsplit(za,"")),nbyte(za),cchar(za))
# The first to the fourth characters
substr(za,1,4)
# The characters of za in the first to the fourth column.
substr2(za,1,4)


# Another example
{if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 )
 nn <- c(87, 101,  32, 104,  97, 118, 101,  32, 228, 184, 141, 232, 184
        , 176, 231, 159, 169,  32, 121, 101, 116,  33)
else
 nn <- c(87, 101,  32, 104,  97, 118, 101,  32, 149, 115, 230, 250, 139
         , 233, 32, 121, 101, 116,  33)}
za2=num2char(nn)
printm( nchar(za2), nchar2(za2), nchar(za2)+sum(is.DBCS(za2)) )
printm(1:nchar(za2),unlist(strsplit(za2,"")),nbyte(za2),cchar(za2))

# Note that start and stop must be the numbers contained in, resp,
# the first and the second columns of the returning matrix from
# cchar function.
substr2(za2,4,7)    # ok
substr2(za2,4,9)    # NG
substr2(za2,4,10)   # ok
substr2(za2,4,11)   # NG
substr2(za2,4,12)   # ok
substr2(za2,4,9, strict=1)    # NG
substr2(za2,4,11, strict=1)   # NG
substr2(za2,9,10, strict=1)   # ok
substr2(za2,9,11, strict=1)   # NG




# reading Shift-JIS files on from R.4.1.3
nobs <- 10
infile1 <- system.file("TestData1.txt",package="lazy.tools")
if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 ){
 temp1 <- ReadLines(infile1, n=nobs+2)[-(1:2)]
} else{
 # The result will be Double Byte.
 temp1 <- ReadLines(infile1, n=nobs+2, asDB=1)[-(1:2)]
 # The result will be Triple Byte.
 temp11 <- ReadLines(infile1, n=nobs+2)[-(1:2)]

 nbyte(temp1)
 nbyte(temp11)
 identical(temp1[-nobs],temp11[-nobs])  # true
 identical(nbyte(temp1[-nobs]),nbyte(temp11[-nobs])) # false
}

# reading UTF-8
nobs <- 10
infile2 <- system.file("TestData2.txt",package="lazy.tools")
temp2 <- ReadLines(infile2, n=nobs+2, encoding="UTF-8")[-(1:2)]
nbyte(temp2)

# starting and ending columns of each field
from=c(1, 7,25,26,27,28,29,30,31,32,34)
to=c(  5,23,25,26,27,28,29,30,31,32,39)

# character locations corresponding to from and to: should be equal.
FTc1=cchar( temp1, from, to )
FTc2=cchar( temp2, from, to )
char=strsplit(temp1[1],"")[[1]]
printm( char, from,to, FTc1[[1]],FTc2[[1]] )

# substringing using FTc
substring( temp2[1], FTc1[[1]][,1], FTc2[[1]][,2] )
substring( temp2[2], FTc1[[2]][,1], FTc2[[2]][,2] )

# get the field values as substrings  using substr2 function
s12=substr2(temp1,from[2],to[2])
s14=substr2(temp1,from[4],to[4])
s111=substr2(temp1,from[11],to[11])
s22=substr2(temp2,from[2],to[2])
s24=substr2(temp2,from[4],to[4])
s211=substr2(temp2,from[11],to[11])

# comparison
printm(s14,s24)

# Note that s111 and s211 are trimmed because they come from
# the variable length last columns.
# (On R4.1.3  s21 and s11 cannot be printmed together by printm.)
print.listof(list(s12=s12,s22=s22,s111=s111,s211=s211))

# On R4.1.3,
# substr or substr2 does not change the encoding.
# lazy.tools::trim changes the encoding.
# s12t=trim(s12)
# printm( s12[1], s12t[1] )
# printm( nbyte(s12[1]),nbyte(s12t[1]) )
# This does not.
# s12tt=substr(s12,1,nchar(trim(s12)))
# printm(s12[1],s12tt[1])
# printm(nbyte(s12[1]),nbyte(s12tt[1]))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/design_expand.R
\name{design_expand}
\alias{design_expand}
\title{Dummy Expantion of a Vector}
\usage{
design_expand(
  x,
  type = 0,
  drop = "last",
  levels = NULL,
  froot = "level",
  sort = 0
)
}
\arguments{
\item{x}{a vector to be expanded}

\item{type}{= -1 for non-full-rank design matrix used in SAS proc GLM \cr
= 0  for zero-sum design matrix \cr
= 1  for drop-one-column design matrix \cr}

\item{drop}{= "last"   or    "first"  when type != -1\cr}

\item{levels}{= a string vector consisting of the labels for the levels}

\item{froot}{= A character to be used to name the columns of X matrix}

\item{sort}{= 1 to sort the levels as the native factor function does. \cr
This is the default behavior of native model.matrix function.}
}
\value{
Design matrix of the size length(x) times length(factor(x))
or length(x) times length(factor(x))-1
}
\description{
Dummy Expantion of a Vector
}
\details{
By default, the zero-sum design matrix with the last level dropped
will be generated.
}
\examples{
design_expand( c("a","b","c") )
design_expand( c("a","xb","c") )
design_expand( c("a","b","c"), sort=1 )

design_expand( c("a","b","c"), type=-1 )
design_expand( c("a","b","c"), levels=c("c","a","b"), type=-1 )

design_expand( 1:3, type=0 )
design_expand( c("a","b","c"), froot="A" )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UTF8pgm.R
\name{ReadLines}
\alias{ReadLines}
\alias{WriteLines}
\title{Read Lines or Wite Lines with or without Encoding Option}
\usage{
ReadLines(filename, encoding = "Shift-JIS", asDBCS = 0, ...)

WriteLines(text, filename, encoding = "Shift-JIS", ...)
}
\arguments{
\item{filename}{an input text file name}

\item{encoding}{= "Shift-JIS" | "UTF-8" | NULL
where NULL is for system default and should be equal to "native.enc".}

\item{asDBCS}{= 1 to create double byte characters when
encoding="Shift-JIS" is specified.}

\item{...}{aditional parameters to readLines and writeLines}

\item{text}{a text vector to be written}
}
\value{
ReadLines returns a vector of strings.
}
\description{
Read Lines or Wite Lines with or without Encoding Option
}
\details{
Note that, as described in the Arguments section of native readLines,
specifying encoding option when calling readLines cannot read lines
from a file using the specified encoding. \cr
This function specifies encoding when defining the connection
and closes it upon return.

The following code
\preformatted{
 con <- file( filename, encoding="Shift-JIS" )
 return( readLines( con, ... )  )
}
makes the Zenkaku characters 3 byte chracters.
Therefore, when asDBCS=1, native \code{readLines} will be used.

When writing UTF-8 characters which are not included in native encoding,
specify \code{useBytes=TRUE} option.\cr
Check to see that \code{Encoding(text)} returns "UTF-8", not "unknown".

Above can avoid the following default action of \code{writeLines}:
\preformatted{
 useBytes is for expert use. Normally (when false) character strings with
 marked encodings are converted to the current encoding before being passed
 to the connection (which might do further re-encoding).
 useBytes = TRUE suppresses the re-encoding of marked strings so they are
 passed byte-by-byte to the connection: this can be useful when strings
 have already been re-encoded by e.g. iconv.
 (It is invoked automatically for strings with marked encoding "bytes".)
 }
}
\examples{
# In the examples below, TestData1.txt is encoded in Shift-JIS
# and  TestData2.txt, in UTF-8. The last line of TestData2.txt contains
# non Shif-JIS characters.
#
# These files are stored in the folder where lazy.tools package is stored.

# reading Shift-JIS files on from R.4.1.3
nobs <- 10
infile1 <- system.file("TestData1.txt",package="lazy.tools")
if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) > 0 ){
 temp1 <- ReadLines(infile1, n=nobs+2)[-(1:2)]
} else{
 # The result will be Double Byte.
 temp1 <- ReadLines(infile1, n=nobs+2, asDB=1)[-(1:2)]
 # The result will be Triple Byte.
 temp11 <- ReadLines(infile1, n=nobs+2)[-(1:2)]

 nbyte(temp1)
 nbyte(temp11)
 identical(temp1[-nobs],temp11[-nobs])  # true
 identical(nbyte(temp1[-nobs]),nbyte(temp11[-nobs])) # false
}



# reading UTF-8
nobs <- 10
infile2 <- system.file("TestData2.txt",package="lazy.tools")
temp2 <- ReadLines(infile2, n=nobs+2, encoding="UTF-8")[-(1:2)]
nbyte(temp2)

identical(temp1[-nobs],temp2[-nobs])  # true
if( regexpr("utf8",Sys.getlocale (category = "LC_CTYPE")) <= 0 ){
 identical(temp11[-nobs],temp2[-nobs])  # true
 identical(nbyte(temp11[-nobs]),nbyte(temp2[-nobs]))  # false
}



# reading UTF-8 with BOM
nobs <- 10
infile3 <- system.file("TestData3.txt",package="lazy.utf8")
temp3 <- ReadLines(infile3, n=nobs+2, encoding="UTF-8")[-(1:2)]
nbyte(temp3)
identical(temp2[-nobs],temp3[-nobs])  # true



\dontrun{
# These two work
WriteLines( temp1, "temp11.txt" )
WriteLines( temp1, "temp12.txt", encoding="UTF-8" )

# These two work exept for the last obs which has non Shift-JIS characters.
WriteLines( temp2, "temp21.txt" )
WriteLines( temp2, "temp22.txt", encoding="UTF-8"  )

# This works.
con <- file( "temp222.txt", encoding="native.enc" )
writeLines( temp2, con , useBytes=TRUE )
close(con)
}


# creating data frame using column specifications
# column spec
from <- c(1, 7,25,26,27,28,29,30,31,32,34)
to <-   c(5,23,25,26,27,28,29,30,31,32,39)
name <- c("id", "name",paste("Q",1:8,sep=""),"comment")

# convert to a character matrix by stacking each row (record, observation)
datamat2 <- Reduce( rbind, substr2(temp2, from,to), init=NULL )
colnames(datamat2) <- name
# matrix to data frame
df2 <- as.data.frame(datamat2)

# Alternative way by stacking each column (variable)
datamat22 <- Reduce( cbind, mapply( function(x,y) substr2(temp2,x,y)
                   , from, to, SIMPLIFY=FALSE ), init=NULL )
colnames(datamat22)=name
# matrix to data frame
df22 <- as.data.frame(datamat22)

identical(datamat2,datamat22)  # true
identical(df2,df22)  # true

print(datamat2)

# character to numeric conversion
df2[,paste("Q",1:8,sep="")] <- apply(df2[,paste("Q",1:8,sep="")],2
                                    ,as.numeric)
str(df2)

# cannto print UTF-8 data frame nicely by native print.data.frame.
print(df2)
print.listof(df2)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reflect_fa.R
\name{reflect_fa}
\alias{reflect_fa}
\title{Reflect the signs of the columns of the factor pattern matrix
so that the number of positive loadings is larger than the negative ones.}
\usage{
reflect_fa(A, sortcols = 0, unclass = 0)
}
\arguments{
\item{A}{Factor Pattern matrix or an object with $loadings.}

\item{sortcols}{= 1 to sort factors (columns of A) according to
factor contributions.}

\item{unclass}{= 1 to unclass the result}

\item{print}{= 1 to print the result}
}
\value{
A list of \cr
loadings = reordered loadings matrix \cr
col_order = sort order of the cols \cr
sign = a vector indicating if the signs were preserved or not
}
\description{
Reflect the signs of the columns of the factor pattern matrix
so that the number of positive loadings is larger than the negative ones.
}
\details{
First, if sortcols == 1, the columns of \code{A} are reordered
according to the magnitude of \code{diag(t(A)\%*\%A)}. \cr
Then, for each column of \code{A},
the sign will be reflected if the number of negative loadings is
greater than the positive ones.
}
\examples{
set.seed(1701)
n <- 20; ndim <- 4
A0 <- matrix(runif(n*ndim),n,ndim)
dimnames(A0)=list( paste("v",1:n,sep=""),paste("f",1:ndim,sep=""))
A0v <- varimax(A0)$loadings
A0v <- A0v*matrix(sample(c(-1,1,1),n*ndim,replace=TRUE),n,ndim)
A0v[,3:4] <- -A0v[,3:4]

A0vs <- reflect_fa(A0v)$loadings
Print(A0v,A0vs, fmt="6.3")

A0vsc <- reflect_fa(A0v, sortcol=1)$loadings
Print(A0v,A0vsc, fmt="6.3")

# attributes
print(reflect_fa(A0v)$loadings)
print(reflect_fa(A0v, unclass=1)$loadings)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sort_missing_data.R
\name{sort_missing_data}
\alias{sort_missing_data}
\title{Sort Matrix/Data Frame According to the Pattern of Missing Elements}
\usage{
sort_missing_data(X, misslast = 1, nobs = 0)
}
\arguments{
\item{X}{a data frame or matrix to be sorted.}

\item{misslast}{= 1 to treat NA as the largest,
= 1 to treat NA as the smallest.}

\item{nobs}{= 1 to sort by the # of observed elements first \cr
= -1 to sort by the descending # of observed elements first \cr
= 0 not to pay any attention to the # of missing elements.}
}
\value{
A list of (sorted_data, freq, Nobs, class, od) \cr
where sorted_data is of the same size as X, \cr
freq is the vector consisting of the # of observatons for each pattern, \cr
Nobs is the vector of length nrow(X)
containing the # of observed elements for each row of X.\cr
class is the interger in [1,length(freq)] indicating the classification
of each obs according to the missing pattern. \cr
od is the order to be used to sort X as X_sorted=X[od,].
}
\description{
Sort Matrix/Data Frame According to the Pattern of Missing Elements
}
\examples{
set.seed(17029)
nn=50
dataframe=0

if( dataframe ){
 x=sample(0:9,nn,replace=1)
 x[runif(nn) < 0.5]=NA
 x2=sample(letters[1:10],nn,replace=1)
 x2[runif(nn) < 0.5]=NA
 x3=sample(LETTERS[1:10],nn,replace=1)
 x3[runif(nn) < 0.5]=NA
 X=data.frame(x,x2,x3, stringsAsFactors=0)
} else{
 x=sample(0:9,nn,replace=1)
 x[runif(nn) < 0.3]=NA
 x2=sample(1:10,nn,replace=1)
 x2[runif(nn) < 0.3]=NA
 x3=sample(1:10,nn,replace=1)
 x3[runif(nn) < 0.3]=NA
 X=cbind(x,x2,x3)
}
res=sort_missing_data( X, misslast=1, nobs=-1 )
Xs=res$sorted_data
freqs=res$freq
Nobss=res$Nobs
od=res$od

# Nobsss should be equal to Nobs, Xss, to Xs
Nobsss=ncol(X)-apply(Xs,1,function(x) sum(is.na(x)))
Xss=X[od,]
pat=rep(1:length(freqs),freqs)
Print(Xs,Xss,Nobss, pat,od)

misspat=Xs[cumsum(res$freq),]
nobs=ncol(misspat)-apply(misspat,1,function(x) sum(is.na(x)))
Print(misspat,nobs)

mands( Xs, by=pat )


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pref2dist.R
\name{pref2dist}
\alias{pref2dist}
\title{Convert Japanese Prefecture to District}
\usage{
pref2dist(pref = NULL, type = 1, dist = NULL, dump = 0)
}
\arguments{
\item{pref}{string vector containing prefecture names}

\item{type}{= 1 to use 8 districs
= 2 to use 14 districts}

\item{dist}{Not Yet Available}

\item{dump}{= 1 to return the prefecture-district table}
}
\value{
a string vector of the same length as pref containing district names
\cr
or, when dump=1, a table of prefecturesn and the corresponding districts.
}
\description{
Convert Japanese Prefecture to District
}
\examples{

# string vector containing prefecture names
Prefectures <- pref2dist(dump=1)$pref

# prefecture name to prefecture number
PrefecturesNo <- 1:47
names(PrefecturesNo) <- Prefectures
pn <- PrefecturesNo[Prefectures[c(13,26)]]

# prefecture name to district
d1 <- pref2dist( Prefectures )
d2 <- pref2dist( Prefectures, type=2 )
pn <- PrefecturesNo[Prefectures]
printm( pn, Prefectures, d1, d2 )

# invalid prefecture names
pref <- c(Prefectures[13],"Iowa", Prefectures[26])
pref2dist( pref, type=1 )
pref2dist( pref, type=2 )

}
